<?php
require_once('..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'secure.php');
require_once('..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'common.php');
require_once('followup_form_shared_constants.php');
require_once("Renew" . DS . "Utilities" . DS . "Validator.php");
require_once("Renew" . DS . "DbManagerWifi.php");
require_once("Renew" . DS . "StringHelper.php");
use \Renew\DbManagerWifi as DbManagerWifi;
use \Renew\Validator as Validator;

$validator = new Validator();

// Suffixes used to get icon, title, and type from checkbox value
define('ICON_SUFFIX', '_icon');
define('TITLE_SUFFIX', '_title');
define('TYPE_SUFFIX', '_type');

// Constants
define('STRING_NO_ICON', 'none');

// ERRORS
define('ERR_MAX_FIELDS_EXCEEDED', sprintf(_('The maximum number of form fields is %s'), MAX_FIELDS));

if (isset($_POST) && isset($_POST['ajax'])) {
  $isSetupWizard = FALSE;
  if (isset($_POST['is_setup_wizard'])) {
    $isSetupWizard = TRUE;
  }

  // Enable checkbox
  $formFollowupEnabled = Validator::formatCheckbox(@$_POST['form_followup_enabled']);

  if ($formFollowupEnabled === Validator::STRING_TRUE) {

    // Send email checkbox
    $formEmailEnabled = Validator::formatCheckbox(@$_POST['form_email_enabled']);

    if ($formEmailEnabled === Validator::STRING_TRUE) {
      // Reporting email (not allowed to be false here, since that wouldn't make any sense)
      $reportingEmail = $_POST['reporting_email'];
      $validator->validateInputEmail($reportingEmail, _('Reporting Email'), FALSE);

      // Send email stats checkbox
      $formEmailStatsEnabled = Validator::formatCheckbox(@$_POST['form_email_stats_enabled']);
    }

    // Text fields
    $title = $_POST['title'];
    $validator->validateTextInputLength($title, _('Form Title'));

    if (!$isSetupWizard) {
      $submitSuccessTitle = $_POST['submit_success_title'];
      $submitButtonTitle = $_POST['submit_button_title'];
      $cancelButtonTitle = $_POST['cancel_button_title'];
      $buttonTitle = $_POST['button_title'];

      // Associative array of titles and corresponding error prefixes
      $textFields = array(
        $submitSuccessTitle => _('Submit Success Title'),
        $submitButtonTitle => _('Submit Button Title'),
        $cancelButtonTitle => _('Cancel Button Title'),
        $buttonTitle => _('Homepage Button Title'),
      );

      // Validate text fields
      foreach ($textFields as $text => $errorPrefix) {
        $validator->validateTextInputLength($text, $errorPrefix);
      }

      // Form Description
      $description = $_POST['description'];
      $validator->validateTextFieldLength($description, _('Form Description'));

      // Button icon
      $icon = $_POST['icon'];
      $validator->validateNotEmpty($icon, _('Form Button Icon'));
      if ($icon === STRING_NO_ICON) {
        $icon = '';
      }
      // Save as a hex code
      $icon = \Renew\StringHelper::ConvertHTMLToHex($icon);
    }

    $formFieldSaveArray = array();

    // Validate form fields
    if (isset($_POST['field_checkbox_array'])) {
      $fieldCheckboxArray = $_POST['field_checkbox_array'];

      // Validate max number of fields and areas
      $totalChecked = count($fieldCheckboxArray);
      if ($totalChecked > MAX_FIELDS) {
        $validator->addFormErrorString(ERR_MAX_FIELDS_EXCEEDED);
      }

      foreach ($fieldCheckboxArray as $fieldCheckboxValue) {
        // Title
        $fieldTitle = $_POST[$fieldCheckboxValue . TITLE_SUFFIX];
        $validator->validateTextInput($fieldTitle, _('Problem with a field title'));

        // Type
        $type = $_POST[$fieldCheckboxValue . TYPE_SUFFIX];
        $validator->validateNotEmpty($type, _('Problem with a field type'));

        // Icon
        $fieldIcon = $_POST[$fieldCheckboxValue . ICON_SUFFIX];
        $validator->validateNotEmpty($fieldIcon, _('Problem with a field icon'));
        if ($fieldIcon === STRING_NO_ICON) {
          $fieldIcon = '';
        }
        $fieldIcon = \Renew\StringHelper::ConvertHTMLToHex($fieldIcon);

        $formFieldSaveArray[] = array(
          'title' => $fieldTitle,
          'type' => $type,
          'icon' => $fieldIcon
        );
      }
    }

    // Form general settings
    $saveSettingsForm = array(
      'form_followup_enabled' => $formFollowupEnabled,
      'form_email_enabled' => $formEmailEnabled
    );

    // If email enabled, save email stats setting
    if ($formEmailEnabled === Validator::STRING_TRUE) {
      $saveSettingsForm['form_email_stats_enabled'] = $formEmailStatsEnabled;
      $saveSettingsForm['reporting_email'] = $reportingEmail;
    }

    // Followup form settings
    $saveSettingsFollowupForm = array(
      'title' => $title,
      'button_title' => $title // If in setup wizard, set button title to title
    );
    if (!$isSetupWizard) {
      $saveSettingsFollowupForm['button_title'] = $buttonTitle;
      $saveSettingsFollowupForm['icon'] = $icon;
      $saveSettingsFollowupForm['description'] = $description;
      $saveSettingsFollowupForm['submit_success_title'] = $submitSuccessTitle;
      $saveSettingsFollowupForm['submit_button_title'] = $submitButtonTitle;
      $saveSettingsFollowupForm['cancel_button_title'] = $cancelButtonTitle;
    }

  } else { // followup form disabled
    $saveSettingsForm = array(
      'form_followup_enabled' => $formFollowupEnabled
    );
  }

  // Save
  if ($validator->formErrorsIsEmpty()) {
    $dbManagerWifi->openDb();
      // Save Form settings
      if (!$dbManagerWifi->settingsSetKeyValueArray($saveSettingsForm)) {
        $validator->addFormErrorSaveFailed();
      }

      // Save options and insert fields only if form is enabled
      if ($formFollowupEnabled === Validator::STRING_TRUE) {
        $success = TRUE;
        if (!($dbManagerWifi->formsUpdateAttrsForFormWithId(DbManagerWifi::FOLLOWUP_FORM_ID, $saveSettingsFollowupForm))
            || !($dbManagerWifi->formFieldsUpdateOrInsertFieldsForForm(DbManagerWifi::FOLLOWUP_FORM_ID, $formFieldSaveArray))) {
          $success = FALSE;
        }
        if ($success === FALSE) {
          $validator->addFormErrorSaveFailed();
        }
      }
    $dbManagerWifi->closeDb();
  }

  // Return to ajax
  echo $validator->ajaxFormResponse();
  exit();
}

// NOTE: includes variables, databases calls, and smarty assignments.
// Used on settings.php and setup_followup_form.php
require_once('.' . DS . 'followup_form_settings_variables.php');

$t->assign("page_title", _('Follow-up Form Settings'));
$t->assign('current_page', 'manage');
$t->assign('current_manage', 'followup_form_settings');
templateDisplayPageAdmin('admin' . DS . 'manage' . DS . 'followup_form' . DS . 'settings.tpl');
