<?php
require_once('..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'secure.php');
require_once('..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'common.php');
require_once("Renew" . DS . "DateHelper.php");
require_once("Renew" . DS . "DbManagerMedia.php");
require_once("Renew" . DS . "Utilities" . DS . "Validator.php");
require_once('..' . DS . '..' . DS . 'includes' . DS . 'add_image_thumbnail.php'); // shared code with add_video_thumbnail.php
require_once('..' . DS . '..' . DS . 'includes' . DS . 'validate_language_iso.php');
use \Renew\Validator;
use \Renew\DbManagerMedia;
use \Renew\StringHelper;

define('IMAGE_CURRENT', 'current');
define('PREFIX_PASSWORD', _('Password'));

$dbManagerMedia = new DbManagerMedia(MEDIA_DB, MEDIA_DIR, USER_IMAGES_DIR);
$validator = new Validator();


if (isset($_POST, $_POST['ajax'])) {

  $isMultiEditForm = isset($_POST['multi_edit']);
  $isUploadForm = isset($_POST['upload_form']);
  $addToDatabase = isset($_POST['add_to_database']);
  $isLangugeDirectory = isset($_POST['is_language_directory']);

  $selectImageErrorPrefix = _('Select Image');
  $descriptionErrorPrefix = _('Description');
  $titleErrorPrefix = _('Title');
  if ($isLangugeDirectory) {
    $selectImageErrorPrefix = _('Greeting Image');
    $descriptionErrorPrefix = _('Greeting Message');
    $titleErrorPrefix = _('Language Display Name');
  }

  // Media image
  $selectImage = $_POST['select_image'];
  // if there is no select image, then assume you want to keep current
  if (empty($selectImage)) {
  	$selectImage = Validator::IMAGE_SELECT_CURRENT_STRING;
  }

  $validator->validateNotEmpty($selectImage, $selectImageErrorPrefix);

  // Use 0, no image if user has selected no image
  if ($selectImage === Validator::IMAGE_SELECT_DEFAULT_STRING) {
    $selectImage = 0;
  } else if ($selectImage === Validator::IMAGE_SELECT_CURRENT_STRING) {
    $selectImage = IMAGE_CURRENT;
  } else if ($selectImage === Validator::IMAGE_SELECT_CURRENT_THUMBNAIL_STRING) {
    if (!empty($_POST['thumbnail_id'])) {
      $selectImage = $_POST['thumbnail_id'];
    } else {
      $fileId = $_POST['file_id'];
      $newThumbnailId = addImageThumbnail($fileId);
      $selectImage = IMAGE_CURRENT;
      if ($newThumbnailId !== FALSE) {
        $selectImage = $newThumbnailId;
        $newThumbnailSet = TRUE;
      }
    }
  }

  // Input: description
  $description = $_POST['description'];
  $validator->validateTextFieldLength($description, $descriptionErrorPrefix);

  if ($isMultiEditForm || $isUploadForm) {

    // Availability
    $wifiStatus = Validator::formatCheckboxAvailability(@$_POST['wifi_status']);
    // $bluetoothStatus = Validator::formatCheckboxAvailability(@$_POST['bluetooth_status']);
    $usbStatus = Validator::formatCheckboxAvailability(@$_POST['usb_status']);

  } else { // for media_item.tpl
    // Title
    $title = $_POST['title'];
    $validator->validateTextInput($title, $titleErrorPrefix);

    // Filename
    $filename = $_POST['filename'];
    $validator->validateTextInput($filename, _('File Name'));

    // Advanced Settings
    
    // Password
    $isPasswordProtected = Validator::formatCheckboxOneZero(@$_POST['is_password_protected']);
    $isPasswordProtectedBoolean = Validator::formatCheckboxBoolean(@$_POST['is_password_protected']);
    $isPasswordProtectedInitialValueBoolean = Validator::formatCheckboxBoolean(@$_POST['is_password_protected_initial_value']);
    if ($isPasswordProtectedBoolean) {
      $password = StringHelper::RemoveWhitespaceFromPassword($_POST['password']);

      // If the password was enabled before, allow password to remain blank (unchanged)
      if ($isPasswordProtectedInitialValueBoolean) {
        $validator->validateTextInputLength($password, PREFIX_PASSWORD);
      } else {
        // If they have just enabled the password, don't allow blank password
        $validator->validateTextInput($password, PREFIX_PASSWORD);
      }
    }

    // Is Download Disabled Checkbox
    $isDownloadDisabled = Validator::formatCheckboxOneZero(@$_POST['is_download_disabled']);
    $isGlobal = Validator::formatCheckboxOneZero(@$_POST['is_global']);

    if ($isLangugeDirectory) {
      $languageIso = validateLanguageIso();

      // Greeting Title
      $greetingTitle = $_POST['greeting_title'];
      $validator->validateTextInputLength($greetingTitle, _('Greeting Title'));

      // Promted media file
      $promotedMediaFileId = $_POST['promoted_media_file_id'];
      $validator->validateNotEmpty($promotedMediaFileId, _('Promote a media file'));
    }
  }

  // Save
  if ($validator->formErrorsIsEmpty()) {
      $updateArray = array(
        'datetime_edited' => DateHelper::DateTimeStringSQLite3(),
      );

      if ($isMultiEditForm || $isUploadForm) {

        // Used with multi-edit form
        $shouldChangeAvailability = TRUE;
        if ($isMultiEditForm) {
          $shouldChangeAvailability = Validator::formatCheckboxBoolean(@$_POST['change_availability']);
        }

        // Conditionally include settings to update
        if ($shouldChangeAvailability) {
          $updateArray['wifi_status'] = $wifiStatus;
          // $updateArray['bluetooth_status'] = $bluetoothStatus;
          $updateArray['usb_status'] = $usbStatus;
        }
        if ($description != '') { // Blank descriptions don't change things on multi-edit or upload
          $updateArray['description'] = $description;
        }
        if ($selectImage !== IMAGE_CURRENT) {
          $updateArray['image_id'] = $selectImage;
        }

        if ($isMultiEditForm) {
          $selectedMediaArray = $_POST['selected_media_array'];
          $dbManagerMedia->openDb();
            foreach($selectedMediaArray as $fileId) {
              if(!$dbManagerMedia->mediaUpdateAttrsForFileWithId($fileId, $updateArray)) {
                $validator->addFormErrorSaveFailed();
              }
            }
          $dbManagerMedia->closeDb();
        } else if ($isUploadForm && $addToDatabase) {
          $updateArray['description'] = $description;

          // Prepare to add to database
          $tempFilePath = DOC_ROOT . $_POST['temp_file_path']; // Starts with '/'
          $directory = $_POST['directory'];

          // Add to database
          $dbManagerMedia->openDb();
            $fileId = $dbManagerMedia->mediaAddFileWithPathToMediaDirAndDb($tempFilePath, $directory);
            if ($fileId === FALSE) {
              error_log("Error: Add to database failed.");
              $validator->addFormErrorSaveFailed();
            } else {
              // Save
              if (!$dbManagerMedia->mediaUpdateAttrsForFileWithId($fileId, $updateArray)) {
                error_log("Error: Save media failed.");
                $validator->addFormErrorSaveFailed();
              }
            }
          $dbManagerMedia->closeDb();
        }
      } else { // this is for updating a single piece of media ...
        $updateArray['title'] = $title;
        $updateArray['description'] = $description;
        $updateArray['is_password_protected'] = $isPasswordProtected;
        $updateArray['is_download_disabled'] = $isDownloadDisabled;
        $updateArray['is_global'] = $isGlobal;
        $updateArray['image_id'] = $selectImage;
        $fileId = $_POST['file_id'];

        if ($isLangugeDirectory) {
          $updateArray['greeting_title'] = $greetingTitle;
          $updateArray['language_iso'] = $languageIso;
          $updateArray['promoted_media_file_id'] = $promotedMediaFileId;
        }

        // Save
        $dbManagerMedia->openDb();
          // Save settings, rename file
          $success = TRUE;
          $success &= $dbManagerMedia->mediaUpdateAttrsForFileWithId($fileId, $updateArray);
          $success &= $dbManagerMedia->mediaRenameFileWithId($fileId, $filename);
          if ($isPasswordProtectedBoolean) {
            $success &= $dbManagerMedia->mediaPasswordsSetPasswordWithFileId($fileId, $password);
          } else {
            $success &= $dbManagerMedia->mediaPasswordsRemovePasswordWithFileId($fileId);
          }

          if (!$success) {
            $validator->addFormErrorString($adminErrorMessages['ERR_FILE_NAME_MAY_ALREADY_EXIST']);
          }
        $dbManagerMedia->closeDb();
      }
  }

  if ($validator->formErrorsIsEmpty() && $isUploadForm && $addToDatabase) {
    $dbManagerMedia->openDb();
      $mediaItem = $dbManagerMedia->mediaGetFileWithId($fileId);
      $mediaItem = $dbManagerMedia->mediaAddFileDetailsToFile($mediaItem);
      $mediaItem['datetime_edited'] = ''; // Reset edited time
      $imagesArray = $dbManagerMedia->imagesGetAll();
    $dbManagerMedia->closeDb();

    // Put media item and directory data together to send to JS
    $mediaItemArray['media_item'] = $mediaItem;
    $mediaItemArray['images_array'] = $imagesArray;
  }

  // Return to Ajax
  if (isset($mediaItemArray) && $mediaItemArray['media_item'] && $isUploadForm && $addToDatabase) {
    echo $validator->ajaxFormResponse(json_encode($mediaItemArray));
  } else if (isset($newThumbnailSet)) { // New thumbnail has been saved
    echo $validator->ajaxFormResponse($newThumbnailId);
  } else {
    echo $validator->ajaxFormResponse();
  }
  exit();
}
