<?php
require_once('Renew' . DS . 'Utilities' . DS . 'CSVGenerator.php');
require_once('Renew' . DS . 'DbManagerBluetooth.php');
require_once('Renew' . DS . 'DbManagerSd.php');
require_once('Renew' . DS . 'DbManagerWifi.php');
require_once('Renew' . DS . 'DiskInfo.php');
require_once('Renew' . DS . 'DeviceInfo.php');
require_once('Renew' . DS . 'Utilities' . DS . 'Validator.php');
use \Renew\Validator;

/*
  NOTE: IMPORTANT! Do not use '&' in strings. Some browsers cannot send this in an email mailto call.
*/
global $dbManagerWifi, $dbManagerSd, $dbManagerBluetooth, $statisticsIncludeRawData;
$dbManagerWifi = new \Renew\DbManagerWifi(LSWIFI_DB);
$dbManagerSd = new \Renew\DbManagerSd(LSSD_DB);
$dbManagerBluetooth = new Renew\DbManagerBluetooth(LSBT_DB);
$statisticsIncludeRawData = Validator::StringToBoolean($dbManagerWifi->settingsGetValueForKey('statistics_include_raw_data'));

/**
 * Handle version information
 *
 **/
function printGeneralStatistics(&$csvGenerator) {
  printGeneralStatisticsFollowupForm($csvGenerator);
  $humanReadableDiskTotalSpace = Renew\DiskInfo::HumanReadableDiskTotalSpace(MEDIA_DIR);
  $csvGenerator->addRow(array('Total Disk Space', $humanReadableDiskTotalSpace));
  $humanReadableDiskFreeSpace = Renew\DiskInfo::HumanReadableDiskFreeSpace(MEDIA_DIR);
  $csvGenerator->addRow(array('Free Disk Space', $humanReadableDiskFreeSpace));
  $csvGenerator->addRow(array());
  $csvGenerator->addRow(array());
}

/*
  Print general statistics, used both in followup form email and in main CSV
*/
function printGeneralStatisticsFollowupForm(&$csvGenerator) {
  $firmwareVersion = \Renew\DeviceInfo::GetLightStreamFirmwareVersion();
  $csvGenerator->addRow(array('Firmware Version', $firmwareVersion));
  $raspberryPiSerialNumber = \Renew\DeviceInfo::GetLightStreamSerialNumber();
  $csvGenerator->addRow(array('System ID', $raspberryPiSerialNumber));
}

/*
  TODO Comment
*/
function printOverviewStatisticsWifi(&$csvGenerator) {
  $csvGenerator->addRow(array('WiFi Overview'));
  printOverviewStatisticsWifiFollowupForm($csvGenerator);
  $csvGenerator->addRow(array());
  $csvGenerator->addRow(array());
}

/*
  Print WiFi stats for followup form email and main CSV
*/
function printOverviewStatisticsWifiFollowupForm(&$csvGenerator) {
  global $dbManagerWifi;
  $csvGenerator->addRow(array('WiFi Total Downloads', $dbManagerWifi->statsGetSumTotal('download')));
  $csvGenerator->addRow(array('WiFi Total Streams', $dbManagerWifi->statsGetSumTotal('stream')));
  $csvGenerator->addRow(array('WiFI Total Unique Connected Devices', $dbManagerWifi->statsGetOverTimeUniqueDevices()['all']));
}

/*
  TODO Comment
*/
function printOverviewStatisticsSd(&$csvGenerator) {
  $csvGenerator->addRow(array('SD / USB Overview'));
  printOverviewStatisticsSdFollowupForm($csvGenerator);
  $csvGenerator->addRow(array());
  $csvGenerator->addRow(array());
}

/*
  Print SD copy stats for followup form email and main CSV
*/
function printOverviewStatisticsSdFollowupForm(&$csvGenerator) {
  global $dbManagerSd;
  $sdStats = $dbManagerSd->statsFindAll();
  $csvGenerator->addRow(array('SD / USB Number of Download Sessions', $sdStats['NUMBER_OF_EXECUTIONS']));
  $csvGenerator->addRow(array('SD / USB Number of Files Downloaded', $sdStats['NUMBER_OF_FILES']));
  $csvGenerator->addRow(array('SD / USB Total Data Downloaded', $sdStats['READABLE_SIZE']));
}

/*
  TODO Comment
*/
function printOverviewStatisticsBluetooth(&$csvGenerator) {
  global $dbManagerBluetooth;
  $csvGenerator->addRow(array('Bluetooth Overview'));
  $csvGenerator->addRow(array('Unique Devices Seen', $dbManagerBluetooth->statsCountUniqueDevices('all')));
  $csvGenerator->addRow(array('Unique Devices Transferred To', $dbManagerBluetooth->statsCountUniqueDevices('success')));
  $csvGenerator->addRow(array('Successful File Transfers', $dbManagerBluetooth->statsCountPushes('success')));
  // $csvGenerator->addRow(array('Rejected Transfers', $dbManagerBluetooth->statsCountPushes('rejected')));
  // $csvGenerator->addRow(array('Failed Transfers', $dbManagerBluetooth->statsCountPushes('failed')));
  $csvGenerator->addRow(array());
  $csvGenerator->addRow(array());
}

/*
  TODO Comment
*/
function printTopFiles(&$csvGenerator) {
  global $dbManagerWifi;

  $wifiTopFiles = $dbManagerWifi->statsGetStreamsAndDownloadsByFileid(5, TRUE);
  $csvGenerator->addRow(array(
    'Top Files',
    'Downloads',
    'Streams'
  ));
  foreach ($wifiTopFiles as $file) {
    $csvGenerator->addRow(array($file['path_rel'], $file['total_downloads'], $file['total_streams']));
  }
  $csvGenerator->addRow(array());
  $csvGenerator->addRow(array());
}

/*
  Get string with basic general, WiFi, and SD statistics for followup form email
  @return String CSV text representing the statistis
*/
function getFollowupFormEmailStatsString() {
  global $dbManagerWifi;
  global $dbManagerSd;
  global $dbManagerBluetooth;

  // Create CSV
  $randomString = uniqid();
  $csvFile = SYS_ROOT_DIR . DS . 'tmp' . DS . "{$randomString}_emailStats.csv";
  $csvGenerator = new Renew\Utilities\CSVGenerator($csvFile);

  // Print stats to csv
  printGeneralStatisticsFollowupForm($csvGenerator);
  printOverviewStatisticsWifiFollowupForm($csvGenerator);
  printOverviewStatisticsSdFollowupForm($csvGenerator);

  // Return String
  $answer = $csvGenerator->getString();
  $csvGenerator->close();
  unlink($csvFile);
  return $answer;
}

/*
  TODO Comment
*/
function getSummaryHelper($shouldDownload) {
  global $dbManagerWifi;
  global $dbManagerSd;
  global $dbManagerBluetooth;
  global $statisticsIncludeRawData;

  $csvFile = BASE_DIR . DS . 'reports' . DS . 'dashboard_summary.csv';
  $csvGenerator = new Renew\Utilities\CSVGenerator($csvFile);

  // General
  printGeneralStatistics($csvGenerator);

  // WiFi
  printOverviewStatisticsWifi($csvGenerator);
  $csvGenerator->addRow(array('WiFi File Totals'));
  $files = $dbManagerWifi->statsGetStreamsAndDownloadsByFileid();
  $csvGenerator->addRow(array('File Path', 'Total Downloads', 'Total Streams', 'File Name', 'File Extension', 'File Id'));
  foreach ($files as $file) {
    $csvGenerator->addRow(array($file['path_rel'], $file['total_downloads'], $file['total_streams'], $file['file_name'], $file['file_extension'], $file['file_id']));
  }
  $csvGenerator->addRow(array());
  $csvGenerator->addRow(array());
  $csvGenerator->addRow(array('WiFi Monthly Totals'));
  $year_months = $dbManagerWifi->statsGetStreamsAndDownloadsByMonth();
  $csvGenerator->addRow(array('Year'.'-'.'Month', 'Total Downloads', 'Total Streams'));
  foreach ($year_months as $year_month) {
    $csvGenerator->addRow(array($year_month['year_month'], $year_month['total_downloads'], $year_month['total_streams']));
  }
  $csvGenerator->addRow(array());
  $csvGenerator->addRow(array());
  if ($statisticsIncludeRawData) {
    $csvGenerator->addRow(array('WiFi Interactions'));
    $interactions = $dbManagerWifi->statsGetRecentActivity();
    $csvGenerator->addRow(array('Interaction Id', 'Date / Time' . ' (UTC)', 'Interaction Type', 'File Path', 'File Name', 'File Extension', 'File Id', 'User Id', 'Device Id'));
    foreach ($interactions as $interaction) {
      $csvGenerator->addRow(array($interaction['interaction_id'], $interaction['date_time'], $interaction['interaction_type'], $interaction['path_rel'], $interaction['file_name'], $interaction['file_extension'], $interaction['file_id'], $interaction['device_num'], $interaction['device_id']));
    }
    $csvGenerator->addRow(array());
    $csvGenerator->addRow(array());
  }

  // SD
  printOverviewStatisticsSd($csvGenerator);

  // Bluetooth
  printOverviewStatisticsBluetooth($csvGenerator);
  $files = $dbManagerBluetooth->statsFindAllSuccessfulPushedFiles();
  $csvGenerator->addRow(array('Bluetooth File Transfer Count'));
  $csvGenerator->addRow(array('File Path', 'Successful'));
  foreach ($files as $file) {
    $csvGenerator->addRow(array($file['file_path'], $file['total_successes']));
  }
  $csvGenerator->addRow(array());
  $csvGenerator->addRow(array());
  if ($statisticsIncludeRawData) {
    $filesAndDevices = $dbManagerBluetooth->statsFindAllSuccessfulAndScans();
    $csvGenerator->addRow(array('Bluetooth Device Data'));
    $csvGenerator->addRow(array('Date / Time' . ' (UTC)', 'Device Name', 'Device ID', 'Interaction Type'));
    foreach ($filesAndDevices as $file) {
      $csvGenerator->addRow(array($file['date_time'], $file['device_name'], $file['device_id'], $file['interaction_type']));
    }
    $csvGenerator->addRow(array());
    $csvGenerator->addRow(array());
    $blacklistedDevices = $dbManagerBluetooth->statsFindAllBlacklistedDevices();
    $csvGenerator->addRow(array('Bluetooth Blacklist Data'));
    $csvGenerator->addRow(array('Device ID', 'Device Name'));
    foreach ($blacklistedDevices as $device) {
      $csvGenerator->addRow(array($device['device_id'], $device['device_name']));
    }
    $csvGenerator->addRow(array());
    $csvGenerator->addRow(array());
  }


  // Close and download the csv file.
  $csvGenerator->close();
  if ($shouldDownload) {
    $csvGenerator->download();
    return;
  } else {
  	return $csvGenerator->getString(); //mail($email, 'LS Stats', $csvGenerator->getString());
  }
}

/*
  TODO Comment
*/
function downloadSummary() {
  getSummaryHelper(TRUE);
}

/*
  TODO Comment
*/
function getSummaryText() {
  return getSummaryHelper(FALSE);
}
