<?php
require_once('..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'secure.php');
require_once('..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'common.php');
use \Renew\DeviceInfo;
use \Renew\Validator;
use \Renew\DbManagerWifi;
use \Renew\SessionHelper;
use Jenssegers\Agent\Agent;
use \Renew\StringHelper;

define('ERR_PREFIX_PASSWORD', _('Password'));

$validator = new Validator();
$currentDeviceType = DeviceInfo::GetDeviceIdentifierWithAgent(new Agent());
$setReportingEmail = FALSE;

if (isset($_GET) && isset($_GET['set_reporting_email'])) {
  $setReportingEmail = TRUE;
}

if (isset($_POST) && isset($_POST['ajax'])) {
  $isSetupWizard = FALSE;
  if (isset($_POST['is_setup_wizard'])) {
    $isSetupWizard = TRUE;
  }

  // Password
  $password = StringHelper::RemoveWhitespaceFromPassword($_POST['password']);
  if ($isSetupWizard) {
    $validator->validateTextInput($password, ERR_PREFIX_PASSWORD);
    if ($password === 'admin') {
      $validator->addFormErrorString(_('Please Enter a New Password'));
    }
  } else {
    $validator->validateTextInputLength($password, ERR_PREFIX_PASSWORD);
  }

  // Confirm password
  $confirmPassword = StringHelper::RemoveWhitespaceFromPassword($_POST['confirm_password']);
  if ($password !== $confirmPassword) {
    $validator->addFormError('', Validator::ERR_PASSWORDS_DO_NOT_MATCH);
  }

  $passwordHint = $_POST['password_hint'];
  $validator->validateTextInputLength($passwordHint, _('Password Hint'));

  if (!$isSetupWizard) {
    // Username
    $userName =  StringHelper::RemoveWhitespaceFromPassword($_POST['user_name']);
    $validator->validateTextInput($userName, _('Username'));

    // Confirm Username
    $confirmUsername =  StringHelper::RemoveWhitespaceFromPassword($_POST['confirm_user_name']);
    if ($userName !== $confirmUsername) {
      $validator->addFormErrorString(_('Usernames do not match'));
    }

    // Reporting email
    $reportingEmail = $_POST['reporting_email'];
    $validator->validateInputEmail($reportingEmail, _('Reporting Email'), TRUE);

    // Advanced options
    $statisticsIncludeRawData = Validator::formatCheckbox(@$_POST['statistics_include_raw_data']);
    $setupShouldShow = Validator::formatCheckbox(@$_POST['setup_should_show']);
    $helpMessagesHighlight = Validator::formatCheckbox(@$_POST['help_messages_highlight']);
    
    // Trusted Devices
    $trustedDeviceSaveArray = array();
    $trustedDevicesDeviceNumArray = @$_POST['trusted_devices_device_num_array'];
    $trustedDevicesDeviceIdArray = @$_POST['trusted_devices_device_id_array'];
    $trustedDevicesDeviceTypeArray = @$_POST['trusted_devices_device_type_array'];
    if ($trustedDevicesDeviceNumArray) {
      foreach($trustedDevicesDeviceNumArray as $index => $value) {
        $trustedDeviceSaveArray[] = array(
          'device_num' => $trustedDevicesDeviceNumArray[$index],
          'device_id' => $trustedDevicesDeviceIdArray[$index],
          'device_type' => $trustedDevicesDeviceTypeArray[$index]
        );
      }
    }
  }

  // Save
  if ($validator->formErrorsIsEmpty()) {
    $success = TRUE;
    $userData = array();

    // If password is supposed to be set
		if (strlen($password) > 0) {
			$userData['password'] = $password;
			$userData['confirm_password'] = $confirmPassword;
		}

    // If username is supposed to be set
		if (isset($userName) && strlen($userName) > 0) {
			$userData['user_name'] = $userName;
		}

    $dbManagerWifi->openDb();
      $success &= $dbManagerWifi->usersSave($userData, $_SESSION['userId']);
      $success &= $dbManagerWifi->settingsSetKeyValue("password_hint", $passwordHint);

      if (!$isSetupWizard) {
        $success &= $dbManagerWifi->settingsSetKeyValue("reporting_email", $reportingEmail);
        $success &= $dbManagerWifi->settingsSetKeyValue("statistics_include_raw_data", $statisticsIncludeRawData);
        $success &= $dbManagerWifi->settingsSetKeyValue("setup_should_show", $setupShouldShow);
        $success &= $dbManagerWifi->settingsSetKeyValue("help_messages_highlight", $helpMessagesHighlight);
        $success &= $dbManagerWifi->trustedDevicesUpdateWithArray($trustedDeviceSaveArray);
      }
      
      // NOTE: this must be after trustedDevicesUpdateWithArray and before usersSave
      $passwordIsSetNotDefault = isset($userData['password']) && $userData['password'] !== DbManagerWifi::ADMIN_DEFAULT_PASSWORD;
      if ($passwordIsSetNotDefault) {
        $currentDevice = $dbManagerWifi->devicesGetDeviceWithId($_SESSION['device_id']);
        $currentDevice['device_type'] = $currentDeviceType;
        $dbManagerWifi->trustedDevicesAddDevice($currentDevice);
      }

      if (!$success) {
        $validator->addFormErrorSaveFailed();
      } else {
				if (isset($userData['password'])) {
					$user = $dbManagerWifi->usersFindById($_SESSION['userId']);
					$dbManagerWifi->usersLogout();
          SessionHelper::StartSecureSession();
					$dbManagerWifi->usersAuthenticate($user['user_name'], $password);
				}
      }
    $dbManagerWifi->closeDb();
  }

  // Return to ajax
  echo $validator->ajaxFormResponse();
  exit();
}

$deviceId = $_SESSION['device_id'];
$dbManagerWifi->openDb();
  $user = $dbManagerWifi->usersFindById($_SESSION['userId']);
  $reportingEmail = $dbManagerWifi->settingsGetValueForKey('reporting_email');
  $passwordHint = $dbManagerWifi->settingsGetValueForKey('password_hint');
  $statisticsIncludeRawData = Validator::StringToBoolean($dbManagerWifi->settingsGetValueForKey('statistics_include_raw_data'));
  $setupShouldShow = Validator::StringToBoolean($dbManagerWifi->settingsGetValueForKey('setup_should_show'));
  $helpMessagesHighlight = Validator::StringToBoolean($dbManagerWifi->settingsGetValueForKey('help_messages_highlight'));
  $currentDevice = $dbManagerWifi->devicesGetDeviceWithId($deviceId);
  $currentDevice['device_type'] = $currentDeviceType;
  $trustedDevicesArray = $dbManagerWifi->trustedDevicesGetAll();
$dbManagerWifi->closeDb();

$t->assign('trusted_devices_array', $trustedDevicesArray);
$t->assign('current_device', $currentDevice);
$t->assign('set_reporting_email', $setReportingEmail);
$t->assign('reporting_email', $reportingEmail);
$t->assign('statistics_include_raw_data', $statisticsIncludeRawData);
$t->assign('setup_should_show', $setupShouldShow);
$t->assign('help_messages_highlight', $helpMessagesHighlight);
$t->assign('user_name', $user['user_name']);
$t->assign('password_hint', $passwordHint);
$t->assign('page_title', _('Account Settings'));
$t->assign('current_page', 'settings');
$t->assign('current_settings', 'account');
templateDisplayPageAdmin('admin' . DS . 'settings' . DS . 'account.tpl');
