<?php
require_once('..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'secure.php');
require_once('..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'common.php');
require_once("Renew" . DS . "SystemHelper.php");
use \Renew\SystemHelper;
use \Renew\Validator;

$validator = new Validator();

define('TYPE_UPDATE', 'update');
define('TYPE_UPDATE_SET_FLAG', 'update_set_flag');
define('TYPE_MIRROR', 'mirror');
define('TYPE_MIRROR_SET_FLAG', 'mirror_set_flag');
define('TYPE_MIRROR_RUN', 'mirror_run');
define('TYPE_MIRROR_RUN_SET_FLAG', 'mirror_run_set_flag');
define('TYPE_SD_CARD_STOP_SERVICE', 'sd_card_stop_service');
define('ERR_NOT_MIRROR', _('The inserted card is not a mirror SD card'));

// Mirror status errors
$statusErrorMessages = array(
  'STATUS_ERROR_FAILED' => _('Process failed'),
  'STATUS_ERROR_TOO_SMALL' => _('SD card too small'),
  'STATUS_ERROR_TIMEOUT' => _('Process took too long and failed')
);

function checkStatusFormErrors($status) {
  global $validator;
  global $statusErrorMessages;
  if ($status === 'STATUS_ERROR_FAILED'
   || $status === 'STATUS_ERROR_TOO_SMALL'
   || $status === 'STATUS_ERROR_TIMEOUT') {
    $validator->addFormErrorString($statusErrorMessages[$status]);
  }
}

if (isset($_POST, $_POST['ajax'])) {
  // error_log(print_r($_POST, TRUE));
  $setupConfirmed = FALSE;
  if (isset($_POST['setup_confirmed'])
   && Validator::StringToBoolean($_POST['setup_confirmed'])) {
    $setupConfirmed = TRUE;
  }
  switch($_POST['type']) {
    case TYPE_UPDATE:
      // error_log('UPDATE DRIVE!');
      $status = FALSE;
      $driveIsInSlot = SystemHelper::UsbDriveIsInSlot();
      if ($driveIsInSlot === TRUE && $setupConfirmed) {
        // error_log('Update, setup confirmed, get status');
        $status = SystemHelper::DeviceLauncherUpdateDriveCreateGetStatus();
        checkStatusFormErrors($status);
      }
      echo $validator->ajaxFormResponse(array(
        'drive_is_in_slot' => $driveIsInSlot,
        'setup_confirmed' => $setupConfirmed,
        'status' => $status
      ));
      exit();
    break;
    case TYPE_UPDATE_SET_FLAG:
      if (!SystemHelper::DeviceLauncherUpdateDriveCreateSetFlag()) {
        $validator->addFormErrorString($statusErrorMessages['STATUS_ERROR_FAILED']);
      }
      // error_log('*** update flag set!!! ***');
      echo $validator->ajaxFormResponse();
      exit();
    break;
    case TYPE_MIRROR:
      $driveIsInSlot = SystemHelper::SdCardIsInSlot();
      $status = FALSE;
      $percent = FALSE;
      if ($driveIsInSlot === TRUE && $setupConfirmed) {
        $status = SystemHelper::DeviceLauncherMirrorCreateGetStatus();
        $percent = SystemHelper::DeviceLauncherMirrorCreatePercentage();
        checkStatusFormErrors($status);
      }
      echo $validator->ajaxFormResponse(array(
        'drive_is_in_slot' => $driveIsInSlot,
        'setup_confirmed' => $setupConfirmed,
        'status' => $status,
        'percent' => $percent
      ));
      exit();
    break;
    case TYPE_MIRROR_SET_FLAG:
      $driveIsInSlot = SystemHelper::SdCardIsInSlot();
      if ($driveIsInSlot) {
        $success = SystemHelper::DeviceLauncherMirrorCreateSetFlag();
      }
      if (!$driveIsInSlot || !$success) {
        $validator->addFormErrorString($statusErrorMessages['STATUS_ERROR_FAILED']);
      }
      // error_log('*** flag set!!! ***');
      echo $validator->ajaxFormResponse();
      exit();
    break;
    case TYPE_MIRROR_RUN:
      // error_log('MIRROR RUN!');
      $driveIsInSlot = SystemHelper::SdCardIsInSlot();
      $isMirrorCard = SystemHelper::SdCardIsInSlotAndIsMirrorCard();
      if ($driveIsInSlot && !$isMirrorCard) {
        $validator->addFormErrorString(ERR_NOT_MIRROR);
      }
      echo $validator->ajaxFormResponse(array(
        'drive_is_in_slot' => $driveIsInSlot,
        'setup_confirmed' => $setupConfirmed
      ));
      exit();
    break;
    case TYPE_MIRROR_RUN_SET_FLAG:
      $isMirrorCard = SystemHelper::SdCardIsInSlotAndIsMirrorCard();
      if ($isMirrorCard) {
        $success = SystemHelper::DeviceLauncherMirrorRunSetFlag();
      } else {
        $validator->addFormErrorString(ERR_NOT_MIRROR);
      }
      if (!$success) {
        $validator->addFormErrorString($statusErrorMessages['STATUS_ERROR_FAILED']);
      }
      // error_log('*** run flag set!!! ***');
      echo $validator->ajaxFormResponse();
      exit();
    break;
    case TYPE_SD_CARD_STOP_SERVICE:
      // error_log('SD CARD STOP SERVICE');
      SystemHelper::SdCardStopService();
      echo $validator->ajaxFormResponse();
      exit();
    break;
  }

  // Return to ajax
  echo $validator->ajaxFormResponse();
  exit();
}

$t->assign('page_title', _('Mirror & Update Drives'));
$t->assign('current_page', 'settings');
$t->assign('current_settings', 'drives');
templateDisplayPageAdmin('admin' . DS . 'settings' . DS . 'drives.tpl');
