<?php
require_once('..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'secure.php');
require_once('..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'common.php');
require_once('..' . DS . 'utilities' . DS . 'captive_portal_set_default_languages.php');
require_once("Renew" . DS . "DbManagerMedia.php");
require_once("Renew" . DS . "Utilities" . DS . "LightStreamServices.php");
require_once("Renew" . DS . "DbManagerWifi.php");
require_once("Renew" . DS . "SystemHelper.php");
require_once("Renew" . DS . "ArrayHelper.php");
require_once("Renew" . DS . "Utilities" . DS . "Validator.php");
require_once('..' . DS . 'includes' . DS . 'wifi_password.php'); // shared code with security.php
use \Renew\ArrayHelper;
use \Renew\Validator;
use \Renew\SystemHelper;
use \Renew\DbManagerMedia;

$validator = new Validator();
$dbManagerMedia = new DbManagerMedia(MEDIA_DB, MEDIA_DIR, USER_IMAGES_DIR);

// Strings
define('PAGE_TITLE', _('WiFi Settings'));
define('MODE_DEFAULT', 'default');
define('MODE_MESSAGE', 'message');
define('MODE_DISABLED', 'disabled');
define('CAPTIVE_PORTAL_PREFIX', _('Captive Portal'));
define('WEBSITE_URL', _('Website URL'));

// Sufixes
define('SSID_SUFFIX', '_ssid');
define('PASSWORD_SUFFIX', '_password');
define('MAX_WIFI_NETWORKS', 5);
define('MESSAGE_MAX_ALLOWED', _('Maximum number allowed: %s'));

// Captive Portal Suffixes
define('DISPLAY_NAME_SUFFIX', '_display_name');
define('MESSAGE_SUFFIX', '_message');
define('LANGUAGE_ISO_SUFFIX', '_language_iso');

// Min and max values
define('WIFI_MAX_DOWNLOADS_MIN', 1);
define('WIFI_MAX_DOWNLOADS_MAX', 10);
define('WIFI_MAX_DOWNLOADS_NO_LIMIT', 1000);
define('WIFI_CHANNEL_MIN', 0); // 0 is random
define('WIFI_CHANNEL_MAX', 11);
define('WIFI_MAX_USERS_MIN', 1);
define('WIFI_MAX_USERS_MAX', 50);

// Prefixes
define('PREFIX_WIFI_CHANNEL', _('WiFi Channel'));
define('PREFIX_WIFI_MAX_DOWNLOADS', _('Max Number of File Downloads At Once '));
define('PREFIX_WIFI_MAX_USERS', _('WiFi Max Users'));


if (isset($_POST) && isset($_POST['ajax'])) {
  // Get a default message from a given language ISO
  // Used by captive portal language creation JS
  if (isset($_POST['_method']) && $_POST['_method'] === 'GET') {
    if (isset($_POST['language_iso'])) {
      $languageIso = $_POST['language_iso'];
      $dbManagerWifi->openDb();
        $defaultMessage = $dbManagerWifi->captivePortalGetDefaultMessageFromLanguageIso($languageIso);
      $dbManagerWifi->closeDb();
      echo $defaultMessage;
      exit();
    }
    echo FALSE;
    exit();
  }

  // Reset captive portal languages button
  if (isset($_POST['reset_captive_portal_languages'])) {
    $success = captivePortalSetDefaultLanguages();
    $dbManagerWifi->openDb();
      $success &= $dbManagerWifi->settingsSetKeyValueArray(array(
        'captive_portal_mode' => 'default',
      ));
    $dbManagerWifi->closeDb();
    if (!$success) {
      $validator->addFormErrorSaveFailed();
    }
    echo $validator->ajaxFormResponse();
    exit();
  }

  $isSetupWizard = FALSE;
  if (isset($_POST['is_setup_wizard'])) {
    $isSetupWizard = TRUE;
  }

  // wifi ssid must be less than 32 bytes to comply
  $wifiSsidStr = strip_tags($_POST['wifi_ssid']);
  $validator->validateTextInputMaxLengthNotBlank($wifiSsidStr, _('WiFi SSID'), MAX_SSID_LENGTH);

  if (!$isSetupWizard) {
    // URL must be a url
    $websiteUrl = Validator::formatURLRemoveHTTPStrings($_POST['site_url']);
    $validator->validateLegalUrl($websiteUrl, WEBSITE_URL);
    $validator->validateSimpleUrl($websiteUrl, WEBSITE_URL, MAX_URL_LENGTH);

    // WiFi Password
    validateWifiPasswordHelper();

    // Validate max downloads
    $wifiMaxDownloads = $_POST['wifi_max_downloads'];
    $validator->validateNotEmpty($wifiMaxDownloads, PREFIX_WIFI_MAX_DOWNLOADS);
    if (intval($wifiMaxDownloads) !== WIFI_MAX_DOWNLOADS_NO_LIMIT) {
      $validator->validateIntInputBetween($wifiMaxDownloads, PREFIX_WIFI_MAX_DOWNLOADS, WIFI_MAX_DOWNLOADS_MIN, WIFI_MAX_DOWNLOADS_MAX);
    }

    // the channel must be between 0 and 11, and 0 is random
    $wifiChannelInt = $_POST['wifi_channel'];
    $validator->validateNotEmpty($wifiChannelInt, PREFIX_WIFI_CHANNEL);
    $validator->validateIntInputBetween($wifiChannelInt, PREFIX_WIFI_CHANNEL, WIFI_CHANNEL_MIN, WIFI_CHANNEL_MAX);

    // Validate max users
    $wifiMaxUsers = $_POST['wifi_max_users'];
    $validator->validateNotEmpty($wifiMaxUsers, PREFIX_WIFI_MAX_USERS);
    $validator->validateIntInputBetween($wifiMaxUsers, PREFIX_WIFI_MAX_USERS, WIFI_MAX_USERS_MIN, WIFI_MAX_USERS_MAX);

    // Captive portal mode
    $captivePortalMode = $_POST['captive_portal_mode'];
    $validator->validateNotEmpty($captivePortalMode, CAPTIVE_PORTAL_PREFIX);

    if ($captivePortalMode === MODE_DEFAULT) {
      // Captive Portal Languages
      $captivePortalLanguagesSaveArray = array();
      if (isset($_POST['captive_portal_languages'])) {
        $captivePortalLanguagesArray = $_POST['captive_portal_languages'];
        foreach ($captivePortalLanguagesArray as $captivePortalLanguageValue) {
          $displayName = $_POST[$captivePortalLanguageValue . DISPLAY_NAME_SUFFIX];
          $validator->validateNotEmpty($displayName, $adminErrorMessages['ERR']);

          // Language ISO
          $languageIsoName = $captivePortalLanguageValue . LANGUAGE_ISO_SUFFIX;
          if (isset($_POST[$languageIsoName])) {
            $languageIso = $_POST[$languageIsoName];
          } else {
            $languageIso = '';
          }

          // Message
          $message = $_POST[$captivePortalLanguageValue . MESSAGE_SUFFIX];
          $validator->validateTextFieldLength($message, CAPTIVE_PORTAL_PREFIX);

          $captivePortalLanguagesSaveArray[] = array(
            'language_display_name' => $displayName,
            'language_iso' => $languageIso,
            'message' => $message
          );
        }
      } else {
        // No captive portal languages to save
        $captivePortalMode = MODE_DISABLED;
        $validator->addFormErrorString(_('Please add at least one language for the WiFi Login Popup'));
      }
    } else if ($captivePortalMode === MODE_MESSAGE) {
      // Title
      $captivePortalTitle = $_POST['captive_portal_title'];
      $validator->validateTextInputLength($captivePortalTitle, _('Title'));

      /// Message
      $captivePortalMessage = $_POST['captive_portal_message'];
      $validator->validateTextFieldLength($captivePortalMessage, _('Message'));

      // Checkbox for allowing video to promote (if this is not selected, video should be default '0')
      // This is not a setting in the db, but is inferred on page load if $captivePortalModeMessageMediaFileId === '0'
      $captivePortalVideoCheckbox = Validator::formatCheckboxBoolean(@$_POST['captive_portal_video_checkbox']);
      $captivePortalModeMessageMediaAutoplay = Validator::STRING_FALSE;

      // Video to promote
      $captivePortalModeMessageMediaFileId = DbManagerMedia::FILE_ID_DEFAULT;
      if ($captivePortalVideoCheckbox === TRUE) {
        $captivePortalModeMessageMediaFileId = $_POST['captive_portal_mode_message_media_file_id'];
        $validator->validateNotEmpty($captivePortalModeMessageMediaFileId, _('Please select a file from the dropdown'));

        // Video autoplay
        $captivePortalModeMessageMediaAutoplay = Validator::formatCheckbox(@$_POST['captive_portal_mode_message_media_autoplay']);
      }
    }

    // Validate remembered wifi networks
    if (isset($_POST['wifi_networks_array'])) {
      $wifiNetworksArray = $_POST['wifi_networks_array'];

      // Validate max number of networks
      $totalNetworks = count($wifiNetworksArray);
      if ($totalNetworks > MAX_WIFI_NETWORKS) {
        $validator->addFormErrorString(sprintf(MESSAGE_MAX_ALLOWED, MAX_WIFI_NETWORKS));
      }
    }
  }

  if ($validator->formErrorsIsEmpty()) {

    $saveSettings = array(
      'ssid' => $wifiSsidStr,
    );

    if (!$isSetupWizard) {
      $saveSettings['site_url'] = $websiteUrl;
      $saveSettings['wifi_max_downloads'] = intval($wifiMaxDownloads);
      $saveSettings['channel'] = intval($wifiChannelInt);
      $saveSettings['wifi_max_users'] = intval($wifiMaxUsers);
      $saveSettings['captive_portal_mode'] = $captivePortalMode;
      if ($captivePortalMode === MODE_MESSAGE) {
        $saveSettings['captive_portal_title'] = $captivePortalTitle;
        $saveSettings['captive_portal_message'] = $captivePortalMessage;
        $saveSettings['captive_portal_mode_message_media_file_id'] = $captivePortalModeMessageMediaFileId;
        $saveSettings['captive_portal_mode_message_media_autoplay'] = $captivePortalModeMessageMediaAutoplay;
      }
      $saveSettings['wifi_password_enabled'] = $wifiPasswordEnabled;
      if ($wifiPasswordEnabled === Validator::STRING_TRUE) {
        $saveSettings['wifi_password'] = $wifiPassword;
      }
    }

    // Save
    $success = TRUE;
    $dbManagerWifi->openDb();
      $success &= $dbManagerWifi->settingsSetKeyValueArray($saveSettings);
      if (isset($captivePortalLanguagesSaveArray)) {
        $success &= $dbManagerWifi->captivePortalLanguagesUpdateWithArray($captivePortalLanguagesSaveArray);
      }
    $dbManagerWifi->closeDb();
    if ($success === FALSE) {
      $validator->addFormErrorSaveFailed();
    } 
  }

  echo $validator->ajaxFormResponse();
  exit();
}

$wifiChannelArray = ArrayHelper::GetAssociativeArrayOfNumbers((WIFI_CHANNEL_MIN + 1), WIFI_CHANNEL_MAX);
$wifiMaxDownloadsArray = ArrayHelper::GetAssociativeArrayOfNumbers(WIFI_MAX_DOWNLOADS_MIN, WIFI_MAX_DOWNLOADS_MAX);
$wifiMaxDownloadsArray[_('No Max Number')] = WIFI_MAX_DOWNLOADS_NO_LIMIT;
$wifiChannelArray[_("Random")] = 0;

$t->assign("page_title", PAGE_TITLE);
$t->assign('current_page', 'settings');
$t->assign('current_settings', 'wifi');
$t->assign('max_url_length', MAX_URL_LENGTH);
$t->assign('max_ssid_length', MAX_SSID_LENGTH);
$t->assign('max_wifi_networks', MAX_WIFI_NETWORKS);
$t->assign('message_max_allowed', MESSAGE_MAX_ALLOWED);
$t->assign('wifi_channel_array', $wifiChannelArray);
$t->assign('wifi_max_downloads_array', $wifiMaxDownloadsArray);
$t->assign('wifi_max_users_array', ArrayHelper::GetAssociativeArrayOfNumbers(WIFI_MAX_USERS_MIN, WIFI_MAX_USERS_MAX));
$dbManagerWifi->openDb();
  $t->assign('wifi_ssid', $dbManagerWifi->settingsGetValueForKey('ssid'));
  $t->assign('site_url', $dbManagerWifi->settingsGetValueForKey('site_url'));
  $t->assign('wifi_channel', $dbManagerWifi->settingsGetValueForKey('channel'));
  $t->assign('wifi_max_users', $dbManagerWifi->settingsGetValueForKey('wifi_max_users'));
  $t->assign('wifi_max_downloads', $dbManagerWifi->settingsGetValueForKey('wifi_max_downloads'));
  $t->assign('captive_portal_mode', $dbManagerWifi->settingsGetValueForKey('captive_portal_mode'));
  $captivePortalModeMessageMediaFileId = $dbManagerWifi->settingsGetValueForKey('captive_portal_mode_message_media_file_id');
  $t->assign('captive_portal_mode_message_media_file_id', $captivePortalModeMessageMediaFileId);
  $t->assign('captive_portal_mode_message_media_autoplay', $dbManagerWifi->settingsGetValueForKey('captive_portal_mode_message_media_autoplay'));
  $t->assign('captive_portal_title', $dbManagerWifi->settingsGetValueForKey('captive_portal_title'));
  $t->assign('captive_portal_message', $dbManagerWifi->settingsGetValueForKey('captive_portal_message'));
  $t->assign('captive_portal_languages', $dbManagerWifi->captivePortalLanguagesGetAll());
  $t->assign('wifi_networks_array', $dbManagerWifi->wifiNetworksGetNetworksArray());
  $t->assign('wifi_password_enabled', Validator::StringToBoolean($dbManagerWifi->settingsGetValueForKey('wifi_password_enabled')));
  $t->assign('wifi_password', $dbManagerWifi->settingsGetValueForKey('wifi_password'));
$dbManagerWifi->closeDb();
$dbManagerMedia->openDb();
  $captivePortalModeMessageMediaFileSelectArray = NULL;
  if ($captivePortalModeMessageMediaFileId !== DbManagerMedia::FILE_ID_DEFAULT) {
    $captivePortalModeMessageMediaFileSelectArray = $dbManagerMedia->mediaGetFileWithIdFormatAsSelectArray($captivePortalModeMessageMediaFileId);
  }
  $t->assign('captive_portal_mode_message_media_file_select_array', $captivePortalModeMessageMediaFileSelectArray);
$dbManagerMedia->closeDb();
templateDisplayPageAdmin('admin' . DS . 'settings' . DS . 'wifi.tpl');
