<?php
require_once('..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'secure.php');
require_once('..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'common.php');
require_once("Renew" . DS . "Utilities" . DS . "Validator.php");
require_once("Renew" . DS . "SystemHelper.php");
use \Renew\Validator;
use \Renew\SystemHelper;

define('MAX_WIFI_NETWORKS', 5);
define('MESSAGE_MAX_ALLOWED', _('Maximum number allowed: %s'));
define('PREFIX_WIFI_PASSWORD', _('WiFi Password'));

if (isset($_POST, $_POST['ajax'], $_POST['_method'])) {
  $validator = new Validator();

  function saveNetwork($ssid, $password, $isUpdate = FALSE) {
    global $validator;
    global $dbManagerWifi;
    $success = TRUE;

    // SSID
    $success &= $validator->validateTextInput($ssid, _('WiFi SSID'));

    // PASSWORD
    $success &= $validator->validateTextInputLength($password, PREFIX_WIFI_PASSWORD);

    if (!$isUpdate) {
      // Validate max number of networks
      $dbManagerWifi->openDb();
        $totalNetworks = count($dbManagerWifi->wifiNetworksGetNetworksArray());
      $dbManagerWifi->closeDb();
      if (($totalNetworks + 1) > MAX_WIFI_NETWORKS) {
        $validator->addFormErrorString(sprintf(MESSAGE_MAX_ALLOWED, MAX_WIFI_NETWORKS));
        $success = FALSE;
      }
    }

    // Save
    if ($validator->formErrorsIsEmpty()) {
      $dbManagerWifi->openDb();
        if (!$dbManagerWifi->wifiNetworksAddOrUpdateNetworkWithSsid($ssid, $password)) {
          $validator->addFormErrorSaveFailed();
          $success = FALSE;
        }
      $dbManagerWifi->closeDb();
    }
    error_log('SAVE SUCCESS: ' . $success);
    return $success;
  }

  if ($_POST['_method'] === 'POST') { // Add new WiFi network
    $ssid = $_POST['ssid'];
    $password = $_POST['password'];
    saveNetwork($ssid, $password);

    // Return to Ajax
    echo $validator->ajaxFormResponse(json_encode(array(
      'ssid' => $ssid,
      'password' => $password
    )));
    exit();
  } else if ($_POST['_method'] === 'DELETE') { // Delete WiFi Network
    $ssid = $_POST['ssid'];

    $dbManagerWifi->openDb();
      if(!$dbManagerWifi->wifiNetworksDeleteNetworkWithSsid($ssid)) {
        $validator->addFormErrorString(_('Delete Failed'));
      }
    $dbManagerWifi->closeDb();

    echo $validator->ajaxFormResponse();
    exit();
  } else if ($_POST['_method'] === 'PUT') { // Update WiFi Network
    $ssid = $_POST['ssid'];
    $password = $_POST['password'];
    saveNetwork($ssid, $password, TRUE);

    echo $validator->ajaxFormResponse();
    exit();
  } else if ($_POST['_method'] === 'CONNECT') {
    $ssid = $_POST['ssid'];
    $password = $_POST['password'];
    $saveSuccess = FALSE;

    if (IS_DEV_ENV) {
      error_log('IS_DEV_ENV: CONNECTING TO: ' . $ssid);
      $saveSuccess = saveNetwork($ssid, $password);
    } else {
      SystemHelper::Wlan1Disconnect();
      if (SystemHelper::Wlan1ConnectToSsid($ssid) === FALSE) {
        $validator->addFormErrorString(sprintf(_('Connect to %s failed. Make sure SSID and password are correct.'), $ssid));
      }
    }

    $responseJSON = json_encode(array(
      'password' => $password, // Password needed when connecting to a new network
      'save_success' => $saveSuccess
    ));

    error_log(print_r($responseJSON, TRUE));

    // Same response for success and failure
    echo $validator->ajaxFormResponse($responseJSON, $responseJSON);
    exit();
  } else if ($_POST['_method'] === 'DISCONNECT') {
    if (IS_DEV_ENV) {
      error_log('IS_DEV_ENV: DISCONNECTING');
    } else {
      if (SystemHelper::Wlan1Disconnect()) {
        $validator->addFormErrorString(_('Disconnect Failed.'));
      }
    }
    echo $validator->ajaxFormResponse();
    exit();
  }
}
