<?php
require_once('Renew/Utilities/SlimDataHelper.php');
use \Renew\DbManagerMedia;
use \Renew\DbManagerWifi;
use \Renew\SlimDataHelper;

// String that lets slim know when not to cache get requests
define('NO_CACHE_URL_STRING', '/no-cache');
define('ADMIN_VUE_URL_STRING', '/admin-v');

// String that prevents access from requests not at device IP address (used for node calls)
define('NODE_ONLY_URL_STRING', '/node-only');
define('ADMIN_ONLY_URL_STRING', '/admin-only');

/////////////////////////////////////////////////////////
// INITIALIZE SLIM APP
$config = array(
  'displayErrorDetails' => true, // Remove this in production
  'addContentLengthHeader' => false,
);
$app = new \Slim\App(['settings' => $config]);

/////////////////////////////////////////////////////////
// SLIM CONTAINER DEPENDENCIES

// Accessed by $this-> in routes
$container = $app->getContainer();

$container['dbManagerWifi'] = function() {
  return new DbManagerWifi(LSWIFI_DB);
};

$container['dbManagerMedia'] = function() {
  return new DbManagerMedia(MEDIA_DB, MEDIA_DIR, USER_IMAGES_DIR);
};

$container['t'] = function($container) {
  global $t;
  return $t;
};

$container['slimDataHelper'] = function($container) {
  global $app;
  global $serialNumber;
  return new SlimDataHelper($app, $container, $serialNumber, $_SESSION['device_id']);
};

$container['indexHTMLFile'] = function($container) {
  return $container->slimDataHelper->getIndexHTMLFile();
};

// Use slim notFoundHandler 404 feature to allow routing in vue if a route isn't defined in slim api
$container['notFoundHandler'] = function ($container) {
  return function ($request, $response) use ($container) {
    $uri = $request->getUri();
    $path = $uri->getPath();

    // Admin route if we have /admin-v at the start of URL
    $isAdminVueRoute = substr($path, 0, strlen(ADMIN_VUE_URL_STRING)) == ADMIN_VUE_URL_STRING;

    if ($isAdminVueRoute) {
      // Make secure able to use variables from common.php
      foreach($GLOBALS as $name => $value) global $$name;
      require_once(DOC_ROOT . '/secure.php');
      $indexHTMLFileAdmin = $container->slimDataHelper->getIndexHTMLFileAdmin($path, function($templatePath) {
        return templateDisplayPageAdmin($templatePath, TRUE);
      });
      return $container['response']->write($indexHTMLFileAdmin);
    } else {
      return $container['response']->write($container['indexHTMLFile']);
    }
  };
};

// If server has error, send 500 code
$container['errorHandler'] = function($container) {
  return function ($request, $response, $exception) use ($container) {
      error_log('Exception: ' . $exception->getMessage());
      return $response->withJson(FALSE, 500);
  };
};

/////////////////////////////////////////////////////////
// MIDDLEWARE

/////////////////////////////////////////////////////////
// Middleware: permanently redirect paths with a trailing slash to their non-trailing counterpart
// Source: https://www.slimframework.com/docs/v3/cookbook/route-patterns.html
$app->add(function ($request, $response, $next) {
  $uri = $request->getUri();
  $path = $uri->getPath();
  if ($path != '/' && substr($path, -1) == '/') {
    $uri = $uri->withPath(substr($path, 0, -1));
    if ($request->getMethod() == 'GET') {
      return $response->withRedirect((string)$uri, 301);
    }
    else {
      return $next($request->withUri($uri), $response);
    }
  }
  return $next($request, $response);
});


/////////////////////////////////////////////////////////
// Doctrine PhpFileCache cache middleware
$app->add(function($request, $response, $next)  {
  global $slimCache;

  $path = $request->getUri()->getPath();
  $method = $request->getMethod();

  // If GET  call is being made to the api
  if (strpos($path, '/api/') === 0
      && strpos($path, NO_CACHE_URL_STRING) === FALSE // If path does not contain 'no-cache'
      && strpos($path, NODE_ONLY_URL_STRING) === FALSE // If path does not contain 'node-only'
      && strpos($path, ADMIN_ONLY_URL_STRING) === FALSE // If path does not contain 'admin-only'
      && $method === 'GET') {
    // Add URL query to path
    $query = $request->getUri()->getQuery();
    if ($query) {
      $path .= '?' . $query;
    }

    // Return cache if exists
    if ($slimCache->cache->contains($path)) {
      return $response->withJson(json_decode($slimCache->cache->fetch($path), TRUE));
    }

    // Set cache
    $response = $next($request, $response);
    if ($response->getStatusCode() !== 404) {
      $body = $response->getBody()->__toString();
      $slimCache->cache->save($path, $body);
    }
  } else {
    // Process response
    $response = $next($request, $response);
  }
  return $response;
});


///////////////////// FOR VUE DEV ENVIRONMENT /////////////////////
if (IS_DEV_ENV) {
  // Enable CORS for dev environment
  $app->options('/{routes:.+}', function ($req, $res, $args) {
    return $res;
  });

  // Middleware, invoked for every oncoming HTTP request
  $app->add(function ($req, $res, $next) {
    $response = $next($req, $res);
    return $response
            ->withHeader('Access-Control-Allow-Origin', '*')
            ->withHeader('Access-Control-Allow-Headers', 'X-Requested-With, Content-Type, Accept, Origin, Authorization')
            ->withHeader('Access-Control-Allow-Methods', 'GET, POST, PUT, DELETE, PATCH, OPTIONS');
  });
}
