<?php
require_once('Renew/Utilities/Validator.php');
require_once('Renew/Utilities/SlimHelper.php');
require_once('Renew/SessionHelper.php');
use \Renew\DbManagerMedia;
use \Renew\SessionHelper;

/////////////////////////////////////////////////////////
// MAIN ROUTES
$app->get('/', function ($req, $res, $args) {
  return $res->write($this->indexHTMLFile);
});

/////////////////////////////////////////////////////////
// AUTHENTICATE STREAMED FILES
// (/media/master directory is hidden so we can authenticate)
$app->get('/file/{fileId}', function ($req, $res, $args) {
  $fileId = $args['fileId'];
  if ($fileId) {    
    $mediaItem = $this->slimDataHelper->subRequestHelper("/media-item/${fileId}");
    if ($mediaItem !== FALSE) {
      $isAuthenicated = !$mediaItem['is_password_protected'] || SessionHelper::CheckFileIdIsAuthenticated($fileId);
      $isAdmin = FALSE;
      if ($isAuthenicated === FALSE) {
        $this->dbManagerWifi->openDb();
          $isAdmin = $this->dbManagerWifi->usersCheckLogin();
        $this->dbManagerWifi->openDb();
      }
      if ($isAuthenicated || $isAdmin) {
        $relPath = DbManagerMedia::MediaGetPathRelWithFile($mediaItem);
        $fullPath = MEDIA_DIR . DS . $relPath;
        $mime = $mediaItem['mime_part1'] . '/' . $mediaItem['mime_part2'];

        header('Content-Length:' . filesize($fullPath));
        header('Content-Type: ' . $mime);
        header("Content-Transfer-Encoding: binary");
        header('Pragma: no-cache');
        header('Expires: 0');
        header('Cache-Control: no-cache, no-store, must-revalidate, post-check=0, pre-check=0');
        header('X-Content-Type-Options: nosniff'); // Security: prevent browser from trying its own mime
        header('X-Accel-Redirect: /media/master/' . rawurlencode($relPath));
        exit();
      } else {
        error_log('ERR: not authenticated');
      }
    } else {
      error_log('ERR: no media item');
    }
  }
  error_log('ERR: ID undefined');
});


/////////////////////////////////////////////////////////
// API V2 ROUTES
$app->group('/api/v2', function() {
  require_once('./api/v2/api_routes_general.php');
  require_once('./api/v2/api_routes_media.php');
  require_once('./api/v2/api_routes_no_cache.php');
  require_once('./api/v2/api_routes_admin_only.php');
  require_once('./api/v2/api_routes_internal.php');

  /////////////////////////////////////////////////////////
  // Invalid route handler for /api/v2 routes
  // Different than nprmal 404 handler, since we don't want to serve up HTML here, where people are expecting data
  $this->map(['GET', 'POST', 'PUT', 'DELETE', 'PATCH'], '/{routes:.+}', function($req, $res) {
    return $res->withStatus(404);
  });
});

/////////////////////////////////////////////////////////
// 404 Catch-all route to serve up vue front-end, which has its own routing
// NOTE: make sure this route is defined last
$app->map(['GET', 'POST', 'PUT', 'DELETE', 'PATCH'], '/{routes:.+}', function($req, $res) {
  $handler = $this->notFoundHandler;
  return $handler($req, $res);
});
