<?php
require_once('Renew/Utilities/CurlHelper.php');
use \Renew\SystemHelper;
use \Renew\CurlHelper;

/////////////////////////////////////////////////////////
// ADMIN ROUTES
// ONLY CALLABLE BY ADMIN
// Starts with /admin-only
$this->group(ADMIN_ONLY_URL_STRING, function() {
  /////////////////////////////////////////////////////////
  // GET ALL WIFI NETWORKS
  // /admin-only/...
  $this->get('/wifi/networks', function ($req, $res, $args) {
    if (IS_DEV_ENV) {
      return $res->withJson(array(
        array(
          'ssid' => 'Test Network',
          'quality' => 80,
          'password_protected' => true,
          'security' => 'wep2',
          'ssid_hex' => '0000',
        ),
        array(
          'ssid' => 'BurgerFlurger',
          'quality' => 40,
          'password_protected' => false,
          'security' => 'none',
          'ssid_hex' => '0002',
        ),
        array(
          'ssid' => 'My Device',
          'quality' => 60,
          'password_protected' => false,
          'security' => 'none',
          'ssid_hex' => '0003',
        ),
      ), 200); 
    }
    $wifiNetworks = SystemHelper::Wlan1ScanForWifiNetworksReturnSSIDArray();

    if ($wifiNetworks === FALSE) {
      throw new Exception('Scan for wifi failed');
    }
    return $res->withJson($wifiNetworks, 200);
  });

  /////////////////////////////////////////////////////////
  // CONNECT TO WIFI NETWORK
  // /admin-only/...
  $this->post('/wifi/networks/{ssidHex}', function ($req, $res, $args) {
    if (IS_DEV_ENV) {
      sleep(1);
      return $res->withStatus(200);
    }
    $ssidHex = $args['ssidHex'];
    $postData = $req->getParsedBody();
    $password = '';
    if (isset($postData['password'])) {
      $password = $postData['password'];
    }
    $success = SystemHelper::Wlan1ConnectToSsid($ssidHex, $password);
    if ($success === FALSE) {
      throw new Exception('Connect to wifi failed');
    }
    return $res->withStatus(200);
  });

  /////////////////////////////////////////////////////////
  // GET WIFI CONNECT STATUS
  // /admin-only/...
  $this->get('/wifi/connect-status', function ($req, $res, $args) {
    // TODO remove this
    if (IS_DEV_ENV) {
      sleep(1);
      return $res->withJson(array(
        'ssid_hex' => '0000',
        'ssid' => 'Test Network',
        'status' => 'connected',
      ), 200);
    }

    $wifiStatus = SystemHelper::Wlan1ConnectGetStatus();
    if ($wifiStatus === FALSE) {
      throw new Exception('Get status failed');
    }
    return $res->withJson($wifiStatus, 200);
  });

  /////////////////////////////////////////////////////////
  // DISCONNECT WIFI
  // /admin-only/...
  $this->get('/wifi/disconnect', function ($req, $res, $args) {
    // TODO remove this
    if (IS_DEV_ENV) {
      sleep(1);
      return $res->withStatus(200);
    }

    $success = SystemHelper::Wlan1Disconnect();
    if ($success === FALSE) {
      throw new Exception('Disconnect failed');
    }
    return $res->withStatus(200);
  });

  /////////////////////////////////////////////////////////
  // GET RECENT ACTIVITY BY DEVICE NUM AND 
  // /admin-only//...
  $this->get('/recent-activity/{deviceNum}/{idGreaterThan}[/{limit}]', function ($req, $res, $args) {
    $deviceNum = $args['deviceNum'];
    $idGreaterThan = $args['idGreaterThan'];
    $limit = 100;
    if (isset($args['limit'])) {
      $limit = $args['limit'];
    }
  
    $this->dbManagerWifi->openDb();
      $deviceId = $this->dbManagerWifi->devicesGetDeviceIdWithDeviceNum($deviceNum);
      $recentActivity = $this->dbManagerWifi->statsGetRecentActivity($limit, TRUE, $idGreaterThan, $deviceId);
    $this->dbManagerWifi->closeDb();

    return $res->withJson($recentActivity);
  });

  // Middleware, if not from same IP address, return false
})->add(function ($request, $response, $next) {
  $this->dbManagerWifi->openDb();
    $isAdmin = $this->dbManagerWifi->usersCheckLogin();
  $this->dbManagerWifi->openDb();
  if ($isAdmin) {
    return $next($request, $response);
  } else {
    error_log('ERR: client tried to access admin route.');
    require_once(DOC_ROOT . '/utilities/secure_clear_cache.php');
    return $response->withJson(FALSE, 401);
  }
});