<?php
use \Renew\DbManagerWifi;
use \Renew\DeviceInfo;
use \Renew\Validator;

/////////////////////////////////////////////////////////
// ARCLIGHT JSON ENDPOINT
$this->get('/arclight-json', function ($req, $res, $args) {
  $this->dbManagerMedia->openDb();
    $arclightJson = $this->dbManagerMedia->mediaGetArclightJsonFile();
  $this->dbManagerMedia->closeDb();
  if ($arclightJson === FALSE) {
    throw new Exception('Get arclight json failed');
  }
  return $res->withJson($arclightJson);
});

/////////////////////////////////////////////////////////
// GET CURRENT FIRMWARE VERSION
// In form #.#.#
$this->get('/firmware-version', function ($req, $res, $args) {
  return $res->withJson(DeviceInfo::GetLightStreamFirmwareVersion());
});


/////////////////////////////////////////////////////////
// LANGUAGE DIRECTORIES ROUTE
$this->get('/language-directories', function ($req, $res, $args) {
  $languageDirectoriesArray = $this->slimDataHelper->getLanguageDirectories();
  return $res->withJson($languageDirectoriesArray);
});



/////////////////////////////////////////////////////////
// CACHED SETTINGS ROUTE
$this->get('/settings-cached', function ($req, $res, $args) {
  $settings = $this->slimDataHelper->getSettings();
  return $res->withJson($settings);
});



/////////////////////////////////////////////////////////
// FORMS ROUTE
$this->get('/forms[/{formId}]', function ($req, $res, $args) {
  $formId = isset($args['formId']) ? $args['formId'] : DbManagerWifi::FOLLOWUP_FORM_ID;
  $form = $this->slimDataHelper->getForm($formId);
  return $res->withJson($form);
});



/////////////////////////////////////////////////////////
// FORM SUBMIT ROUTE
$this->post('/forms[/{formId}]', function ($req, $res, $args) {
  $formId = isset($args['formId']) ? $args['formId'] : DbManagerWifi::FOLLOWUP_FORM_ID;

  $this->dbManagerWifi->openDb();
    $formSettings = $this->dbManagerWifi->settingsGetFormAll();
  $this->dbManagerWifi->closeDb();
  $formEnabled = Validator::StringToBoolean($formSettings['form_followup_enabled']['value']);
  if (!$formEnabled) {
    // TODO ERROR
    error_log('ERROR: Form not enabled.');
    return $res->withJson(FALSE, 500);
  }

  // Get POST data
  $fields = $req->getParsedBody();

  // Get data from each field
  foreach($fields as $fieldId => $value) {
    if (!empty($value)) {
      $saveArray[] = array(
        'field_id' => $fieldId,
        'data' => $value
      );
    }
  }

  // Get device ID
  if (!isset($_SESSION['device_id']) || empty($_SESSION['device_id'])) {
    // TODO call session function?
    $deviceId = '';
  } else {
    $deviceId = $_SESSION['device_id'];
  }

  // Save
  $this->dbManagerWifi->openDb();
    $responseId = $this->dbManagerWifi->formResponsesInsertResponseAndData($deviceId, DbManagerWifi::FOLLOWUP_FORM_ID, $saveArray);
    $this->dbManagerWifi->statsInsert(DbManagerWifi::INTERACTION_TYPE_FORM, $_SESSION['device_id'], NULL, DbManagerWifi::FOLLOWUP_FORM_ID);

  $responsesArray = $this->dbManagerWifi->formResponsesGetFormattedResponsesForFormId(DbManagerWifi::FOLLOWUP_FORM_ID, FALSE, 0, 10);
  $this->dbManagerWifi->closeDb();

  // Success
  if ($responseId !== FALSE) {
    return $res->withJson(array(
      'response_id' => $responseId
    ), 200);
  }
  throw new Exception('Form save failed');
});
