<?php
use \Renew\DeviceInfo;
use \Renew\Validator;
use \Renew\DiskInfo;
use \Renew\StringHelper;
use \Renew\SystemHelper;

/////////////////////////////////////////////////////////
// NODE ROUTES
// ONLY CALLABLE BY NODE
// Starts with /node-only
$this->group(NODE_ONLY_URL_STRING, function() {
  
  $this->post('/is-session-logged-in-admin', function ($req, $res, $args) {
    $postData = $req->getParsedBody();
    if (!isset($postData['session_id'], $postData['user_agent'])) {
      return $res->withJson(FALSE, 400);
    }
    $sessionId = $postData['session_id'];
    $userAgent = $postData['user_agent'];

    $this->dbManagerWifi->openDb();
      $isLoggedInAdmin = $this->dbManagerWifi->usersCheckLogin($sessionId, $userAgent);
    $this->dbManagerWifi->closeDb();

    return $res->withJson($isLoggedInAdmin);
  });

  /////////////////////////////////////////////////////////
  // CHECK IF USER IS LOGGED IN
  // Spoof Cookies in request to check.
  // /node-only/...
  $this->get('/is-logged-in-admin', function ($req, $res, $args) {
    $this->dbManagerWifi->openDb();
      $isLoggedInAdmin = $this->dbManagerWifi->usersCheckLogin();
    $this->dbManagerWifi->closeDb();

    // Clear cache if accessing admin route
    if ($isLoggedInAdmin) {
      require_once(DOC_ROOT . '/utilities/secure_clear_cache.php');
    }

    return $res->withJson($isLoggedInAdmin, 200);
  });

  /////////////////////////////////////////////////////////
  // GET SETTING FROM DB
  // /node-only/...
  $this->get('/settings-server/serial-number', function ($req, $res, $args) {
    global $serialNumber;
    return $res->withJson($serialNumber, 200);
  });

  /////////////////////////////////////////////////////////
  // GET SETTING FROM DB
  // /node-only/...
  $this->get('/settings-server/{key}', function ($req, $res, $args) {
    $key = $args['key'];
    $this->dbManagerWifi->openDb();
      $value = $this->dbManagerWifi->settingsGetValueForKey($key);
    $this->dbManagerWifi->closeDb();
    return $res->withJson($value, 200);
  });

  /////////////////////////////////////////////////////////
  // Create Language Directory if one doesn't exist
  // /node-only/...
  // @post language_iso, name_english, name_native
  // @returns fileId
  $this->post('/media/language-directories', function ($req, $res, $args) {
    $postData = $req->getParsedBody();
    if (!isset($postData['language_iso'])) {
      return $res->withJson(FALSE, 400);
    }

    $languageIso = $postData['language_iso'];
    $title = $postData['title'];

    $this->dbManagerMedia->openDb();
      $languageDirectories = $this->dbManagerMedia->mediaGetLanguageDirectoriesWithLanguageIso($languageIso);
      if ($languageDirectories === FALSE) {
        throw new Exception('Getting language directories failed');
      }

      // If none exist, make one
      if (count($languageDirectories) === 0) {
        $fileId = $this->dbManagerMedia->mediaCreateLanguageDirectoryWithLanguageIso($postData['language_iso'], $title);
        if ($fileId === FALSE) {
          throw new Exception('Create folder failed');
        }
      } else {
        $fileId = $languageDirectories[0]['file_id'];
      }
    $this->dbManagerMedia->closeDb();

    return $res->withJson($fileId, 200);
  });

  /////////////////////////////////////////////////////////
  // Get first language directory by language iso
  // /node-only/...
  $this->get('/media/language-directories/{languageIso}', function ($req, $res, $args) {
    if (!isset($args['languageIso'])) throw new Exception('Err: no language iso');
    $languageIso = $args['languageIso'];
    $this->dbManagerMedia->openDb();
      $languageDirectories = $this->dbManagerMedia->mediaGetLanguageDirectoriesWithLanguageIso($languageIso);
    $this->dbManagerMedia->closeDb();
    if ($languageDirectories && count($languageDirectories) > 0) {
      $firstLanguageDirectory = $languageDirectories[0];
      return $res->withJson($firstLanguageDirectory, 200);
    }
    return $res->withJson(FALSE, 404);
  });

  /////////////////////////////////////////////////////////
  // Add media item to database
  // /node-only/...
  // @post downloader_media_item
  $this->post('/media', function ($req, $res, $args) {
    $postData = $req->getParsedBody();
    if (!isset($postData['downloader_media_item'], $postData['file_path_abs'])) {
      return $res->withJson(FALSE, 400);
    }
    $filePathAbs = $postData['file_path_abs'];
    $parentDirId = $postData['parent_dir_id'];
    $saveDirId = $parentDirId;
    $downloaderMediaItem = $postData['downloader_media_item'];
    $nestedDirName = $postData['nested_dir_name'];
    $updateArray = array(
      'description' => $downloaderMediaItem['description_native'],
      // 'title' => $downloaderMediaItem['title_native'], Comment out to use filename, which includes language name
      'featured' => $downloaderMediaItem['featured'],
      'language_iso' => $downloaderMediaItem['language_iso'],
      'downloader_source' => $downloaderMediaItem['downloader_source'],
      'downloader_json' => json_encode($downloaderMediaItem['downloader_json']),
      'image_id' => $downloaderMediaItem['image_id'],
    );
    $this->dbManagerMedia->openDb();
      if ($nestedDirName) {
        $nestedDirName = StringHelper::FormatFilename($nestedDirName);
        $nestedDirId = $this->dbManagerMedia->mediaCreateDirectoryWithFilenameAndParentId($nestedDirName, $parentDirId, FALSE, TRUE);
        if ($nestedDirId !== FALSE) {
          $saveDirId = $nestedDirId;
        }
      }
      $fileId = $this->dbManagerMedia->mediaAddFileWithPathToMediaDirAndDb($filePathAbs, $saveDirId, $updateArray);
      if ($fileId === FALSE) {
        throw new Exception('Add new media item failed');
      }
    $this->dbManagerMedia->closeDb();

    return $res->withJson($fileId, 204);
  });

  /////////////////////////////////////////////////////////
  // Get image with imageTitle
  // /node-only/...
  $this->get('/images/{imageTitle}', function ($req, $res, $args) {
    if (!isset($args['imageTitle'])) {
      throw new Exception('Err: no image title');
    }
    $imageTitle = $args['imageTitle'];
    $this->dbManagerMedia->openDb();
      $image = $this->dbManagerMedia->imagesGetImageWithTitle($imageTitle);
    $this->dbManagerMedia->closeDb();
    if ($image === FALSE) {
      return $res->withStatus(404);
    }

    return $res->withJson($image['image_id'], 200);
  });

  /////////////////////////////////////////////////////////
  // Add image to image database
  // /node-only/...
  // @return string the image id of the newly created image
  $this->post('/images', function ($req, $res, $args) {
    $postData = $req->getParsedBody();
    if (!isset($postData['image_path_abs'], $postData['image_title'])) {
      throw new Exception('Image add post misformed');
    }
    $this->dbManagerMedia->openDb();
      $imageId = $this->dbManagerMedia->imagesAddImage($postData['image_title'], $postData['image_path_abs']);
    $this->dbManagerMedia->closeDb();
    if ($imageId === FALSE) {
      throw new Exception('Add image failed');
    }

    return $res->withJson($imageId, 200);
  });

  /////////////////////////////////////////////////////////
  // Disconnect from WiFi
  // /node-only/...
  $this->get('/wifi/disconnect', function ($req, $res, $args) {
    // TODO remove this
    if (IS_DEV_ENV) {
      sleep(1);
      return $res->withStatus(200);
    }

    $success = SystemHelper::Wlan1Disconnect();
    if ($success === FALSE) {
      throw new Exception('Disconnect failed');
    }
    return $res->withStatus(200);
  });

  $this->get('/disk/free-bytes', function($req, $res, $args) {
    $diskFreeSpaceInB = DiskInfo::DiskFreeSpaceInB(MEDIA_DIR, LS_RESERVE_SIZE);
    return $res->withJson($diskFreeSpaceInB, 200);
  });

  // Middleware, if not from same IP address, return false
})->add(function ($request, $response, $next) {
  if (DeviceInfo::RequestingDeviceIpIsLocalhost($_SERVER)) {
    return $next($request, $response);
  } else {
    throw new Exception('Client tried to access private route');
  }
});