<?php
use \Renew\SlimHelper;
use \Renew\DbManagerMedia;
use \Renew\DbManagerWifi;
use \Renew\Validator;

/////////////////////////////////////////////////////////
// MEDIA SINGLE ITEM ROUTE
$this->get('/media-item[/{fileId}]', function ($req, $res, $args) {
  if (!isset($args['fileId'])) {
    $fileId = DbManagerMedia::MEDIA_ROOT_DIRECTORY_FILE_ID;
  } else {
    $fileId = (int) $args['fileId'];
  }
  $this->dbManagerMedia->openDb();
    $mediaItem = $this->dbManagerMedia->mediaGetFileWithId($fileId, TRUE, TRUE);
  $this->dbManagerMedia->closeDb();

  // TODO: should we move this into the mediaGetFileWithId query?
  // if ($mediaItem['is_deleted']) {
  //   $mediaItem = FALSE;
  // }

  return $res->withJson($mediaItem);
});



/////////////////////////////////////////////////////////
// MEDIA ROUTE
$this->get('/media[/{directoryId}]', function ($req, $res, $args) {
  define('DEPTH_DEFAULT', 2); // 3
  define('LIMIT_ARRAY_DEFAULT', [30, 10]);
  define('OFFSET_DEFAULT', 0);

  $directoryId = SlimHelper::getDirectoryIdFromArgs($args);

  // Get query parameters
  $depth = (int) $req->getQueryParam('depth', DEPTH_DEFAULT);
  $offset = (int) $req->getQueryParam('offset', OFFSET_DEFAULT);
  $limitArray = $req->getQueryParam('limit-array', LIMIT_ARRAY_DEFAULT);

  // TODO validate input
  // Turn limit list of values delimted by ',' into an array
  if ($limitArray !== LIMIT_ARRAY_DEFAULT) {
    $limitArray = json_decode('[' . $limitArray . ']', TRUE);
  }

  // dbManagerMedia expects the deepest levels first
  $limitArray = array_reverse($limitArray);

  $this->dbManagerMedia->openDb();
    $currentDirectory = $this->dbManagerMedia->mediaGetFileWithId($directoryId, TRUE, TRUE);
    if ($currentDirectory !== FALSE) {
      $currentDirectoryPathRel = DbManagerMedia::MediaGetPathRelWithFile($currentDirectory);
      $mediaArray = $this->dbManagerMedia->mediaGetHomepageWithParentPath($currentDirectoryPathRel, $offset, $limitArray, $depth);
      $shouldGetGlobalMedia = DbManagerMedia::MediaItemIsLanguageDirectory($currentDirectory) && $offset === OFFSET_DEFAULT;
      if ($shouldGetGlobalMedia) {
        $globalMedia = $this->dbManagerMedia->mediaGetGlobalMedia();
        if ($globalMedia !== FALSE) {
          $mediaArray = DbManagerMedia::MediaItemArrayRemoveGlobal($mediaArray); // Don't allow duplicates in directory
          $mediaArrayCombined = array_merge($mediaArray, $globalMedia);
          // $mediaArrayCombined = DbManagerMedia::MediaItemArrayUsort($mediaArrayCombined);
          $currentDirectory['children'] = $mediaArrayCombined;
        }
      } else {
        $currentDirectory['children'] = $mediaArray;
      }
    } else {
      throw new Exception('Current directory does not exist');
    }
  $this->dbManagerMedia->closeDb();

  return $res->withJson($currentDirectory);
});



/////////////////////////////////////////////////////////
// MEDIA FEATURED ROUTE
$this->get('/media-featured[/{directoryId}]', function ($req, $res, $args) {
  $directoryId = SlimHelper::getDirectoryIdFromArgs($args);
  $this->dbManagerMedia->openDb();
    if ($directoryId === DbManagerMedia::MEDIA_ROOT_DIRECTORY_FILE_ID) {
      // TODO JOSIAH move this to dbManagerWifi, or remove and make a new function to get the names in an array
      function getKeyArrayFromLanguageDirectoriesArray($languageDirectoriesArray, $key) {
        $languageDirectoryFilenamesArray = array();
        foreach ($languageDirectoriesArray as $language) {
          $languageDirectoryFilenamesArray[] = $language[$key];
        }
        return $languageDirectoryFilenamesArray;
      }
      $languageDirectoriesArray = $this->dbManagerMedia->mediaGetLanguageDirectories(TRUE);
      $ignoreDirectoryFilenamesArray = getKeyArrayFromLanguageDirectoriesArray($languageDirectoriesArray, 'file_name');
      $featuredMediaArray = $this->dbManagerMedia->mediaGetFeaturedMedia(TRUE, TRUE, $ignoreDirectoryFilenamesArray);
    } else { // Language directory
      $currentDirectory = $this->dbManagerMedia->mediaGetFileWithId($directoryId);
      $featuredMediaArray = $this->dbManagerMedia->mediaGetFeaturedMedia(TRUE, TRUE, FALSE, $currentDirectory['file_name']);
      $featuredMediaArrayGlobal = $this->dbManagerMedia->mediaGetGlobalMedia(TRUE);
      $featuredMediaArray = array_merge($featuredMediaArray, $featuredMediaArrayGlobal);
    }
  $this->dbManagerMedia->closeDb();
  return $res->withJson($featuredMediaArray);
});



/////////////////////////////////////////////////////////
// PROMOTED MEDIA ITEM ROUTE
$this->get('/media-promoted[/{directoryId}]', function ($req, $res, $args) {
  $directoryId = SlimHelper::getDirectoryIdFromArgs($args);
  $promotedMediaItem = null;
  $this->dbManagerMedia->openDb();
    if ($directoryId === DbManagerMedia::MEDIA_ROOT_DIRECTORY_FILE_ID) {
      // If in root, get promoted media from greeting panel settings (if it's enabled)
      $this->dbManagerWifi->openDb();
        $greetingSettings = $this->dbManagerWifi->settingsGetGreetingAll();
      $this->dbManagerWifi->closeDb();
      if ($greetingSettings['greeting_mode']['value'] == DbManagerWifi::GREETING_MODE_MEDIA) {
        $greetingModeMediaFileId = $greetingSettings['greeting_mode_media_file_id']['value'];
        $promotedMediaItem = $this->dbManagerMedia->mediaGetFileWithId($greetingModeMediaFileId, TRUE, TRUE);
      }
    } else {
      // Get promoted media item if directory has one
      $currentDirectory = $this->dbManagerMedia->mediaGetFileWithId($directoryId);
      $promotedMediaFileId = $currentDirectory['promoted_media_file_id'];
      if ((int) $promotedMediaFileId !== 0) {
        $promotedMediaItem = $this->dbManagerMedia->mediaGetFileWithId($promotedMediaFileId, TRUE, TRUE);
      }
    }
  $this->dbManagerMedia->closeDb();
  return $res->withJson($promotedMediaItem);
});

/////////////////////////////////////////////////////////
// SEARCH FOR MEDIA BY NAME
$this->get('/media-search', function ($req, $res, $args) {
  $search = $req->getQueryParam('search', '');
  $onlyVideo = Validator::StringToBoolean($req->getQueryParam('onlyVideo', 'false'));
  $page = (int) $req->getQueryParam('page', 0);
  $this->dbManagerMedia->openDb();
    $paginatedResponse = $this->dbManagerMedia->mediaSearch($search, $page, DbManagerMedia::LIMIT_SEARCH_DEFAULT, $onlyVideo);
  $this->dbManagerMedia->closeDb();
  return $res->withJson($paginatedResponse, 200);
});