<?php
use \Renew\SessionHelper;
use \Renew\DbManagerWifi;
use \Renew\DbManagerMedia;
use \Renew\StringHelper;
use \Renew\Validator;

/////////////////////////////////////////////////////////
// GET ROUTES INCLUDED HERE WILL NOT BE CACHED
// ALL ROUTES WILLS START WITH /no-cache (NO_CACHE_URL_STRING)
$this->group(NO_CACHE_URL_STRING, function() {
  /////////////////////////////////////////////////////////
  // SETTINGS ROUTE (half cached, half uncached)
  // /no-cache/settings
  $this->get('/settings', function ($req, $res, $args) {
    $settingsCached = $this->slimDataHelper->subRequestHelper('/settings-cached');
    $settingsNoCache = $this->slimDataHelper->getSettingsNoCache();
    $settings = array();
    $settings = $settingsCached + $settingsNoCache;
    return $res->withJson($settings);
  });

  /////////////////////////////////////////////////////////
  // CONFIRM STILL CONNECTED TO WIFI ROUTE
  // /no-cache/device-is-still-connected
  $this->get('/device-is-still-connected', function ($req, $res, $args) {
    global $serialNumber;
    return $res->withJson($serialNumber);
  });

  /////////////////////////////////////////////////////////
  // MEDIA PASSWORD AUTHENTICATE ROUTE
  // /no-cache/media...
  $this->post('/media/{fileId}/password', function ($req, $res, $args) {
    $success = TRUE;
    $fileId = $args['fileId'];
    $postData = $req->getParsedBody();
    if (isset($postData['password'])) {
      $password = $postData['password'];

      // TODO: check to see if password matches the actual fileID password
      $this->dbManagerMedia->openDb();
        $success = $this->dbManagerMedia->mediaPasswordsCheckFileIdHasPassword($fileId, $password);
      $this->dbManagerMedia->closeDb();


      if ($success === TRUE) {
        SessionHelper::AuthenticateFileId($fileId);
      }
    } else {
      $success = FALSE;
    }
    return $res->withJson($success);
  });



  /////////////////////////////////////////////////////////
  // MEDIA CHECK IS PASSWORD AUTHENTICATED ROUTE
  // /no-cache/media...
  $this->get('/media/{fileId}/password', function ($req, $res, $args) {
    $fileId = $args['fileId'];
    return $res->withJson(SessionHelper::CheckFileIdIsAuthenticated($fileId));
  });
  
  

  /////////////////////////////////////////////////////////
  // FORM MAILTO from responseId
  // /no-cache/form-mailto
  $this->get('/form-mailto/{responseId}', function ($req, $res, $args) {
    require_once(DOC_ROOT . '/admin/reports/dashboard_summary_helper.php');

    if (!isset($args['responseId'])) {
      throw new Exception('Reseponse id undefined');
    }
    $responseId = $args['responseId'];

    $this->dbManagerWifi->openDb();
      // Get form information
      $followupForm = $this->dbManagerWifi->formsGetFormWithId(DbManagerWifi::FOLLOWUP_FORM_ID);
      $reportingEmail = $this->dbManagerWifi->settingsGetValueForKey('reporting_email');
      $formSettings = $this->dbManagerWifi->settingsGetFormAll();
      $formResponseAndData = $this->dbManagerWifi->formResponsesGetResponseAndDataWithId($responseId);
    $this->dbManagerWifi->closeDb();

    $emailBodyString = '';
    if ($formResponseAndData !== FALSE) {
      // Create email from response data
      foreach ($formResponseAndData['data'] as $responseData) {
        $emailBodyString .= $responseData['title'] . NEWLINE;
        $emailBodyString .= $responseData['data'] . NEWLINE;
        $emailBodyString .= NEWLINE;
      }

      // Append stats to email if enabled
      $formEmailStatsEnabled = Validator::StringToBoolean($formSettings['form_email_stats_enabled']['value']);

      if ($formEmailStatsEnabled) {
        $statsString = getFollowupFormEmailStatsString();
        for ($i = 0; $i < 10; $i++) {
          $emailBodyString .= NEWLINE;
        }
        $emailBodyString .= EMAIL_DIVIDER;
        $emailBodyString .= NEWLINE . NEWLINE;
        $emailBodyString .= $statsString;
      }

      // Create email url for mailto redirect
      $emailSubject = $followupForm['title'] . ' (' . $formResponseAndData['response'][0]['date_time'] . ')';
      $mailto_url = StringHelper::CreateMailtoUrl(
        $reportingEmail,
        $emailSubject,
        $emailBodyString
      );

      return $res->withJson(array(
        'mailto_url' => $mailto_url,
      ));
    }
    throw new Exception('Form response and data undefined.');
  });



  /////////////////////////////////////////////////////////
  // SESSION ROUTE
  // /no-cache/session
  $this->get('/session', function ($req, $res, $args) {
    $session = array();
    $this->dbManagerWifi->openDb();
      $isFirstHomepageVisit = $this->dbManagerWifi->checkIsFirstWifiVisitByDeviceId($_SESSION['device_id']);
    $this->dbManagerWifi->closeDb();
    $session['is_first_homepage_visit'] = $isFirstHomepageVisit;

    return $res->withJson($session);
  });



  /////////////////////////////////////////////////////////
  // LANGUAGE DIRECTORY FOR BROWSER TO JUMP INTO
  // /no-cache/browser-language-directory
  $this->get('/browser-language-directory', function ($req, $res, $args) {
    $browserLanguageDirectory = $this->slimDataHelper->getBrowserLanguageDirectory();
    return $res->withJson($browserLanguageDirectory);
  });

  /////////////////////////////////////////////////////////
  // GET SESSION ID
  // Used by node to check authentication
  $this->get('/session-id', function ($req, $res, $args) {
    $sessionId = session_id();
    return $res->withJson($sessionId);
  });

  /////////////////////////////////////////////////////////
  // NODE GATEWAY ROUTE THAT ONLY CALLS NODE IF AUTHENTICATED
  // /no-cache/forward-route-if-authenticated
  // $this->post('/forward-route-if-authenticated', function ($req, $res, $args) {
  //   $postData = $req->getParsedBody();
  //   error_log(print_r($postData, TRUE));

  //   $routeString = $postData['route_string'];
  //   unset($postData['route_string']);
  //   $postData['ip_address'] = DeviceInfo::GetRequestingDeviceIp($_SERVER);

  //   $ch = curl_init();
  //   $url = "http://localhost{$routeString}";
  //   error_log($url);
  //   curl_setopt($ch, CURLOPT_URL, $url);
  //   curl_setopt($ch, CURLOPT_POST, 1);
  //   $postDataString = http_build_query($postData);
  //   error_log('QUERY');
  //   error_log($postDataString);
  //   curl_setopt($ch, CURLOPT_POSTFIELDS, $postDataString);
  //   curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
  //   $response = curl_exec($ch);
  //   curl_close($ch);

  //   error_log('GETTING RESPONSE!');
  //   error_log(print_r($response, TRUE));

  //   return $res->withJson($response);
  // });
});