<?php

/**
 * Setup the database connection
 *
 * @author JPul
 **/
require_once(dirname(dirname(__FILE__)) . DIRECTORY_SEPARATOR . 'config' . DIRECTORY_SEPARATOR . 'settings.php');
set_include_path('.' . PS . BASE_DIR . DS . 'lib' . PS . get_include_path());
require_once('vendor' . DS . 'autoload.php');
require_once('Renew' . DS . 'TemplateManager.php');
require_once('Renew' . DS . 'DbManagerWifi.php');
require_once('Renew' . DS . 'DeviceInfo.php');
require_once('Renew' . DS . 'StringHelper.php');
require_once('Renew' . DS . 'SessionHelper.php');
require_once('Renew' . DS . 'CacheHelper.php');
require_once('Renew' . DS . 'Utilities' . DS . 'Validator.php');
use \Renew\SessionHelper;
use \Renew\CacheHelper;
use \Renew\DeviceInfo;
use \Renew\Validator;

// Start session with session_start() for all pages
SessionHelper::StartSecureSession();

// Doctrine cache for slim API calls
// See middleware in api_config.php
// use $slimCache->flushAll() to clear cache
// PhpFileCache (can have better performance with opcache?)
$slimCache = new CacheHelper(new Doctrine\Common\Cache\FilesystemCache(DOC_ROOT . '/cache'));

// Smarty::CACHING_OFF enabled by default. To turn on, set $t->caching = 1;
$t = new Renew\TemplateManager(BASE_DIR);
$t->default_modifiers = array('escape:"html"');

$dbManagerWifi = new Renew\DbManagerWifi(LSWIFI_DB);

/**
 * Function to take associative array of settings and apply them to the template manager
 * @param $settings Associative array of key, value settings
 **/
 function templateAssignSettings($settings) {
   global $t;
   foreach($settings as $setting) {
     $key = $setting['key'];
     $value = Validator::StringToBoolean($setting['value']);
     $t->assign($key, $value);
   }
 }

function setDeviceIdSessionVariable($dbManagerWifi) {
  try {
    $dbManagerWifi->openDb();
    $ipAddr = DeviceInfo::GetRequestingDeviceIp($_SERVER);
    if ($ipAddr !== NULL && strlen($ipAddr) > 6 ) {
      $deviceId = $dbManagerWifi->ipAddrsGetDeviceIdWithIpAddr($ipAddr);
    } else {
      error_log("Had trouble getting the ip_addr: ${ipAddr}");
    }

    if ($deviceId === NULL || strlen($deviceId) < 5) { // TODO should this <5 be <= 5?
      error_log("Creating a temporary device_id for ip_addr: ${ipAddr}");
      $deviceId = 'temp-';
      $deviceId .= \Renew\StringHelper::GetRandomMd5String();
      $dbManagerWifi->devicesInsertDeviceId($deviceId);
      $dbManagerWifi->ipAddrsInsertOrReplace($ipAddr, $deviceId);
    }

    $_SESSION['device_id'] = $deviceId;
  } catch (\Exception $e) {
    error_log($e);
  }
}

// Do devices if needed
if (!isset($_SESSION['device_id']) || strlen($_SESSION['device_id']) < 5) {
  $dbManagerWifi->openDb();
    setDeviceIdSessionVariable($dbManagerWifi);
  $dbManagerWifi->closeDb();
}

// set the timezone to UTC
date_default_timezone_set('UTC');

// pass the serial number of the lightstream to the front end to prevent image caching
$serialNumber = '0000000000000000';
if (isset($_SERVER, $_SERVER['SERIAL_NUMBER'])) {
  $serialNumber = $_SERVER['SERIAL_NUMBER'];
}
