/*
  JS used with the user interface
*/
/*
  === InitUser Class ===
  NOTE: COUPLED TO HTML.
  This is the only class that is allowed to refrence HTML ids and classes.
  Functions used in <script> tags on the front-end to initialize template JS
*/
var initUser = {
  init: function(templateName, params) {
    switch(templateName) {
      /* ==================================================================== */
      case 'index.tpl':
        utilUser.setTimestamp();

        // Language folder switcher bar
        $('#languages-bar').on('click', function() {
          $('#languages-collapse').collapse('toggle');
        });

        // Update homepage visit stats
        $.get(
          '/update_stats.php',
          {type: 'homepage'}
        );
      break;
      /* ==================================================================== */
      case 'featured_media_item.tpl':
        var idPrefix = params['id_prefix'];
        var fileId = params['file_id'];
        var mimePart2 = params['mime_part2'];
        var isDirectory = params['is_directory'];

        $("#"+idPrefix+"-"+fileId+"-featured-stream").on('click', function() {
          if (isDirectory) {
            util.redirect("/index.php?id="+fileId);
          } else {
            // Will automatically stream audio and video
            util.redirect("/media_details.php?id="+fileId+"&autoplay=true");
          }
        });
        $("#"+idPrefix+"-"+fileId+"-featured-download").on('click', function() {
          userMedia.download(fileId, mimePart2);
        });
      break;
      /* ==================================================================== */
      case 'greeting_panel.tpl':
        var fileId = params['file_id'];
        var mimePart2 = params['mime_part2'];
        $('#greeting-custom-image').on('click', function() {
          if (params['greeting_mode_media_image_action'] === 'download') {
            userMedia.download(fileId, mimePart2);
          } else {
            userMedia.stream(fileId);
          }
        });
        $('#greeting-play-button').on('click', function() {
          userMedia.stream(fileId);
        });
        $('#greeting-download-button').on('click', function() {
          userMedia.download(fileId, mimePart2);
        });
      break;
      /* ==================================================================== */
      case 'folder_slider.tpl':
        var fileId = params['file_id'];

        folder.initializeArrows("#folder"+fileId, "#arrow-left-"+fileId, "#arrow-right-"+fileId);
        $("#arrow-left-"+fileId).on('click', function() {
          folder.scrollFolderLeft("#folder"+fileId);
        });
        $("#arrow-right-"+fileId).on('click', function() {
          folder.scrollFolderRight("#folder"+fileId);
        });
      break;
      /* ==================================================================== */
      case 'media_details_content.tpl':
        var fileId = params['file_id'];
        var fileType = params['file_type']; // mimepart1
				var mimePart2 = params['mime_part2']; // mimepart2
        var htmlVideoId = '#' + params['html_video_id'];
        var htmlAudioId = '#' + params['html_audio_id'];
        var autoplay = params['autoplay'];
        var streamSupported = params['stream_supported'];

        if (streamSupported) {
          if (fileType === 'video' && util.supportsHTMLVideo()) {
            userMedia.incrementStreamStatsOnHTMLPlay(fileId, htmlVideoId);
          } else if (fileType === 'audio' && util.supportsHTMLAudio()) {
            userMedia.incrementStreamStatsOnHTMLPlay(fileId, htmlAudioId);
          }
        }

        var mediaDetailsStream = function() {
          switch(fileType) {
            case 'video':
              if (!streamSupported) {
                userMedia.download(fileId, mimePart2);
              } else if (util.supportsHTMLVideo()) {
                userMedia.playHTMLAudioVideo(htmlVideoId);
              } else {
                userMedia.stream(fileId);
              }
            break;
            case 'audio':
              if (!streamSupported) {
                userMedia.download(fileId, mimePart2);
              } else if (util.supportsHTMLAudio()) {
                userMedia.playHTMLAudioVideo(htmlAudioId);
              } else {
                userMedia.stream(fileId);
              }
            break;
            default:
              userMedia.stream(fileId);
            break;
          }
        }

        // If URL has autoplay field
        if (autoplay && streamSupported) {
          mediaDetailsStream();
        }
        $('.media-details-stream-class').on('click', function() {
          mediaDetailsStream();
        });
        $('#media-details-download').on('click', function() {
          userMedia.download(fileId, mimePart2);

          // If downloading .apk
          // TODO currently this is always false, until the updater size issue is resolved
          if (params['is_apk']) {
            $('#apk-instructions-video-panel').toggle(true);
            $('#media-details-panel').toggle(false);
          }
        });
      break;
      /* ==================================================================== */
      case 'main_home_media.tpl':
        if (params['should_show_more_button']) {
          sharedAjax.ajaxTemplatesOnButtonClick(
            '#load-more-button',
            '#load-more-wrapper',
            '#main-home-media',
            params['media_limit'],
            params['children_count'],
            'index.php',
            {
              current_directory_id: params['current_directory_id'],
              general_disable_sliders: params['general_disable_sliders']
            }
          );
        }
      break;
      /* ==================================================================== */
      case 'form.tpl':
        var blink = params['blink'];
        if (blink) {
          var inputs = $('.form-inputs');
          inputs.each(function() {
            var currentInputId = '#' + $(this).attr('id');
            util.addClassAndRemoveAfterTime('pulse', currentInputId, 500);
          });
        }

        /* HAS BUGS: attemps to make inputs unfocus upon scrolling to the bottom to fix android issue
        var pageWindow = $(window);
        pageWindow.on('scroll', function() {
          if(pageWindow.scrollTop() + pageWindow.height() == $(document).height()) {
            var currentScrollTop = pageWindow.scrollTop();
            var focusedInput = $(':focus');
            focusedInput.blur(); // unfocuses element
            pageWindow.scrollTop(currentScrollTop);
          }
        });
        */

        $("#cancel-button").on('click', function(e) {
          util.goBack();
        });
      break;
      /* ==================================================================== */
      case 'form_success.tpl':
        var redirect = params['redirect'];
        var mailtoUrl = params['mailto_url'];
        if (redirect) {
          $('#redirect-form').submit();
        } else {
          setTimeout(function() {
            if (!util.iOSDevice()) {
              util.redirect("/");
            }
          }, 3000);
          if (mailtoUrl) {
            util.redirect(mailtoUrl);
          }
        }
        $("#cancel-button").on('click', function() {
          util.redirect('/');
        });
      break;
      /* ==================================================================== */
      default:
        console.log('Error: ' + templateName + ' is not a valid template name.');
      break;
    }
  }
};

/*
  === Slider class ===
  Helper functions for slick slider plugin
*/
var slider = {
  init: function(sliderId, autoplaySpeedSeconds) {
    /* TODO transition over to siema slider
    // HOW TO DO DOTS: https://codepen.io/pawelgrzybek/pen/boQQWy
    new Siema({
      selector: sliderId,
      duration: 200,
      easing: 'ease-out',
      perPage: 1,
      startIndex: 0,
      draggable: true,
      multipleDrag: true,
      threshold: 20,
      loop: true
    });
    */
      /*
    var slider = tns({
      container: util.addPound(sliderId),
      items: 1,
      slideBy: 'page',
      autoplay: true,
      mouseDrag: true,
      speed: 200,
      autoplayTimeout: autoplaySpeedSeconds * 1000
    });
    */


      var sliderInput = $(sliderId);
      var slickObject = {
        nextArrow: '<i class="material-icons slick-next-material button-hover">&#xE315;</i>',
        prevArrow: '<i class="material-icons slick-prev-material button-hover">&#xE314;</i>',
        waitForAnimate: false,
        touchThreshold: 10,
        dots: true,
        speed: 200,
      };
      autoplaySpeedSeconds = parseInt(autoplaySpeedSeconds);
      if (autoplaySpeedSeconds) {
        slickObject['autoplay'] = true;
        slickObject['autoplaySpeed'] = autoplaySpeedSeconds * 1000;
      }

      sliderInput.slick(slickObject);
  }
};

/*
  === Media class ===
  Downloading media, streaming media, playing HTML5 video and audio players, etc.
*/
var userMedia = {
  download: function(fileId, mimePart2) {
    // Show no download popup if IOS device
    if (util.iOSDevice()) {
      if (mimePart2 === 'pdf') {
        userMedia.stream(fileId);
      } else {
        var confirmButtonId = '#ios-popup-confirm';
        sharedAjax.template(
          'shared/popup_custom.tpl',
          {
            include_file: 'user/components/popup_ios_content.tpl',
            hide_cancel: true,
            confirm_text: '<i class="material-icons">&#xE5CA;</i>',
            confirm_button_id: util.removePound(confirmButtonId)
          },
          'body',
          false,
          function() {
            var confirmButtonTimeout = setTimeout(function() {
              $(confirmButtonId).click();
            }, 3000);

            // If confirmed early, clear it.
            $(confirmButtonId).on('click', function() {
              clearTimeout(confirmButtonTimeout);
            })
          }
        );
      }
    } else {
      userMedia.downloadClickedHelper(fileId, true, true, false);
    }
  },
  downloadDir: function(fileId) {
    if (!util.iOSDevice()) {
      userMedia.downloadClickedHelper(fileId, true, true, true);
    }
  },
  stream: function(fileId) {
    userMedia.downloadClickedHelper(fileId, false, true, false);
  },
  playHTMLAudioVideo: function(htmlMediaId) {
	  // [0] used to select JS DOM object from jQuery selection, play is native JS way to play HTML player
    if (userMedia.HTMLAudioVideoIsPlaying(htmlMediaId)) {
      $(htmlMediaId)[0].pause();
    } else {
      $(htmlMediaId)[0].play();
    }
  },
  HTMLAudioVideoIsPlaying: function(htmlMediaId) {
    var htmlPlayer = $(htmlMediaId)[0];
    return (htmlPlayer.currentTime > 0 && !htmlPlayer.paused && !htmlPlayer.ended && htmlPlayer.readyState > 2);
  },
  /*
    If HTML5 player is being played for the first time since page load,
    increment streaming stats.

    @param fileId Database id of the file being streamed
    @param htmlMediaId HTML id of the HTML5 player being played
  */
  incrementStreamStatsOnHTMLPlay: function(fileId, htmlMediaId) {
    // Firstplay variable is reset on each refresh
		// check to make sure the object exists
		if (!document.getElementById(util.removePound(htmlMediaId))) {
			console.log('The element:' + htmlMediaId + ' does not exist on the page');
			return;
		}
    var firstPlay = true;

    //$(htmlMediaId)[0].onplay = function() {
    $(htmlMediaId).on('play', function() {
      // Increment stats if the first time playing the video/audio
      if (firstPlay) {
        userMedia.incrementDownloadStats(fileId, false);
      }
      firstPlay = false;
    });
  },
  /*
    Helper to allow streaming or downloading of media files.

    @param fileId Database id of file to access
    @param forceDl If true, file will download, else, file will stream or open
    @param addToStats If true, stats will be updated with the current interaction
    @param isDir If true, directory will be zipped and downloaded
  */
  downloadClickedHelper: function(fileId, forceDl, addToStats, isDir) {
    if (isDir == true){
      console.log('Error: tried to download directory');
      return;
    }

  	var download_url_base = '/download.php';
  	var download_url;
  	// check if you should add it to the stats
  	if (addToStats == true ){
  		userMedia.incrementDownloadStats(fileId, forceDl);
  	}

    var downloadString = "&f=false";
  	if (forceDl == true){
      downloadString = "&f=true";
  	}
    download_url = download_url_base + "?id=" + fileId + downloadString;

    // FROM download.html
    // var request = new XMLHttpRequest();
    // request.addEventListener('readystatechange', function(e) {
    // 	if(request.readyState == 2 && request.status == 200) {
    //     console.log('Ready state 2');
    // 		// document.querySelector('#start-download').style.display = 'block';
    // 		// document.querySelector('#download-button').style.display = 'none';
    // 	}
    // 	else if(request.readyState == 3) {
    //     console.log('Ready state 3');
    // 		// document.querySelector('#download-progress-container').style.display = 'block';
    // 		// document.querySelector('#start-download').style.display = 'none';
    // 	}
    // 	else if(request.readyState == 4) {
    //     console.log('Ready state 4');
    //     _OBJECT_URL = URL.createObjectURL(request.response);
    //     console.log(_OBJECT_URL);

    // 		// document.querySelector('#save-file').setAttribute('href', _OBJECT_URL);
    // 		// document.querySelector('#save-file').setAttribute('download', 'img.jpeg');
    		
    // 		// document.querySelector('#save-file').style.display = 'block';
    // 		// document.querySelector('#download-progress-container').style.display = 'none';

    // 		// setTimeout(function() {
    // 		// 	window.URL.revokeObjectURL(_OBJECT_URL);

    // 		// 	document.querySelector('#download-button').style.display = 'block';
    // 		// 	document.querySelector('#save-file').style.display = 'none';
    // 		// }, 60*1000);
    // 	}
    // });
    // request.addEventListener('progress', function(e) {
    // 	var percent_complete = (e.loaded / e.total)*100;
    //   console.log(percent_complete);
    //   //document.querySelector('#download-progress').style.width = percent_complete + '%';
    // });
    // request.responseType = 'blob';
    // console.log(download_url);
    // request.open('get', download_url); 
    // request.send(); 

    // redirect to the download page
  	util.redirect(download_url);
  },
  /*
    Update stats for streaming or downloading of media files

    @param fileId Database id of file accessed
    @param forceDl If true, stats will be increased for file download, else, stats
    will be increased for file stream
  */
  incrementDownloadStats: function(fileId, forceDl){
  	// update the stats URL
  	var update_stats_url;
  	if (forceDl == true){
  		update_stats_url = "/update_stats.php?id=" + fileId + "&type=download";
  	}
  	else{
  		update_stats_url = "/update_stats.php?id=" + fileId + "&type=stream";
  	}
  	// make thae AJAX call to the server to increment the stats
  	$.get(update_stats_url, function (data){});
  }
};

/*
  === Folder class ===
  Navigation and initialization for folder sliders
*/
var folder = {
  scrollFolderRight: function(folderId) {
    folder.scrollFolder(folderId, '+');
  },
  scrollFolderLeft: function(folderId) {
    folder.scrollFolder(folderId, '-');
  },
  /*
    Scroll scrollable div either right or left, moving the width of the div.
    Helper for folder.scrollFolderRight and folder.scrollFolderLeft.

    @param folderId the folder_slider.tpl folder id
    @param direction '+' or '-', '+' for right, '-' for left
  */
  scrollFolder: function(folderId, direction) {
    var folderIn = $(folderId);

    // location where we can't scroll to the right any more
    // Width of the scrollable div - the width of the folder
    var endScrollRight = folderIn[0].scrollWidth - folderIn.width();

    // Don't scroll if there isn't anywhere to scroll;
    if (endScrollRight <= 0) {
      return;
    }

    folderIn.animate({scrollLeft: direction + '=' + folderIn.width()}, 300);
  },
  /*
    Set up folder arrows so that they are hidden when user scrolls to edge of folder
  */
  initializeArrows: function(folderId, arrowLeftId, arrowRightId) {
    var folderIn = $(folderId);
    var arrowLeft = $(arrowLeftId);
    var arrowRight = $(arrowRightId);

    // Disable right arrow if small folder
    if (folderIn[0].scrollWidth <= folderIn.width()) {
      arrowRight.toggle(false);
    }

    // Hide arrows on scroll to edges
    folderIn.on('scroll', function() {
      var endScrollRight = folderIn[0].scrollWidth - folderIn.width();
      arrowRight.toggle(true);
      arrowLeft.toggle(true);
      if (folderIn.scrollLeft() >= endScrollRight) {
        arrowRight.toggle(false);
      } else if (folderIn.scrollLeft() <= 0) {
        arrowLeft.toggle(false);
      }
    });
  },
};

/*
  === Client Data Class ===
  Gets data about the users like language, etc.
*/
var clientData = {
  getUserLanguage: function() {
    if(navigator.languages) {
      return navigator.languages[0]; // Chrome
    } else {
      return (navigator.language || navigator.userLanguage); // Other browsers
    }
  },
};

/*
  === User Utility Class ===
*/
var utilUser = {
  /*
    NOTE: Function is duplicated in captive_portal.tpl for code-reuse purposes.
    If you change this, update function in captive_portal.tpl.
  */
  setTimestamp: function() {
    // set the timestamp
    var utc_timestamp = new Date().getTime();
    utc_timestamp = Math.floor(utc_timestamp / 1000);
    $.get(
      '/client_date.php',
      {timestamp: utc_timestamp}
    );
  }
};
