/*
  JS used with the admin interface

  @author JP
*/

var constantsAdmin = {
  /* COUPLED TO FILESYSTEM */
  CLASS_RAISED_BUTTON_GLOBAL_GIF: '.submit-button-with-gif', // Coupled to raised_button.tpl
  CLASS_LOADING_GIF: '.loading-gif',
  ID_ADMIN_MAIN_PAGE: '#main-page-admin',
  ID_LOADING_GIF: '#loading-gif',
  ID_POPUP: '#popup',
  ID_POPUP_ERROR_MESSAGE: '#popup-error-message',
  ID_MEDIA_EDIT_NO_SELECTION: '#no-media-selected-message-wrapper', // Location of edit message when no selection has been made
  ID_MEDIA_EDIT_MULTI: '#multi-edit-wrapper', // Multi edit form, for when selection is made
  ID_FINEUPLOADER_DEFAULT: 'qq-default-template',
  SELECT2_UPLOAD: 'upload',
  SELECT2_GENERATE_THUMBNAIL: 'generate_thumbnail',
  SELECT2_CURRENT_THUMBNAIL: 'current_thumbnail',
  SELECT2_NONE: 'none',
  SELECT2_DEFAULT: 'default',
  SELECT2_CURRENT: 'current',
  SELECT2_CUSTOM_LANGUAGE: 'und',
  SELECT2_THUMBNAIL: 'thumbnail',
  SELECT2_DEFAULT_CLASS_NAME: '.select2',
  SELECT2_POPUP_Z_INDEX: 40,
  MEDIA_SELETED_MEDIA_ARRAY_NAME: 'selected_media_array[]', // Used when inserting selected media as additional form data
  PULSE_INTERVAL_MS: 2000,
  LOCALIZED_LANGUAGE_DELIMITER: ' | ',
  LANGUAGE_FOLDER_UNDEFINED: 'und',
  PHP_DRIVES: '/admin/settings/drives.php'
};

/*
  === InitAdmin Class ===
  NOTE: COUPLED TO HTML.
  This is the only class that is allowed to refrence HTML ids and classes.
  Functions used in <script> tags on the front-end to initialize template JS
*/
var initAdmin = {
  /*
    @param templateName String representing the template to be initialized with
    JS. Passed to a switch statement that will choose the correct initalization
    code.
    @param params an object that has all of the parameters necessary for the
    relevant initialization code.
  */
  init: function(templateName, params) {
    switch(templateName) {
      /* ==================================================================== */
      case 'media_item.tpl':
        var id = '#' + params['id'];
        var idEdit = '#' + params['id_edit'];
        var mediaDisplayTitle = params['media_item']['title'];
        var nameInputId = idEdit+"-title";
        if (params['is_language_directory']) {
          // mediaDisplayTitle = params['media_item']['file_name'];
          nameInputId = '#' + params['name_input_id'];
          var languageSelectId = '#' + params['language_select_id'];
          var languageIso = params['media_item']['language_iso'];
          // var fileName = params['media_item']['file_name'];

          // TODO Josiah make this include file name also (english name) instead of just display title
          // Add default selection, based on language_iso
          if (languageIso !== constantsAdmin['SELECT2_CUSTOM_LANGUAGE']) {
            $(languageSelectId).append(
              '<option selected value="'
              + languageIso
              + '">'
              + mediaDisplayTitle
              + ' ['
              +  languageIso
              + ']'
              + '</option>'
            );
          }
        }
        var isDirectory = params['is_directory'];
        var isAjax = params['ajax'];
        var collapseId = id+"-edit";
        var mediaImageId = id+"-media-item-image";
        var mediaItemTitleId = id+"-media-item-title";
        var selectImageId = id+"-select";
        var selectThumbnailInputId = selectImageId+'-thumbnail-input';

        collapse.toggleOnClick(collapseId, id+"-button-edit");
        collapse.showOnClick(id+"-edit", id+"-button-change-image");
        select2Class.openOnClick(selectImageId, id+"-button-change-image");

        // Directory image goes into directory, doesn't toggle edit
        if (!isDirectory) {
          collapse.toggleOnClick(collapseId, mediaImageId);
        }

        multiSelect.initSelectCheckbox(id+"-select-checkbox", ".media-selected-count", true);

        var editFormId = id+"-form";
        var deleteFormId = id+"-delete-form";
        var availabilityFormId = id+"-availability-form";
        var showFirstFormId= id+"-sort-order-form";

        // Edit form
        ajax.submitAjaxFormSuccessWithCallback(editFormId, function(newThumbnailId) {
          // Collapse media_item_edit.tpl
          collapse.hide(collapseId);

          // New image is already set if thumbnail is selected
          var newImageId = $(selectImageId).val();

          // Set value from invisible input to new thumbnail, if created in media_edit.php
          if (newThumbnailId) {
            $(selectThumbnailInputId).val(newThumbnailId);
            newImageId = newThumbnailId;
          }

          if (newImageId !== constantsAdmin['SELECT2_CURRENT_THUMBNAIL'] || newThumbnailId) {
            utilAdmin.setBackgroundImage(mediaImageId, newImageId);
          }

          // Set new title
          var currentTitle = $(nameInputId).val();
          $(mediaItemTitleId).text(currentTitle);
        });

        // Delete
        ajax.submitAjaxForm(
          deleteFormId, 
          function() { // Success
            popup.removeLoadingGif();
            ajax.appendSuccessMessage();
            util.fadeOutCSS(id+"-media-item");
          },
          function() { // Fail
            popup.removeLoadingGif();
          }
        );
        $(id+"-delete-button").on('click', function() {
          mediaDisplayTitle = $(mediaItemTitleId).text();
          popup.confirmPopup(
            utilAdmin.sprintf(
              params['message_confirm'],
              mediaDisplayTitle
            ),
            id+"-popup-confirm-button",
            function() {
              $(id+"-popup-confirm-button").on('click', function() {
                popup.loadingGif();
                form.submitForm(deleteFormId);
              });
            }
          );
        });

        // availability
        ajax.submitAjaxFormSuccess(availabilityFormId);
        availabilityForm = $(availabilityFormId);
        form.submitFormOnChange(availabilityFormId, id+"-wifi-status, "+id+"-usb-status, "+id+"-availability-featured");

        // Sort order
        ajax.submitAjaxForm(showFirstFormId, function() {
          // Don't refresh page if we are in the uploader
          if (!params['is_uploader']) {
            utilAdmin.refreshPageSuccess();
          } else {
            ajax.appendSuccessMessage();
          }
        });
        form.submitFormOnChange(showFirstFormId, id+"-show-first-checkbox");
        $(id+"-show-first-button").on('click', function() {
          utilAdmin.triggerCheckbox(id+"-show-first-checkbox");
        });

        if (isAjax) {
          select2Class.initImages(id+"-select");
        }
      break;
      /* ==================================================================== */
      case 'manage_images_icon.tpl':
        var imageId = params['image_id'];
        var imageTitleWrapper = $("#"+imageId+"-title");

        // Delete button
        var deleteFormId = "#"+imageId+"-delete-form";
        ajax.submitAjaxForm(deleteFormId, function() { util.fadeOutCSS("#"+imageId+"-icon"); });
        $("#"+imageId+"-delete").on('click', function() {
          popup.confirmPopup(
            utilAdmin.sprintf(
              params['confirm_message'],
              imageTitleWrapper.text()
            ),
            "#"+imageId+"-popup-confirm-button",
            function() {
              form.submitFormOnClick(deleteFormId, "#"+imageId+"-popup-confirm-button");
            }
          );
        });

        // Edit button
        var confirmButtonId = "#"+imageId+"-confirm";
        var inputId = "#"+imageId+"-input";
        var editFormId = "#"+imageId+"-edit-form";
        $("#"+imageId+"-edit").on('click', function() {
          popup.inputPopup(
            params['rename_heading'],
            '',
            confirmButtonId,
            inputId,
            params['rename_confirm_text'],
            '', // Placeholder
            imageTitleWrapper.text(), // Value
            function() { // Successful creation of template
              $(confirmButtonId).on('click', function() {
                var newTitle = $(inputId).val();
                var editForm = $(editFormId);
                var additionalFormDataArray = [{
                  key: 'title',
                  value: newTitle
                }];
                ajax.submitAjaxFormHelper(
                  editFormId,
                  additionalFormDataArray,
                  function() { // On success
                    imageTitleWrapper.text(newTitle);
                  },
                  false
                );
                form.submitForm(editForm);

                /*
                var createFolderForm = $(createFolderFormId);
                var title = $(inputId).val(); // Get value from text area

                var additionalFormDataArray = [{
                  key: 'title',
                  value: title
                }];
                ajax.submitAjaxFormHelper(
                  createFolderForm,
                  additionalFormDataArray,
                  function() {
                    util.refreshPage();
                  },
                  false
                );
                form.submitForm(createFolderForm);
                */
              });
            }
          );
        });
      break;
      /* ==================================================================== */
      case 'media_nav.tpl':
        var id = '#' + params['id'];
        collapse.toggleOnClick("#multi-edit-collapse", id+"-button-edit");
        multiSelect.initMultiselectCheckbox(id+"-checkbox-multiselect", 'input[name=\'' + params['media_checkbox_array_name'] + '\']');

        // Delete
        var multiDeleteFormId = id+"-delete-multi-form";
        var deletePopupConfirmButtonId = id+"-popup-confirm-button";
        multiSelect.clickActionButton(
          id+"-delete-multi-button",
          function() {
            popup.confirmPopup(
              utilAdmin.sprintf(
                params['delete_message'],
                multiSelect.multiSelectArray.length
              ),
              deletePopupConfirmButtonId,
              function() {
                // Get selected media
                ajax.submitAjaxFormHelper(
                  multiDeleteFormId,
                  multiSelect.getMultiSelectArrayAsFormDataArray(constantsAdmin['MEDIA_SELETED_MEDIA_ARRAY_NAME']),
                  function() {
                    utilAdmin.refreshPageSuccess();
                  },
                  false
                );
                $(deletePopupConfirmButtonId).on('click', function() {
                  popup.loadingGif();
                  $(multiDeleteFormId).submit();
                });
              }
            );
          },
          params['no_media_selected_message']
        );

        // Move
        var moveFormId = id+"-move-form";
        multiSelect.clickActionButton(
          id+"-move-button",
          function() {
            var confirmButtonId = id+'-popup-select-confirm-button';
            var selectId = id+'-popup-select';
            popup.popupHelper(
              'shared/popup_custom.tpl',
              {
                include_file: 'admin/components/media/media_move_select.tpl',
                heading: params['move_title_message'],
                confirm_text: params['move_title_message'],
                message: utilAdmin.sprintf(
                  params['move_message'],
                  multiSelect.multiSelectArray.length
                ),
                confirm_button_id: util.removePound(confirmButtonId),
                select_id: util.removePound(selectId),
                language_directories_array: params['language_directories_array'],
                directory_array: params['directory_array']
              },
              function() {
                select2Class.initInPopup(selectId);
                $(confirmButtonId).on('click', function() {
                  var selectedDirectoryIdFile = $(selectId).val();
                  var additionalFormDataArray = multiSelect.getMultiSelectArrayAsFormDataArray(constantsAdmin['MEDIA_SELETED_MEDIA_ARRAY_NAME']);
                  additionalFormDataArray.push({
                    key: 'directory',
                    value: selectedDirectoryIdFile
                  });
                  popup.loadingGif();
                  ajax.submitAjaxFormHelper(
                    moveFormId,
                    additionalFormDataArray,
                    function() {
                      utilAdmin.refreshPageSuccess();
                    },
                    false
                  );
                  form.submitForm(moveFormId);
                });
              }
            );

            /*
            // heading, message, confirmButtonId, popupSelectId, selectArrayJSON, callback
            popup.selectPopup(
              params['move_title_message'],
              utilAdmin.sprintf(
                params['move_message'],
                multiSelect.multiSelectArray.length
              ),
              id+"-popup-select-confirm-button",
              id+"-popup-select",
              params['directory_array'],
              function() {
                $(id+"-popup-select-confirm-button").on('click', function() {

                  var selectedDirectoryId = $(id+"-popup-select").val();
                  var additionalFormDataArray = multiSelect.getMultiSelectArrayAsFormDataArray(constantsAdmin['MEDIA_SELETED_MEDIA_ARRAY_NAME']);
                  additionalFormDataArray.push({
                    key: "directory",
                    value: selectedDirectoryId
                  });

                  ajax.submitAjaxFormHelper(
                    moveFormId,
                    additionalFormDataArray,
                    function() {
                      utilAdmin.refreshPageSuccess();
                    },
                    false
                  );
                  form.submitForm(moveFormId);
                });
              });*/
            },
            params['no_media_selected_message']
          );

        // Multi edit
        var multiEditFormId = "#multi-edit-form";
        $("#multi-edit-submit-button").on('click', function() {
          // Get selected media
          popup.loadingGif();
          ajax.submitAjaxFormHelper(
            multiEditFormId,
            multiSelect.getMultiSelectArrayAsFormDataArray(constantsAdmin['MEDIA_SELETED_MEDIA_ARRAY_NAME']),
            function() {
              utilAdmin.refreshPageSuccess();
            },
            false
          );
          form.submitForm(multiEditFormId);
        });
      break;
      /* ==================================================================== */
      case 'color_picker.tpl':
        var id = '#' + params['id'];
        reveal.revealWithInput(
          id+"-collapse",
          id+"-image",
          params['name']
        );
        form.colorpicker(id+"-input-color", id+"-input-text");
      break;
      /* ==================================================================== */
      case 'form_deletable_item_extend.tpl':
        var checkboxId = '#' + params['checkbox_id']; // Checkbox ID set to value
        deletableList.formDeletableItem(checkboxId+"-remove", checkboxId, checkboxId+"-tr", params['ajax']);
      break;
      /* ==================================================================== */
      case 'fineuploader_upload.tpl':
        var isMedia = params['is_media'];
        var errorPopupMessage = params['upload_failed_message'];
        var templateId = util.addPound(constantsAdmin['ID_FINEUPLOADER_DEFAULT']);
        if (params['fineuploader_template_id']) {
          var templateId = util.addPound(params['fineuploader_template_id']);
        }

        if (isMedia) {
          var formId = '#' + params['form_id'];
          var uploadedFilesId = '#' + params['uploaded_files_id'];
          var selectedDirectoryId = '#' + params['select_directory_id'];
          var step1ClickId = '#' + params['step1_id'];
          var step2ClickId = '#' + params['step2_id'];
          var step3ClickId = '#' + params['step3_id'];
          var groupClass = '.' + params['group_class'];
          var uploadedFilesLoadingMessage = $('#uploaded-files-loading-message');
          var mediaItemWrapperClass = '.media-item-wrapper';

          collapse.initAccordion(step2ClickId, groupClass, true);

          // Fineuploader instantiation
          var uploadForm = $(formId);
          var selectDirectory = $(selectedDirectoryId);
          var fineuploaderObject = uploader.initUploaderMulti({
            onComplete: function(id, fileName, responseJSON) {
              var fineuploaderObject = this;
              if (responseJSON['success']) {
                var formData = new FormData(uploadForm[0]);
                formData.append('directory', selectDirectory.val()); // Select dropdown
                formData.append('add_to_database', true);
                formData.append('temp_file_path', uploader.getTempFilePath(responseJSON, fileName)); // Location of file

                // Add file to database
                ajax.phpHelper(
                  '/admin/manage/media_forms/media_edit.php',
                  formData,
                  function(mediaItemArrayJSON) { // On success, return media_item as json
                    var mediaItemArray = JSON.parse(mediaItemArrayJSON);
                    var mediaItem = mediaItemArray['media_item'];
                    var imagesArray = mediaItemArray['images_array'];
                    sharedAjax.template(
                      'admin/components/media/media_item.tpl',
                      {
                        // TODO this should be making them open?
                        collapse: false,
                        media_item: mediaItem,
                        images_array: imagesArray,
                        is_uploader: true,
                        current_directory_id: params['current_directory_id']
                      },
                      uploadedFilesId,
                      false,
                      function() {
                        uploadedFilesLoadingMessage.toggle(false);
                        popup.removeLoadingGif();
                      }
                    );
                  },
                  function() { // Failure
                    popup.errorPopup(errorPopupMessage);
                  },
                  true
                );
              } else {
                popup.errorPopup(errorPopupMessage);
              }

              // Check to see if there are items queued up
              // Fixes bug where items that users queue up mid progress don't start
              var uploadsRemaining = fineuploaderObject.getUploads({ status: 'submitted' }).length > 0;
              if (uploadsRemaining) {
                uploader.fineuploaderStartUpload(fineuploaderObject);
              }
            },
            onSubmit: function(id) {
              var fineuploaderObject = this;
              utilAdmin.userConfirmBack(true);
            },
            onAllComplete: function() {
              popup.loadingGif(); // Removed by ajax.template
              // Show loading message only if there are no media items
              if ($(mediaItemWrapperClass).length === 0) {
                uploadedFilesLoadingMessage.toggle(true);
              }
              uploader.initUploaderMultiShowUpload();
              collapse.openAccordion(step3ClickId, groupClass);
              $('#finish-uploading-banner').toggle(true);
              uploader.fineuploaderReset(fineuploaderObject);
              translateFineuploaderStrings(); // Must be re-run after reset
            }
          },
          false,
          params['storage_free_b'],
          params['message_storage_full'],
          util.removePound(templateId)
          );
        } else { // Images Fineuploader
          var acceptFiles = params['image_allowed_mime_types'];
          var allowedExtensions = params['image_allowed_extensions'];
          var isDropdownImageUploader = params['is_dropdown_image_uploader'];

          var fineuploaderObject = uploader.initUploaderMulti({
            onComplete: function(id, fileName, responseJSON) {
              var currentFineuploaderObject = this;
              if (responseJSON['success']) {
                var uploadEndpoint = '/admin/manage/images_forms/upload_image.php';

                // If logo image, use different endpoint
                if (uploader.currentDropdownObject && uploader.currentDropdownObject['is_logo_image']) {
                  uploadEndpoint = '/admin/manage/images_forms/upload_image_logo.php';
                }

                // Add image to database
                ajax.php(
                  uploadEndpoint,
                  {
                    'temp_file_path': uploader.getTempFilePath(responseJSON, fileName),
                    'file_name': fileName
                  },
                  function(response) { // Success
                    var filesLeft = currentFineuploaderObject.getInProgress();
                    if (filesLeft < 1) {
                      popup.removeLoadingGif();
                      if (isDropdownImageUploader) {
                        if (uploader.currentDropdownObject['is_logo_image']) {
                          uploader.currentDropdownObject['toggle_image'](true, response);
                        } else {
                          response = JSON.parse(response);
                          select2Class.addImage(
                            uploader.currentDropdownObject['select_id'],
                            uploader.currentDropdownObject['select_group_class'],
                            uploader.currentDropdownObject['image_insert_class'],
                            response['image_id'],
                            response['filename']
                          );
                        }
                        uploader.currentDropdownObject = false;
                      } else { // If last item, refresh page
                        utilAdmin.refreshPageSuccess();
                      }
                    }
                  },
                  function() { // Failure
                    popup.errorPopup(errorPopupMessage);
                  }
                );
              } else {
                popup.errorPopup(errorPopupMessage);
              }
            },
            onSubmit: function(id) {
              utilAdmin.userConfirmBack(true);

              if (isDropdownImageUploader) {
                popup.loadingGif();
              }
              // Upload file automatically when it's selected
              var fineuploaderObject = this;
              uploader.fineuploaderStartUploadAfterTimeout(fineuploaderObject);
            },
            onAllComplete: function() {
              popup.loadingGif();
              // Page is refreshed in ajax.php() callback
            }
          },
          // Additional options
          {
            acceptFiles: acceptFiles,
            allowedExtensions: allowedExtensions,
            multiple: false // was var multiple
          },
          params['storage_free_b'],
          params['message_storage_full'],
          util.removePound(templateId)
          );
        }

        // Button clicks
        var cancelUploadId = templateId + '-cancel';
        var submitButtonId = templateId + '-submit';
        var backButtonId = templateId + '-back';
        $(submitButtonId).on('click', function() {
          uploader.fineuploaderStartUpload(fineuploaderObject);
        });
        $(cancelUploadId).on('click', function() {
          uploader.initUploaderMultiShowUpload();
          uploader.fineuploaderCancel(fineuploaderObject);
        });
        $(backButtonId).on('click', function() {
          collapse.openAccordion(step1ClickId, groupClass);
        });

        // Translate fineuploader dynamic strings
        // See https://developer.mozilla.org/en-US/docs/Web/API/MutationObserver
        var translateFineuploaderStrings = function() {
          var fineuploaderDialog = $(templateId + ' .fineuploader-dialog').first();
          var observer = new MutationObserver(function() {
            var dialogText = fineuploaderDialog.text();
            if (dialogText === 'No files to upload.') {
              fineuploaderDialog.text(params['fineuploader_message_no_files']);
            } else if (dialogText.includes('Valid extension(s)')) {
              fineuploaderDialog.text(params['fineuploader_message_invalid_extension']);
            }
          });
          observer.observe(fineuploaderDialog[0], { childList: true });
        }
        translateFineuploaderStrings();
      break;
      /* ==================================================================== */
      case 'upload_media_form.tpl':
        var step1ClickId = "#" + params['step1_id'];
        var step2ClickId = "#" + params['step2_id'];
        var groupClass = "." + params['group_class'];
        var formId = "#" + params['form_id'];

        collapse.initAccordion(step1ClickId, groupClass, true);
        ajax.submitAjaxForm(formId, function() {
          collapse.openAccordion(step2ClickId, groupClass); // Success
        });
      break;
      /* ==================================================================== */
      case 'theme.tpl':
        var themeAction = '/admin/manage/user_interface_forms/theme.php';
        var themeMessage = params['theme_message'];
        var themePopupId = '#' + params['theme_popup_id'];

        var confirmApplyTheme = function(formData) {
          popup.confirmPopup(
            themeMessage,
            themePopupId,
            function() {
              $(themePopupId).on('click', function() {
                ajax.php(themeAction, formData, function() {
                  utilAdmin.refreshPageSuccess();
                }, false);
              });
            }
          );
        };

        var data = {
          'apply_theme': true,
        };

        $('#' + params['apply_light_them_id']).on('click', function() {
          data.type = 'light';
          confirmApplyTheme(data);
        });
        $('#' + params['apply_dark_them_id']).on('click', function() {
          data.type = 'dark';
          confirmApplyTheme(data);
        });

        var collapseId = collapse.getAccordionCollapseIdFromClickId(params['click_id']);
        ajax.submitAjaxFormSuccessWithCallback('#' + params['form_theme_id'], function() {
          collapse.hide(collapseId);
        });
      break;
      /* ==================================================================== */
      case 'featured_media.tpl':
        var collapseId = collapse.getAccordionCollapseIdFromClickId(params['click_id']);
        ajax.submitAjaxFormSuccessWithCallback("#form-featured-media", function() {
          collapse.hide(collapseId);
        });
        /*
        reveal.revealWithInput(
          '#slider-autoplay-collapse',
          '#slider-autoplay-checkbox',
          false,
          true
        );
        */
        reveal.revealWithInput(
          '#slider-autoplay-collapse',
          '#radio-feautred-media-show-options-1',
          'featured_slider_disabled'
        );
      break;
      /* ==================================================================== */
      case 'general_settings.tpl':
        var collapseId = collapse.getAccordionCollapseIdFromClickId(params['click_id']);
        ajax.submitAjaxFormSuccessWithCallback("#form-general-settings", function() {
          collapse.hide(collapseId);
        });

        // If open, open the advanced settings
        if (params['open'] === 'general') {
          $('#general-settings-advanced-button').click();
        }

        reveal.revealWithInput(
          '#general-logo-enabled-collapse',
          '#general-logo-enabled'
        );
        reveal.revealWithInput(
          '#radio-logo-options-2-collapse',
          '#radio-logo-options-2',
          'general_logo_mode'
        );
        reveal.revealWithInput(
          '#radio-logo-options-3-collapse',
          '#radio-logo-options-3',
          'general_logo_mode'
        );

        var customImage = $('#custom-image-img');
        var customImageLabel = $('#custom-image-label');
        var customImageSrcInput = $('#custom-image-src');
        var customImageWrapper = $('#custom-image-wrapper');
        var toggleImage = function(enable, imgSrc) { // Function called in uploader to show / hide image
          if (!imgSrc) {
            imgSrc = '';
          }
          customImage.attr('src', imgSrc);
          customImageSrcInput.val(imgSrc);
          customImageWrapper.toggle(enable);
          customImageLabel.toggle(!enable);
        };
        $('#upload-custom-image-button').on('click', function() {
          uploader.currentDropdownObject = {
            is_logo_image: true,
            toggle_image: toggleImage
          };
          uploader.fineuploaderOpenDialog();
        });

        $('#custom-image-button-remove').on('click', function() {
          ajax.php(
            '/admin/manage/images_forms/upload_image_logo.php',
            {
              _method: 'DELETE'
            },
            function() { // Success
              toggleImage(false);
            }
          );
        });
      break;
      /* ==================================================================== */
      case 'uploaded_files.tpl':
        var groupClass = '.' + params['group_class'];
        var step1ClickId = '#' + params['step1_id'];
        var step3ClickId = '#' + params['step3_id'];
        collapse.initAccordion(step3ClickId, groupClass, true);

        $('#media-manager-button').on('click', function() {
         var currentDirectoryId = $('#' + params['select_directory_id']).val();
         util.redirect('/admin/manage/media.php?id=' + currentDirectoryId);
        });
        $('#upload-more-button').on('click', function() {
          util.refreshPage();
        });
      break;
      /* ==================================================================== */
      case 'greeting_panel.tpl':
        var collapseId = collapse.getAccordionCollapseIdFromClickId(params['click_id']);
        ajax.submitAjaxFormSuccessWithCallback('#form-greeting-panel', function() {
          collapse.hide(collapseId);
        });

        // Open up theme collapse to change greeting panel background
        $('#change-theme-settings').on('click', function() {
          collapse.hide('#accordion-2-collapse');
          collapse.show('#accordion-4-collapse');
        });

        reveal.revealWithInput(
          '#greeting-enabled-collapse',
          '#greeting-enabled-checkbox'
        );
        reveal.revealWithInput(
          '#select-greeting-media-custom-image-collapse',
          '#checkbox-greeting-media-custom-image'
        );
        reveal.revealWithInput(
          '#radio-greeting-options-2-collapse',
          '#radio-greeting-options-2',
          'greeting_mode'
        );
        reveal.revealWithInput(
          '#radio-greeting-options-3-collapse',
          '#radio-greeting-options-3',
          'greeting_mode'
        );
      break;
      /* ==================================================================== */
      case 'images.tpl':
        $('#upload-images-top-button').on('click', function() {
          // collapse.show('#fineuploader-image-upload-collapse');
          $('#fineuploader-image-upload').toggle(true);
          $('input[name="qqfile"]').click();
        });

        $('#images-delete-all-button').on('click', function() {
          var deleteAllImagesFormId = '#form-delete-all-images';
          ajax.submitAjaxForm(deleteAllImagesFormId, function() {
            utilAdmin.refreshPageSuccess();
          });
          var confirmButtonId = '#delete-all-images-confirm';
          var checkboxId = '#delete-all-images-checkbox';
          popup.checkboxPopup(
            params['delete_popup_message'],
            confirmButtonId,
            checkboxId,
            'delete_all_images_checkbox',
            deleteAllImagesFormId
          );
        });

        // Load more media on click
        if (params['should_show_more_button']) {
          sharedAjax.ajaxTemplatesOnButtonClick(
            '#show-more-images',
            '#show-more-images-wrapper',
            '#image-icon-container',
            params['images_limit'],
            params['total_images'],
            '/admin/manage/images.php',
            {_method: 'GET'}
          );
        }
      break;
      /* ==================================================================== */
      case 'update_manually_accordion.tpl':
        // collapse.initAccordion('#' + params['click_id'], '.' + params['group_class']);

        uploader.initUploaderHelper({
          // Upload file automatically when it's selected
          onSubmit: function(id) {
            var fineuploaderObject = this;
            uploader.fineuploaderStartUploadAfterTimeout(fineuploaderObject);
          },
          // Finished uploading
          onComplete: function(id, fileName, responseJSON) {
            utilAdmin.userConfirmBack(false);
            var fineuploaderObject = this;

            if (responseJSON['success']) {
              var formData = new FormData();
              formData.append('temp_file_path', uploader.getTempFilePath(responseJSON, fileName));
              formData.append('file_name', fileName);

              popup.loadingGif();

              // Send update file to php
              ajax.phpHelper(
                '/admin/utilities/upload_firmware.php',
                formData,
                function() { // Success
                  popup.successPopup(params['message_success']);
                  utilAdmin.hideAdminContent();
                  // Remove the uploader template
                  //$(util.addPound(constantsAdmin['ID_FINEUPLOADER_DEFAULT'])).remove();
                },
                function() { // Failure
                  popup.removeLoadingGif();
                  uploader.fineuploaderReset(fineuploaderObject);
                },
                true
              );
            } else { // Fineuploader failed
              popup.errorPopup(params['message_failure']);
            }
          },
        },
        {
          multiple: false,
        }
        );
        /*
          params['storage_free_b'],
          params['message_storage_full']
        */
      break;
      /* ==================================================================== */
      case 'update_online_accordion.tpl':
        collapse.initAccordion('#' + params['click_id'], '.' + params['group_class']);
        var buttonScanWifiId = '#button-scan-wifi';
        $(buttonScanWifiId).on('click', function() {
          scanWifiNetworks();
        });

        // TODO buttonCheckWifi

        var scanWifiNetworks = function() {
          utilAdmin.enableRaisedButtonGif(buttonScanWifiId);
          var wifiNetworksCollapseId = '#wifi-networks-collapse';
          ajax.ajax(
            '/admin/settings/update.php',
            function(responseHTML) {
              $(wifiNetworksCollapseId).empty();
              $(wifiNetworksCollapseId).append(responseHTML);
              utilAdmin.disableRaisedButtonGif(buttonScanWifiId);
              collapse.show(wifiNetworksCollapseId);
            },
            {
              _method: 'GET'
            }
          );
        };
        scanWifiNetworks();
      break;
      /* ==================================================================== */
      case 'media_item_edit.tpl':
        if (params['is_multi_edit']) {
          select2Class.select2Nav('#' + params['select_id']);
          reveal.revealWithInput(
            '#change-availability-collapse',
            '#change-availability'
          );
        } else {
          reveal.revealWithInput(
            util.addPound(params['id'] + '-is-password-protected-collapse'),
            util.addPound(params['id'] + '-is-password-protected-checkbox')
          );
        }
      break;
      /* ==================================================================== */
      case 'new_directory_select_wrapper.tpl':
        var languageSelectId = util.addPound(params['language_select_id']);
        var customNameCheckbox = $(languageSelectId + '-checkbox');
        var nameInput = $('#' + params['name_input_id']);
        var filenameInput = $(util.addPound(params['filename_input_id']));
        var currentSearchPage = 0;
        var currentSearchTerm = '';

        select2Class.initHelper(languageSelectId, {
          ajax: {
            url: '/api/v1/languages.php',
            data: function(params) {
              var searchTerm = '';
              if (params.term !== undefined) {
                searchTerm = params.term.trim();
              }
              if (searchTerm !== currentSearchTerm) {
                currentSearchPage = 0;
              }
              currentSearchTerm = searchTerm;
              return {
                search: searchTerm,
                page: currentSearchPage,
                select: 'true'
              };
            },
            processResults: function(data) {
              if (data.more) {
                currentSearchPage += 1;
              }
              var results = [];
              var searchArray = data.data;
              searchArray.forEach(function(obj) {
                var id = obj.iso;
                var text =  obj.name_english + constantsAdmin['LOCALIZED_LANGUAGE_DELIMITER'] + ' [' + obj.iso + ']';
                if (obj.name_localized && obj.name_localized !== obj.name_english) {
                  text = obj.name_localized + constantsAdmin['LOCALIZED_LANGUAGE_DELIMITER'] + text;
                }
                results.push({
                  id: id,
                  text: text
                });
              });
              return {
                results: results,
                pagination: {
                  more: data.more,
                },
              };
            },
            delay: 250,
            dataType: 'json'
          }
        });
        select2Class.popupZIndex(languageSelectId);
        $(languageSelectId).on('select2:select', function() {
          var languageIso = this.value.trim();

          // Get english name and localized name from PHP using the languageIso
          ajax.ajax(
            '/admin/manage/media_forms/language_directory_get_language_iso_strings.php',
            function(response) {
              response = JSON.parse(response);

              // Set inputs with formatted strings based on ISO
              filenameInput.val(response['file_name']);

              if (params['is_captive_portal_language']) {
                // TODO currently this is the same as the title, consider removing captive_portal_language_display_name from 
                // language_directory_get_language_iso_strings.php
                nameInput.val(response['captive_portal_language_display_name']);
              } else {
                nameInput.val(response['title']);
              }
            },
            {
              language_iso: languageIso
            }
          );
        });

        $(nameInput).on('change', function() {
          var nameInputValue = this.value;
          if (customNameCheckbox.is(':checked')) {
            filenameInput.val(nameInputValue + ' [' + constantsAdmin['LANGUAGE_FOLDER_UNDEFINED'] + ']');
          }
        });
      break;
      /* ==================================================================== */
      case 'media_select.tpl':
        var mediaSelectId = util.addPound(params['media_select_id']);
        var defaultOptionText = params['default_option_text'];
        var defaultOptionValue = params['default_option_value'];
        var onlyVideo = params['only_video'];
        var currentSearchPage = 0;
        var currentSearchTerm = '';

        select2Class.initHelper(mediaSelectId, {
          ajax: {
            url: '/api/v2/media-search',
            data: function(params) {
              var searchTerm = '';
              if (params.term !== undefined) {
                searchTerm = params.term.trim();
              }
              if (searchTerm !== currentSearchTerm) {
                currentSearchPage = 0;
              }
              currentSearchTerm = searchTerm;
              return {
                search: searchTerm,
                page: currentSearchPage,
                onlyVideo: onlyVideo
              };
            },
            processResults: function(data, params) {
              var results = [];
              
              // Add in default value to top of search
              // params will be _type 'query:append' for results from scrolling, and we don't want default value added on these
              if (params._type === 'query' && defaultOptionText) {
                results.unshift({
                  id: defaultOptionValue,
                  text: defaultOptionText
                });
              }
              if (data.more) {
                currentSearchPage += 1;
              }
              var searchArray = data.data;
              searchArray.forEach(function(obj) {
                var id = obj.file_id;
                var text =  obj.title;
                if (obj.file_extension || obj.title !== obj.file_name) {
                  text += " (" + obj.file_name;
                  if (obj.file_extension) {
                    text += "." + obj.file_extension;
                  }
                  text += ")";
                }
                results.push({
                  id: id,
                  text: text
                });
              });
              return {
                results: results,
                pagination: {
                  more: data.more,
                },
              };
            },
            delay: 250,
            dataType: 'json'
          }
        });
        select2Class.popupZIndex(languageSelectId);
      break;
      /* ==================================================================== */
      case 'media.tpl':
        // Load more media on click
        if (params['should_show_more_button']) {
          sharedAjax.ajaxTemplatesOnButtonClick(
            '#show-more-media-button',
            '#show-more-media-button-wrapper',
            '#media-foreach-wrapper',
            params['media_limit'],
            params['children_count'],
            '/admin/manage/media.php',
            {
              current_directory_id: params['current_directory_id'],
              media_checkbox_array_name: params['media_checkbox_array_name']
            },
            function() { // After ajax callback
              init.materialDesign();
              multiSelect.initMultiselectCheckbox("#media-nav-checkbox-multiselect", 'input[name=\'' + params['media_checkbox_array_name'] + '\']');
            }
          );
        }

        var confirmButtonId = '#' + params['confirm_button_id'];
        var inputId = '#' + params['input_id'];
        var createFolderFormId = '#' + params['create_folder_form_id'];
        var message = utilAdmin.sprintf(
          params['message'],
          params['current_directory_title']
        );

        // Set buffer on bottom of page based on nav height (in case it grows from breadcrumbs)
        var navHeight = $('#media-nav').height();
        $('#media-nav-buttom-buffer').attr('style', 'height:' + navHeight + 'px');

        // New folder
        $('#new-folder-button').on('click', function() {
          popup.addDirectoryPopup(false, params);
        });
        $('#new-language-folder-button').on('click', function() {
          popup.addDirectoryPopup(true, params);
        });
        // var addNewFolder = function(isLanguageDirectory) {
        //   var popupConfirmButtonId = '#new-folder-popup-confirm';
        //   var nameInputId = '#name-input-select';

        //   var popupHelperData = {
        //     include_file: 'admin/components/media/new_directory_select_wrapper.tpl',
        //     heading: params['heading'],
        //     confirm_text: params['confirm_text'],
        //     message: '',
        //     name_input_id: util.removePound(nameInputId),
        //     is_language_directory: isLanguageDirectory,
        //     confirm_button_id: util.removePound(popupConfirmButtonId),
        //     current_directory_id: params['current_directory_id'],
        //     filename: '',
        //     title: '',
        //     is_popup: true
        //   }

        //   if (isLanguageDirectory) {
        //     var languageSelectId = '#new-language-select';
        //     popupHelperData['language_select_id'] = util.removePound(languageSelectId);
        //     popupHelperData['language_iso'] = '';
        //     // popupHelperData['ajax_get_data'] = 'language_isos_array';
        //   }

        //   popup.popupHelper(
        //     'shared/popup_custom.tpl',
        //     popupHelperData,
        //     function() {
        //       select2Class.popupZIndex(selectId);
        //       var newDirectoryFormId = '#new-directory-form';
        //       ajax.submitAjaxForm(
        //         newDirectoryFormId,
        //         function() {
        //           utilAdmin.refreshPageSuccess();
        //         },
        //         function() {
        //           popup.removeLoadingGif();
        //         }
        //       );
        //       popup.hideOnFormSubmit(newDirectoryFormId); // IMPORTANT: if user presses enter, close the popup
        //       $(popupConfirmButtonId).on('click', function() {
        //         popup.loadingGif();
        //         $(newDirectoryFormId).submit();
        //       });
        //     }
        //   );
        // };
      break;
      /* ==================================================================== */
      case 'followup_form_responses.tpl':
        var groupClass = '.' + params['group_class'];
        var clickId = '#' + params['click_id'];
        var checkboxId = '#' + params['checkbox_id'];
        var mediaSelectedCountClass = '.' + params['media_select_count_class'];
        var timeAgoId = utilAdmin.addPound(clickId + '-title-time-ago');

        utilAdmin.updateTimestamp(params['date_time'], timeAgoId, params);

        multiSelect.initSelectCheckbox(checkboxId, mediaSelectedCountClass);
        collapse.initAccordion(clickId, groupClass, false, true);
      break;
      /* ==================================================================== */
      case 'responses.tpl':
        var groupClass = '.' + params['group_class'];
        var group = $(groupClass);
        $('#open-responses').on('click', function() {
          collapse.show(groupClass);
        });
        $('#button-email-responses').on('click', function() {
          if (params['reporting_email_set']) {
            ajax.redirectToAjaxResponseString('/admin/reports/followup_form_summary.csv.php?email=true');
          } else {
            util.redirect('/admin/settings/account.php?set_reporting_email=true');
          }
        });
        $('#delete-all').on('click', function() {
          var deleteAllFormId = '#delete-all-form';
          ajax.submitAjaxForm(deleteAllFormId, function() {
            utilAdmin.refreshPageSuccess();
          });
          var confirmButtonId = '#delete-all-confirm';
          var checkboxId = '#delete-all-checkbox';
          popup.checkboxPopup(
            params['delete_popup_message'],
            confirmButtonId,
            checkboxId,
            'delete_all_checkbox',
            deleteAllFormId
          );
        });
        // Load more responses on click
        if (params['should_show_more_button']) {
          sharedAjax.ajaxTemplatesOnButtonClick(
            '#show-more-responses-button',
            '#show-more-responses-button-wrapper',
            '#responses-foreach-wrapper',
            params['responses_limit'],
            params['responses_total_count'],
            '/admin/manage/followup_form/responses.php',
            {
              _method: 'GET'
            },
            function() { // After ajax callback
              init.materialDesign();
              multiSelect.initMultiselectCheckbox("#responses-checkbox-multiselect", ".followup-form-multiselect-checkboxes");
            }
          );
        }
      break;
      /* ==================================================================== */
      case 'followup_form_responses_nav.tpl':
        var errorMessage = params['error_message'];
        var deleteFormId = '#delete-responses-form';

        //reveal.revealOnScroll("#followup-responses-nav", "#responses-nav-reveal", 0);

        // Delete
        multiSelect.initMultiselectCheckbox('#responses-checkbox-multiselect', '.' + params['checkbox_class']);
        multiSelect.clickActionButton(
          '#delete-responses-button',
          function() {
            var popupConfirmButtonId = '#popup-delete-responses-button';

            popup.confirmPopup(
              utilAdmin.sprintf(
                params['delete_message'],
                multiSelect.multiSelectArray.length
              ),
              popupConfirmButtonId,
              function() {
                ajax.submitAjaxFormHelper(
                  deleteFormId,
                  multiSelect.getMultiSelectArrayAsFormDataArray(params['selected_array_name']), // Get selected items
                  function() {
                    utilAdmin.refreshPageSuccess();
                  },
                  false
                );
                $(popupConfirmButtonId).on('click', function() {
                  popup.loadingGif();
                  $(deleteFormId).submit();
                });
              }
            );
          },
          errorMessage
        );
      break;
      /* ==================================================================== */
      case 'dashboard.tpl':
        var firstRecentActivitiesLoad= true;
        var TIME_STATS_INTERVAL = 5000;

        var updateStats = function(endpoint) {
          var handleStats = function(response) {
            response = JSON.parse(response);
            for (var category in response) {
              for (var type in response[category]) {
                for (var time in response[category][type]) {
                  if (type === "top-files") {
                    var pathrelId = '#' + category + '_' + type + '_' + time + '_path_rel';
                    var pathRelValue = response[category][type][time]['path_rel'];
                    utilAdmin.addClassAndRemoveAfterTimeIfValuesDiffer('pulse', pathrelId, pathRelValue, params['pulse_interval_ms']);

                    // Important: Unhide the row if there was nothing there
                    if (pathRelValue) {
                      $(pathrelId).closest( "tr" ).toggle(true); // Show the hidden TR
                      $('#no-top-files').toggle(false); // Hide no top message
                    }

                    var totalStreamsId = '#' + category + '_' + type + '_' + time + '_total_streams';
                    var totalStreamsValue = response[category][type][time]['total_streams'];
                    utilAdmin.addClassAndRemoveAfterTimeIfValuesDiffer('pulse', totalStreamsId, totalStreamsValue, params['pulse_interval_ms']);

                    var totalDownloadsId = '#' + category + '_' + type + '_' + time + '_total_downloads';
                    var totalDownloadsValue = response[category][type][time]['total_downloads'];
                    utilAdmin.addClassAndRemoveAfterTimeIfValuesDiffer('pulse', totalDownloadsId, totalDownloadsValue, params['pulse_interval_ms']);
                  } else {
                    var statId = '#' + category + '_' + type + '_' + time;
                    var statValue = response[category][type][time];
                    utilAdmin.addClassAndRemoveAfterTimeIfValuesDiffer('pulse', statId, statValue, params['pulse_interval_ms']);
                  }
                }
              }
            }

            // Recent activity
            var newRecentActivity = response['wifi']['recent-activity'];
            var newRecentActivityExists = !$.isEmptyObject(newRecentActivity);
            var recentActivityId = "#recent-activity";
            var noRecentActivityMessageId ="#no-recent-activity-message";

            // If first load
            if (newRecentActivityExists) {
              if (newRecentActivity['interaction-0'].length == 0) {
                newRecentActivityExists = false;
              }
            }

            // If there is new recent activity
            if (newRecentActivityExists) {
              $(noRecentActivityMessageId).toggle(false);
              initAdmin.recentActivity = newRecentActivity;
              sharedAjax.template(
              'admin/components/dashboard/tile_content/recent_activity_foreach.tpl',
              {
                items: initAdmin.recentActivity,
                show_user_details_link: true
              },
              recentActivityId,
              false,
              // Pulse the new items
              function() {
                var recentActivityLength = Object.keys(initAdmin.recentActivity).length; // NOTE: this may break some older browsers
                var newItems = $(recentActivityId).children().slice(0, recentActivityLength);
                newItems.each(function() {
                  if (!firstRecentActivitiesLoad) {
                    util.addClassAndRemoveAfterTime('pulse', '#' + $(this).attr('id'), params['pulse_interval_ms']);
                  }
                });
                firstRecentActivitiesLoad = false;
              });
            } else if (!newRecentActivityExists && $(recentActivityId).children().length == 1) {
              $(noRecentActivityMessageId).toggle(true);
            }
          }

          ajax.ajax(
            endpoint,
            function(response) {
              handleStats(response);
            },
            {},
            true,
            TIME_STATS_INTERVAL - 500
          );
        }

        var apiString = '/api/v1/stats.php?wifi=true&bluetooth=true&sd=true';
        updateStats(apiString);
        setInterval(function() {
          if ($.isEmptyObject(initAdmin.recentActivity)) {
            updateStats(apiString);
          } else {
            // New interactions last time
            var topInteractionId = parseInt(initAdmin.recentActivity['interaction-0']['interaction_id']);
            updateStats(apiString + '&wifi_id_gt=' + topInteractionId);
          }
        }, TIME_STATS_INTERVAL);

        // Email Stats
        $('#button-email-stats').on('click', function() {
          if (params['reporting_email_set']) {
            ajax.redirectToAjaxResponseString('/admin/reports/dashboard_summary.csv.php?email=true');
          } else {
            util.redirect('/admin/settings/account.php?set_reporting_email=true');
          }
        });

        // Delete stats
        var deleteAllStatsButton = $('#delete-all-stats-button');
        var deleteAllStatsFormId = '#delete-all-stats-form';
        ajax.submitAjaxForm(deleteAllStatsFormId, function() {
          utilAdmin.refreshPageSuccess();
        });
        $(deleteAllStatsFormId).on('submit', function() {
          popup.loadingGif();
        });

        deleteAllStatsButton.on('click', function() {
          var confirmButtonId = '#delete-stats-confirm';
          var checkboxId = '#delete-stats-checkbox';
          popup.checkboxPopup(
            params['delete_popup_message'],
            confirmButtonId,
            checkboxId,
            'delete_stats_checkbox',
            deleteAllStatsFormId
          );
        });

      break;
      /* ==================================================================== */
      case 'recent_activity_tile.tpl':
        var interaction = params['interaction'];
        utilAdmin.updateTimestamp(interaction.date_time, '#' + interaction.interaction_id + '-time-ago', params);
      break;
      /* ==================================================================== */
      case 'setup_section_form.tpl':
        select2Class.initIcons('#select-form-icon', 100);
        ajax.submitAjaxFormSuccess('#form-followup-form-settings');
        reveal.revealWithInput(
          '#checkbox-enable-send-email-collapse',
          '#checkbox-enable-send-email'
        );
        reveal.revealWithInput(
          '#followup-form-enable-collapse',
          '#followup-form-enable-checkbox'
        );
      break;
      /* ==================================================================== */
      case 'power.tpl':
        var shutdownFormId = '#shutdown-form';
        var powerFormId = '#power-form';
        var shutdownConfirmButtonId = '#power-confirm-shutdown'

        reveal.revealWithInput(
          '#radio-startup-options-3-collapse',
          '#radio-startup-options-3',
          'startup_mode'
        );

        $('#shutdown-btn').on('click', function() {

          // Are you sure message
          popup.confirmPopup(
            params['message_confirm_shutdown'],
            shutdownConfirmButtonId,
            function() { // On load
              $(shutdownConfirmButtonId).on('click', function() { // On confirm
                // Start shutdown, show confirmation message
                var shutdown = function() {
                  popup.successPopup(params['message_shutdown']);
                  utilAdmin.hideAdminContent();
                  $(shutdownFormId).submit();
                };

                // Shut down after successful, or unsuccessful submission of data
                ajax.submitAjaxFormHelper(
                  powerFormId,
                  false,
                  shutdown, // on success
                  shutdown, // on failure
                  true // Hide errors from power form, so that shutdown message can be shown
                );
                $(powerFormId).submit();
                ajax.submitAjaxFormSuccess(powerFormId);
              });
            }
          );
        });
        ajax.submitAjaxFormSuccess(powerFormId);
        ajax.submitAjaxForm(shutdownFormId);
      break;
      /* ==================================================================== */
      case 'setup_section_connect_wifi.tpl':
        var defaultOptionValue = params['default_option_value'];
        var reScanButtonId = '#re-scan-button';
        deletableList.initWifiList(
          "#wifi-networks",
          "#add-wifi-network-select",
          params['checkbox_array_name'],
          params['max_wifi_networks'],
          params['message_max_allowed'] // From wifi.php
        );
        var getWifiNetworkSelectedOptions = function() {
          ajax.getSelectOptions(
            '#add-wifi-network-select',
            '/admin/settings/wifi.php',
            {
              _method: 'GET',
            },
            true,
            defaultOptionValue,
            function() { // On getting the media
              utilAdmin.disableRaisedButtonGif(reScanButtonId);
            }
          );
        };
        getWifiNetworkSelectedOptions();
        $(reScanButtonId).on('click', function() {
          utilAdmin.enableRaisedButtonGif(reScanButtonId);
          getWifiNetworkSelectedOptions();
        });
      break;
      /* ==================================================================== */
      case 'drives.tpl':
        // var phpEndpoint = '/admin/settings/drives.php';
        utilAdmin.showText('.animate-dots', '...', 0, 500, true);
      break;
      /* ==================================================================== */
      case 'advanced_options_end.tpl':
        var id = util.addPound(params['id']);
        var buttonWrapperId = id+'-button-wrapper';
        var collapseId = id+'-collapse';
        var showButton = $(id + '-button');

        // Show button
        showButton.on('click', function() {
          $(buttonWrapperId).toggle(false);
          collapse.show(collapseId);
        });

        // Hide button
        $(collapseId + ' .advanced-hide-button').on('click', function() {
          $(buttonWrapperId).toggle(true);
          collapse.hide(collapseId);
        });

        if (params['is_open']) {
          showButton.click();
        }
      break;
      /* ==================================================================== */
      case 'security.tpl':
        reveal.revealWithInput(
          '#disable-followup-form-collapse',
          '#disable-followup-form-checkbox',
          false,
          true
        );
        ajax.submitAjaxForm('#security-form', function() {
          popup.successPopup(params['message_save_successful'] + ' ' + params['message_restart'] + '.');
        });
      break;
      /* ==================================================================== */
      case 'wide_button_factory_restore.tpl':
        var factoryRestoreFormId = '#factory-restore-form';
        ajax.submitAjaxForm(factoryRestoreFormId);
        $(factoryRestoreFormId).on('submit', function() {
          popup.successPopup(params['message_factory_restore_success']);
          utilAdmin.hideAdminContent();
        });
        $('#factory-restore-button').on('click', function() {
          popup.checkboxPopup(
            params['factory_restore_message'],
            '#factory-restore-confirm',
            '#factory-restore-checkbox',
            'factory_restore_checkbox',
            factoryRestoreFormId,
            params['factory_restore_confirm_text'],
            true
          );
        });
      break;
      case 'form_select_image.tpl':
        var selectId = params['select_id'];
        var selectGroupClass = params['select_group_class'];
        var imageInsertClass = params['image_insert_class'];
        var select = $(selectId);

        // Input used to send up data about the current thumbnail chosen
        var thumbnailInput = $(util.addPound(selectId) + '-thumbnail-input');

        // Determine if default value exists, so we can reset to it on start upload
        var defaultOptionValue = '';
        var defaultOptionValueExists = ($(selectId + " option[value='" + constantsAdmin['SELECT2_DEFAULT'] + "']").length > 0);
        if (defaultOptionValueExists) {
          defaultOptionValue = constantsAdmin['SELECT2_DEFAULT'];
        }

        // initialize for images
        select2Class.initImages(selectGroupClass);

        select.on('select2:select', function() {
          if (this.value === constantsAdmin['SELECT2_UPLOAD']) {

            // Give fineuploader object content it needs for the callback
            uploader.currentDropdownObject = {
              select_id: selectId,
              select_group_class: selectGroupClass,
              image_insert_class: imageInsertClass
            };

            // Open fineuploader dialog
            fineuploaderId = false;
            var fineuploaderTemplateId = params['fineuploader_template_id'];
            if (fineuploaderTemplateId) {
              fineuploaderId = util.addPound(fineuploaderTemplateId);
            }
            uploader.fineuploaderOpenDialog(fineuploaderId);
            select.val(defaultOptionValue).trigger('change'); // Reset the select
          } else if (this.value === constantsAdmin['SELECT2_GENERATE_THUMBNAIL']) {
            popup.loadingGif();
            var fileId = params['file_id'];
            var mediaImageId = '#id-media-' + fileId + '-media-item-image'; // COUPLED to media_edit.tpl
            // Call php
            ajax.php(
              '/admin/manage/media_forms/add_video_thumbnail.php',
              {
                file_id: fileId
              },
              function(newThumbnailId) { // Success
                popup.removeLoadingGif();
                // Set current thumbnail to new id
                thumbnailInput.val(newThumbnailId);
                select.val(constantsAdmin['SELECT2_CURRENT_THUMBNAIL']).trigger('change');

                utilAdmin.setBackgroundImage(mediaImageId, newThumbnailId);
              },
              function() { // Failure
                popup.removeLoadingGif();
                select.val(defaultOptionValue).trigger('change');
              }
            );
          }
        });
      break;
      /* ==================================================================== */
      case 'form_security_button.tpl':
        var buttonId = '#' + params['security_button_id'];
        $(buttonId).on('click', function() {
          popup.errorPopupHeading(
            params['popup_message'],
            params['popup_heading'],
            {
              is_warning: true
            }
          )
        });
      break;
      /* ==================================================================== */
      case 'help_message.tpl':
        var helpMessageCollaseId = '#help-message';
        var helpMessageCollapse = $(helpMessageCollaseId);
        var helpMessageToggleButtonId =' #help-message-toggle-button'
        var helpMessageToggleButton = $(helpMessageToggleButtonId);
        var activeClass = 'help-message-toggle-button-active';

        var toggleHelpMessage = function() {
          helpMessageToggleButton.removeClass('pulse');
          if (helpMessageCollapse.hasClass('in')) {
            collapse.hide(helpMessageCollaseId);
            helpMessageToggleButton.removeClass(activeClass);
          } else {
            collapse.show(helpMessageCollaseId);
            helpMessageToggleButton.addClass(activeClass);
          }
        }
        $('#help-message-button, '  + helpMessageToggleButtonId).on('click', function() {
          // Disable pulse
          if (params['help_messages_highlight']) {
            ajax.php('/admin/utilities/help_messages.php', {});
          }
          toggleHelpMessage();
        });

        /*
        var helpMessageCheckboxId = '#help-message-checkbox';
        var helpMessageFormId = '#help-message-form';
        ajax.submitAjaxForm(helpMessageFormId);
        form.submitFormOnChange(helpMessageFormId, helpMessageCheckboxId);
        */
      break;
      /* ==================================================================== */
      case 'toggle_group.tpl':
        utilAdmin.initToggleGroup(
          '#'+params['button_id'],
          '#'+params['content_id'],
          '.'+params['button_group_class'],
          '.'+params['content_group_class'],
          '.'+params['section_group_class'],
          '#'+params['back_button_id']);
      break;
      /* ==================================================================== */
      case 'toggle_group_mirror_drive.tpl':
        var sectionGroup = $('.'+params['section_group_class']);
        var state2 = $(util.addPound(params['state2_id']));
        var state3 = $(util.addPound(params['state3_id']));

        // Percent complete loading bar
        var percentCompleteDiv = $('#mirror-drive-progress-percent-complete');
        var percentRemainingDiv = $('#mirror-drive-progress-percent-remaining');
        var setMirrorPercentComplete = function(percentComplete) {
          if (!percentComplete) {
            percentComplete = 0;
          }
          var percentRemaining = 100 - percentComplete;
          percentCompleteDiv.attr('style', 'width: ' + percentComplete + '%');
          percentRemainingDiv.attr('style', 'width: ' + percentRemaining + '%');
        }

        // Fake initial percent shown after 1 second, so users know progress is happening
        var initialFakePercentComplete = 2;

        drives.setupDrive(
          params,
          'mirror',
          function(callbackParams) { // confirmSetupCallback
            // Start mirror in PHP
            var backButton = callbackParams['back_button'];
            var checkDriveStatus = callbackParams['check_drive_status'];

            // Move to state2 before flag is set, so user sees change
            drives.toggleState(state2, sectionGroup);
            setTimeout(function() {
              setMirrorPercentComplete(initialFakePercentComplete);
            }, 1000);

            // Set flag
            ajax.php(
              constantsAdmin['PHP_DRIVES'],
              {
                type: 'mirror_set_flag'
              },
              function() {
                checkDriveStatus();
              },
              function() {
                backButton.click();
              }
            );
          },
          function(callbackParams) { // Setupcallback, called on each ping
            var response = callbackParams['response'];
            var toggleBackButtonCancelOkay = callbackParams['toggle_back_button_cancel_okay'];

            switch(response['status']) {
              case 'STATUS_IN_PROGRESS':
                drives.toggleState(state2, sectionGroup);
                var percentComplete = response['percent'];
                if (typeof percentComplete === 'number' && (percentComplete > initialFakePercentComplete)) {
                  setMirrorPercentComplete(percentComplete);
                }
              break;
              case 'STATUS_SUCCESS':
                setMirrorPercentComplete();
                drives.toggleState(state3, sectionGroup);
                toggleBackButtonCancelOkay(true);
              break;
            }
          },
          function() { // On cancel
            setMirrorPercentComplete();
            ajax.php(
              constantsAdmin['PHP_DRIVES'],
              {
                type: 'sd_card_stop_service'
              }
            );
          }
        );
      break;
      /* ==================================================================== */
      case 'toggle_group_mirror_drive_run.tpl':
        var sectionGroup = $('.'+params['section_group_class']);
        var state2 = $(util.addPound(params['state2_id']));

        drives.setupDrive(
          params,
          'mirror_run',
          function(callbackParams) { // confirmSetupCallback
            var backButton = callbackParams['back_button'];
            var toggleBackButtonCancelOkay = callbackParams['toggle_back_button_cancel_okay'];
            var checkDriveStatus = callbackParams['check_drive_status'];
            drives.toggleState(state2, sectionGroup);
            toggleBackButtonCancelOkay(true);
            ajax.php(
              constantsAdmin['PHP_DRIVES'],
              {
                type: 'mirror_run_set_flag'
              },
              function() { // On success set flag
                checkDriveStatus();
                popup.successPopup(params['message_restarting']);
                // backButton.click();
              },
              function() { // Error
                backButton.click();
              }
            );
          }
        );
      break;
      /* ==================================================================== */
      case 'toggle_group_update_drive.tpl':
        var sectionGroup = $('.'+params['section_group_class']);
        var state2 = $(util.addPound(params['state2_id']));
        var state3 = $(util.addPound(params['state3_id']));
        drives.setupDrive(
          params,
          'update',
          function(callbackParams) { // confirmSetupCallback
            var backButton = callbackParams['back_button'];
            var toggleBackButtonCancelOkay = callbackParams['toggle_back_button_cancel_okay'];
            var checkDriveStatus = callbackParams['check_drive_status'];
            drives.toggleState(state2, sectionGroup);
            toggleBackButtonCancelOkay(true);
            ajax.php(
              constantsAdmin['PHP_DRIVES'],
              {
                type: 'update_set_flag'
              },
              function() { // Success
                checkDriveStatus();
              },
              function() {
                backButton.click();
              }
            )
          },
          function(callbackParams) { // Setupcallback, called on each ping
            var response = callbackParams['response'];
            var toggleBackButtonCancelOkay = callbackParams['toggle_back_button_cancel_okay'];
            switch(response['status']) {
              case 'STATUS_IN_PROGRESS':
                drives.toggleState(state2, sectionGroup);
                toggleBackButtonCancelOkay(true);
              break;
              case 'STATUS_SUCCESS':
                drives.toggleState(state3, sectionGroup);
                toggleBackButtonCancelOkay(true);
              break;
            }
          }
        );
      break;
      /* ==================================================================== */
      case 'user_details.tpl':
        $('#current-user-select').on('select2:select', function() {
          var deviceNum = this.value;
          var backParam = params['back_param'];
          util.redirect('/admin/user_details.php?back=' + backParam + '&device_num=' + deviceNum);
        });
      break;
      /* ==================================================================== */
      case 'form_sortable_item_captive_portal_language_list.tpl':
        var wrapperId = util.addPound(params['wrapper_id']);
        var listWrapper = $(wrapperId);
        var clickInput = $(util.addPound(params['click_id']));
        var buttonReset = $(util.addPound(params['button_reset_id']));
        utilAdmin.sortableInit(wrapperId);

        // Render template
        var getCaptivePortalTemplate = function (templateData) {
          sharedAjax.template(
            'admin/components/settings/wifi/form_sortable_item_captive_portal_language.tpl',
            templateData,
            wrapperId,
            false
          );
        }

        /*
        <form id="{$list_id}-form-reset" class="display-none" action="/admin/settings/wifi.php">
          <input type="hidden" name="reset_captive_portal_languages" value="true" />
        </form>
        */
        
        var butonConfirmId = '#button-reset-confirm';
        buttonReset.on('click', function() {
          popup.confirmPopup(
            params['button_reset_message'],
            butonConfirmId,
            function() {
              $(butonConfirmId).on('click', function() {
                popup.loadingGif();
                ajax.php(
                  '/admin/settings/wifi.php',
                  { reset_captive_portal_languages: true },
                  function () {
                    utilAdmin.refreshPageSuccess();
                  },
                  function() {
                    popup.removeLoadingGif();
                  }
                );
              });
            }
          );
        });

        clickInput.on('click', function() {
          popup.addDirectoryPopup(true, params, function() { // On confirm
            var languageIso = $('#new-language-select').val();
            var languageDisplayName = $('#name-input-select').val();
            var isLanguageAvailable = !($('#new-language-select-checkbox').is(':checked'));
            utilAdmin.userConfirmBack(true);

            // Default data for new captive portal language
            var data = {
              checkbox_array_name: params['checkbox_array_name'],
              checkbox_value: 'new_item_' + (listWrapper.children().length + 1),
              captive_portal_display_name: languageDisplayName,
            };

            // If there is a language ISO selected
            if (isLanguageAvailable) {
              data['captive_portal_language_iso'] = languageIso;

              // TODO ajax the message if possible
              ajax.ajax(
                '/admin/settings/wifi.php',
                function (defaultMessage) { // Callback
                  if (defaultMessage) {
                    data['captive_portal_message'] = defaultMessage;
                  }

                  // Ajax in a template for the new captive portal language
                  getCaptivePortalTemplate(data);
                },
                {
                  _method: 'GET',
                  language_iso: languageIso
                }
              );
            } else {
              getCaptivePortalTemplate(data);
            }
          });
        });
      break;
      /* ==================================================================== */
      case 'form_list_item.tpl':
        deletableList.formDeletableItem(
          util.addPound(params['remove_button_id']),
          util.addPound(params['wrapper_id']),
          params['is_ajax']
        );
      break;
      /* ==================================================================== */
      case 'form_list_item_array.tpl':
        var addButtonId = util.addPound(params['add_button_id']);
        var wrapperId = util.addPound(params['wrapper_id']);
        var wrapper = $(wrapperId);

        if (params['sortable']) {
          utilAdmin.sortableInit(wrapperId);
        }

        $(addButtonId).on('click', function () {
          var arrayLength = wrapper.children().length;
          utilAdmin.userConfirmBack(true);
          var data = params['data'];
          data['form_list_item_id'] = params['wrapper_id'] + '-new-' + arrayLength;

          // Call ajax
          sharedAjax.template(
            'admin/components/reusable/form_list_item.tpl',
            params['data'],
            wrapperId,
            true,
            params['callback']
          );
        });
      break;
      /* ==================================================================== */
      default:
        console.log('Error: ' + templateName + ' is not a valid template name.');
      break;
    }
  },
  // Persistant variables
  recentActivity: {},
}; // End initAdmin

/*
  === Drives Class ===

  Helper functions for the drives page templates
*/
var drives = {
  /*
    Show or hide different sections in the drive creation process
  */
  toggleState: function(stateSelector, sectionGroup) {
    sectionGroup.toggle(false);
    stateSelector.toggle(true);
  },
  /*
    Initialize the creation of a mirror drive, mirror run drive, or update drive
  */
  setupDrive: function(params, type, confirmSetupCallback, setupCallback, onCancelCallback) {
    var button = $('#'+params['button_id']);
    var backButtonId = '#'+params['back_button_id'];
    var backButton = $(backButtonId);
    var backButtonCancel = $(backButtonId+'-cancel');
    var backButtonOkay = $(backButtonId+'-okay');
    var toggleBackButtonCancelOkay = function(setOkay) {
      backButtonCancel.toggle(!setOkay);
      backButtonOkay.toggle(setOkay);
    }
    var state1 = $(util.addPound(params['state1_id']));
    var state2 = $(util.addPound(params['state2_id']));
    var sectionGroup = $('.' + params['section_group_class']);

    var shouldCreateConfirmPopup = true;
    var shouldCheckDriveStatus = false;
    var setupConfirmed = false;
    var checkDriveStatusInterval = null;
    var checkDriveStatus = function() {
      // Break out of function if user hasn't clicked the create button yet
      if (!shouldCheckDriveStatus) {
        return;
      }

      // Ping PHP every 2 seconds
      ajax.php(
        constantsAdmin['PHP_DRIVES'],
        {
          type: type,
          setup_confirmed: setupConfirmed
        },
        function(response) { // Success
          var callbackParams = {
            response: response,
            back_button: backButton,
            toggle_back_button_cancel_okay: toggleBackButtonCancelOkay,
            check_drive_status: function() {
              checkDriveStatus();
            }
          };

          // If drive is in the slot, ask user to confirm mirror
          if (response['drive_is_in_slot'] && !response['setup_confirmed']) {
            var confirmButtonId = '#confirm-drive-button';
            var cancelButtonId = '#cancel-drive-button';

            // If confirm popup is not created, create it
            if (shouldCreateConfirmPopup) {
              popup.removeAllPopups();
              shouldCreateConfirmPopup = false;
              // TODO add checkbox instead? checkboxPopup: function(message, confirmButtonId, checkboxId, checkboxName, actionForm, confirmText, removeOnConfirm, heading, checkboxLabel, callback)
              popup.confirmPopup(
                params['message_confirm'],
                confirmButtonId,
                function() {
                  // Set boolean to true if confirm
                  $(confirmButtonId).on('click', function() {
                    if (confirmSetupCallback) {
                      confirmSetupCallback(callbackParams);
                    }
                    // Enable getting status for next checkDriveStatus() call
                    setupConfirmed = true;
                    shouldCreateConfirmPopup = true;
                    drives.toggleState(state2, sectionGroup);
                  });
                  // If cancel, go back to buttons view
                  $(cancelButtonId).on('click', function() {
                    backButton.click();
                  });
                },
                {
                  cancel_button_id: util.removePound(cancelButtonId),
                  remove_exit: true
                }
              );
            }
          }

          // Only check status if mirror has been confirmed
          if (response['setup_confirmed']) {
            if (setupCallback) {
              setupCallback(callbackParams);
            }
          }
        },
        function() { // Error
          backButton.click();
        }
      );
    }

    button.on('click', function() {
      drives.toggleState(state1, sectionGroup);
      shouldCheckDriveStatus = true; // Allows setInterval
      checkDriveStatus();
      checkDriveStatusInterval = setInterval(checkDriveStatus, 2000);
    });
    backButton.on('click', function() {
      shouldCreateConfirmPopup = true;
      shouldCheckDriveStatus = false;
      setupConfirmed = false;
      clearInterval(checkDriveStatusInterval);
      toggleBackButtonCancelOkay(false);
    });

    if (onCancelCallback) {
      backButtonCancel.on('click', function() {
        onCancelCallback();
      });
    }
  }
};

/*
  === Deletable List Class  ===

  Used for lists of items, where user can add and remove content
*/
var deletableList = {
  initWifiList: function(wrapperDivId, selectId, checkboxArrayName, maxNumberOfInputs, maxNumberMessage) {
    deletableList.initListHelper(wrapperDivId, selectId, checkboxArrayName, maxNumberOfInputs, maxNumberMessage, 'admin/components/settings/form_sortable_item_wifi_network.tpl', true, 'wifi', false);
  },
  initList: function(tableBodyId, addButtonId, checkboxArrayName, maxNumberOfInputs, maxNumberMessage, selectIconArray) {
    deletableList.initListHelper(tableBodyId, addButtonId, checkboxArrayName, maxNumberOfInputs, maxNumberMessage, 'admin/components/followup_form/form_sortable_item_followup.tpl', true, false, selectIconArray);
  },
  initListFeatured: function(tableBodyId, selectId, checkboxArrayName, maxNumberOfInputs, maxNumberMessage) {
    deletableList.initListHelper(tableBodyId, selectId, checkboxArrayName, maxNumberOfInputs, maxNumberMessage, 'admin/components/user_interface/featured_media_sortable_item.tpl', false, 'featured', false);
  },
  initListHelper: function (wrapperId, clickId, checkboxArrayName, maxNumberOfInputs, maxNumberMessage, template, sortable, type, selectIconArray) {
    if (maxNumberOfInputs) {
      maxNumberMessage = utilAdmin.sprintf(
        maxNumberMessage,
        maxNumberOfInputs
      );
      maxNumberOfInputs = parseInt(maxNumberOfInputs);
    }

    var clickInput = $(clickId);
    var listWrapper = $(wrapperId);
    var clickIsSelect = type === 'featured' || type === 'wifi';

    // add on click, or select 2 select
    var onString = 'click';
    if (clickIsSelect) {
      onString = 'select2:select';
    }

    if (sortable) {
      utilAdmin.sortableInit(wrapperId);
    }

    clickInput.on(onString, function() {

      // Stop user from leaving
      utilAdmin.userConfirmBack(true);

      var totalCheckedInputs = listWrapper.find(":checkbox:checked").length;
      var selectionAlreadyAdded = false;

      if (maxNumberOfInputs && (totalCheckedInputs >= maxNumberOfInputs)) {
        popup.errorPopup(maxNumberMessage);
        if (clickIsSelect) {
          select2Class.unselectCurrentSelection(clickId);
        }
      } else {
        // Setup default ajax callback variables
        var callback = false;
        var data = {
          checkbox_array_name: checkboxArrayName,
          checkbox_value: 'new_item_' + (listWrapper.children().length + 1)
        };
        if (selectIconArray) {
          data['select_icon_array'] = selectIconArray;
        }

        if (clickIsSelect) {
          var value = this.value;
          if (type === 'featured') {
            data['checkbox_value'] = value;
          }

          // Select2
          var text = $(this).select2('data')[0].text.trim();

          callback = function() {
            select2Class.unselectCurrentSelection(clickId);
            // var currentOption = clickInput.find(':selected');
            // if ((type === 'wifi' && value !== 'default') || type !== 'wifi') {
            //   currentOption.attr('disabled', 'disabled'); // Disable option
            //   select2Class.init(clickId);
            // }
            // // COUPLED to form_deletable_item_extend.tpl;
            // var removeButtonId = '#' + data['checkbox_value'] + '-remove';

            // // Enable option in dropdown if delete has been clicked
            // $(removeButtonId).on('click', function() {
            //   currentOption.removeAttr('disabled');
            //   select2Class.init(clickId);
            //   select2Class.unselectCurrentSelection(clickId);
            // });
          }

          if (type === 'featured') {
            data['title'] = text;
          } else {
            data['ssid'] = text;
            if (value === 'default') {
              data['ssid'] = '';
            }
          }
          selectionAlreadyAdded = document.querySelector("input[value='" + data['checkbox_value'] + "']") !== null;
        }

        // Call ajax
        if (!selectionAlreadyAdded) {
          sharedAjax.template(
            template,
            data,
            wrapperId,
            true,
            callback
          );
        } else {
          select2Class.unselectCurrentSelection(clickId);
        }
      }
    });
  },
  // TODO replace initListHelper with this
  // formListItemArray: function(wrapperId, clickId, template, data, sortable, callback) {
    
  // },
  formDeletableItem: function(removeButtonId, wrapperId, isAjax, checkboxId) {
    if (isAjax) {
      init.materialDesign();
    }
    var removeButton = $(removeButtonId);

    removeButton.on('click', function() {
      // Prevent user from leaving
      utilAdmin.userConfirmBack(true);
      var itemWrapper = $(wrapperId);
      util.fadeOutCSS(wrapperId, false);
      itemWrapper.after('<div class="hidden"></div>');

      if (checkboxId) {
        $(checkboxId)[0].checked = false;
      }
    });
  },
};

/*
  === Admin Utility Class ===
  Used for utility functions that won't be used on the user view
*/
var utilAdmin = {
  sortableInit: function(wrapperId) {
    var sortableObject = Sortable.create($(wrapperId)[0]);
    sortableObject.option('handle', '.sortable-handle');
  },
  hideAdminContent: function() {
    $(constantsAdmin['ID_ADMIN_MAIN_PAGE']).toggle(false);
  },
  /*
    Takes a group of buttons, and a group of hidden material, and hides the buttons
    when one is pressed, showing the relevant hidden material. Similar to
    accordion.initAccordion.
    @param buttonId ID of button that shows/hides content
    @param contentId ID of content to show
    @param buttonGroupClass class of container for buttons that show/hide content
    @param contentGroupClass class of container for all hidden content
    @param sectionGroupClass class of each section in a content container
  */
  initToggleGroup: function(buttonId, contentId, buttonGroupClass, contentGroupClass, sectionGroupClass, backButtonId) {
    var button = $(buttonId);
    var content = $(contentId);
    var buttonGroup = $(buttonGroupClass);
    var contentGroup = $(contentGroupClass);
    var sectionGroup = $(sectionGroupClass);
    var backButton = $(backButtonId);

    // State switcher
    var toggleInitialGroupState = function(shouldShowButtons) {
      // Never show more than one content item, or section
      contentGroup.toggle(false);
      sectionGroup.toggle(false);

      buttonGroup.toggle(shouldShowButtons);
      content.toggle(!shouldShowButtons);
      backButton.toggle(!shouldShowButtons);
    }

    // Initial state
    toggleInitialGroupState(true);

    // Show content
    button.on('click', function() {
      toggleInitialGroupState(false);
    });

    // Hide content
    backButton.on('click', function() {
      toggleInitialGroupState(true);
    });
  },
  /*
    Function to animate text onto page
    Source: https://stackoverflow.com/questions/7264974/show-text-letter-by-letter (mrtsherman)
  */
  showText: function(selectId, message, index, updateInterval, shouldLoop) {
    if (index < message.length) {
      $(selectId).append(message[index++]);
      setTimeout(function() { utilAdmin.showText(selectId, message, index, updateInterval, shouldLoop); }, updateInterval);
    } else if (shouldLoop) {
      $(selectId).empty();
      setTimeout(function() { utilAdmin.showText(selectId, message, 0, updateInterval, shouldLoop); }, updateInterval);
    }
  },
  setBackgroundImage: function(divId, newImageId) {
    // Set image to currently selected image
    var currentImageIdentifier = parseInt(newImageId);
    if (Number.isInteger(currentImageIdentifier)) {
      $(divId).attr('style', "background-image: url('/images/user_images/4x3/" + currentImageIdentifier + ".jpeg')")
    } else {
      $(divId).attr('style', ''); // If no image set
    }
  },
  /*
    Remove file extension, like '.jpeg' or '.mp4'.
  */
  removeExtension: function(string) {
    return string.replace(/\.[^/.]+$/, "");
  },
  /*
    Remove path from filename, so you just have the filename.
  */
  removePath: function(string) {
    return string.split('\\').pop();
  },
  /*
    Add a '#' symbol to the front of a string, if one isn't there already.
    Used when making jQuery selections, with $() syntax
  */
  addPound: function(string) {
    if (string && string.charAt(0) !== '#') {
      return '#' + string;
    }
    return string;
  },
  /*
    Simulate a user checkbox click by toggling checkbox and triggering a 'change'
    event.
  */
  triggerCheckbox: function(checkboxId) {
    var checkbox = $(checkboxId);
    if (checkbox.is(':checked')) {
      checkbox.prop('checked', false).trigger('change');
    } else {
      checkbox.prop('checked', true).trigger('change');
    }
  },
  setCheckbox: function(checkboxId, setToBoolean) {
    var checkbox = $(checkboxId);
    checkbox.prop('checked', setToBoolean).trigger('change');
  },
  mergeObjects: function(object1, object2) {
    Object.keys(object2).forEach(function(key) { object1[key] = object2[key]; });
    return object1;
  },
  enableRaisedButtonGif: function(buttonId) {
    utilAdmin.raisedButtonGifHelper(buttonId, true);
  },
  disableRaisedButtonGif: function(buttonId) {
    utilAdmin.raisedButtonGifHelper(buttonId, false);
  },
  /*
    Enables and disables a loading gif in the middle of a raised button.
    COUPLED TO HTML - see raised_button.tpl.
  */
  raisedButtonGifHelper: function(buttonId, showGif) {
    var button = $(buttonId);

    // If button exists
    if (button.length) {
      var gif = $(buttonId + '-gif');
      var content = $(buttonId + '-content');
      if (showGif) {
        button.addClass('pointer-events-none');
        content.toggle(false);
        gif.toggle(true);
      } else {
        button.removeClass('pointer-events-none');
        content.toggle(true);
        gif.toggle(false);
      }
    }
  },
  /*
    Goes through string and replaces '%s' with a value

    @param string
    @param value
  */
  sprintf: function(string, value) {
    return string.replace('%s', value);
  },
  /*
    Refresh page with post request to PHP. Endpoint must check
    isset($_POST['form_success']) and send down a $form_success variable,
    allowing the front-end to pop up a success message after a refresh
    (Popup message check is in footer_admin.tpl)
  */
  refreshPageSuccess: function() {
    var url = window.location.href;
    var successForm = $(
      '<form action="' + url + '" method="POST">' +
        '<input type="text" class="hidden" name="form_success" value="TRUE" />' +
      '</form>'
    );
    $('body').append(successForm);
    successForm.submit();
  },
  /*
    Converts a PHP timestamp string to a string like '4 hours ago, etc.'

    @param timestampString PHP string, with Date / Time
    @param translatedStrings Object with translated strings (using {t}{/t})
  */
  convertTimestampStringToRelativeTime: function(timestampString, translatedStrings) {
    var secInDay = 86400;
    var secInHour = 3600;
    var secInMinute = 60;

    // iOS / Safari fix, requires '/' instead of '-', and removal of period.
    if (timestampString.indexOf('.') !== -1) {
      timestampString = timestampString.substring(0, timestampString.indexOf('.'));
    }
    timestampString = timestampString.replace(/-/g, '/');
    timestampString += ' UTC';

    var currentTimeStamp = new Date().getTime();
    var interactionTimeStamp = new Date(timestampString).getTime();
    var timeAgo = parseInt((currentTimeStamp - interactionTimeStamp) / 1000);

    // Don't allow negative time for minor clock differences
    if (timeAgo < 0) {
      timeAgo = 0;
    }

    // Convert seconds to days, hours, or minutes string
    if (timeAgo >= secInDay) {
      return utilAdmin.sprintf(translatedStrings['days_string'], (timeAgo / secInDay).toFixed(0));
    } else if (timeAgo >= secInHour) {
      return utilAdmin.sprintf(translatedStrings['hours_string'], (timeAgo / secInHour).toFixed(0));
    } else if (timeAgo >= secInMinute) {
      return utilAdmin.sprintf(translatedStrings['minutes_string'], (timeAgo / secInMinute).toFixed(0));
    } else {
      return utilAdmin.sprintf(translatedStrings['seconds_string'], timeAgo);
    }
  },
  /*
    Updates a span with 'seconds ago' etc.

    @param dateTime initial time to update from
    @param spanId span where the text should go
    @param params translated Strings (see shared_translated_strings.tpl)
  */
  updateTimestamp: function(dateTime, spanId, params) {
    var updateRelativeTimes = function() {
      var timeString = utilAdmin.convertTimestampStringToRelativeTime(dateTime, params);
      timeString = utilAdmin.sprintf(params['time_ago_string'], timeString);

      // Update
      $(spanId).text(timeString);
    }

    // Update the "Seconds ago" every 15 seconds
    updateRelativeTimes();
    setInterval(updateRelativeTimes, 15000);
  },
  /*
    Used to add 'pulse' class to updated items.

    @param classNameToAdd class to add
    @param objectId object that will receive the class
    @param objectNewValue text in the object that is being updated
    @param timeoutMs time that class should be added
  */
  addClassAndRemoveAfterTimeIfValuesDiffer: function(classNameToAdd, objectId, objectNewValue, timeoutMs) {
		var objectCurrentValue = $(objectId).text().trim();
		if (typeof objectNewValue == 'undefined') {
			var objectNewValue = '';
		} else {
      var objectNewValue = ('' + objectNewValue).trim();
    }

    $(objectId).text(objectNewValue);

    // If the value has changed
		if (objectCurrentValue != objectNewValue) {
      // Only pulse if updating a value already on the page
      if (objectCurrentValue !== '-' && objectCurrentValue !== '') {
        util.addClassAndRemoveAfterTime(classNameToAdd, objectId, timeoutMs);
      }
		}
	},
  /*
    If a true parameter is passed, window will stop users from leaving page.

    @param enable if true, stop users, if false, shut off dialog
  */
  userConfirmBack: function(enable) {
    if (enable) {
      window.onbeforeunload = function(e) {
        return true;
      };
    } else {
      window.onbeforeunload = undefined;
    }
  },
  /*
    Each time a user changes a form, the formId is added to this array.
  */
  userFormChangesIdArray: [],
  /*
    NOTE: Appears to be working! Not yet used. Needs to be tested!
    For all forms on a given page, if users change the forms, prevent user
    from leaving page (going back) if they have not submitted the form yet.
  */
  userConfirmBackOnFormChange: function() {
    var pageForms = $('form');

    // Add change listener to each form on the page
    pageForms.each(function() {
      var keepUserOnPageIfChanged = function() {
        window.onbeforeunload = function(e) {
          if ((utilAdmin.userFormChangesIdArray.length != 0)) {
              return true;
          }
        };
      }

      // If form has an ID
      var currentFormIdNoPound = $(this).attr('id');
      if (currentFormIdNoPound) {
        var currentFormId = '#' + currentFormIdNoPound;
        var currentForm = $(currentFormId);
        currentForm.find(':input').on('change', function() {
          // Add to changed array if not currently in it
          if (utilAdmin.userFormChangesIdArray.indexOf(currentFormId) === -1) {
            utilAdmin.userFormChangesIdArray.push(currentFormId);
          }
          // Keep on page if there are changes recorded in the change ID array
          keepUserOnPageIfChanged();
        });

        // Used Javascript syntax for .on('submit') because jQuery was not firing
        currentForm[0].onsubmit = function() {
          // On submit, remove form ID from changes array
          utilAdmin.userFormChangesIdArray.splice(
            utilAdmin.userFormChangesIdArray.indexOf(currentFormId), 1
          );
          keepUserOnPageIfChanged();
        };
      }
    });
  },
  revealCapsLockWarning: function(warningId) {
    warning = $(warningId);

    // Shut off caps on keyup
    document.addEventListener('keyup', function(e) {
      if (e.key === 'CapsLock') {
        warning.toggle(false);
      }
    });

    // Turn on caps lock on any keydown click
    document.addEventListener('keydown', function(e) {
      if (e.getModifierState && e.getModifierState('CapsLock')) {
        warning.toggle(true);
      } else {
        warning.toggle(false);
      }
    });
  }
}

/*
  === Reveal class ===
  Class used to show hidden elements on click, select, scroll, etc.
*/
var reveal = {
  revealOnScroll: function(revealId, scrollTargetId, offset) {
    var reveal = $(revealId);
    var scrollTarget = $(scrollTargetId).offset().top + offset;
    $(window).scroll(function() {
      if ($(window).scrollTop() >= scrollTarget && reveal.is(":hidden")) {
        reveal.toggle(true);
      } else if ($(window).scrollTop() < scrollTarget && reveal.is(":visible")) {
        reveal.toggle(false);
      }
    });
  },
  revealWithInput: function(contentId, inputId, radioGroupName, reverseToggleOrder) {
    var content = $(contentId);
    var input = $(inputId);

    content.addClass("collapse"); // Instantiate classes
    reveal.toggleRevealChecked(content, input, reverseToggleOrder);

    var selector; // Check for input radio button, or checkbox
    if (radioGroupName) {
      selector = $('input[name=' + radioGroupName + ']:radio');
    } else {
      selector = input;
    }

    selector.on('click', function() { // Toggle on click
      reveal.toggleRevealChecked(content, input, reverseToggleOrder);
    });
  },
  revealOnClick: function(contentId, inputId) {
    var content = $(contentId);
    var input = $(inputId);

    input.on("click", function() {
      content.collapse('show');
    });
  },
  /* Input: jquery selection */
  toggleRevealChecked: function(content, input, reverseToggleOrder) {
    if (input.is(":checked")) {
      if (reverseToggleOrder) {
        content.collapse('hide');
      } else {
        content.collapse('show');
      }
    } else {
      if (reverseToggleOrder) {
        content.collapse('show');
      } else {
        content.collapse('hide');
      }
    }
  },
  /* Shows and hides without collapse */
  show: function(showId, hideId, clickId) {
    var show = $(showId);
    var hide = $(hideId);
    var click = $(clickId);

    click.on('click', function() {
      show.toggle(true);
      hide.toggle(false);
    })
  }
};

/*
  === SELECT2 PLUGIN CLASS ===
  Class with helpers for the select2 dropdown select menu plugin (select2.full.min.js)
*/
var select2Class = {
  /*
    Initialize without formatting function.
  */
  init: function(selectId) {
    select2Class.initHelper(selectId, false, false, false);
  },
  initInPopup: function(selectId) {
    select2Class.init(selectId)
    select2Class.popupZIndex(selectId);
  },
  popupZIndex: function(selectId) {
    select2Class.select2ZIndex(selectId, constantsAdmin['SELECT2_POPUP_Z_INDEX']);
  },
  /*
    Initialize with function that includes images with options.
  */
  initImages: function(selectId) {
    select2Class.initHelper(selectId, false, select2Class.formatImages, false);
  },
  /*
    Initialize with function that shows icons next to options.
  */
  initIcons: function(selectId, minimumResultsForSearch) {
    select2Class.initHelper(selectId, false, select2Class.formatIcons, true, minimumResultsForSearch);
  },
  /*
    Helper to format select2 dropdown <select> tag

    @param selectId HTML id of <select> tag
    @param formatFunction custom Select2 function that changes the way options
    look.
    @param showFormatInSelection If true, selected items will be formatted as well,
    Not just items in the dropdown menu
  */
  initHelper: function(selectId, initialOptions, formatFunction, showFormatInSelection, minimumResultsForSearch) {
    if (!minimumResultsForSearch) {
      var minimumResultsForSearch = 10;
    }

    // If selectId is undefined, use default .select2 class
    if (!selectId) {
      selectId = constantsAdmin['SELECT2_DEFAULT_CLASS_NAME'];
    }

    if (!initialOptions) {
      initialOptions = {};
    }
    var options = initialOptions;
    options['minimumResultsForSearch'] = minimumResultsForSearch,
    options['width'] = '100%';
    if (formatFunction) {
      options['templateResult'] = formatFunction;
      if (showFormatInSelection) {
        options['templateSelection'] = formatFunction;
      }
    }
    $(selectId).select2(options);
  },
  /*
    Open select2 dropdown upon clicking element

    @param selectId HTML id of <select> tag
    @param clickId item clicked to open select2 dropdown
  */
  openOnClick: function(selectId, clickId) {
    select2Class.onClick(selectId, clickId, 'open');
  },
  /*
    Helper to perform action to select2 dropdown upon clicking another element

    @param selectId
    @param clickId
    @param selectInput Action to be peformed on the select2 dropdown, like 'open',
    or 'close'.
  */
  onClick: function(selectId, clickId, selectInput) {
    var select = $(selectId);
    var click = $(clickId);
    click.on('click', function() {
      select.select2(selectInput);
    });
  },
  /*
    Changes the z-index of the global select2 popup so a select2 can be
    used in a navigation menu, and is not behind the menu.
  */
  select2Nav: function(selectId) {
    select2Class.select2ZIndex(selectId, 20)
  },
  /*
    The Select2 plugin sometimes has issues with getting above or behind elements
    upon opening. This function changes the z-index of the dropdown upon open
    and close, to allow Select2 to be used in nav bars, etc.

    @param selectId
    @param zIndex The z-index to be applied on open ('inherit' will be used
    otherwise).
  */
  select2ZIndex: function(selectId, zIndex) {
    var select = $(selectId);

    select.on('select2:open', function() {
      var dropdown = $('.select2-dropdown');
      dropdown.css('z-index', zIndex);
    });

    select.on('select2:close', function() {
      var dropdown = $('.select2-dropdown');
      dropdown.css('z-index', 'inherit');
    });
  },
  /*
    Custom Select2 'templateResult' formatting function.
    Takes image ids stored as option values and inserts images using the ids.

    @param option Parameter representing each option in the dropdown, since this
    is called for each option
  */
  formatImages: function(option) {
    // Return text for options that aren't images
    var optionText = option.text;
    if (!option.id){
      return optionText;
    }
    var optionValue = option.element.value;
    if (optionValue === constantsAdmin['SELECT2_DEFAULT']
        || optionValue === constantsAdmin['SELECT2_NONE']
        || optionValue === constantsAdmin['SELECT2_THUMBNAIL'] // TODO Are we using this?
        || optionValue === constantsAdmin['SELECT2_CURRENT']
        || optionValue === constantsAdmin['SELECT2_CURRENT_THUMBNAIL']) {
      return optionText;
    }

    // Image upload button
    if (optionValue === constantsAdmin['SELECT2_UPLOAD'] || optionValue === constantsAdmin['SELECT2_GENERATE_THUMBNAIL']) {
      var icon = '&#xE3F4;'; // Generate thumbnail
      if (optionValue === constantsAdmin['SELECT2_UPLOAD']) {
        icon = '&#xE2C6;'; // Upload
      }
      return $('<span><i class="material-icons">' + icon + '</i> ' + optionText + '</span>');
    }

    var formattedOption = $(
      '<span><img src="/images/user_images/4x3/' + optionValue + '.jpeg" class="select2-media-image" /> ' + optionText + '</span>'
    );
    return formattedOption;
  },
  /*
    Custom Select2 'templateResult' formatting function.
    Takes icon hex code stored stored as option value and inserts Google material
    design icons into the dropdown.

    @param option Parameter representing each option in the dropdown
  */
  formatIcons: function(option) {
    var optionText = option.text;
    if (!option.id){
      return optionText;
    }
    var optionValue = option.element.value;

    // NOTE: this was breaking stuff when icons showed up as &#xE87D; instead of a single character icon
    // Don't format if not an icon
    // var charCode = optionValue.charCodeAt(0); // Charcodes for icons will be > 255
    // if (charCode <= 255) {
    //   return optionText;
    // }

    var formattedOption = $(
      '<span>' + optionText + '<i class="material-icons margin-top-minus-3">' + option.element.value + '</i></span>'
    );
    return formattedOption;
  },
  /*
    Adds image to the form_select_image.tpl <select>, called after upload success.

    @param selectId HTML id of the Select2 <select>
    @param imageInsertClass class of the current options in the <select> to prepend to
    @param imageIdentifier Database id of the new image that was recently uploaded
  */
  addImage: function(selectId, selectGroupClass, imageInsertClass, imageIdentifier, filename) {
    $(imageInsertClass).toggle(true); // Make sure image insert class is visible
    popup.removeLoadingGif();

    $(imageInsertClass).prepend('<option value="' + imageIdentifier + '">' + filename + '</option>');
    $(selectId).val(imageIdentifier).trigger('change');
    select2Class.initImages(selectGroupClass);
  },
  /*
    Reset select option, and input

    @param inputId HTML id of the file upload input
    @param selectId HTML id of the Select2 <select>
  */
  removeImage: function(inputId, selectId) {
    popup.removeLoadingGif();
    var input = $(inputId);
    var select = $(selectId);

    select.val('').trigger('change');
    input.val(''); // Reset the input
  },
  /*
    If you have multiple identical select2s with the same class, you can make
    them all change together when one changes.

    @param select2ClassName the class of the set of select2s
  */
  onChangeUpdateAllSelect2WithClass: function(select2ClassName) {
    select2ClassSet = $(select2ClassName);
    select2ClassSet.on('select2:select', function() {
      $(select2ClassName).val(this.value).trigger('change');
    });
  },
  /*
    Removes the current selection, returns to default.
  */
  unselectCurrentSelection: function(select2ClassName) {
    $(select2ClassName).val('').trigger('change');
  },
  /*
    Appends options to a currently selected object

    @param select2Object NOT and ID
  */
  appendOption: function(select2Object, value, text) {
    select2Object.append('<option value="' + value + '">' + text + '</option>');
  }
}; // End select2Class
// NOTE: Primary, default select2 initialization occurs here.
select2Class.init();

/*
  === Popup class ===
  Class used for popup error messages and other popup templates.
*/
var popup = {
  /*
    Create popup to add directories, language folders, and captive portal languages
  */
  addDirectoryPopup: function (isLanguageDirectory, params, confirmCallback) {
    var popupConfirmButtonId = '#new-folder-popup-confirm';
    var nameInputId = '#name-input-select';

    var currentDirectoryId = 0;
    if (params['current_directory_id']) {
      currentDirectoryId = params['current_directory_id'];
    }
    var popupHelperData = {
      include_file: 'admin/components/media/new_directory_select_wrapper.tpl',
      heading: params['heading'],
      confirm_text: params['confirm_text'],
      message: '',
      name_input_id: util.removePound(nameInputId),
      is_language_directory: isLanguageDirectory,
      confirm_button_id: util.removePound(popupConfirmButtonId),
      current_directory_id: currentDirectoryId,
      filename: '',
      title: '',
      is_popup: true
    }
    if (params['is_captive_portal_language']) {
      popupHelperData['is_captive_portal_language'] = params['is_captive_portal_language'];
    }
    
    if (isLanguageDirectory) {
      var languageSelectId = '#new-language-select';
      popupHelperData['language_select_id'] = util.removePound(languageSelectId);
      popupHelperData['language_iso'] = '';
      // popupHelperData['ajax_get_data'] = 'language_isos_array';
    }

    popup.popupHelper(
      'shared/popup_custom.tpl',
      popupHelperData,
      function () {
        if (isLanguageDirectory) {
          select2Class.popupZIndex(selectId);
        }
        var newDirectoryFormId = '#new-directory-form';
        if (confirmCallback) {
          $(newDirectoryFormId).on('submit', function(e) {
            e.preventDefault();
            confirmCallback();
          });
        } else {
          ajax.submitAjaxForm(
            newDirectoryFormId,
            function () {
              utilAdmin.refreshPageSuccess();
            },
            function () {
              popup.removeLoadingGif();
            }
          );
        }
        popup.hideOnFormSubmit(newDirectoryFormId); // IMPORTANT: if user presses enter, close the popup
        $(popupConfirmButtonId).on('click', function () {
          if (!confirmCallback) {
            popup.loadingGif(); 
          }
          $(newDirectoryFormId).submit();
        });
      }
    );
  },
  /*
    Open error popup template.

    @param message Error message to display to users in popup body.
  */
  errorPopup: function(message, callback) {
    popup.errorPopupHeading(
      message,
      false,
      {
        is_success: false
      },
      callback
    );
  },
  /*
    Popup message with success styling

    @param message
  */
  successPopup: function(message) {
    popup.errorPopupHeading(message, false, {
      is_success: true
    });
  },
  /*
    Helper for errorPopup.

    @param message error message
    @param heading
  */
  errorPopupHeading: function(message, heading, additionalData, callback) {
    popup.removeLoadingGif();
    var appendErrorMessage = function(errorMessage) {
      $(constantsAdmin['ID_POPUP_ERROR_MESSAGE']).append('<br /> ' + errorMessage);
    }
    var popupErrorMessageSelection = $(constantsAdmin['ID_POPUP_ERROR_MESSAGE']);
    if (popupErrorMessageSelection.length > 0) { // If there is a current error popup, then don't call ajax
      appendErrorMessage(message);
    } else {
      var data = {};
      if (additionalData) {
        data = additionalData
      }
      data['message'] = message;
      if (heading) {
        data['heading'] = heading;
      }

      popup.removeAllPopups();
      sharedAjax.template(
        'shared/popup_error.tpl',
        data,
        'body',
        false,
        function() {
          var errorMessagesRemoved = popup.removeDuplicatePopups(constantsAdmin['ID_POPUP_ERROR_MESSAGE']);
          errorMessagesRemoved.forEach(function(errorMessage) {
            appendErrorMessage(errorMessage);
          });
          if (callback) {
            callback();
          }
        }
      );
    }
  },
  /*
    Open confirm popup template, with cancel and confirm buttons.

    @param message Message to display to users
    @param confirmButtonId HTML id of button representing confirmation
    @param callback Function called when template is loaded
  */
  confirmPopup: function(message, confirmButtonId, callback, additionalData) {
    popup.removeLoadingGif();

    if (additionalData) {
      data = additionalData;
    } else {
      data = {};
    }

    // Add in data
    data['message'] = message;
    if (confirmButtonId) {
      data['confirm_button_id'] = util.removePound(confirmButtonId);
    }

    popup.removeAllPopups();
    sharedAjax.template(
      'shared/popup_confirm.tpl',
      data,
      'body',
      false,
      callback
    );
  },
  /*
    Open select popup template, with Select2 <select> dropdown.

    @param popupSelectId HTML id of Select2 <select> options dropdown
    @param selectArrayJSON JSON containing the keys and values for the select
    dropdown (see select_array in form_select.tpl)
  */
  selectPopup: function(heading, message, confirmButtonId, popupSelectId, selectArrayJSON, callback) {
    popup.cancelConfirmPopupHelper(
      heading,
      message,
      confirmButtonId,
      'shared/popup_select.tpl',
      {
        popup_select_id: util.removePound(popupSelectId),
        select_array: selectArrayJSON
      },
      function() {
        select2Class.initInPopup(popupSelectId);
        callback();
      }
    );
  },
  /* TODO: Comment */
  inputPopup: function(heading, message, confirmButtonId, inputId, confirmText, placeholder, value, callback) {
    popup.cancelConfirmPopupHelper(
      heading,
      message,
      confirmButtonId,
      'shared/popup_input.tpl',
      {
        input_id: util.removePound(inputId),
        placeholder: placeholder,
        value: value,
        confirm_text: confirmText
      },
      function() {
        callback();
      }
    );
  },
  /* TODO: Comment */
  checkboxPopup: function(message, confirmButtonId, checkboxId, checkboxName, actionForm, confirmText, removeOnConfirm, heading, checkboxLabel, callback) {
    if (removeOnConfirm === undefined) {
      removeOnConfirm = true;
    }
    popup.cancelConfirmPopupHelper(
      heading,
      message,
      confirmButtonId,
      'shared/popup_checkbox.tpl',
      {
        checkbox_id: util.removePound(checkboxId),
        checkbox_name: checkboxName,
        checkbox_label: checkboxLabel,
        confirm_text: confirmText
      },
      function() {
        $(confirmButtonId).on('click', function() {
          if ($(checkboxId).is(":checked")) {
            form.submitForm(actionForm);
            if (removeOnConfirm) {
              sharedPopup.hide();
            }
          } else {
            var checkboxParentId = '#' + $(checkboxId).parent().attr('id');
            util.addClassAndRemoveAfterTime('pulse', checkboxParentId, constantsAdmin['PULSE_INTERVAL_MS']);
          }
        });
        if (callback) {
          callback();
        }
      }
    );
  },
  /*
    Helper function for popups that have a cancel button, and confirm button

    @param heading
    @param message
    @param confirmButtonId
    @param template
    @param additionalTemplateParameters Array of objects formatted as {key: '', value: ''},
    added to FormData object
    @param callback Function called when template is loaded
  */
  cancelConfirmPopupHelper: function(heading, message, confirmButtonId, template, additionalTemplateParameters, callback) {
    var templateParameters = {
      heading: heading,
      message: message,
      confirm_button_id: util.removePound(confirmButtonId),
    };
    templateParameters = utilAdmin.mergeObjects(templateParameters, additionalTemplateParameters);
    popup.popupHelper(
      template,
      templateParameters,
      callback
    );
  },
  popupHelper: function(template, data, callback) {
    popup.removeAllPopups();
    popup.removeLoadingGif();
    sharedAjax.template(
      template,
      data,
      'body',
      false,
      function() {
        callback();
      }
    );
  },
  /*
    Start the loading Gif, disabling screen clicks.
  */
  loadingGif: function() {
    util.preventBodyScroll();
    var loadingGif = $(constantsAdmin['ID_LOADING_GIF']);

    // No loading gif currently present
    if (loadingGif.length === 0) {
      ajax.prependTemplate(
      'shared/loading_gif.tpl',
      {},
      'body');
    }
  },
  /*
    Remove the loading Gif. IMPORTANT: always call this function on success, or
    failure after an event requires calling the loadingGif() function. If you don't
    remove the loading gif, the user will be not be able to click on anything (until refresh).
  */
  removeLoadingGif: function() {
    util.enableBodyScroll();
    var loadingGif = $(constantsAdmin['CLASS_LOADING_GIF']);
    util.fadeOutCSS(loadingGif);
  },
  /*
    Remove current popups on the page
  */
  removeAllPopups: function() {
    $(constantsShared['CLASS_POPUP']).remove();
  },
  /*
    Iterates through current popups on the page, and removes all popups except the first one found
    @param messageId optional, the Id of the messages that you want to retrieve from the removed popups
    @return Array of the messages from the removed popups, or empty strings if messageId is not defined
  */
  removeDuplicatePopups: function(messageId) {
    var popups = $(constantsShared['CLASS_POPUP']);
    var messagesRemoved = [];
    if (popups.length > 1) {
      popups.each(function(index, element) {
        if (index > 0) {
          if (messageId) {
            var message = $(element).find(messageId);
            // If message is defined, add it to the removed array
            if (message.length > 0) {
              messagesRemoved.push(message.text());
            }
          }
          element.remove();
        }
      });
    }
    return messagesRemoved;
  },
  /*
    IMPORTANT: If your custom popup has a form in it, and a user presses enter,
    not pressing the popup confirm button, the popup will not close.
    This wil close the popup on submit.
  */
  hideOnFormSubmit: function(formId) {
    $(formId).on('submit', function() {
      sharedPopup.hide();
    });
  }
}

/*
  === COLLAPSE CLASS ===
  Helper functions for Bootstrap collapse plugin (in bootstrap.min.js).
*/
var collapse = {
  hide: function(contentId) {
    $(contentId).collapse('hide');
  },
  show: function(contentId) {
    $(contentId).collapse('show');
  },
  toggle: function(contentId) {
    $(contentId).collapse('toggle');
  },
  showOnClick: function(contentId, inputId) {
    collapse.onClick(contentId, inputId, 'show');
  },
  hideOnClick: function(contentId, inputId) {
    collapse.onClick(contentId, inputId, 'hide');
  },
  toggleOnClick: function(contentId, inputId) {
    collapse.onClick(contentId, inputId, 'toggle')
  },
  /*
    Helper for showOnClick, hideOnClick, and toggleOnClick.
    Calls collapse on HTML content to show, hide, or toggle it when clicking
    another element.

    @param contentId HTML id of content to show/hide on click
    @param inputId HTML id of item that, on click, triggers reveal of content
    @param collapseInput collapse command, like 'show', 'hide', 'toggle', etc.
  */
  onClick: function(contentId, inputId, collapseInput) {
    var content = $(contentId);
    var input = $(inputId);

    input.on('click', function() {
      content.collapse(collapseInput);
    });
  },
  /*
    Initialize a set of elements that open like an "accordion," where only one
    element can be open at a time

    @param clickId HTML id of item that opens the collapse on click
    @param groupClass the class applied to all of the collapse elements in the
    accordion
    @param disableClick If true, accordion can be opened programatically, but
    not on user click
    @param disableOneAtATime If true, accordions can open independently of each
    other
  */
  initAccordion: function(clickId, groupClass, disableClick, disableOneAtATime) {
    click = $(clickId);
    collapseSelect = $(collapse.getAccordionCollapseIdFromClickId(clickId));
    group = $(groupClass);

    /*
    var setArrowBasedOnCollapse = function() {
      if (collapseSelect.hasClass('in')) {
        collapse.arrowUp(clickId);
      } else {
        collapse.arrowDown(clickId);
      }
    };
    */

    //setArrowBasedOnCollapse();
    if (!disableClick) {
      click.on('click', function() {
        //setArrowBasedOnCollapse();
        group.collapse('hide');
        $(this).next().collapse('toggle');
      });
    }
  },
  /*
    Returns the Id of the accordion collapse from the clickId

    COUPLED TO HTML

    @param clickId the clickId of the accordion template
  */
  getAccordionCollapseIdFromClickId: function(clickId) {
    return utilAdmin.addPound(clickId) + '-collapse';
  },

  /*
  getAccordionArrowIdFromClickId: function(clickId) {
    return utilAdmin.addPound(clickId) + '-arrow';
  },
  arrowUp: function(clickId) {
    collapse.arrowRotateHelper(clickId, true);
  },
  arrowDown: function(clickId) {
    collapse.arrowRotateHelper(clickId, false);
  },

  // Animate arrow on dropdowns, to make it rotate

  arrowRotateHelper: function(clickId, rotateUp) {
    var arrow = $(collapse.getAccordionArrowIdFromClickId(clickId));
    if (rotateUp) {
      arrow.addClass('rotate-180');
      arrow.removeClass('rotate-0');
    } else {
      arrow.addClass('rotate-0');
      arrow.removeClass('rotate-180');
    }
  },
  */
  /*
    Programatically open an accordion item, using its clickId.

    @param clickId The item that opens the accordion item on click
    @param groupClass The class applied to the group of accordion items, on
    the collapsable element
  */
  openAccordion: function(clickId, groupClass, disableOneAtATime) {
    if (!disableOneAtATime) {
      group.collapse('hide');
    }
    // collapse.arrowUp(clickId);
    $(clickId).next().collapse('toggle');
  },
};

/*
  === FORM INITIALIZATION CLASS ===
  Form submission,
*/
var form = {
  /*
    Initialize color picker used in theme.tpl.

    @param colorPickerId HTML id of input with type="color"
    @param colorInputId HTML id of text input with color hex value
  */
  colorpicker: function(colorpickerId, colorInputId) {
    var colorpicker = $(colorpickerId);
    var colorInput = $(colorInputId);

    // On color input change, update hex input
    colorpicker.on('change', function() {
      colorInput.val(this.value);
    });

    // On hex input change, convert to hex if input is invalid
    colorInput.on('change', function() {
      var input = this.value;

      // Format as a hex value
      input = input.replace(/[^A-Fa-f0-9]/g, '');

      // TODO Make sure conversion from 3 digit to 6 digit is correct
      if (input.length > 0) {
        input = utilAdmin.addPound(input);

        // Duplicate input to ensure 6-digit (plus #) hex value
        while (input.length < 7) {
          input = input + util.removePound(input);
        }

        // Crop if too long
        if (input.length > 7) {
          input = input.substr(0, 7);
        }
      } else {
        input = '#000000';
      }

      // Set value
      this.value = input;
      colorpicker.val(input);
    });
  },
  /*
    Submit the closest outer form when file is uploaded.

    @param inputId HTML id of input with type="file" that will make form submit when a file has
    been uploaded
  */
  submitOnFileUpload: function(inputId) {
    var input = $(inputId);
    var closestForm = input.closest('form');
    form.submitFormOnFileUpload(utilAdmin.addPound(closestForm.attr('id')), inputId);
  },
  /*
    Submit any form on file upload.

    @param formId Form to submit
    @param inputId Input that will submit the form on upload
  */
  submitFormOnFileUpload: function(formId, inputId) {
    var inputForm = $(formId);
    form.onFileUploadHelper(inputId, function() {
      inputForm.trigger('submit');
    });
  },
  /*
    Helper for submitFormOnFileUpload and submitOnFileUpload.
    Calls a callback when a file is uploaded in an input.

    @param inputId HTML id of input that fires an event on file upload
    @param callback that calls when file upload has occured.
  */
  onFileUploadHelper: function(inputId, callback) {
    var input = $(inputId);

    // Reset on click, to make sure the loading gif doesn't fire on cancel
    input.on('click touchstart' , function() {
      popup.removeLoadingGif();
      $(this).val('');
    });

    input.on('change', function(e) {
      // Start loading gif
      popup.loadingGif();

      if (this.files && this.files[0]) {
        popup.removeLoadingGif();
        callback(this.files[0]);
      } else {
        /* TODO error popup? */
      }
    });
  },
  /*
    For ajax forms. Append success message at bottom of page, useful when
    form has been successfully submitted.
  */
  animateSuccessOutTimeout: undefined, // Global timeout for removing succcess messages
  submitForm: function(formId) {
    inputForm = $(formId);
    inputForm.submit();
  },
  submitFormOnClick: function(formId, clickId) {
    form.submitFormHelper(formId, clickId, 'click');
  },
  /*
    Submit form when an element fires a 'change' event.

    @param formId
    @param changeId
  */
  submitFormOnChange: function(formId, changeId) {
    form.submitFormHelper(formId, changeId, 'change');
  },
  /*
    Submit form on <select> change, used with Select2 plugin.

    @param formId
    @param selectId HTML id of Select2 <select> tag
  */
  submitFormOnSelect: function(formId, selectId) {
    form.submitFormHelper(formId, selectId, 'select2:select');
  },
  /*
    Helper for submitFormOnSelect, submitFormOnClick, and submitFormOnChange

    @param formId
    @param changeId HTML id of element that will submit the form when the event
    encoded by eventString fires on it
    @param eventString jQuery event string like 'click', 'change', or 'select2:select'
  */
  submitFormHelper: function(formId, changeId, eventString) {
    var change = $(changeId);
    change.on(eventString, function() {
      form.submitForm(formId);
    });
  },
};

/*
  === UPLOADER CLASS ===
  Fineuploader plugin initialization and helper functions.
*/
var uploader = {
  initUploaderHelper: function(callbacks, options, templateId, remainingSpaceBytes, remainingSpaceErrorMessage) {
    // Prevent upload if totalBytes exceeds space on LightStream
    if (remainingSpaceBytes !== undefined && remainingSpaceErrorMessage !== undefined) {
      callbacks['onTotalProgress'] = function(totalUploadedBytes, totalBytes) {
        var fineuploaderObject = this;
        setTimeout(function() { // setTimeout needed to fix fineuploader bug
          var uploads = fineuploaderObject.getUploads({
            status: 'submitted'
          });
          if (totalBytes >= remainingSpaceBytes || totalUploadedBytes >= remainingSpaceBytes) {
            uploader.fineuploaderReset(fineuploaderObject);
            sharedPopup.hide();
            popup.errorPopup(remainingSpaceErrorMessage);
          }
        }, 100);
      };
    }
    if (!templateId) {
      templateId = constantsAdmin['ID_FINEUPLOADER_DEFAULT'];
    }
    templateId = util.removePound(templateId);
    var fineuploaderData = {
      element: document.getElementById(templateId),
      template: uploader.getTemplateScriptId(templateId),
      request: {
          endpoint: '/admin/utilities/fine_uploader/endpoint.php'
      },
      chunking: {
         enabled: true,
         concurrent: {
           enabled: true
         },
         success: {
           endpoint: "/admin/utilities/fine_uploader/endpoint.php?done"
         }
     },
      thumbnails: {
          placeholders: {
            waitingPath: '/vendor/fine_uploader/placeholders/waiting-generic.png',
            notAvailablePath: '/vendor/fine_uploader/placeholders/not_available-generic.png'
          }
      },
      retry: {
        enableAuto: true,
        showButton: true
      },
      callbacks: callbacks,
      autoUpload: false,
    };
    // Add additional options
    if (options) {
      var optionsArray = ['multiple', 'validation'];
      for(i in optionsArray) {
        var item = optionsArray[i];
        if (options[item] !== undefined) {
          fineuploaderData[item] = options[item];
        }
      }
    }
    return new qq.FineUploader(fineuploaderData);
  },
  currentFineuploaderObject: false,
  currentDropdownObject: false,
  initUploaderMulti: function(callbacks, additionalOptions, remainingSpaceBytes, remainingSpaceErrorMessage, templateId) {
    templateId = util.addPound(templateId);

    // Call function from additonalCallbacks if defined
    var callAdditionalCallback = function(name) {
      if(callbacks[name]) {
        callbacks[name]();
      }
    }

    // Acceptable files for images
    var options = {
      validation: {}
    };
    if (additionalOptions['acceptFiles']) {
      options['validation']['acceptFiles'] = additionalOptions['acceptFiles'];
    }
    if (additionalOptions['allowedExtensions']) {
      options['validation']['allowedExtensions'] = additionalOptions['allowedExtensions'];
    }
    if (typeof(additionalOptions['multiple']) === typeof(true)) { // if boolean
      options['multiple'] = additionalOptions['multiple'];
    }

    // Instantiate fineuploader
    return uploader.initUploaderHelper({
      onComplete: callbacks['onComplete'],
      onSubmit: callbacks['onSubmit'],
      // Called when each file is submitted for upload
      onUpload: function(id, fileName) {
        uploader.initUploaderMultiShowCancel(templateId);
        callAdditionalCallback('onUpload');
      },
      onCancel: function() {
        if ($(templateId + ' .fineuploader-upload-files-list').first().children().length <= 1) {
          uploader.initUploaderMultiShowUpload(templateId);
        }
        callAdditionalCallback('onCancel');
      },
      onAllComplete: function(succeededArray, failedArray) {
        utilAdmin.userConfirmBack(false);
        callAdditionalCallback('onAllComplete');
      }
    },
    options,
    util.removePound(templateId),
    remainingSpaceBytes,
    remainingSpaceErrorMessage
    );
  },
  /*
    COUPLED TO HTML
  */
  // Show and hide upload / cancel button
  initUploaderMultiShowCancel: function(templateId) {
    uploader.initUploaderMultiToggle(templateId, false);
  },
  initUploaderMultiShowUpload: function(templateId) {
    uploader.initUploaderMultiToggle(templateId, true);
  },
  initUploaderMultiToggle: function(templateId, showSubmit) {
    $(templateId + '-upload-files-submit').toggle(showSubmit);
    $(templateId + '-upload-files-cancel').toggle(!showSubmit);
  },
  fineuploaderReset: function(fineuploaderObject) {
    fineuploaderObject.reset();
  },
  fineuploaderStartUpload: function(fineuploaderObject) {
    utilAdmin.userConfirmBack(true); // prevent users from leaving page freely
    fineuploaderObject.uploadStoredFiles();
  },
  fineuploaderStartUploadAfterTimeout: function(fineuploaderObject) {
    // If function is run immediately, it doesn't know submit has happened yet
    setTimeout(function() {
      uploader.fineuploaderStartUpload(fineuploaderObject);
    }, 100);
  },
  fineuploaderCancel: function(fineuploaderObject) {
    utilAdmin.userConfirmBack(false);
    fineuploaderObject.cancelAll();
  },
  getTempFilePath: function(responseJSON, fileName) {
    return '/admin/utilities/fine_uploader/files/' + responseJSON['uuid'] + '/' + fileName;
  },
  getTemplateScriptId: function(templateId) {
    return templateId + '-script';
  },
  fineuploaderOpenDialog: function(templateId) {
    if (!templateId) {
      templateId = constantsAdmin['ID_FINEUPLOADER_DEFAULT'];
    }
    $(util.addPound(templateId) + ' .qq-upload-button-selector input').click();
  }
} // Uploader

/*
  === AJAX CLASS ===
  Submitting forms with Ajax, getting and appending template data, calling files
*/
var ajax = {
  /*
    General ajax caller that isn't strictly for getting templates or submitting forms.
    @param action Endpoint in php
    @param data Data object
    @param isGet Boolean, if true, submetted as a get request
    @param callback Function on success
    @param timeout 
  */
  ajax: function(action, callback, data, isGet, timeout) {
    var type = 'POST';
    if (isGet) {
      type = 'GET';
    }
    data['ajax'] = 'true';
    var ajaxObject = {
      type: type,
      url: action,
      data: data,
      success: function(response) {
        if (callback) {
          callback(response);
        }
      }
    }
    if (timeout) {
      ajaxObject['timeout'] = timeout;
    }
    $.ajax(ajaxObject);
  },
  appendTemplate: function(template, attributes, renderId) {
    sharedAjax.template(template, attributes, renderId, true, false);
  },
  prependTemplate: function(template, attributes, renderId) {
    sharedAjax.template(template, attributes, renderId, false, false);
  },
  /*
    Submit form with ajax, without a callback action.
  */
  submitAjaxFormNoCallback: function(formId) {
    ajax.submitAjaxForm(formId, false);
  },
  /*
    Refresh page on success, after ajax form submission.
  */
  // submitAjaxFormRefresh: function(formId) {
  //   ajax.submitAjaxForm(formId, util.refreshPage);
  // },
  submitAjaxFormSuccessRefresh: function(formId) {
    ajax.submitAjaxForm(
      formId,
      utilAdmin.refreshPageSuccess,
      popup.removeLoadingGif
    );
  },
  /*
    Show success message on success.
  */
  submitAjaxFormSuccess: function(formId) {
    ajax.submitAjaxFormSuccessWithCallback(formId, false);
  },
  /*
    Helper for submitAjaxFormSuccess, allows an additional success callback
  */
  submitAjaxFormSuccessWithCallback: function(formId, successCallback) {
    ajax.submitAjaxFormHelper(
      formId,
      false,
      function(successJSON) {
        ajax.appendSuccessMessage();
        if (successCallback) {
          successCallback(successJSON);
        }
      },
      false
    );
  },
  /*
    Submit a form with ajax, with a success callback.
    Doesn't show a success message, but will show error messages.
    Helper for submitAjaxFormNoCallback and submitAjaxFormRefresh.

    @param formId
    @param successCallback Function called on successful submission
  */
  submitAjaxForm: function(formId, successCallback, failureCallback) {
    ajax.submitAjaxFormHelper(
      formId,
      false,
      successCallback,
      failureCallback
    );
  },
  /*
    Helper for submitAjaxForm and submitAjaxFormSuccess. Submits a form with
    ajax.

    @param formId
    @param additionalFormDataArray Array that will be partsed to add in additonal
    POST / FormData sent to PHP. NOTE: Formatted as and arrary of objects with
    'key' and 'value' attributes, like [{key: '1', value: '2'}, {etc...}]
    @param successCallback
    @param failureCallback
  */
  submitAjaxFormHelper: function(formId, additionalFormDataArray, successCallback, failureCallback, hideErrorPopup) {
    var ajaxForm = $(formId);
    var action = ajaxForm.attr('action');
    var ajaxFormInputs = $(formId + ' input');

    // NOTE: this is the only event that can fire on submit, since we want to avoid cases where this is called 2+ times
    ajaxForm.off('submit');

    ajaxForm.on('submit', function(e) {
      e.preventDefault(); // Prevent refresh

      var formData = new FormData(this);

      // Add optional additional form data, not in the HTML form
      if (additionalFormDataArray) {
        additionalFormDataArray.forEach(function(formDataKeyValueObject) {
          var key = formDataKeyValueObject['key'];
          var value = formDataKeyValueObject['value'];
          formData.append(key, value);
        });
      }

      ajax.phpHelper(action, formData, successCallback, failureCallback, true, hideErrorPopup);
    });
  },
  /*
    Call a PHP file with ajax. Similar to submitAjaxFormHelper, but data is
    just a normal JS object, instead of a FormData object.
  */
  php: function(action, data, successCallback, failureCallback) {
    ajax.phpHelper(action, data, successCallback, failureCallback, false);
  },
  /*
    Helper for submitAjaxFormHelper and php. NOTE: PHP is expected to send back
    a specially formatted JSON file as a response. Validator.php has functions
    to do this (see Validator->ajaxFormResponse).

    JSON file can have 3 attributes, which can be defined or undefined:
    1) 'success': if true, ajax has been successful. If false, it has failed.
    2) 'response': if defined, 'response' will contain string of form error data.
    3) 'argument': argument to pass to success or failure function.

    @param action PHP file to POST data to
    @param data FormData object or JS object to send up as POST data
    @param successCallback called when server responds with a JSON file
    containing a 'success' attribute that is set to true
    @param failureClalback called when server responds with a JSON file
    containing a 'success' attribute that is set to false
    @param isForm if true, data is treated as a FormData object, instead of
    a standard JS object
  */
  phpHelper: function(action, data, successCallback, failureCallback, isForm, hideErrorPopup) {
    // Set ajax flag for PHP
    if (isForm) {
      data.append('ajax', 'true');
    } else {
      data.ajax = true;
    }
    var ajaxObject = {
      type: 'POST',
      url: action,
      data: data,
      success: function(response) {
        // response expects a JSON string that has a 'success' boolean field,
        // and a 'response' field containing a string

        // If applicable, disable loading gif on fixed bottom banner
        utilAdmin.disableRaisedButtonGif(constantsAdmin['CLASS_RAISED_BUTTON_GLOBAL_GIF']);

        var responseJSON = JSON.parse(response);
        var responseJSONSuccess = responseJSON['success']; // true or false
        var responseJSONResponse = responseJSON['response']; // For messages
        var responseJSONArgument = responseJSON['argument']; // For callback

        if (responseJSONSuccess) { // If we have succeeded (TRUE)
          if (successCallback) { // If a callback is defined for this (!FALSE)
            if (responseJSONArgument) { // If we have received a param for success (not '')
              successCallback(responseJSONArgument);
            } else { // Generally, responseJSON['response'] is empty
              successCallback();
            }
          }
        } else {
          if (!hideErrorPopup) {
            // Open error popup
            popup.errorPopup(responseJSONResponse); // Response will be the form errors
          }
          if (failureCallback) {
            if (responseJSONArgument) {
              failureCallback(responseJSONArgument);
            } else {
              failureCallback();
            }
          }
        }
      }
    }

    // Modify settings if we are using FormObject as data
    // TODO What is this doing? We need it, but why?
    if (isForm) {
      ajaxObject.cache = false;
      ajaxObject.contentType = false;
      ajaxObject.processData = false;
    }

    // Ajax Call
    $.ajax(ajaxObject);
  },
  /*
    Used for mailto functions. Takes the string returned from a php endpoint and
    redirects the browser to that string.

    @param phpEndpoint The endpoint to get response url from
  */
  redirectToAjaxResponseString: function(phpEndpoint) {
    popup.loadingGif();
    ajax.ajax(
      phpEndpoint,
      function(response) {
        popup.removeLoadingGif();
        util.redirect(response);
      },
      {}
    );
  },
  getSelectOptions: function(selectId, phpEndpoint, data, removeCurrentOptions, defaultText, callback) {
    var select = $(selectId);
    if (removeCurrentOptions) {
      select.children().each(function() {
        var currentChild = $(this);
        if (currentChild.val() !== defaultText && currentChild.val()) { // Don't remove the default
          $(this).remove();
        }
      });
    }
    ajax.ajax(
      phpEndpoint,
      // Expects a array, either json_encode associative or standard.
      function(response) {
        if (callback) {
          callback();
        }
        response = JSON.parse(response);
        if (Array.isArray(response)) {
          response.forEach(function(element) {
            select2Class.appendOption(select, element, element);
          });
        } else {
          for (i in response) {
            select2Class.appendOption(select, i, response[i]);
          }
        }
      },
      data
    );
  },
  appendSuccessMessage: function() {
    var successId = '#form-success-message';

    var animateSuccessIn = function(successId) {
      $(successId).animate({bottom: "0px"}, 500);
    }
    var animateSuccessOut = function(successId) {
      $(successId).animate({bottom: "-100px"}, 500, function() {
        $(successId).remove();
      });
    }

    // if success already is there
    if ($(successId).length) {
      $(successId).remove();
      clearTimeout(form.animateSuccessOutTimeout); // Prevents premature removal of new one
    }

    sharedAjax.template(
      'admin/components/reusable/alert_form_success.tpl',
      {
        'id': util.removePound(successId),
      },
      'body',
      true,
      function() {
        animateSuccessIn(successId);

        form.animateSuccessOutTimeout = setTimeout(function() {
          animateSuccessOut(successId);
        }, 2500);
      }
    );
  }
}; // End ajax class

/*
  === MULTI SELECT CHECKBOX CLASS ===
*/
var multiSelect = {
  multiSelectArray: [],
  clickActionButton: function(actionButtonId, clickFunction, errorMessage) {
    $(actionButtonId).on('click', function() {
      var mediaSelectedCount = multiSelect.multiSelectArray.length;
      if (mediaSelectedCount > 0) {
        clickFunction();
      } else {
        popup.errorPopup(errorMessage);
      }
    });
  },
  getMultiSelectArrayAsFormDataArray: function(postArrayNameForPHP) {
    var answer = [];

    multiSelect.multiSelectArray.forEach(function(multiSelectValue) {
      item = {};
      item['key'] = postArrayNameForPHP;
      item['value'] = multiSelectValue;
      answer.push(item);
    });

    return answer;
  },
  /*
    @param multiselectCheckboxId The ID of the checkbox that will select all children
    @param selectCheckboxesClass Class of children checkboxes
  */
  initMultiselectCheckbox: function(multiselectCheckboxId, selectCheckboxesClass) {
    var multiselectCheckbox = $(multiselectCheckboxId);
    var selectCheckboxes = $(selectCheckboxesClass)

    multiselectCheckbox.on('change', function() {
      if (multiselectCheckbox.is(':checked')) {
        selectCheckboxes.prop('checked', true).trigger('change');
      } else {
        selectCheckboxes.prop('checked', false).trigger('change');
      }
    });
  },
  initSelectCheckbox: function(checkboxId, mediaSelectedCountClass, isMediaPage) {
    var checkbox = $(checkboxId);
    var mediaSelectedCountItems = $(mediaSelectedCountClass);
    var checkboxValue = checkbox.attr('value');

    checkbox.on('change', function() {
      var checkboxValueIndex = multiSelect.multiSelectArray.indexOf(checkboxValue);
      if (checkbox.is(':checked')) { // Add to array
        if (checkboxValueIndex === -1) {
          multiSelect.multiSelectArray.push(checkboxValue);
        }
      } else { // Remove from array
        if (checkboxValueIndex !== -1) {
          multiSelect.multiSelectArray.splice(checkboxValueIndex, 1);
        }
      }

      // Update count text on page
      var selectedCount = multiSelect.multiSelectArray.length;
      mediaSelectedCountItems.text(selectedCount);

      if (isMediaPage) {
        var multiEditFormWrapper = $(constantsAdmin['ID_MEDIA_EDIT_MULTI']); // Hide/show edit form when > 0 selected
        var mediaEditNoSelectionWrapper = $(constantsAdmin['ID_MEDIA_EDIT_NO_SELECTION']); // Edit message when 0 selected

        // Show edit form
        if (selectedCount > 0) {
          multiEditFormWrapper.toggle(true);
          mediaEditNoSelectionWrapper.toggle(false);
        } else {
          multiEditFormWrapper.toggle(false);
          mediaEditNoSelectionWrapper.toggle(true);
        }
      }
    });
  },
};
