<?php
require_once('common.php');
require_once('Renew' . DS . 'Utilities' . DS . 'Validator.php');
require_once('Renew' . DS . 'DiskInfo.php');
require_once('Renew' . DS . 'DeviceInfo.php');
use \Renew\Validator;
use \Renew\DiskInfo;
use \Renew\SessionHelper;
use \Renew\DeviceInfo;

// Constants used on multiple pages
define('MAX_URL_LENGTH', 32);
define('MAX_SSID_LENGTH', 32);
define('MAX_LOGO_TEXT_LENGTH', 50);
define('LOCALE_EN_US', 'en_US.UTF-8');

// Validate user
$dbManagerWifi->openDb();
  if (!$dbManagerWifi->usersCheckLogin()) {
    header('Location: ' . DIRECTORY_SEPARATOR . 'login.php');
    die('redirecting to login');
  }
  $siteLanguage = $dbManagerWifi->settingsGetValueForKey('admin_language');
$dbManagerWifi->closeDb();

// Clear smarty cache and slim cache
require_once(DOC_ROOT . '/utilities/secure_clear_cache.php');

SessionHelper::SetSiteLanguage($siteLanguage, BASE_DIR);

// Show success banner
if (isset($_POST['form_success'])) {
  $t->assign('form_success', TRUE);
}

// Error messages for admin pages
$adminErrorMessages = array(
  // General messages
  'ERR_DELETE_FAILED' => _('Delete failed'),
  'ERR_SAVE_FAILED' => _('Save failed'),
  'ERR' => _('An error occured'),
	'ERR_RENAME_FAILED' => _('Rename failed'),
  'ERR_NOT_EQUAL' => _('Not equal'),

  // mediaDbManager messages
  'ERR_TITLE_EXISTS' => _('Image with this filename already exists. Rename the image and try again.'),
  'ERR_DB_FAILED' => _('Save failed'),
  'ERR_CONVERSION_FAILED' => _('Image conversion failed'),
  'ERR_IMAGE_DNE' => _('Image does not exist'),

  // wifiDbManager messages
  'ERR_THEME_SAVE_FAILED' => _('Theme save failed'),
);
$adminErrorMessages['ERR_FILE_NAME_MAY_ALREADY_EXIST'] = $adminErrorMessages['ERR_SAVE_FAILED'] . '. ' . _('File name name may already exist') . '.';

// Used on language.php and setup_language.php, MUST have .UTF-8 for setlocale to work correctly
$adminLanguagesArray = array(
  'English' => 'en_US.UTF-8',
  'አማርኛ [Amharic]' => 'am_ET.UTF-8',
  'عربى [Arabic]' => 'ar_EG.UTF-8',
  'বাংলা [Bengali]' => 'bn_BN.UTF-8',
  '中文 [Chinese]' => 'zh_CN.UTF-8',
  'Français [French]' => 'fr_FR.UTF-8',
  'हिंदी [Hindi]' => 'hi_IN.UTF-8',
  'Português [Portuguese]' => 'pt_BR.UTF-8',
  'русский [Russian]' => 'ru_RU.UTF-8',
  'Español [Spanish]' => 'es_ES.UTF-8',
);

/**
 * Function to display a page for the admin, setting up necessary variables
 * @param string $templatePath
 */
function templateDisplayPageAdmin($templatePath, $shouldFetch = FALSE) {
  global $t, $dbManagerWifi, $serialNumber, $siteLanguage;

  $satelliteCableConnected = FALSE;
  if (file_exists('/dev/smartone')) {
    $satelliteCableConnected = TRUE;
  }

  $dbManagerWifi->openDb();
    $hashAdminSettings = $dbManagerWifi->settingsGetHashAdminAll();
    $themesCssFile = $dbManagerWifi->settingsGetValueForKey('themes_css_file');
    $logo = $dbManagerWifi->settingsGetValueForKey('general_logo');
    $helpMessageHighlight = Validator::StringToBoolean($dbManagerWifi->settingsGetValueForKey('help_messages_highlight'));
  $dbManagerWifi->closeDb();

  // Check if files are downloading
  $downloaderState = null;
  try {
    $client = new \GuzzleHttp\Client();
    $response = $client->request('GET', 'http://' . DeviceInfo::IP_LOCAL_HOST . '/n/internal-only/downloader/download');
    $downloaderState = $response->getBody();
    if ($response->getStatusCode() === 200) {
      $downloaderState = json_decode($downloaderState, TRUE);
    } else {
      $downloaderState = null;
    }
  } catch (Exception $e) {
    error_log($e);
  }
  
  $t->assign('downloader_state', $downloaderState);
  $t->assign('admin_language', $siteLanguage);
  $t->assign('satellite_cable_connected', $satelliteCableConnected);
  $t->assign('storage_free_b', DiskInfo::DiskFreeSpaceInB(MEDIA_DIR, LS_RESERVE_SIZE));
  $t->assign('text_input_character_limit', Validator::TEXT_INPUT_CHARACTER_LIMIT); // Used to restrict input in fields with JS
  $t->assign('text_field_character_limit', Validator::TEXT_FIELD_CHARACTER_LIMIT);
  $t->assign('is_dev_env', IS_DEV_ENV);
  $t->assign('logo', $logo);
  $t->assign('logo_default', '/images/defaultlogo.png');
  $t->assign('themes_css_file', $themesCssFile);
  $t->assign('serial_number', $serialNumber);
  $t->assign('help_messages_highlight', $helpMessageHighlight);
  templateAssignSettings($hashAdminSettings); // the hashes for the user css and js files
  if ($shouldFetch) {
    return $t->fetch($templatePath);
  } else {
    $t->display($templatePath);
  } 
};
