<?php

class DateHelper {

	/**
	 * Return the current DateTime as a string for use in SQLite
	 *
	 * @return a string of format "YYYY-MM-DD HH:MM:SS.SSS", and FALSE on failure
	 */
	public static function DateTimeStringSQLite3(){
		try {
			$date = new DateTime();
			$date_string = $date->format("Y-m-d H:i:s").'.'.mb_substr($date->format("u"),0,3);
			return $date_string;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public static function DateTimeStringYmdHis(){
		try {
			$date = new DateTime();
			$date_string = $date->format("YmdHis").'.'.mb_substr($date->format("u"),0,3);
			return $date_string;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * @param string $dateTime string with date, space, and time
	 * @return integer time since 1970
	 */
	public static function DateTimeStringToIntTime($dateTime) {
		return strtotime(explode(' ', $dateTime)[1]);
	}

	/**
	 *
	 * Set the system time with a UTC timestamp
	 *
	 * TODO can we remove this? It doesn't seem to be used.
	 * @param integer $timestampUTC
	 * @return TRUE on success and FALSE on failure
	 */
	public static function SetSystemTimeDateWithTimestampUTC($timestampUTC) {
		try {
			// create the new date string
			$new_date_str = self::UnixTimestampToSqliteTime($timestampUTC);
			// disable automatic synchronization to manually set the clock
			exec('sudo /usr/bin/timedatectl set-ntp 0');
			// set the timezone to utc
			exec('sudo /usr/bin/timedatectl set-local-rtc 0');
			exec('sudo /usr/bin/timedatectl set-timezone UTC');
			// set the the time and date
			$set_time_cmd = 'sudo /usr/bin/timedatectl set-time "' . $new_date_str . '"';
			exec($set_time_cmd);
			error_log('System time set: ' . $new_date_str);
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public static function UnixTimestampToSqliteTime($unixTime){
		try {
			$float = (float)$unixTime;
			$floatStr = sprintf('%0f', $float);
			$newDate = DateTime::createFromFormat("U.u", $floatStr);
			$newDate->setTimezone(new DateTimeZone('UTC'));
			$newDateStr = $newDate->format('Y-m-d H:i:s');
			return $newDateStr;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public static function SqliteTimeToUnixTimestamp($time){
		try {
			$newDate = DateTime::createFromFormat("Y-m-d H:i:s.u", $time);
			$newDate->setTimezone(new DateTimeZone('UTC'));
			$newDateStr = $newDate->format('U.u');
			return $newDateStr;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * helper function to get the elapsed time between calls
	 */
 	public static function TimeElapsed() {
		static $last = null;
		$now = microtime(true);
		if ($last != null ) {
			error_log('<<<<<' . ($now - $last) . '>>>>');
		}
		$last = $now;
	} 

}
