<?php

 namespace Renew;
 require_once('DbManager.php');

 class DbManagerSd extends DbManager {

   /**
   * Resets the data in the table
   *
   * @return Boolean TRUE on Success and FALSE on Failure
   * @access public
   * @author CV
   **/
   public function statsDeleteAll() {
     return $this->queryExecute("DELETE FROM sd_interaction_stats");
   }

	 public function statsFindAll() {
		try {
			$array = array();
			$stmt = $this->dbHandle->prepare("SELECT SUM(file_size_bytes) AS READABLE_SIZE, COUNT(interaction_id) AS NUMBER_OF_FILES, COUNT(DISTINCT  session_id) AS NUMBER_OF_EXECUTIONS FROM sd_interaction_stats");
			$results = $stmt->execute();
			$array = $results->fetchArray(SQLITE3_ASSOC);
			$array["READABLE_SIZE"] = $this->bytesToSize($array["READABLE_SIZE"]);
			return $array;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function statsGetOverTimeSessions() {
		return $this->statsGetOverTime('sessions');
	}

	public function statsGetOverTimeFiles() {
		return $this->statsGetOverTime('files');
	}

	public function statsGetOverTimeData() {
		try {
			$retArray = $this->statsGetOverTime('data');
			foreach ($retArray as $key => $value) {
				$retArray[$key] = $this->bytesToSize($value);
			}
			return $retArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	private function statsGetOverTime($interactionType) {
		try {
			$clause="status == 'success'";
			if ($interactionType == 'sessions') {
				$selectStmt = "SELECT COUNT(DISTINCT session_id)";
			}
			elseif ($interactionType == 'files') {
				$selectStmt = "SELECT COUNT(*)";
			}
			elseif ($interactionType == 'data') {
				$selectStmt = "SELECT SUM(file_size_bytes)";
			}
			else {
				return FALSE;
			}

			return $this->tableSelectOverTimeRanges('sd_interaction_stats', $selectStmt, $clause);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

    /**
     * Convert bytes to human readable format
     *
     * @param integer bytes Size in bytes to convert
     * @return string
     * @access private
     * @link http://codeaid.net/php/convert-size-in-bytes-to-a-human-readable-format-(php)
     */
    private function bytesToSize($bytes, $precision = 2) {
			try {
				$kilobyte = 1000;
				$megabyte = 1000000;
				$gigabyte = 1000000000;
				$terabyte = 1000000000000;

				if ($bytes == 0) {
					return '0 B';
				} elseif (($bytes >= 0) && ($bytes < $kilobyte)) {
					return $bytes . ' B';
				} elseif (($bytes >= $kilobyte) && ($bytes < $megabyte)) {
					return round($bytes / $kilobyte, $precision) . ' KB';
				} elseif (($bytes >= $megabyte) && ($bytes < $gigabyte)) {
					return round($bytes / $megabyte, $precision) . ' MB';
				} elseif (($bytes >= $gigabyte) && ($bytes < $terabyte)) {
					return round($bytes / $gigabyte, $precision) . ' GB';
				} elseif ($bytes >= $terabyte) {
					return round($bytes / $terabyte, $precision) . ' TB';
				} else {
					return $bytes . ' B';
				}
			} catch (Exception $e) {
				error_log($e);
				return FALSE;
			}
		}

}
