<?php
namespace Renew;
require_once('DbManager.php');
require_once('DeviceInfo.php');
require_once('SystemHelper.php');
require_once('DateHelper.php');

class DbManagerWifi extends DbManager {

	// ERRORS
	const ERR_THEME_SAVE_FAILED = 'ERR_THEME_SAVE_FAILED';

	// General constants
	const GREETING_MODE_MEDIA = 'media';
	const GREETING_MODE_TEXT = 'text';
	const GREETING_MODE_IMAGE = 'image';

	const GENERAL_LOGO_MODE_DEFAULT = 'default';
	const GENERAL_LOGO_MODE_IMAGE = 'image';
	const GENERAL_LOGO_MODE_TEXT = 'text';

	const ADMIN_DEFAULT_PASSWORD = 'admin';

	const INTERACTION_TYPE_STREAM = 'stream';
	const INTERACTION_TYPE_DOWNLOAD = 'download';
	const INTERACTION_TYPE_CONNECT = 'connect';
	const INTERACTION_TYPE_DISCONNECT = 'disconnect';
	const INTERACTION_TYPE_PORTAL = 'portal';
	const INTERACTION_TYPE_PORTALOK = 'portal-ok';
	const INTERACTION_TYPE_HOMEPAGE = 'homepage';
	const INTERACTION_TYPE_FORM = 'form';

	// Form constants
	const FOLLOWUP_FORM_ID = 0;

	// Theme constants
	const THEME_EFFECT_BLUR = 'BLUR';
	const THEME_EFFECT_TEXT_SHADOW = 'TEXT_SHADOW';
	const THEME_EFFECT_BACKGROUND_IMAGE = 'THEME_EFFECT_BACKGROUND_IMAGE';
	const THEME_EFFECT_REMOVE_TEXT_SHADOW = 'THEME_EFFECT_REMOVE_TEXT_SHADOW';
	const THEME_EFFECT_BACKGROUND_NONE = 'THEME_EFFECT_BACKGROUND_NONE';

	// CSS snippets
	const CSS_IMPORTANT = '!important';

	const THEME_TYPE_BACKGROUND = 'background';
	const THEME_TYPE_IMAGE = 'image';
	const THEME_TYPE_COLOR = 'color';
	const THEME_BG_IMAGES_PATH = '/images/user_images/bg/';
	const THEME_CSS_PREFIX = '.user-theme-';
	const THEME_LUMINANCE_THRESHOLD = 0.6; // Backgrounds darker than this will use color_dark text

	// Theme helper css (used by front-end in css, not in database)
	const THEME_CSS_BACKGROUND_NONE = 'width: inherit !important; height: inherit !important; background: none !important; background-color: none !important;';

	// . self::THEME_CSS_PREFIX . 'background-none::before {content: none;}';
	const THEME_CSS_BACKGROUND_NONE_CLASS = self::THEME_CSS_PREFIX . 'background-none {' . self::THEME_CSS_BACKGROUND_NONE . '}';

	// Theme keys
	const THEME_BACKGROUND_NAV_HEADER = 'background_nav_header';
	const THEME_BACKGROUND_MEDIA_ICON = 'background_media_icon';
	const THEME_BACKGROUND_BUTTON_1 = 'background_button_1';
	const THEME_BACKGROUND_BUTTON_1_COLOR = 'background_button_1_color';
	const THEME_BACKGROUND_MAIN = 'background_main';
	const THEME_BACKGROUND_SECONDARY = 'background_secondary';
	const THEME_BACKGROUND_GREETING = 'background_greeting';
	const THEME_COLOR_LIGHT = 'color_light';
	const THEME_COLOR_DARK = 'color_dark';

	// Theme effects array
	const EFFECT_CSS = array(
		'' => '',
		// User-defined effects
		self::THEME_EFFECT_BLUR => 'filter: blur(25px); width: 120%; height: 120%; left: -40px; top: -40px;',
		// General effects
		self::THEME_EFFECT_TEXT_SHADOW => 'text-shadow: 0.6px 0.6px 1.5px rgba(0,0,0,1);',
		self::THEME_EFFECT_BACKGROUND_IMAGE => 'background-size: cover; background-position: center;', //width: 100%; height: 100%; position: absolute; content: "";
		self::THEME_EFFECT_REMOVE_TEXT_SHADOW => 'text-shadow: none;',
	);

	// Theme presets
	const THEME_PRESET_LIGHT = array(
		array(
			'key' => self::THEME_BACKGROUND_NAV_HEADER,
			'value' => '#939391',
			'type' => self::THEME_TYPE_BACKGROUND,
			'effect' => ''
		),
		array(
			'key' => self::THEME_BACKGROUND_MEDIA_ICON,
			'value' => '#ffffff',
			'type' => self::THEME_TYPE_BACKGROUND,
			'effect' => ''
		),
		array(
			'key' => self::THEME_BACKGROUND_BUTTON_1,
			'value' => '#4d8777',
			'type' => self::THEME_TYPE_BACKGROUND,
			'effect' => ''
		),
		array(
			'key' => self::THEME_BACKGROUND_MAIN,
			'value' => '#cdcdcd',
			'type' => self::THEME_TYPE_BACKGROUND,
			'effect' => ''
		),
		array(
			'key' => self::THEME_BACKGROUND_SECONDARY,
			'value' => '#eeeeee',
			'type' => self::THEME_TYPE_BACKGROUND,
			'effect' => ''
		),
		array(
			'key' => self::THEME_BACKGROUND_GREETING,
			'value' => '#cdcdcd',
			'type' => self::THEME_TYPE_BACKGROUND,
			'effect' => ''
		),
		array(
			'key' => self::THEME_COLOR_LIGHT,
			'value' => '#eeeeee',
			'type' => self::THEME_TYPE_COLOR,
			'effect' => ''
		),
		array(
			'key' => self::THEME_COLOR_DARK,
			'value' => '#49494a',
			'type' => self::THEME_TYPE_COLOR,
			'effect' => ''
		)
	);
	const THEME_PRESET_DARK = array(
		array(
			'key' => self::THEME_BACKGROUND_NAV_HEADER,
			'value' => '#212121',
			'type' => self::THEME_TYPE_BACKGROUND,
			'effect' => ''
		),
		array(
			'key' => self::THEME_BACKGROUND_MEDIA_ICON,
			'value' => '#fcfcfc',
			'type' => self::THEME_TYPE_BACKGROUND,
			'effect' => ''
		),
		array(
			'key' => self::THEME_BACKGROUND_BUTTON_1,
			'value' => '#4d8777',
			'type' => self::THEME_TYPE_BACKGROUND,
			'effect' => ''
		),
		array(
			'key' => self::THEME_BACKGROUND_MAIN,
			'value' => '#363636',
			'type' => self::THEME_TYPE_BACKGROUND,
			'effect' => ''
		),
		array(
			'key' => self::THEME_BACKGROUND_SECONDARY,
			'value' => '#8b8b8b',
			'type' => self::THEME_TYPE_BACKGROUND,
			'effect' => ''
		),
		array(
			'key' => self::THEME_BACKGROUND_GREETING,
			'value' => '#363636',
			'type' => self::THEME_TYPE_BACKGROUND,
			'effect' => ''
		),
		array(
			'key' => self::THEME_COLOR_LIGHT,
			'value' => '#fcfcfc',
			'type' => self::THEME_TYPE_COLOR,
			'effect' => ''
		),
		array(
			'key' => self::THEME_COLOR_DARK,
			'value' => '#1b1b1b',
			'type' => self::THEME_TYPE_COLOR,
			'effect' => ''
		)
	);

	/**
	* Sums up the total of all activity
	*
	* @param string $recordType the type of record (downloads, streams)
	* @return integer the total number of requested $recordType
	* @access public
	* @author JPul
	**/
	public function statsGetSumTotal($recordType) {
		try {
			$stmt = $this->dbHandle->prepare("SELECT COUNT(interaction_id) AS total FROM wifi_interaction_stats WHERE interaction_type=:interaction_type{$this->dateTimeBetweenAndClause}");
			$stmt->bindValue(":interaction_type", $recordType);
			$results = $stmt->execute();
			$row = $results->fetchArray();
			$total = $row['total'];
			if (is_null($total)) {
				$total = 0;
			}
			return $total;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function statsGetOverTimeStreams() {
		return $this->statsGetOverTime('streams');
	}

	public function statsGetOverTimeDownloads() {
		return $this->statsGetOverTime('downloads');
	}

	public function statsGetOverTimeUniqueDevices() {
		return $this->statsGetOverTime('devices');
	}

	private function statsGetOverTime($interactionType) {
		try {
			$selectStmt = "SELECT COUNT(*)";
			$clause="";
			if ($interactionType == 'devices') {
				$selectStmt = "SELECT COUNT(DISTINCT device_id)";
			}
			elseif ($interactionType == 'streams') {
				$clause="interaction_type == 'stream'";
			}
			elseif ($interactionType == 'downloads') {
				$clause="interaction_type == 'download'";
			}
			else {
				return FALSE;
			}
			return $this->tableSelectOverTimeRanges('wifi_interaction_stats', $selectStmt, $clause);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}
	/**
	*
	*
	* @return array an array of associative arrays with keys "total_streams", "total_downloads", "year_month"
	*/
	public function statsGetStreamsAndDownloadsByMonth(){
		try {
			$stmt = $this->dbHandle->prepare("SELECT SUM(CASE WHEN interaction_type='stream' THEN 1 ELSE 0 END) AS 'total_streams',
										SUM(CASE WHEN interaction_type='download' THEN 1 ELSE 0 END) AS 'total_downloads',
										strftime('%Y-%m', date_time) AS 'year_month'
										FROM wifi_interaction_stats{$this->dateTimeBetweenWhereClause} GROUP BY strftime('%Y-%m', date_time)
										ORDER BY 'year_month' ASC");
			return $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	*
	* @param integer $limit the limit of the number of top files it should select
	* @return array
	*/
	public function statsGetStreamsAndDownloadsByFileid($limit = NULL, $returnAssociative = FALSE){
		try {
			$stmt_str = "SELECT COUNT(interaction_id) as 'total_interactions',
						SUM(CASE WHEN interaction_type='stream' THEN 1 ELSE 0 END) AS 'total_streams',
						SUM(CASE WHEN interaction_type='download' THEN 1 ELSE 0 END) AS 'total_downloads',
						*
						FROM wifi_interaction_stats WHERE interaction_type IN ('stream', 'download')
						GROUP BY file_id ORDER BY total_interactions DESC";
			if ($limit !== NULL){
				$stmt_str .= " LIMIT ".$limit;
			}
			$stmt = $this->dbHandle->prepare($stmt_str);
			$returnVal = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			if ($returnAssociative == TRUE){
				$returnAssocArr = array();
				if ($limit !== NULL){
					for ($i=0; $i < $limit; $i++) {
						$returnAssocArr["file-$i"] = array();
					}
				}
				foreach ($returnVal as $key => $value) {
					$returnAssocArr["file-$key"] = $value;
				}
				return $returnAssocArr;
			}
			return $returnVal;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function statsGetRecentActivity($limit = NULL, $returnAssociative = FALSE, $idGreaterThan = 0, $deviceId = NULL, $allowedInteractionTypes = array(self::INTERACTION_TYPE_STREAM, self::INTERACTION_TYPE_DOWNLOAD, self::INTERACTION_TYPE_CONNECT, self::INTERACTION_TYPE_DISCONNECT, self::INTERACTION_TYPE_HOMEPAGE, self::INTERACTION_TYPE_FORM)) {
		try {
			$interactionTypeStr = "('" . implode("', '", $allowedInteractionTypes) . "')";

			$deviceIdAndClause = '';
			if ($deviceId !== NULL && is_string($deviceId) && strlen($deviceId) > 0) {
				$deviceIdAndClause = "AND wifi_interaction_stats.device_id = '{$deviceId}' ";
			}
			// order by interaction_id instead of by date_time because it's more reliable
			$stmt_str = "SELECT * FROM wifi_interaction_stats JOIN devices ON wifi_interaction_stats.device_id = devices.device_id WHERE interaction_id > :interaction_id AND interaction_type IN {$interactionTypeStr} {$deviceIdAndClause}ORDER BY interaction_id DESC";
			if ($limit !== NULL){
				$stmt_str .= " LIMIT ".$limit;
			};

			// error_log('STMT STRING');
			// error_log($stmt_str);

			$stmt = $this->dbHandle->prepare($stmt_str);
			$stmt->bindValue(':interaction_id', $idGreaterThan);
			$returnVal = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			if ($returnAssociative == TRUE){
				$returnAssocArr = array();
				if ($limit !== NULL){
					for ($i=0; $i < $limit; $i++) {
						$returnAssocArr["interaction-$i"] = array();
					}
				}
				foreach ($returnVal as $key => $value) {
					$returnAssocArr["interaction-$key"] = $value;
				}
				return $returnAssocArr;
			}
			return $returnVal;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Find all records from the database
	*
	* @return array of the files, and FALSE on failure
	* @access public
	* @author CV
	**/
	public function statsGetAll($limit = 0) {
		try {
			$stmtStr = "SELECT * FROM wifi_interaction_stats{$this->dateTimeBetweenWhereClause} ORDER BY interaction_id DESC";
			if ( $limit > 0 ){
				$stmtStr =$stmtStr . " LIMIT {$limit}";
			}
			$stmt = $this->dbHandle->prepare($stmtStr);
			return $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function statsGetLastInteractionFromDeviceWithId($deviceId) {
		try {
			// use interaction_id instead of date_time because until the first phone hits javascript/php, the date time may be unreliable
			$stmtStr = "SELECT * FROM wifi_interaction_stats WHERE device_id = :device_id ORDER BY interaction_id DESC LIMIT 1";
			$stmt = $this->dbHandle->prepare($stmtStr);
			$stmt->bindValue(':device_id', $deviceId);
			return $this->executeSqlStmtRetFirstAssociativeArrayIfExists($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Resets the data in the wifi_interaction_stats table
	*
	* @return Boolean TRUE on Success and FALSE on Failure
	* @access public
	* @author CV
	**/
	public function statsDeleteAll() {
		return $this->queryExecute("DELETE FROM wifi_interaction_stats");
	}

	/**
	* Insert a row into the wifi_interaction_stats table
	*
	* @param string $file_id the key that coresponds to the 'media' table's 'file_id' column in the media.sqlite3 database
	* @param string $path_rel the location of the "parent directory", relative to the "master" directory, of the file or directory
	* @param string $file_name the name of the file without any extension
	* @param string $file_extension the extension of the file, if it has one, and '' if it does not (NOT NULL)
	* @param string $date_time the timestamp for the interaction in ISO8601 format "YYYY-MM-DD HH:MM:SS.SSS"
	* @param string $interaction_type
	* @return TRUE on success and FALSE on failure
	*/
	public function statsInsert($interactionType, $deviceId, $dateTime = NULL, $formId = NULL, $fileId = NULL, $pathRel = NULL, $fileName = NULL, $fileExtension = NULL) {
		if ($dateTime === NULL) {
			$dateTime = \DateHelper::DateTimeStringSQLite3();
		}
		// Prevent accidental back to back double counts of downloads
		if ($interactionType === self::INTERACTION_TYPE_DOWNLOAD) {
			$lastInteractionArray = $this->statsGetRecentActivity(1, FALSE, 0, $deviceId);
			if (count($lastInteractionArray) && $lastInteractionArray[0]) {
				$lastInteraction = $lastInteractionArray[0];
				$timeDifferenceIsClose = TRUE;
				if ($dateTime && $lastInteraction['date_time']) {
					$timeLastInteraction = \DateHelper::DateTimeStringToIntTime($lastInteraction['date_time']);
					$timeCurrentDownload = \DateHelper::DateTimeStringToIntTime($dateTime);
					$timeDifferenceSeconds = abs($timeCurrentDownload - $timeLastInteraction);
					$timeDifferenceIsClose = $timeDifferenceSeconds <= 5; // Less than 5s between stat inserts
				}
				if ($lastInteraction['file_id'] === $fileId && $lastInteraction['interaction_type'] === self::INTERACTION_TYPE_DOWNLOAD && $timeDifferenceIsClose) {
					return FALSE;
				}
			}
		}
		try {
			$attrs = array();
			$attrs['file_id'] = $fileId;
			$attrs['path_rel'] = $pathRel;
			$attrs['file_name'] = $fileName;
			$attrs['file_extension'] = $fileExtension;
			$attrs['date_time'] = $dateTime;
			$attrs['interaction_type'] = $interactionType;
			$attrs['device_id'] = $deviceId;
			if ($formId !== NULL) {
				$attrs['form_id'] = $formId;
			}
			return $this->insertOrIgnoreIntoTableAttributes('wifi_interaction_stats', $attrs);
		} catch (\Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Get all of the settings that start with "greeting_"
	* Puts them in an associative array of associative arrays where the index is the key
	* @return Associative array on success and FALSE on failure
	**/
	public function settingsGetGreetingAll($returnSimpleArray = FALSE) {
		return $this->settingsGetAllLikeHelper('greeting_%', $returnSimpleArray);
	}

	public function settingsGetFormAll($returnSimpleArray = FALSE) {
		return $this->settingsGetAllLikeHelper('form_%', $returnSimpleArray);
	}

	public function settingsGetChatAll($returnSimpleArray = FALSE) {
		return $this->settingsGetAllLikeHelper('chat_%', $returnSimpleArray);
	}

	public function settingsGetHashAdminAll($returnSimpleArray = FALSE) {
		return $this->settingsGetAllLikeHelper('hash_admin_%', $returnSimpleArray);
	}

	/**
	* Get all of the settings that start with "featured_"
	* Puts them in an associative array of associative arrays where the index is the key
	* @return Associative array on success and FALSE on failure
	**/
	public function settingsGetFeaturedAll($returnSimpleArray = FALSE) {
		return $this->settingsGetAllLikeHelper('featured_%', $returnSimpleArray);
	}

	/**
	* Get all of the settings that start with "general_"
	* Puts them in an associative array of associative arrays where the index is the key
	* @return Associative array on success and FALSE on failure
	* @author JP
	**/
	public function settingsGetGeneralAll($returnSimpleArray = FALSE) {
		return $this->settingsGetAllLikeHelper('general_%', $returnSimpleArray);
	}

	/**
	* Get all of the settings that start with a certain string
	* Puts them in an associative array of associative arrays where the index is the key
	* @param $likeString The string that the key is like
	* @return Associative array on success and FALSE on failure
	* @author JP
	**/
	public function settingsGetAllLikeHelper($likeString, $returnSimpleArray) {
		$stmt = $this->dbHandle->prepare("SELECT * FROM settings WHERE key LIKE '{$likeString}'");

		$valueCol = NULL;
		if ($returnSimpleArray) {
			$valueCol = 'value';
		}
		return $this->executeSqlStmtRetArrayOfAssocArrays($stmt, 'key', $valueCol);
	}

	/**
	* Get all of the settings that start with a certain string
	* Puts them in an associative array of associative arrays where the index is the key
	* @param $imagePath the path to the image to set as the logo
	* @return Associative array on success and FALSE on failure
	**/
	public function settingsSetGeneralLogo($imagePath, $imagesDir) {
		try {
			// check if the image exists
			if (!file_exists($imagePath)){
				error_log("the image attempted to be set as the general image:{$imagePath} does not exist");
				return FALSE;
			}

			if (!file_exists($imagesDir)) {
				mkdir($imagesDir, 0777, TRUE);
			}

			// set open permissions for the file
			chmod("{$imagePath}", 0777);

			// TODO DEV ENVIRONMENT - This putenv is so we can call gm scripts.
			$cmd = "\"rwo_convertLogoImage.sh\" " . escapeshellarg($imagePath) . " 2>&1";
			SystemHelper::RunCmd($cmd, FALSE);
			// the script outputs the file at the location below
			$imagePathLogo = "{$imagePath}.logo.png";
			// remove the original file
			if (file_exists($imagePath)) unlink($imagePath);

			if (!file_exists($imagePathLogo)){
				if (file_exists($imagePathLogo)) unlink($imagePathLogo);
				error_log("conversion of file:{$imagePath} failed to set as logo");
				return FALSE;
			}

			// check the md5 of the image to make sure one doesn't already exist
			$imageMd5 = md5_file($imagePathLogo);
			// delete the old image
			$this->settingsRemoveCurrentGeneralLogoThrowsExceptions($imagesDir);

			$generalLogoPng = $this->settingsGetValueForKey('general_logo');
			$generalLogoPath = "{$imagesDir}/{$generalLogoPng}";
			if (strlen($generalLogoPng) > 0 && file_exists("{$generalLogoPath}")) {
				unlink("{$generalLogoPath}");
			}

			$imageFileNameLogoNew = "{$imageMd5}.png";
			$imagePathLogoNew = "{$imagesDir}/{$imageFileNameLogoNew}";
			rename($imagePathLogo, $imagePathLogoNew);

			if (!file_exists($imagePathLogoNew)) {
				unlink($imagePathLogo);
				error_log("moving file failed:{$imagePathLogo} to {$imagePathLogoNew} failed to set as logo");
				return FALSE;
			}

			$success = $this->settingsSetKeyValue('general_logo', $imageFileNameLogoNew);

			if ($success === FALSE) {
				unlink($imagePathLogoNew);
				error_log("setting the value in the database failed while trying to set the general_logo!");
				return FALSE;
			}

			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	private function settingsRemoveCurrentGeneralLogoThrowsExceptions($imagesDir) {
		// delete the old image
		$generalLogoPng = $this->settingsGetValueForKey('general_logo');
		$generalLogoPath = "{$imagesDir}/{$generalLogoPng}";
		if (strlen($generalLogoPng) > 0 && file_exists("{$generalLogoPath}")) {
			unlink("{$generalLogoPath}");
		}
	}

	public function settingsRemoveCurrentGeneralLogo($imagesDir) {
		try {
			$this->settingsRemoveCurrentGeneralLogoThrowsExceptions($imagesDir);
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Get all of the information from the themes table
	* Puts them in an associative array of associative arrays where the index is the key
	* @return Associative array on success and FALSE on failure
	**/
	public function themesGetAll(){
		$stmt = $this->dbHandle->prepare("SELECT * FROM themes");
		return $this->executeSqlStmtRetArrayOfAssocArrays($stmt, "key");
	}

	/**
	* Get the keys (names) of all of the themes
	* @author JP
	* @return Array on success and FALSE on failure
	*/
	public function themesGetAllKeys(){
		$stmt = $this->dbHandle->prepare("SELECT key FROM themes");
		return array_keys($this->executeSqlStmtRetArrayOfAssocArrays($stmt, "key"));
	}

	/**
	* Gets all of the themes, formats them, and writes them to a CSS file
	* @author JP
	* @param $filePath The file path of the CSS file
	* @return boolean TRUE on success, and, FALSE on failure
	*/
	// TODO run this function once in a setup place, like config, to take DB and make initial file?
	public function themesWriteCSSFile($fileDir) {
		$themes = $this->themesGetAll();
		$themeCSS = '';

		$colorDark = $themes[self::THEME_COLOR_DARK]['value'];
		$colorLight = $themes[self::THEME_COLOR_LIGHT]['value'];

		// Add color to match background button 1 background color
		// TODO move this into helper function
		$themeBackgroundButton1Value = $themes[self::THEME_BACKGROUND_BUTTON_1]['value'];
		$themes[self::THEME_BACKGROUND_BUTTON_1_COLOR] = array(
			'key' => self::THEME_BACKGROUND_BUTTON_1_COLOR,
			'value' => $themeBackgroundButton1Value,
			'type' => self::THEME_TYPE_COLOR,
			'effect' => ''
		);

		foreach ($themes as $theme) {
			// CSS uses - rather than _ as a convention, so replace underscores
			$key = self::THEME_CSS_PREFIX . str_replace('_', '-', $theme['key']);
			$value = $theme['value'];
			$type = $theme['type'];
			$effect = $theme['effect'];
			$valueCSS = '';

			// Set CSS string
			switch($type) {
				case self::THEME_TYPE_BACKGROUND:
					$color = $colorDark;
					if (self::themesColorIsLight($value)) {
						$color = $colorLight;
					}
					$valueCSS = 'background-color:'
					. $value . self::CSS_IMPORTANT . ';'
					. 'color:' . $color . self::CSS_IMPORTANT . ';'
					. self::EFFECT_CSS[self::THEME_EFFECT_REMOVE_TEXT_SHADOW];
				break;
				case self::THEME_TYPE_IMAGE:
					$valueCSS = 'background-image: url(\''
					. self::THEME_BG_IMAGES_PATH
					. $value . '.jpeg\')' . self::CSS_IMPORTANT . ';'
					. self::EFFECT_CSS[self::THEME_EFFECT_BACKGROUND_IMAGE];
				break;
				case self::THEME_TYPE_COLOR:
					$valueCSS = 'color:' . $value . self::CSS_IMPORTANT . ';';
				break;
				default:
					error_log('Error: Invalid theme type');
					return FALSE;
			}

			// Add effects to css
			$valueCSS .= self::EFFECT_CSS[$effect];
			//$beforeString = '';
			if ($type == self::THEME_TYPE_IMAGE) {
				// Before tag allows background effects to not apply to children
				//$beforeString = '::before';
				$themeCSS .=
				$key . '{'
				. 'color:' . $colorLight . self::CSS_IMPORTANT . ';'
				. self::EFFECT_CSS[self::THEME_EFFECT_TEXT_SHADOW]
				. '}';
			}

			// Create the css
			$themeCSS .=
			$key //. $beforeString
			. '{'
			. $valueCSS
			. '}';
		}

		// Add helper css classes
		$themeCSS .= self::THEME_CSS_BACKGROUND_NONE_CLASS;
		$hash = md5($themeCSS); // take the md5 of the contents of the file
		$fileNamePrefix="application-user-themes-custom-";
		$fileName = $fileNamePrefix . $hash . '.css';
		$filePath = $fileDir . DIRECTORY_SEPARATOR . $fileName;

		// delete any lingering custom user themes before you create the new one
		foreach (glob("$fileDir" . DIRECTORY_SEPARATOR . "{$fileNamePrefix}*.css") as $thisFilePath) {
			unlink($thisFilePath);
		}

		// Write to the file, returns FALSE on failure
		$success = file_put_contents($filePath, $themeCSS);
		if ($success === FALSE) {
			return FALSE;
		}
		// save the setting to the database
		$this->settingsSetKeyValue('themes_css_file', $fileName);
		return TRUE;
	}

	/**
	* For a themes table that has a key column and a value column
	* @param $key String represents the key in a settings database
	* @param $value String represents the value associated with that key
	* @return TRUE on success, and 'ERR_THEME_SAVE_FAILED' on failure
	*/
	public function themesSet($key, $value, $type, $effect) {
		try{
			$attrs = array();
			$attrs['key'] = $key;
			$attrs['value'] = $value;
			$attrs['type'] = $type;
			$attrs['effect'] = $effect;
			return $this->insertOrRepalceIntoTableAttributes('themes', $attrs);
		} catch (Exception $e){
			error_log($e);
			return ERR_THEME_SAVE_FAILED;
		}
	}

	/**
	* TODO Comment
	*/
	public function themesSetAllWithArray($themesArray) {
		$success = TRUE;
		foreach ($themesArray as $theme) {
			$currentSuccess = $this->themesSet($theme['key'], $theme['value'], $theme['type'], $theme['effect']);
			if ($currentSuccess === FALSE) {
				$success = FALSE;
			}
		}
		return $success;
	}

	private static function themesColorIsLight($hexString) {
		$luminance = self::themesCalculateLuminance($hexString);
		if ($luminance > self::THEME_LUMINANCE_THRESHOLD) {
			return FALSE;
		}
		return TRUE;
	}

	/**
	* Returns the luminance of a hexidecimal string, between 0 and 1
	*	@author JP
	* @param hexString a 6-digit hexidecimal string, which can contain #
	*/
	private static function themesCalculateLuminance($hexString) {
		$hexString = str_replace('#', '', $hexString); // Remove pound
		if (strlen($hexString) !== 6) {
			return FALSE;
		}
		$r = hexdec(substr($hexString, 0, 2));
		$g = hexdec(substr($hexString, 2, 2));
		$b = hexdec(substr($hexString, 4, 2));

		// Luminance equation
		$answer = ($r*0.299 + $g*0.587 + $b*0.114) / 256.0;
		return $answer;
	}

	// creates a form object and returns TRUE on success and FALSE on failure
	public function formsCreateForm($title, $imageId, $description){
		try {
			$stmt = $this->dbHandle->prepare( "INSERT OR IGNORE INTO forms (title, image_id, description, is_deleted) VALUES (:title, :image_id, :description, :is_deleted)");
			$stmt->bindValue(':title', $title);
			$stmt->bindValue(':image_id', $imageId);
			$stmt->bindValue(':description', $description);
			$stmt->bindValue(':is_deleted', 0);
			return $this->executeSqlStmt($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	// creates a form object and returns TRUE on success and FALSE on failure
	public function formsDeleteFormWithId($formId) {
		try {
			$stmt = $this->dbHandle->prepare( "UPDATE forms SET is_deleted = 1 WHERE form_id = :form_id)");
			$stmt->bindValue(':form_id', $formId);
			return $this->executeSqlStmt($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function formsUpdateAttrsForFormWithId($formId, $attrs) {
		try {
			return $this->updateTableAttributesForWithColnameAndColvalue('forms', 'form_id', $formId, $attrs);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function formsGetForms() {
		try {
			$stmt = $this->dbHandle->prepare('SELECT * FROM forms ORDER BY form_id DESC');
			$array = $this->executeSqlStmtRetArrayOfAssocArrays($stmt, "form_id");
			return $array;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function formsGetFormWithId($formId){
		try {
			$stmt = $this->dbHandle->prepare('SELECT * FROM forms WHERE form_id = :form_id');
			$stmt->bindValue(':form_id', $formId);
			return $this->executeSqlStmtRetFirstAssociativeArrayIfExists($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function formFieldsUpdateOrInsertFieldsForForm($formId, $newFieldsArray) {
		try {
			$formId = intval($formId);
			// set all of the fields for this form to deleted and we can undelete them
			$stmt = $this->dbHandle->prepare('UPDATE form_fields SET is_deleted = 1, sort_order = NULL WHERE form_id = :form_id');
			$stmt->bindValue(':form_id', $formId);
			$this->executeSqlStmt($stmt);
			// get the fields for the form (including deleted ones)
			$stmt = $this->dbHandle->prepare('SELECT * FROM form_fields WHERE form_id = :form_id');
			$stmt->bindValue(':form_id', $formId);
			$oldFieldsArray = $this->executeSqlStmtRetArrayOfAssocArrays($stmt, 'title');
			// iterate through the fields
			$sortOrder = 0;
			foreach ($newFieldsArray as $newField) {
				// increment the sort order
				$sortOrder += 1;
				$newField['sort_order'] = $sortOrder;
				$newField['form_id'] = $formId;
				$newField['is_deleted'] = 0;
				// check if a field by the same title already exists
				if (array_key_exists($newField['title'], $oldFieldsArray)) {
					$fieldToUpdate = $oldFieldsArray[$newField['title']];
					$fieldIdToUpdate = $fieldToUpdate['field_id'];
					$newField['field_id'] = $fieldIdToUpdate;
					$this->formFieldsUpdateAttrsForFieldWithId($fieldIdToUpdate, $newField);
				}
				else {
					$this->formFieldsCreateField($newField);
				}
			}
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function formFieldsUpdateAttrsForFieldWithId($fieldId, $attrs){
		try {
			unset($attrs['field_id']); // should never have a field_id
			return $this->updateTableAttributesForWithColnameAndColvalue('form_fields', 'field_id', $fieldId, $attrs);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	// creates a form object and returns TRUE on success and FALSE on failure
	public function formFieldsCreateField($fieldAttrs) {
		try {
			unset($fieldAttrs['field_id']); // should never have a field_id
			return $this->insertOrIgnoreIntoTableAttributes('form_fields', $fieldAttrs);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function formFieldsDeleteFieldWithId($fieldId){
		try {
			$this->_formFieldsCreateSortOrderTriggers();
			$stmt = $this->dbHandle->prepare( "UPDATE form_fields SET is_deleted = 1 WHERE field_id = :field_id)");
			$stmt->bindValue(':field_id', $fieldId);
			return $this->executeSqlStmt($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function formFieldsGetFieldsWithFormId($formId){
		try {
			$stmt = $this->dbHandle->prepare('SELECT * FROM form_fields WHERE form_id = :form_id AND is_deleted = 0 ORDER BY sort_order ASC');
			$stmt->bindValue(':form_id', $formId);
			$array = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			return $array;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}


	/**
	* submit a form respomse
	* @param $deviceId string that is the device_id of the submitting device
	* @param $formId integer the id of the form being submitted
	* @param $respomseData an array of associative arrays. Each associative array should have 'field_id' => (Integer), 'data' => (String)
	* @return integer which is the new response_id on success and FALSE on failure
	**/
	public function formResponsesInsertResponseAndData($deviceId, $formId, $responseData){
		try {
			$dateFormatString = self::SQLITE_DATE_FORMAT_STRING;
			$stmt = $this->dbHandle->prepare("INSERT OR IGNORE INTO form_responses (device_id, form_id, date_time) VALUES (:device_id, :form_id, STRFTIME('{$dateFormatString}', 'NOW'))");
			$stmt->bindValue(':device_id', $deviceId);
			$stmt->bindValue(':form_id', $formId);
			if ($this->executeSqlStmt($stmt) === FALSE) {
				return FALSE; // FAIL
			}
			$responseId = $this->dbHandle->lastInsertRowID();
			if ($responseId === FALSE) {
				return FALSE; // FAIL
			}
			foreach ($responseData as $key => $value) {
				$value['response_id'] = $responseId;
				if( $this->insertOrIgnoreIntoTableAttributes('form_data', $value) === FALSE){
					return FALSE;
				}
			}
			return $responseId;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	*
	* @param $responseId integer
	* @return an associative array thta has 2 keys: 'response' => (associative array of response), 'data' => (associative array of the form data including the title)
	**/
	public function formResponsesGetResponseAndDataWithId($responseId){
		try {
			$retArray = array();
			// get all of the fields and their names
			$stmt = $this->dbHandle->prepare("SELECT * FROM form_data JOIN form_fields ON form_fields.field_id = form_data.field_id WHERE form_data.response_id = :response_id ORDER BY form_fields.sort_order IS NULL, form_fields.sort_order ASC");
			$stmt->bindValue(':response_id', $responseId);
			$data = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			// get all of the fields and their names
			$stmt = $this->dbHandle->prepare("SELECT * FROM form_responses WHERE response_id = :response_id");
			$stmt->bindValue(':response_id', $responseId);
			$response = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			$retArray['response'] = $response;
			$retArray['data'] = $data;
			return $retArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function formResponsesDeleteAll() {
		try {
			$stmt = $this->dbHandle->prepare("DELETE FROM form_responses");
			$this->executeSqlStmt($stmt);
			$stmt = $this->dbHandle->prepare("DELETE FROM form_data");
			$this->executeSqlStmt($stmt);
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function formResponsesDeleteResponsesAndDataWithIdsArray($responseIdsArray) {
		try {
			foreach ($responseIdsArray as $responseId) {
				$stmt = $this->dbHandle->prepare("DELETE FROM form_responses WHERE response_id = :response_id");
				$stmt->bindValue(':response_id', $responseId);
				$this->executeSqlStmt($stmt);
				$stmt = $this->dbHandle->prepare("DELETE FROM form_data WHERE response_id = :response_id");
				$stmt->bindValue(':response_id', $responseId);
				$this->executeSqlStmt($stmt);
			}
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/*

	*/
	public function formResponsesGetCount($formId) {
		$stmt = $this->dbHandle->prepare("SELECT * FROM form_responses WHERE form_id = :form_id");
		$stmt->bindValue(':form_id', $formId);
		$formResponses = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
		return count($formResponses);
	}

	/**
	*
	* @param $formId integer form_id that you want to get the data for
	* @return array of arrays The first array contains all of the 'headers' for the table of responses, and each array below that contains the values for each response
	**/
	public function formResponsesGetFormattedResponsesForFormId($formId, $tableFormat = TRUE, $offset = NULL, $limit = NULL, $transDeviceId = 'device_id', $transResponseId = 'response_id', $transDateTime = 'date_time', $transDeviceNum = 'device_num') {
		try {
			$stmtStr = "SELECT * FROM form_responses JOIN devices ON form_responses.device_id = devices.device_id WHERE form_id = :form_id ORDER BY date_time DESC";
			$hasOffset = (isset($offset) && isset($limit));
			if ($hasOffset) {
				$stmtStr .= " LIMIT :lim OFFSET :off";
			}
			$stmt = $this->dbHandle->prepare($stmtStr);
			if ($hasOffset) {
				$stmt->bindValue(':lim', $limit);
				$stmt->bindValue(':off', $offset);
			}
			$stmt->bindValue(':form_id', $formId);
			$responses = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			if ($responses === FALSE) {
				return FALSE; // FAIL
			}
			// get all of the fields and their names
			$stmt = $this->dbHandle->prepare("SELECT * FROM form_fields JOIN form_data ON form_fields.field_id = form_data.field_id WHERE form_data.response_id NOT NULL AND form_fields.form_id = :form_id GROUP BY form_fields.field_id ORDER BY form_fields.sort_order IS NULL, form_fields.sort_order, form_fields.field_id ASC");
			$stmt->bindValue(':form_id', $formId);
			$fieldsWithResponses = $this->executeSqlStmtRetArrayOfAssocArrays($stmt, 'field_id');
			if ($fieldsWithResponses === FALSE) {
				return FALSE; // FAIL
			}
			$retArray = array();
			$headerArray = array();
			$idsArray = array();
			$templateArray = array();

			$headerArray[] = $transResponseId;
			$idsArray[] = 'response_id';
			$templateArray[] = '';

			$headerArray[] = $transDateTime;
			$idsArray[] = 'date_time';
			$templateArray[] = '';

			$headerArray[] = $transDeviceId;
			$idsArray[] = 'device_id';
			$templateArray[] = '';

			$headerArray[] = $transDeviceNum;
			$idsArray[] = 'device_num';
			$templateArray[] = '';

			foreach ($fieldsWithResponses as $fieldId => $fieldData) {
				$headerArray[] = $fieldData['title'];
				$idsArray[] = $fieldId;
				$templateArray[] = '';
			}
			if ($tableFormat) { // only include the header row for table format
				$retArray[] = $headerArray;
			}
			// iterate through all of the response data and each response
			foreach ($responses as $response) {
				$responseId = $response['response_id'];
				if ($tableFormat) {
					$thisResponse = $templateArray; // arrays are assigned by copy, so this copies the array
					$thisResponse[0] = $responseId;
					$thisResponse[1] = $response['date_time'];
					$thisResponse[2] = $response['device_id'];
					$thisResponse[3] = $response['device_num'];
				} else {
					// an array of associative arrays
					$thisResponse = array();
					$thisResponse[$transResponseId] = $responseId;
					$thisResponse[$transDateTime] = $response['date_time'];
					$thisResponse[$transDeviceId] = $response['device_id'];
					$thisResponse[$transDeviceNum] = $response['device_num'];
				}

				$stmt = $this->dbHandle->prepare("SELECT * FROM form_data WHERE response_id = :response_id ORDER BY response_id, field_id DESC");
				$stmt->bindValue(':response_id', $responseId);
				$responseData = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
				if ($responseData === FALSE) {
					continue;
				}
				foreach ($responseData as $field) {
					$fieldId = $field['field_id'];
					$indexOf = array_search($fieldId, $idsArray);
					if ($tableFormat) {
						$thisResponse[$indexOf] = $field['data'];
					}
					else {
						$thisResponse[$headerArray[$indexOf]] = $field['data'];
					}
				}
				$retArray[] = $thisResponse;
			}
			return $retArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	// takes in an ip address of the form NNN.NNN.NNN.NNN and returns a device_id if it can find it, and FALSE if it can not
	public function ipAddrsGetDeviceIdWithIpAddr($ipAddr){
		try {
			// Handle case of internal request
			if ($ipAddr === DeviceInfo::IP_LOCAL_HOST) {
				return DeviceInfo::DEVICE_ID_LOCAL_HOST;
			}
			$stmt = $this->dbHandle->prepare("SELECT * FROM ip_addrs WHERE ip_addr = :ip_addr");
			$stmt->bindValue(":ip_addr", $ipAddr);
			$retArray = $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
			$deviceId = FALSE;
			if ($retArray === FALSE) {
				error_log("WARNING: there is no device_id for the ip address:${ipAddr}");
				return FALSE;
			}
			if (count($retArray) < 1) {
				error_log("WARNING: there is no device_id for the ip address:${ipAddr}");
				return FALSE;
			}
			if (array_key_exists('device_id', $retArray[0])) {
				$deviceId = $retArray[0]['device_id'];
			}
			return $deviceId;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}


	/**
	* Update ip address table with deviceID
	*
	* @param $ipAddr
	* @param $deviceId
	* @return boolean
	**/
	public function ipAddrsInsertOrReplace($ipAddr, $deviceId){
		try {
			$stmt = $this->dbHandle->prepare("INSERT OR REPLACE INTO ip_addrs (ip_addr, device_id) VALUES (:ip_addr, :device_id)");
			$stmt->bindValue(":ip_addr", $ipAddr);
			$stmt->bindValue(":device_id", $deviceId);
			return $this->executeSqlStmt($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Get device number from device id
	*
	* @param $deviceNum
	* @return string device ID, FALSE on failure
	**/
	public function devicesGetDeviceIdWithDeviceNum($deviceNum) {
		try {
			$stmtStr = "SELECT device_id FROM devices WHERE device_num = :device_num LIMIT 1";
			$stmt = $this->dbHandle->prepare($stmtStr);
			$stmt->bindValue(':device_num', $deviceNum);
			return $this->executeSqlStmtRetFirstAssociativeArrayIfExists($stmt)['device_id'];
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Get device number from device id
	*
	* @return Array of associative arrays of device nums
	**/
	public function devicesGetDeviceNums() {
		try {
			$stmt = $this->dbHandle->prepare('SELECT device_num FROM devices WHERE device_id IN (SELECT device_id FROM wifi_interaction_stats) ORDER BY device_num ASC');
			return $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function devicesGetDeviceWithId($deviceId) {
		try {
			$stmt = $this->dbHandle->prepare("SELECT * FROM devices WHERE device_id = :device_id");
			$stmt->bindValue(":device_id", $deviceId);
			return $this->executeSqlStmtRetFirstAssociativeArrayIfExists($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function devicesUpdateDeviceAttrsWithId($deviceId, $attrs){
		try {
			unset($attrs['device_id']); // device_id should never be updated
			return $this->updateTableAttributesForWithColnameAndColvalue('devices', 'device_id', $deviceId, $attrs);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}

	}

	public function devicesInsertDeviceId($deviceId){
		try {
			$attrs = array();
			$attrs['device_id'] = $deviceId;
			return $this->insertOrIgnoreIntoTableAttributes("devices", $attrs); // get element zero
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Autenticate user based on device_id
	 *
	 * @param string $userName
	 * @param string $password
	 * @return boolean
	 */
	public function usersAuthenticateTrustedDevice($deviceId) {
		$deviceIsTrusted = $this->trustedDevicesDeviceIdIsTrusted($deviceId);
		if ($deviceIsTrusted) {
			$user = $this->usersGetUserForAuthentication();
			return $this->usersAutenticateSetSession($user);
		} else {
			return FALSE;
		}
	}

	/**
	 * Get the first user, or a specific user
	 *
	 * @param string $userName
	 * @return AssociativeArray
	 */
	private function usersGetUserForAuthentication($userName = NULL) {
		try {
			$stmtString = "SELECT * FROM users LIMIT 1";
			if ($userName) {
				$stmtString = "SELECT * FROM users WHERE user_name = :user_name LIMIT 1";
			}
			$stmt = $this->dbHandle->prepare($stmtString);
			if ($userName) {
				$stmt->bindValue(':user_name', $userName);
			}
			return $this->executeSqlStmtRetFirstAssociativeArrayIfExists($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Set session to authenticate user on server
	 *
	 * @param ArrayList $user
	 * @return boolean TRUE if session is defined
	 */
	private function usersAutenticateSetSession($user) {
		if (isset($_SESSION)) {
			$browser = $_SERVER['HTTP_USER_AGENT'];
			$_SESSION['userId'] = $user['id'];
			$_SESSION['userName'] = $user['user_name'];
			$_SESSION['loginCode'] = hash('sha256', $user['password'].$browser);
			return TRUE;
		}
		return FALSE;
	}

	/**
	 * Check to see if user password is default
	 *
	 * @param string $userName
	 * @return boolean
	 */
	public function usersCheckPasswordIsDefaultWithUserName($userName) {
		try {
			$user = $this->usersGetUserForAuthentication($userName);
			if ($user) {
				// User exists
				$passwordHashedStored = $user['password'];
				$passwordHashed = hash('sha256', self::ADMIN_DEFAULT_PASSWORD.$user['salt']);
				if ($passwordHashed === $passwordHashedStored) {
					return TRUE;
				} else {
					return FALSE;
				}
			}
			return FALSE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Authenticate an user by user name and password
	 *
	 * @param string $userName
	 * @param string $password
	 * @return boolean
	 */
	public function usersAuthenticate($userName, $password)
	{
		try {
			$user = $this->usersGetUserForAuthentication($userName);
			if ($user) {
				$passwordHashedStored = $user['password'];
				$passwordHashed = hash('sha256', $password.$user['salt']);
				if ($passwordHashed === $passwordHashedStored) {
					// Password correct
					// check for session variable (in case being called from cmdline, will be undefined)
					$this->usersAutenticateSetSession($user);
					return TRUE;
				} else {
					return FALSE;
				}
			}
			return FALSE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Check whether user has logged in.
	 *
	 * @param $sessionId check is specific session logged in (used by node)
	 * @param $userAgent check if session logged in, with user agent (used by node)
	 * @return boolean
	 */
	public function usersCheckLogin($sessionId = null, $userAgent = null)
	{
		try {
			if ($sessionId !== null) {
				// Get the session from the session ID
				session_destroy();
				unset($_SESSION);
				session_id($sessionId);
				session_start();
			}
			if (isset($_SESSION['userId'], $_SESSION['userName'], $_SESSION['loginCode'])) {
				$userId = $_SESSION['userId'];
				$loginCode = $_SESSION['loginCode'];
				$userName = $_SESSION['userName'];
				if ($userAgent !== null) {
					$browser = $userAgent;
				} else {
					$browser = $_SERVER['HTTP_USER_AGENT'];
				}
				$stmt = $this->dbHandle->prepare("select * from users where id = :id LIMIT 1");
				$stmt->bindValue(':id', $userId);
				$rs = $stmt->execute();
				if ($row = $rs->fetchArray()) {
					// User exists
					$password = $row["password"];
					$loginCheck = hash('sha256', $password.$browser);
					if ($loginCheck === $loginCode) {
						return TRUE;
					} else {
						return FALSE;
					}
				} else {
					return FALSE;
				}
			} else {
				return FALSE;
			}
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}
	/**
	 * Logout the current user
	 *
	 * @return void
	 **/
	public function usersLogout()
	{
		try {
			$_SESSION = array();
			$params = session_get_cookie_params();
			setcookie(
				session_name(),
				'',
				time() - 42000,
				$params["path"],
				$params["domain"],
				$params["secure"],
				$params["httponly"]
			);
			session_destroy();
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}


	/**
	* Encrypts the given password
	*
	* @param string $salt the user's password salt
	* @param string $password the new password to encrypt
	* @return string the encrypted string
	* @access private
	* @author JPul
	**/
	private function usersEncryptPassword($salt, $password)
	{
		try {
			return hash('sha256', $password.$salt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* checks to see if the record exists
	*
	* @param integer $id the id of the user
	* @return boolean does it exist?
	* @access private
	* @author JPul
	**/
	private function usersRecordExists($id)
	{
		try {
			$stmt = $this->dbHandle->prepare("SELECT COUNT(*) AS count FROM users WHERE id = :id");
			$stmt->bindValue(':id', intval($id));
			$results = $stmt->execute();
			$row = $results->fetchArray();
			if ($row['count'] == 1) {
				return TRUE;
			} else {
				return FALSE;
			}
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Updates the user's information
	 *
	 * @param array $data the user data to save
	 * @param integer $id the id of the user to save
	 * @return boolean
	 * @access private
	 * @author JPul
	 **/
	private function usersUpdateRecord($data, $id)
	{
		try {
			$currentRecord = $this->usersFindById($id);
			if (isset($data['password'])) {
				if ($data['password'] != $data['confirm_password']) {
					throw new \InvalidArgumentException("The password & confirm password does not match.");
				}
				if (isset($data['confirm_password'])) {
					unset($data['confirm_password']);
				}
				$data['password'] = $this->usersEncryptPassword($currentRecord['salt'], $data['password']);
			}
			$newData = array_merge($currentRecord, $data);
			$stmt = $this->dbHandle->prepare(
				"UPDATE users SET " .
				"user_name = :user_name, " .
				"password = :password " .
				"WHERE id = :id"
			);
			$stmt->bindValue(':user_name', $newData['user_name']);
			$stmt->bindValue(':password', $newData['password']);
			$stmt->bindValue(':id', $id);
			$this->executeSqlStmt($stmt);
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * find the user by the given id
	 *
	 * @param integer $id the id of the given user
	 * @return array
	 * @access public
	 * @author JPul
	 **/
	public function usersFindById($id)
	{
		try {
			$stmt = $this->dbHandle->prepare("SELECT * FROM users WHERE id = :id");
			$stmt->bindValue(':id', intval($id));
			$results = $stmt->execute();
			return $results->fetchArray(SQLITE3_ASSOC);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Save the User information.  It will either create (no id provided), or update (id provided) based
	* on passed params
	*
	* @param array $data the user data to save
	* @param integer $id the id of the user to save (optional)
	* @return boolean
	* @access public
	* @todo implement create if we allow multiple users
	* @author JPul
	**/
	public function usersSave($data, $id = null)
	{
		try {
			if (is_null($id)) {
				return false;
			} else {
				if ($this->usersRecordExists($id) === false) {
					throw new \InvalidArgumentException("The id does not exist in the database.");
				}
				return $this->usersUpdateRecord($data, $id);
			}
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}


	/**
	 *
	 * @return boolean array of associative arrays representing rows and cols in the table if succes, FALSE if failure
	 * @access public
	 * @author CV
	 **/
	public function wifiNetworksGetNetworksArrayForPresentation($scannedNetworksArray = FALSE, $connectedSsid = FALSE) {
		try {
			$stmt = $this->dbHandle->prepare('SELECT *, 1 as is_saved, 0 as is_connected, 0 as quality FROM wifi_networks WHERE is_deleted = 0 ORDER BY sort_order ASC');
			$savedNetworksArray =  $this->executeSqlStmtRetArrayOfAssocArrays($stmt, 'ssid');
			if ($scannedNetworksArray !== FALSE) {
				$savedNetworksArray = array_replace_recursive($savedNetworksArray, $scannedNetworksArray);
			}
			if ($connectedSsid !== FALSE && strlen($connectedSsid) > 0) {
				if (array_key_exists($connectedSsid, $savedNetworksArray)) {
					$savedNetworksArray[$connectedSsid]['is_connected']=1;
				}
				else {
					error_log("Warning, connected ssid:{$connectedSsid} is not in the networks array");
					$savedNetworksArray[$connectedSsid]=array('is_saved' => 0, 'quality' => 0, 'is_connected' => 1, 'ssid' => $connectedSsid);
				}
			}

			foreach ($savedNetworksArray as $ssid => $ssidArray) {
				if (!array_key_exists('network_id', $ssidArray)) {
					$savedNetworksArray[$ssid]['network_id'] = 0;
				}
				if (!array_key_exists('sort_order', $ssidArray)) {
					$savedNetworksArray[$ssid]['sort_order'] = 0;
				}
				if (!array_key_exists('quality', $ssidArray)) {
					$savedNetworksArray[$ssid]['quality'] = 0;
				}
				if (!array_key_exists('is_deleted', $ssidArray)) {
					$savedNetworksArray[$ssid]['is_deleted'] = 0;
				}
				if (!array_key_exists('is_saved', $ssidArray)) {
					$savedNetworksArray[$ssid]['is_saved'] = 0;
				}
				if (!array_key_exists('is_connected', $ssidArray)) {
					$savedNetworksArray[$ssid]['is_connected'] = 0;
				}
			}

			//time to sort the array
			$quality = array();
			$isConnected = array();
			foreach ($inventory as $ssid => $network)
			{
			    $quality[$ssid] = $network['quality'];
					$isConnected[$ssid] = $network['is_connected'];
			}
			array_multisort($isConnected, SORT_DESC, $quality, SORT_DESC, $savedNetworksArray);

			return $savedNetworksArray;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}


	public function wifiNetworksGetNetworksArray() {
		try {
			$stmt = $this->dbHandle->prepare('SELECT * FROM wifi_networks WHERE is_deleted = 0 ORDER BY sort_order ASC');
			return $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 *
	 * @param string $ssid the ssid string you want to add
	 * @param string $password the password you want to add or blank for an open network
	 * @return boolean TRUE if succes, FALSE if failure
	 * @access public
	 * @author CV
	 **/
	public function wifiNetworksAddOrUpdateNetworkWithSsid($ssid, $password = '') {
		try {
			if ($password === NULL || $password === FALSE) {
				$password = '';
			}
			// first delete any existing network with this name
			$this->wifiNetworksDeleteNetworkWithSsid($ssid);
			// next get the array of wifi networks
			$networksArray =  $this->wifiNetworksGetNetworksArray();
			// add your new item to the beginning of the array
			$newNetwork = array();
			$newNetwork['ssid'] = $ssid;
			$newNetwork['password'] = $password;
			array_unshift($networksArray , $newNetwork);
			return $this->wifiNetworksSetNetworksArray($networksArray);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 *
	 * @param string $ssid the string for the ssid that you want to delete
	 * @return boolean TRUE if succes, FALSE if failure
	 * @access public
	 * @author CV
	 **/
	public function wifiNetworksDeleteNetworkWithSsid($ssid) {
		try {
			$stmt = $this->dbHandle->prepare('UPDATE wifi_networks SET is_deleted = 1 WHERE ssid = :ssid');
			$stmt->bindValue(':ssid', $ssid);
			$success = $this->executeSqlStmt($stmt);
			if ($success !== FALSE) {
				$networksArray = $this->wifiNetworksGetNetworksArray();
				$success = $this->wifiNetworksSetNetworksArray($networksArray);
			}
			return $success;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 *
	 * @param array $newNetworksArray is an array of associative arrays that represent the rows and cols in the wifi_networks database
	 * @return boolean TRUE if succes, FALSE if failure
	 * @access public
	 * @author CV
	 **/
	public function wifiNetworksSetNetworksArray($newNetworksArray) {
		try {
			// set all of the networks for this form to deleted and we can undelete them
			$stmt = $this->dbHandle->prepare('UPDATE wifi_networks SET is_deleted = 1, sort_order = NULL');
			$this->executeSqlStmt($stmt);

			// get the old networks
			$stmt = $this->dbHandle->prepare('SELECT * FROM wifi_networks');
			$oldNetworksArray = $this->executeSqlStmtRetArrayOfAssocArrays($stmt, 'ssid');

			$sortOrder = 0;
			foreach ($newNetworksArray as $newNetwork) {
				// increment the sort order
				$sortOrder += 1;
				$newNetwork['sort_order'] = $sortOrder;
				$newNetwork['is_deleted'] = 0;
				// check if a field by the same title already exists
				if (array_key_exists($newNetwork['ssid'], $oldNetworksArray)) {
					$networkToUpdate = $oldNetworksArray[$newNetwork['ssid']];
					$networkIdToUpdate = $networkToUpdate['network_id'];
					$newNetwork['network_id'] = $networkIdToUpdate;
					$this->wifiNetworksUpdateAttrsForNetworkWithId($networkIdToUpdate, $newNetwork);
				}
				else {
					$this->wifiNetworksFieldsCreateNetwork($newNetwork);
				}
			}
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	private function wifiNetworksUpdateAttrsForNetworkWithId($networkId, $attrs){
		try {
			unset($attrs['network_id']); // should never have a network_id
			return $this->updateTableAttributesForWithColnameAndColvalue('wifi_networks', 'network_id', $networkId, $attrs);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	private function wifiNetworksFieldsCreateNetwork($attrs) {
		try {
			unset($attrs['network_id']); // should never have a network id
			return $this->insertOrIgnoreIntoTableAttributes('wifi_networks', $attrs);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	public function checkIsFirstWifiVisitByDeviceId($deviceId) {
		try {
			$lastTwoInteractionsConnectHomepage = $this->statsGetRecentActivity(2, FALSE, 0, $deviceId, array(self::INTERACTION_TYPE_CONNECT, self::INTERACTION_TYPE_HOMEPAGE));
			if (count($lastTwoInteractionsConnectHomepage) == 0) {
				// TODO figure out why this always fires for .map js files
				return TRUE;
			} else if ($lastTwoInteractionsConnectHomepage[0]['interaction_type'] === self::INTERACTION_TYPE_HOMEPAGE) {
				return FALSE;
			}
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Update ip address table with deviceID
	*
	* @param Array $captivePortalLanguagesArray Array of ArrayLists that updates the
	* list of captive portal languages
	* @return boolean
	**/
	public function captivePortalLanguagesUpdateWithArray($captivePortalLanguagesArray) {
		$success = TRUE;
		$success &= $this->captivePortalLanguagesRemoveAll();
		foreach($captivePortalLanguagesArray as $index => $captivePortalLanguage) {
			$captivePortalLanguage['sort_order'] = $index;
			$success &= $this->captivePortalLanguagesAddLanguage($captivePortalLanguage);
		}
		return $success;
	}

	/**
	* Update ip address table with deviceID
	*
	* @param ArrayList $captivePortalLanguage ArrayList representing a language
	* @return boolean
	**/
	public function captivePortalLanguagesAddLanguage($captivePortalLanguage) {
		try {
			$stmt = $this->dbHandle->prepare(
				'INSERT OR REPLACE INTO captive_portal_languages '
				. '(language_iso, message, language_display_name, sort_order) '
				. 'VALUES '
				. '(:language_iso, :message, :language_display_name, :sort_order)'
			);
			$stmt->bindValue(':language_iso', strtolower($captivePortalLanguage['language_iso']));
			$stmt->bindValue(':message', $captivePortalLanguage['message']);
			$stmt->bindValue(':language_display_name', $captivePortalLanguage['language_display_name']);
			$stmt->bindValue(':sort_order', $captivePortalLanguage['sort_order']);
			return $this->executeSqlStmt($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Delete all captive portal languages
	*
	* @return boolean
	**/
	public function captivePortalLanguagesRemoveAll() {
		try {
			$stmt = $this->dbHandle->prepare('DELETE FROM captive_portal_languages');
			return $this->executeSqlStmt($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Get all captive portal languages
	*
	* @return boolean
	**/
	public function captivePortalLanguagesGetAll() {
		try {
			$stmt = $this->dbHandle->prepare('SELECT * FROM captive_portal_languages');
			return $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Get captive portal language by id
	*
	* @return boolean
	**/
	public function captivePortalLanguagesGetWithId($id) {
		try {
			$stmt = $this->dbHandle->prepare('SELECT * FROM captive_portal_languages WHERE id = :id');
			$stmt->bindValue(':id', $id);
			return $this->executeSqlStmtRetFirstAssociativeArrayIfExists($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Get default captive portal message for a language iso
	*
	* @return String
	**/
	public function captivePortalGetDefaultMessageFromLanguageIso($languageIso) {
		try {
			$languageIso = strtolower($languageIso);
			$stmt = $this->dbHandle->prepare('SELECT message FROM captive_portal_languages_default_messages WHERE language_iso = :language_iso');
			$stmt->bindValue(':language_iso', $languageIso);
			$answer = $this->executeSqlStmtRetFirstAssociativeArrayIfExists($stmt);
			if (isset($answer['message'])) {
				return $answer['message'];
			}
			return FALSE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Get array of all default messages
	*
	* @return Array
	**/
	public function captivePortalGetAllDefaultMessages() {
		try {
			$stmt = $this->dbHandle->prepare('SELECT * FROM captive_portal_languages_default_messages');
			return $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Add array of all new devices
	*
	* @param Array $trustedDevicesArray Array of ArrayLists that updates the
	* trusted devices
	* @return boolean
	**/
	public function trustedDevicesUpdateWithArray($trustedDevicesArray) {
		$success = TRUE;
		$success &= $this->trustedDevicesRemoveAll();
		foreach($trustedDevicesArray as $device) {
			$success &= $this->trustedDevicesAddDevice($device);
		}
		return $success;
	}

	/**
	* Add a device to trusted devices
	*
	* @param ArrayList $device
	* @return Boolean
	*/
	public function trustedDevicesAddDevice($device) {
		if (empty($device['device_num']) 
			|| empty($device['device_id']) 
			|| empty($device['device_type'])) {
			return FALSE;
		}
		try {
			$stmt = $this->dbHandle->prepare(
				'INSERT or REPLACE INTO trusted_devices '
				. '(device_num, device_id, device_type) '
				. 'VALUES '
				. '(:device_num, :device_id, :device_type)'
			);
			$stmt->bindValue(':device_num', $device['device_num']);
			$stmt->bindValue(':device_id', $device['device_id']);
			$stmt->bindValue(':device_type', $device['device_type']);
			return $this->executeSqlStmt($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Remove all trusted devices
	*
	* @return Boolean
	**/
	private function trustedDevicesRemoveAll() {
		try {
			$stmt = $this->dbHandle->prepare('DELETE FROM trusted_devices');
			return $this->executeSqlStmt($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Get array of all trusted devices
	*
	* @return Array
	**/
	public function trustedDevicesGetAll() {
		try {
			$stmt = $this->dbHandle->prepare('SELECT * FROM trusted_devices');
			return $this->executeSqlStmtRetArrayOfAssocArrays($stmt);
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	* Check if a given device is a trusted device
	*
	* @return boolean
	**/
	public function trustedDevicesDeviceIdIsTrusted($deviceId) {
		try {
			$stmt = $this->dbHandle->prepare('SELECT device_id FROM trusted_devices WHERE device_id = :device_id');
			$stmt->bindValue(':device_id', $deviceId);
			$answer = $this->executeSqlStmtRetFirstAssociativeArrayIfExists($stmt);
			if (!empty($answer)) {
				return TRUE;
			}
			return FALSE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}
}
