<?php

namespace Renew;

/*
 * Description of DeviceInfo
 */
class DeviceInfo {

		// Constants
		const IP_LOCAL_HOST = '127.0.0.1';
		const DEVICE_ID_LOCAL_HOST = 'device-id-internal';

    public static function GetLightStreamSerialNumber() {
        // return raspberrypi serial number from /proc/cpuinfo
        $deviceIdentifier = `rwo_system.sh getSerialNumber`;
        if ($deviceIdentifier){
            $deviceIdentifier = trim($deviceIdentifier);
        }
        return $deviceIdentifier;
    }

		public static function GetLightStreamFirmwareVersion() {
				// return raspberrypi serial number from /proc/cpuinfo
				$deviceIdentifier = `rwo_system.sh getVersion`;
				if ($deviceIdentifier){
						$deviceIdentifier = trim($deviceIdentifier);
				}
				return $deviceIdentifier;
		}

    public static function GetCurrentlyConnectedDevicesCount(){
      try {
        return `iw dev wlan0 station dump | grep "^Station" | wc -l`;
      } catch (\Exception $e) {
        error_log($e);
        return 0;
      }
    }

	/**
	*
	*
	* @return string of the hardware platform (rpi1, rpi2, rpi3, pocket .. ) on success and FALSE on failure
	**/
	public static function GetLightStreamHwPlatform() {
		try {
      // return raspberrypi serial number from /proc/cpuinfo
      $hwPlatform = `rwo_system.sh getHwPlatform`;
      if ($hwPlatform){
          $hwPlatform = trim($deviceIdentifier);
      }
			return $hwPlatform;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Check to see if device requesting is the server itself, using the $_SERVER object
	 * 
	 * @param $serverArray the $_SERVER object
	 * @return boolean TRUE if request is from server
	 */
	public static function RequestingDeviceIpIsLocalhost($serverArray = NULL) {
		return self::GetRequestingDeviceIp($serverArray) === self::IP_LOCAL_HOST;
	}

	// return the IP of the device making the request, or NULL
	public static function GetRequestingDeviceIp($serverArray = NULL) {
		try {
			if ($serverArray == NULL || empty($serverArray)) {
				$serverArray = $_SERVER;
			}
			$clientIp = NULL;
			if ( array_key_exists('HTTP_CLIENT_IP', $serverArray) && strlen($serverArray['HTTP_CLIENT_IP']) ) {
				$clientIp = $serverArray['HTTP_CLIENT_IP'];
			}
			elseif ( array_key_exists('HTTP_X_FORWARDED_FOR', $serverArray) && strlen($serverArray['HTTP_X_FORWARDED_FOR']) ) {
				$clientIp = $serverArray['HTTP_X_FORWARDED_FOR'];
			}
			elseif ( array_key_exists('REMOTE_ADDR', $serverArray) && strlen($serverArray['REMOTE_ADDR']) ) {
				$clientIp = $serverArray['REMOTE_ADDR'];
			}
			return $clientIp;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

	/**
	 * Get string identifying a device
	 * 
	 * @param $agent Jenssegers\Agent\Agent object
	 */
	public static function GetDeviceIdentifierWithAgent($agent) {
		$platform = $agent->platform(); // OS
		$device = $agent->device();
		$type = _('Other');
		if ($agent->isTablet()) {
			$type = _('Tablet');
		} else if ($agent->isMobile()) {
			$type = _('Mobile');
		} else if ($agent->isDesktop()) {
			$type = _('Desktop');
		}
		return "${platform} - ${type} (${device})";
	}

	// public function GetRequestingDeviceInfo() {
	// 	$detect = new Mobile_Detect;
	// 	$retVal = array();

	// 	// get the device_type
	// 	if ($detct->isMobile()) {
	// 		if ($detect->isTablet()) {
	// 			$retVal['type'] = "tablet";
	// 		}
	// 		else{
	// 			$retVal['type'] = "mobile";
	// 		}
	// 		$retVal['type'] = "computer";
	// 	}

	// 	// get the device OS
	// 	if ($detct->isAndroid()) {
	// 		$retVal['os'] =  "Android";
	// 	} elseif ($detect->isiOS()){
	// 		$retVal['os']  =  "iOS";
	// 	} elseif ($detect->isWin()) {
	// 		$retVal['os'] = "Windows";
	// 	} elseif ($detect->isMac()) {
	// 		$retVal['os'] = "Mac";
	// 	} elseif ($detect->isLinux()) {
	// 		$retVal['os'] = "Linux";
	// 	} else {
	// 		$retVal['os'] = "Other";
	// 	}

	// 	// get the Browser
	// 	if ($detct->isChrome()) {
	// 		$retVal['browser'] =  "Chrome";
	// 	} elseif ($detect->isSafari()){
	// 		$retVal['browser'] =  "Safari";
	// 	} elseif ($detect->isIE() || $detect->isInternetExplorer()) {
	// 		$retVal['browser'] =  "IE";
	// 	} elseif ($detect->isFirefox()) {
	// 		$retVal['browser'] =  "Firefox";
	// 	} else {
	// 		$retVal['browser'] =  "Other";
	// 	}

	// 	return $retVal;
	// }



}
