<?php
namespace Renew;

/**
 * Description of DiskInfo
 *
 * @author CV
 */
class DiskInfo {

  public static function HumanReadableDiskFreeSpace($directory = "/", $bufferBytes = FALSE) {
    return self::BytesToHumanReadableSize(self::DiskFreeSpaceInB($directory, $bufferBytes));
  }

  public static function HumanReadableDiskTotalSpace($directory = "/", $bufferBytes = FALSE) {
    return self::BytesToHumanReadableSize(self::DiskTotalSpaceInB($directory, $bufferBytes));
  }

	public static function DiskTotalSpaceInGB($directory = "/", $bufferBytes = FALSE) {
    return self::BytesToGB(self::DiskTotalSpaceInB($directory, $bufferBytes));
	}

	public static function DiskFreeSpaceInGB($directory = "/", $bufferBytes = FALSE) {
    $bytes = self::DiskFreeSpaceInB($directory, $bufferBytes);
    return self::BytesToGB($bytes);
	}

  public static function DiskFreeSpaceInB($directory = "/", $bufferBytes = FALSE) {
    $space = disk_free_space($directory);
    if ($bufferBytes !== FALSE) {
      $space -= $bufferBytes;
    }
    return $space;
  }

  public static function DiskTotalSpaceInB($directory = "/", $bufferBytes = FALSE) {
    $space = disk_total_space($directory);
    if ($bufferBytes !== FALSE) {
      $space -= $bufferBytes;
    }
    return $space;
  }

	public static function DiskFreePercent($directory = "/", $bufferBytes = FALSE) {
    $freeSpace = self::DiskFreeSpaceInB($directory, $bufferBytes);
    $totalSpace = self::DiskTotalSpaceInB($directory, $bufferBytes);
		$percent = ($freeSpace / $totalSpace) * 100.0;
		return round($percent);
	}

  /*
    source: https://gist.github.com/eusonlito/5099936
  */
  public static function FolderSize($dir) {
    $size = 0;
    foreach (glob(rtrim($dir, '/').'/*', GLOB_NOSORT) as $each) {
      $size += is_file($each) ? filesize($each) : self::FolderSize($each);
    }
    return $size;
  }

  public static function BytesToHumanReadableSize($bytes) {
    $suffix = " B";
    $number = $bytes;

    if ($bytes >= 1000000000000) {
        $number = $bytes / 1000000000000;
        $suffix = " TB";
    } else if ($bytes >= 1000000000) {
        $number = $bytes / 1000000000;
        $suffix = " GB";
    } else if ($bytes >= 1000000) {
        $number = $bytes / 1000000;
        $suffix = " MB";
    } else if ($bytes >= 1000) {
        $number = $bytes / 1000;
        $suffix = " kB";
    }

    return number_format($number, 1) . $suffix;
  }

  public static function BytesToGB($bytes) {
    $gb = $bytes / 1000000000;
		return number_format($gb, 1);
  }
}
