<?php

namespace Renew;

/**
 * Functions having to do with files
 */

class FilesHelper {

    /**
     * Get the estimated size of the contents for a particular directory
     * 
     * @param string $dir the directory to estimate the size of
     * @return int the size in bytes of all of the hard links 
     */
    public static function DirectorySizeEstimate($dir) {
        if (is_dir($dir) === FALSE) {
            return 0;
        }
        $recursive_iterator = new \RecursiveIteratorIterator(new \RecursiveDirectoryIterator($dir . '/', \RecursiveDirectoryIterator::SKIP_DOTS), \RecursiveIteratorIterator::SELF_FIRST);
        $size_estimate = 0;
        foreach ($recursive_iterator as $this_file) {
            if (is_dir($this_file)) {
                
            } elseif (is_file($this_file)) {
                $size_estimate += filesize($this_file);
            }
        }
        return $size_estimate;
    }
    
    /**
     * Check if directory is completely empty (not just size 0)
     * 
     * @param string $dir the directory to estimate the size of
     * @return boolean
     */
    public static function DirectoryIsEmpty($dir) {
        if (!is_readable($dir)) {
            return FALSE; 
        }
        return (count(scandir($dir)) === 2); // '.' and '..'
    }

    /**
     * Delete a directory's contents
     * 
     * @param string $dir the directort to delete the contents of
     * @param boolean $deleteRoot should $dir be deleted too?
     * @return boolean TRUE on success and FALSE on failure
     */
    public static function RemoveDirectoryAndContents($dir, $deleteRoot = TRUE) {
        
        $dir = rtrim($dir, '/');

        if (is_dir($dir) === FALSE) {
            return FALSE;
        }

        if (!$dh = @opendir($dir)) {
            return FALSE;
        }

        while (false !== ($obj = readdir($dh))) {

            if ($obj == '.' || $obj == '..') {
                continue;
            }

            if (!@unlink($dir . '/' . $obj)) {
                // recursive call
                FilesHelper::RemoveDirectoryAndContents($dir . '/' . $obj, TRUE);
            }
        }

        closedir($dh);

        if ($deleteRoot) {
            @rmdir($dir);
        }
        
        return TRUE;
    }

}
