<?php

namespace Renew;

class SessionHelper {
  const SESSION_MEDIA_AUTHENTICATED_ARRAY = 'media_autenticated_array';
  const LOCALE_EN_US = 'en_US.UTF-8';
  const LOCALE_DOMAIN = 'light_stream';

  /**
   * Start a session
   */
  public static function StartSecureSession() {
    try {
			$session_name = 'sec_session_id';
			$secure = false;
			$httponly = true;
			ini_set('session.use_only_cookies', 1);
			$cookieParams = session_get_cookie_params();
			// set cookie params, 86400 seconds (24 hours)
			session_set_cookie_params(
				86400,
				$cookieParams['path'],
				$cookieParams['domain'],
				$secure,
				$httponly
			);
			session_name($session_name);
			session_start();
			session_regenerate_id();
			return TRUE;
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
  }

  /**
   * Set the language for admin
   */
  public static function SetSiteLanguage($siteLanguage, $baseDirectory) {
    if ((!isset($siteLanguage)) || empty($siteLanguage)) {
      $siteLanguage = self::LOCALE_EN_US;
    }

    // Locale is always going to be en_US.UTF-8, so we include a en_US.UTF-8 folder in each language folder,
    // and use /locale/$siteLanguage as our locale so that the locales don't need to be installed in linux
    bindtextdomain(self::LOCALE_DOMAIN, $baseDirectory . DS . 'locale' . DS . $siteLanguage);
    bind_textdomain_codeset(self::LOCALE_DOMAIN, 'UTF-8');
    textdomain(self::LOCALE_DOMAIN);
  }

  /**
   * Remove session variables that shouldn't persist for admin if they log in
   */
  public static function CleanUpSessionForAdmin() {
    $_SESSION[self::SESSION_MEDIA_AUTHENTICATED_ARRAY] = array();
  }

  /**
   * Check to see if a password-protected file has been authenticated
   */
  public static function CheckFileIdIsAuthenticated($fileId) {
    return isset($_SESSION[self::SESSION_MEDIA_AUTHENTICATED_ARRAY]) 
      && isset($_SESSION[self::SESSION_MEDIA_AUTHENTICATED_ARRAY][$fileId])
      && $_SESSION[self::SESSION_MEDIA_AUTHENTICATED_ARRAY][$fileId] === TRUE;
  }

  /**
   * Authenticate a password-protected file
   */
  public static function AuthenticateFileId($fileId) {
    if (!isset($_SESSION[self::SESSION_MEDIA_AUTHENTICATED_ARRAY])) {
      $_SESSION[self::SESSION_MEDIA_AUTHENTICATED_ARRAY] = array();
    }
    $_SESSION[self::SESSION_MEDIA_AUTHENTICATED_ARRAY][$fileId] = TRUE;
  }
}
    