<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Renew;

class StringHelper {

    /**
     * Check if a given string ends with another string
     *
     * @param string $string the main string
     * @param string $subString the substring to search for
     * @return TRUE if the $string ends with $subString
     * @access public
     * @author CV
     **/
    public static function EndsWith($string, $subString) {
        $length = (strlen($subString) * -1);
        return StringHelper::ContainsSubStrAtIndex($string, $subString, $length);
    }

    /**
     * Check if a given string starts with another string
     *
     * @param string $string the main string
     * @param string $subString the substring to search for
     * @return TRUE if the $string starts with $subString
     * @access public
     * @author CV
     **/
    public static function StartsWith($string, $subString) {
        return StringHelper::ContainsSubStrAtIndex($string, $subString, 0);
    }


    /**
     * Does a string contain a particular substring at a specified index
     *
     * @param string $string
     * @param string $subString
     * @param string $index
     * @return boolean TRUE if the string at the specified index matches the substring, FALSE if not
     */
    private static function ContainsSubStrAtIndex($string, $subString, $index) {

        $length = strlen($subString);

        if ($length === 0) {
            return TRUE;
        }

        if ($length > strlen($string)) {
            return FALSE;
        }

        return (substr($string, $index, $length) === $subString);
    }



    /**
     * Takes in a file name, and returns a "Windows Safe" version of the string
     *
     * @param string $str the file name
     * @return string the windows safe file name
     */
    public static function StringToWindowsSafeFileName($str){

        /*  Define list of reserved characters as string.
          These characters are reserved on Windows-supported file systems (such as FAT)
          and should therefore not be permitted in filenames which might be destined for such file systems. */
        $reserved_characters_string = "\\/:*?\"<>|";
        $safe_str = StringHelper::StringReplaceCharactersWithUnicodeReplacementChar($str, $reserved_characters_string);
        return $safe_str;

    }

    /**
     * Takes in a file path, and returns a windows safe path
     *
     * @param string $str the path to the file
     * @return string the windows safe path that is still pieced together by "/"
     */
    public static function StringToWindowsSafePathName($str){

        /*  Define list of reserved characters as string.
          These characters are reserved on Windows-supported file systems (such as FAT)
          and should therefore not be permitted in filenames which might be destined for such file systems. */
        $reserved_characters_string = "\:*?\"<>|";
        $safe_str = StringHelper::StringReplaceCharactersWithUnicodeReplacementChar($str, $reserved_characters_string);
        return $safe_str;

    }

    /**
     * Replace all the individual unique characters in the $needles_str with the Ubicode replacement character (�)
     *
     * @param string $haystack_str the string to do the replacement on
     * @param string $needles_str the string containing the characters that need replacement
     * @return string the string after the replacement has happened
     */
    public static function StringReplaceCharactersWithUnicodeReplacementChar($haystack_str, $needles_str){

        // Turn list of reserved characters into an array. Each element is a single reserved character.
        $reserved_characters_array = str_split($needles_str);

        // Replace each occurrence of a reserved character in $newTitle with the Unicode replacement character (�).
        $safe_str = str_replace($reserved_characters_array, "�", $haystack_str);

        return $safe_str;

    }

    /**
     * Returns a raw urlencoded string with everything except the / character encoded
     * Source:
     * http://stackoverflow.com/a/2834613/5364953
     *
     * @param string $string_to_encode the string to encode
     * @return string the rawurlencoded string
     */
    public static function RawUrlEncodeStringExceptFwdSlash($string_to_encode){
        return implode("/", array_map("rawurlencode", explode("/", $string_to_encode))) ;
    }

    /*
      TODO Comment
    */
    public static function SecondsToFormattedTime($seconds) {
      return gmdate("H:i:s", $seconds);
    }
  
  /**
   * @return string random MD5 String
   */
	public static function GetRandomMd5String() {
		try {
			return md5(mt_rand());
		} catch (Exception $e) {
			error_log($e);
			return FALSE;
		}
	}

  /**
  * Creates a URL string that can be re-directed to to open up an email client
  *
  * @param $email email address to send message to
  * @param $subject email $subject
  * @param $body body of email
  */
  public static function CreateMailtoUrl($email, $subject, $body) {
    return "mailto:" . rawurlencode($email)
     . "?subject=" . rawurlencode($subject)
     . "&body=" . rawurlencode($body);
  }

  /*
    TODO Comment
    source: https://stackoverflow.com/questions/13280200/convert-unicode-to-html-entities-hex
  */
  public static function ConvertHTMLToHex($HTMLIcon) {
    if (!isset($HTMLIcon) || empty($HTMLIcon)) {
      return $HTMLIcon;
    }
    $answer = preg_replace_callback('/[\x{80}-\x{10FFFF}]/u', function ($m) {
      $char = current($m);
      $utf = iconv('UTF-8', 'UCS-4', $char);
      return sprintf("&#x%s;", ltrim(strtoupper(bin2hex($utf)), "0"));
    }, '' . $HTMLIcon);
    return $answer;
  }

  /**
   * Convert a string to UTF8 encoding
   * @param $string
   */
  // public static function ConvertToUTF8($string) {
  //   return mb_convert_encoding($string, 'UTF-8');
  // }

  /**
   * Convert a hex string (of all numbers and symbols with no escapes) to a UTF-16 String.
   * Used to convert SSIDs from hex.
   */
  public static function ConvertHexToText($stringHex) {
    return hex2bin($stringHex);
  }

  /**
	* Remove illegal characters from filenames
	*
	* @param $fileName
	**/
	public static function FormatFilename($fileName) {
		$fileName = str_replace('/', '', $fileName);
		return $fileName;
  }

  /**
   * Remove whitespace login
   */
  public static function RemoveWhitespaceFromPassword($string) {
    return trim($string);
  }
}
