<?php

namespace Renew\Utilities;

/**
 * This is a wrapper class for PHP's CSV Generator methods for easier integration
 *
 * @package default
 * @author JPul
 **/
class CSVGenerator
{
    /**
     * The pointer to the file object
     *
     * @var Object
     * @access private
     **/
    private $filePointer;
    /**
     * The path to the created file
     *
     * @var string
     * @access private
     **/
    private $file;
    /**
     * Intialize the class
     *
     * @return void
     * @access public
     * @author JPul
     **/
    public function __construct($file)
    {
        $this->file = $file;
        $this->filePointer = fopen($this->file, 'w');
    }
    /**
     * Add a row to the CSV document
     *
     * @param array $content an array of data to add to CSV
     * @return void
     * @access public
     * @author JPul
     **/
    public function addRow($content)
    {
        fputcsv($this->filePointer, $content);
    }
    /**
     * Close the CSV File
     *
     * @return void
     * @access public
     * @author JPul
     **/
    public function close()
    {
        fclose($this->filePointer);
    }
    /**
     * Trigger the download for the given file by sending appropriate headers, and content
     * This method exits your code
     *
     * @return void
     * @access public
     * @author JPul
     **/
    public function download()
    {
        $filename = basename($this->file);
        /**
         * disable caching
         *
         * @author JPul
         **/
        $now = gmdate("D, d M Y H:i:s");
        header("Expires: Tue, 03 Jul 2001 06:00:00 GMT");
        header("Cache-Control: max-age=0, no-cache, must-revalidate, proxy-revalidate");
        header("Last-Modified: {$now} GMT");

        /**
         * force download
         *
         * @author JPul
         **/
        header("Content-Type: application/force-download");
        header("Content-Type: application/octet-stream");
        header("Content-Type: application/download");

        /**
         * disposition / encoding on response body
         *
         * @author JPul
         **/
        header("Content-Disposition: attachment;filename={$filename}");
        header("Content-Transfer-Encoding: binary");
        echo file_get_contents($this->file);
        exit;
    }

		public function getString(){
			return file_get_contents($this->file);
		}
}
