<?php

/*
 * A script that will add pieces of media with specified full paths
 * To call this script from the command line:
 * $ php <-wbu> "/full/path/1" "/full/path/2" ... "/full/path/n"
 * Will add media with the full path to the sqlite database
 * Options:
 * -w make available over wifi
 * -b make available over bluetooth
 * -u make available over usb
 *
 * All options must be specified in first parameter
 * If options are omitted, the media will be added to the database without making it available to any of the destinations
 */

// include the settings for php functions to run from the command line
require_once(dirname(dirname(dirname(dirname(__FILE__)))) . DIRECTORY_SEPARATOR . 'config' . DIRECTORY_SEPARATOR . 'settings_cmdline.php');
require_once('Renew' . DIRECTORY_SEPARATOR . 'StringHelper.php');
require_once('Renew' . DIRECTORY_SEPARATOR . 'DbManagerMedia.php');
require_once('Renew' . DIRECTORY_SEPARATOR . 'ArrayHelper.php');
use \Renew\dbManagerMedia;
use \Renew\ArrayHelper;
$mediaDbManager = new DbManagerMedia(MEDIA_DB, MEDIA_DIR, USER_IMAGES_DIR);

if ($argc < 2){
    exit ();
}

$addBluetooth = FALSE;
$addWifi = FALSE;
$addUsb = FALSE;

// check for options arguments like -b, -w, etc.
$startingIndex = 1;
$firstArg = $argv[1];
if (\Renew\StringHelper::StartsWith($firstArg, '-')) {
    $startingIndex = 2;
    $addWifi = (strpos($firstArg, 'w') !== FALSE);
    $addBluetooth = (strpos($firstArg, 'b') !== FALSE);
    $addUsb = (strpos($firstArg, 'u') !== FALSE);
}

$pathFull = $argv[$startingIndex];
$arclightJsonPath = $argv[$startingIndex + 1] ?? NULL;
$originalFilePathRel = $argv[$startingIndex + 2] ?? NULL;

$mediaDbManager->openDb();  
  // Add in arclight information if there's an arclight.json file
  $arclightJsonObject = NULL;
  if ($arclightJsonPath !== NULL && $originalFilePathRel !== NULL) {
    $arclightJsonString = file_get_contents($arclightJsonPath);
    if ($arclightJsonString !== false) {
      $arclightJson = json_decode($arclightJsonString, TRUE);
      if ($arclightJson !== NULL) { // json_decode returns null on failure
        $arclightJsonArray = $arclightJson['mediaAssets'];
        $arclightJsonObject = ArrayHelper::GetArrayListWithKeyValueFromArray($arclightJsonArray, 'urlPath', $originalFilePathRel, NULL);
      }
    }
  }
  $fileId = $mediaDbManager->mediaAddOrUndeleteFileWithPath($pathFull, NULL, $arclightJsonObject);
  // do the availability
  if ($fileId !== FALSE) {
    $attrs = array();
    $attrs['wifi_status'] = 0;
    $attrs['bluetooth_status'] = 0;
    $attrs['usb_status'] = 0;
    if ($addWifi) {
      $attrs['wifi_status'] = 2;
    }
    if ($addBluetooth) {
      $attrs['bluetooth_status'] = 2;
    }
    if ($addUsb) {
      $attrs['usb_status'] = 2;
    }

    // check for default images
    $mediaDbManager->mediaSetDefaultsForFileNameWithFileIdIfAvailable($fileId);
    $mediaDbManager->mediaUpdateAttrsForFileWithId($fileId, $attrs);
  }
$mediaDbManager->closeDb();
