<?php

// Define the namespace.
namespace Renew\Utilities;

/**
 * the incron services
 * @link http://inotify.aiken.cz/?section=incron&page=about&lang=en
 *
 * @package default
 * @author JPul
 **/
class LightStreamServices
{

    /**
     * change the settings of the WiFi
     *
     *  $expectedSettings = array(
     *      'ssid'       =>  'My SSID',
     *      'channel'    =>  13
     *  );
     *
     * @param array $newSettings An array of new settings
     * @access public
     **/
    public function changeWiFiSettings($newSettings, $lswifidb)
    {
      try {
        if (empty($newSettings)) {
            throw new \InvalidArgumentException("The WiFi Settings array must not be empty.");
        }
        $fileContent = "";
        foreach ($newSettings as $key => $value) {
            $fileContent .= $key . "=" . $value . "\n";
        }

        $db = new \SQLite3($lswifidb);
        $db->busyTimeout(5000);
        $stmt1 = $db->prepare("UPDATE settings SET value = :ssid WHERE key = 'ssid'");
        $stmt1->bindValue(":ssid", $newSettings['ssid']);
        $stmt1->execute();
        $stmt2 = $db->prepare("UPDATE settings SET value = :channel WHERE key = 'channel'");
        $stmt2->bindValue(":channel", $newSettings['channel']);
        $stmt2->execute();
        $db->close();

        return TRUE;
      } catch (\Exception $e) {
        error_log($e);
        return FALSE;
      }
    }

    /**
     * clear the given log file
     *
     * @param string $log The short name for the log to clear (allowed: access, dmesg, error, kern, lsbt, and ls_cron)
     * @access public
     **/
    public function clearLog($log)
    {
      try {
        if (empty($log)) {
            throw new \InvalidArgumentException("The log short name must not be empty.");
        }
        if (!in_array($log, array('access', 'error', 'lsbt', 'incron', 'satellite'))) {
            throw new \InvalidArgumentException("The log short name can only be access, error, lsbt, and cron.");
        }
        // TODO implement a new way of doing this ... not in use currently CV 20180206
        return TRUE;
      } catch (\Exception $e) {
        error_log($e);
        return FALSE;
      }
    }

    /**
     * clear the SD Copy Stats file
     *
     * @return boolean
     * @access public
     * @author JPul
     **/
    public function clearSDCopyStats()
    {
			// TODO implement a new way of doing this ... CV 20180206
        return TRUE;
    }

    /**
     * Reboots the Light Stream system
     *
     * @access public
     **/
    public function shutdown(){
        exec('sudo /usr/bin/rwo_system.sh systemShutdownNow');
        return TRUE;
    }

		public function reboot(){
				exec('sudo /usr/bin/rwo_system.sh systemRebootNow');
				return TRUE;
		}

    /**
     * reboot the Light Stream Bluetooth Service
     *
     **/
    public function rebootBluetooth()
    {
        exec('sudo /usr/bin/systemctl restart rwo_bluetooth');
        return TRUE;
    }

    public function rebootWifi() {
        // use the systemd-run timer so that you can give php time to give it's response to the user
        exec('sudo /usr/bin/systemd-run --on-active=3 "systemctl restart rwo_wifi@wlan0"');
        return TRUE;
    }

    /**
     * Update the firmware.
     *
     */
    public function updateFirmware()
    {
      exec('sudo /usr/bin/systemctl start rwo_updateFirmware@findAndRunAdminPanelUpload.service');
      return TRUE;
    }

}
