<?php

namespace Renew;

require_once('Renew/Utilities/Validator.php');
require_once('Renew/DbManagerMedia.php');
require_once('Renew/DbManagerWifi.php');
require_once('Renew/DbManager.php');
require_once("Renew" . DS . "DiskInfo.php");
use Jenssegers\Agent\Agent;
use \Renew\Validator;
use \Renew\DbManagerMedia;
use \Renew\DbManagerWifi;
use \Renew\DbManager;
use \Renew\DiskInfo as DiskInfo;

/**
 * Functions to get data from database, shared between slim routes
 */
class SlimDataHelper {
  private $app;
  private $container;
  private $serialNumber;
  private $deviceId;

  public function __construct(&$app, &$slimContainer, $serialNumber, $deviceId) {
    $this->app = $app;
    $this->container = $slimContainer;
    $this->serialNumber = $serialNumber;
    $this->deviceId = $deviceId;
  }

  public function subRequestHelper($url) {
    $response = $this->app->subRequest('GET', "api/v2{$url}");
    return json_decode($response->getBody()->__toString(), TRUE);
  }

  public function getIndexHTMLFile($isAdmin = FALSE) {
    $file = 'dist_vue/index.html';
    $indexHTMLFile = file_get_contents($file);
    $settingsObject = $this->subRequestHelper('/no-cache/settings');
    $settingsObject['is_admin'] = $isAdmin;

    // Inject CSS into html
    $indexHTMLFile = str_replace(
      '<meta server_inject_theme_css>',
      "<link rel=\"stylesheet\" href=\"/css/{$settingsObject['themes_css_file']}\">",
      $indexHTMLFile
    );

    if ($isAdmin) {
      $settingsObject['translations'] = $this->getVueTranslations();
      $settingsObject['disk_free_space_bytes'] = DiskInfo::DiskFreeSpaceInB(MEDIA_DIR, LS_RESERVE_SIZE);
    } else {
      $initialAppState = array(
        'browser_language_directory' => $this->subRequestHelper('/no-cache/browser-language-directory'), // Todo, stop using the endpoint to do this and use getBrowserLanguageDirectory
        'form_followup' => $this->subRequestHelper('/forms'),
        'language_directory_array' => $this->subRequestHelper('/language-directories'),
      );
    }

    $initialAppState['settings_object'] = $settingsObject;
    $initialStateScriptId = 'id-initial-state';
    $initialAppState['initial_state_script_id'] = $initialStateScriptId;
    $initialAppState = json_encode($initialAppState); // JSON_INVALID_UTF8_SUBSTITUTE
    
    // Inject initial JS state object into html, to increase load speeds
    $indexHTMLFile = str_replace(
      '<meta server_inject_js>',
      "
        <script id=\"{$initialStateScriptId}\">
          window.__INITIAL_STATE__ = {$initialAppState};
        </script>
      ",
      $indexHTMLFile
    );

    return $indexHTMLFile;
  }

  public function getIndexHTMLFileAdmin($path, $fetchFunction) {
    $path = explode('/admin-v/', $path)[1];
    $currentPage = explode('/', $path)[0];
    $this->container->t->assign('current_page', $currentPage);
    $this->container->t->assign('is_admin_vue', TRUE);

    $indexHTMLFileAdmin = $this->getIndexHTMLFile(TRUE);

    // Add header, nav, & footer into index.html file
    $indexHTMLFileAdmin = str_replace(
      '<meta server_inject_admin_header>',
      $fetchFunction('header_admin_content.tpl'),
      $indexHTMLFileAdmin
    );
    $indexHTMLFileAdmin = str_replace(
      '<meta server_inject_admin_nav>',
      $fetchFunction('admin/top_nav.tpl'),
      $indexHTMLFileAdmin
    );
    $indexHTMLFileAdmin = str_replace(
      '<meta server_inject_admin_footer>',
      $fetchFunction('footer_admin_content.tpl'),
      $indexHTMLFileAdmin
    );
    return $indexHTMLFileAdmin;
  }

  public function getForm($formId = DbManagerWifi::FOLLOWUP_FORM_ID) {
    $form = array();
    $this->container->dbManagerWifi->openDb();
      $form = $this->container->dbManagerWifi->formsGetFormWithId($formId);
      $form['fields'] = $this->container->dbManagerWifi->formFieldsGetFieldsWithFormId($formId);
    $this->container->dbManagerWifi->closeDb();
    return $form;
  }

  public function getLanguageDirectories() {
    $this->container->dbManagerMedia->openDb();
      // TODO make a separate function instead of passing TRUE
      $languageDirectoriesArray = $this->container->dbManagerMedia->mediaGetLanguageDirectories(TRUE);
    $this->container->dbManagerMedia->closeDb();
    return $languageDirectoriesArray;
  }

  public function getSettingsNoCache() {
    $this->container->dbManagerWifi->openDb();
      $isFirstHomepageVisit = $this->container->dbManagerWifi->checkIsFirstWifiVisitByDeviceId($this->deviceId);
      $device = $this->container->dbManagerWifi->devicesGetDeviceWithId($this->deviceId);
      $themesCSSFile = $this->container->dbManagerWifi->settingsGetValueForKey('themes_css_file'); // Get CSS and JS content for intial HTML file
      $helpMessagesHighlight = Validator::StringToBoolean($this->container->dbManagerWifi->settingsGetValueForKey('help_messages_highlight'));
      $this->container->dbManagerWifi->closeDb();
    
    $settingsNoCache = array();
    $settingsNoCache['chat_user_name'] = $device['chat_user_name'];
    $settingsNoCache['device_num'] = (string) $device['device_num'];
    $settingsNoCache['is_first_homepage_visit'] = $isFirstHomepageVisit;
    $settingsNoCache['themes_css_file'] = $themesCSSFile;
    $settingsNoCache['help_messages_highlight'] = $helpMessagesHighlight;
    return $settingsNoCache;
  }

  public function getSettings($type = FALSE) {
    // Get setting type, or if no type, get all settings
    $settings = array();
    $settingsFunctionTitles = array(
      'general' => 'settingsGetGeneralAll',
      'greetings' => 'settingsGetGreetingAll',
      'featured' => 'settingsGetFeaturedAll',
      'forms' => 'settingsGetFormAll',
      'chat' => 'settingsGetChatAll',
    );
    $this->container->dbManagerWifi->openDb();
      foreach ($settingsFunctionTitles as $settingsType => $functionTitle) {
        if ($type === FALSE || $type === $settingsType) {
          $settings += $this->container->dbManagerWifi->$functionTitle(TRUE);
        }
      }

      // Get number of items in home directory to see if we can hide home button in language dropdown
      $this->container->dbManagerMedia->openDb();
        $homeMediaCount = count($this->container->dbManagerMedia->mediaGetHomepageWithParentPath(''));
      $this->container->dbManagerMedia->closeDb();
      
      // Additional settings (without database getters)
      $settings += array('themes_css_file' => $this->container->dbManagerWifi->settingsGetValueForKey('themes_css_file'));
      $settings += array('ssid' => $this->container->dbManagerWifi->settingsGetValueForKey('ssid'));
      $settings += array('serial_number' => $this->serialNumber);
      $settings += array('home_media_count' => $homeMediaCount);
      $settings += array('translations' => array());
    $this->container->dbManagerWifi->closeDb();

    return DbManager::formatAssociativeArrayConvertBooleans($settings);
  }

  /**
   * Get translations for vue admin views. TODO: in the future, implement GetText in Vue?
   * @return Associative Array vue translation strings
   */
  private function getVueTranslations() {
    return array(
      'chat_message_disappear' => _('All chat messages will disappear from this device when it is restarted.'),
      'chat_enable_chat' => _('Enable Chat'),
      'chat_enable_user_chat' => _('Enable User to User Chat'),
      'chat_enable_global_chat' => _('Enable Group Chat'),
      'chat_global_chat' => _('Group Chat'),
      'chat_new_message' => _('New Message'),
      'chat_settings' => _('Settings'),
      'chat_connected_users' => _('Connected Users'),
      'downloader_clips' => _('Clips - %s'),
      'wifi_disconnect' => _('Disconnect'),
      'wifi_cancel' => _('Cancel'),
      'wifi_enter_password' => _('Enter password for WiFi - %s'),
      'wifi_connect' => ('Connect to Internet'),
      'wifi_connect_message' => _('Note: only password-protected WiFi networks are supported. While connecting to the internet, you could be automatically disconnected from this device. If this occurs, please reconnect your phone or laptop to %s and return to this page.'),
      'general_error' => _('An error occured'), // TODO combine this with secure.php string
      'general_load_more' => _('Load More'),
      'downloader_current_downloads' => _('Current Downloads'),
      'downloader_download_new_media' => _('Download New Media Online'),
      'downloader_search_by_language' => _('Search By Language'),
      'downloader_add_more' => _('Add more downloads'),
      'downloader_select_language' => _('Select language for title and description localization'),
      'downloader_auto' => _('Auto'),
      'downloader_download' => _('Download'),
      'downloader_connect_charger' => _('Please connect charger during downloads'),
      'downloader_done' => _('Done'),
      'downloader_selected' => _('Selected: %s'),
      'downloader_space_available' => _('Space available: %s'),
      'downloader_select_less_media' => _('Please select less media and try again.'),
      'downloader_in_progress' => _('Current download is in progress.'),
      'downloader_downloading_files' => _('Downloading Files'),
      'downloader_number_finished' => _('Number of downloads finished: %s'),
      'downloader_number_succeeded' => _('Succeeded: %s'),
      'downloader_number_failed' => _('Failed: %s'),
      'help' => _('Help'),
      'help_settings' => _('Settings'),
      'help_chat' => _('Chat'),
      'help_chat_introduction' => _('Chat allows devices that are currently connected to the WiFi to message each other.'),
      'help_chat_introduction_note' => _('Note: You can use this page as the Admin to directly message any user, even if chat is disabled.'),
      'help_chat_group_chat' => _('To message all users connected to device at the same time, click "Group Chat".'),
      'help_chat_new_message' => _('To directly message an individual, click New Message.'),
      'help_chat_restarted' => _('Note: When this device is restarted, all chat messages will disappear.'),
      'help_chat_username' => _('Username'),
      'help_chat_username_desc' => _('The Admin and all chat users can optionally set a custom username for their device.'),
      'help_chat_enable_chat' => _('Enable Chat'),
      'help_chat_enable_chat_desc' => _('Disable this option to hide the chat functionality from users.'),
      'help_chat_enable_group_chat' => _('Enable Group Chat'),
      'help_chat_enable_group_chat_desc' => _('Group Chat enables connected users to chat with each other in one group.'),
      'help_chat_enable_user_chat' => _('Enable User to User Chat'),
      'help_chat_enable_user_chat_desc' => _('User to User Chat allows users to communicate directly with each other. The Admin will not be able to monitor these conversations.'),
      'help_downloader' => _('This page allows you to download additional Gospel media straight to your device by connecting to the internet.'),
    );
  }

  public function getBrowserLanguageDirectory() {
    $this->container->dbManagerWifi->openDb();
      $generalLanguageRedirect = $this->container->dbManagerWifi->settingsGetValueForKey('general_language_redirect');
      $isFirstHomepageVisit = $this->container->dbManagerWifi->checkIsFirstWifiVisitByDeviceId($_SESSION['device_id']);
    $this->container->dbManagerWifi->closeDb();
    
    // If jump into folder is enabled
    if ($generalLanguageRedirect === Validator::STRING_TRUE) {
      $this->container->dbManagerMedia->openDb();
        $languageDirectoriesArray = $this->container->dbManagerMedia->mediaGetLanguageDirectories(TRUE);
        if ($isFirstHomepageVisit && !empty($languageDirectoriesArray)) {
          $agent = new Agent();
          $browserAcceptLanguages = $agent->languages();
          foreach($browserAcceptLanguages as $language) {
            $languageFolderFileId = $this->container->dbManagerMedia->mediaGetHomepageLanguageFolderFileIdForBrowserLanguage($language);
            if ($languageFolderFileId !== FALSE) {
              $this->container->dbManagerMedia->closeDb();
              return $languageFolderFileId;
            }
          }
        }
      $this->container->dbManagerMedia->closeDb();
    }
    
    return DbManagerMedia::MEDIA_ROOT_DIRECTORY_FILE_ID;
  }
}
