<?php

namespace Renew;

class VersionReader
{
    /**
     * The text file for the version information
     *
     * @var string
     * @access private
     **/
    private $versionFile;
    /**
     * The pointer to the file object
     *
     * @var Object
     * @access private
     **/
    private $filePointer;
    /**
     * Initialize the class
     *
     * @param string $versionFile the path to the file holding version information
     * @return void
     * @access public
     * @author JPul
     **/
    public function __construct($versionFile)
    {
      try {
        if (file_exists($versionFile) === false) {
            throw new \InvalidArgumentException("The Version file is missing: " . $versionFile);
        }
        $this->versionFile = $versionFile;
        $this->filePointer = fopen($this->versionFile, 'r');
      } catch (\Exception $e) {
        error_log($e);
      }
    }
    /**
     * retrieve all the version information
     *
     * @return array
     * @access public
     * @author JPul
     **/
    public function findAll()
    {
        $versionData = array();
        /**
         * 1) Match letters and spaces
         * 2) Match version numbers
         */
        $pattern = '/([A-Za-z\s-]*)([0-9]*[.][0-9]*.*)/';
        while (($line = fgets($this->filePointer)) !== false) {
            preg_match($pattern, $line, $matches);
            if ((!empty($matches)) && (isset($matches[1])) && (isset($matches[2]))) {
                switch (trim($matches[1])) {
                    case 'LightStream Version':
                        $versionData['lightstream'] = $matches[2];
                        break;
                }
            }
        }
        return $versionData;
    }
    /**
     * The class is finished being used
     *
     * @return void
     * @access public
     * @author JPul
     **/
    public function __destruct()
    {
        fclose($this->filePointer);
    }
}
