#!/bin/bash

# include logging
LOG_SILENT_STDOUT=1
. /usr/bin/rwo_includeLogging.sh
. /usr/bin/rwo_includeHelpers.sh


# check for which hardware you are running on
hwPlatform=$(rwo_system.sh getHwPlatform)

if [[ "${hwPlatform}" == 'pocket-v1' ]]; then

	batVoltageFull="4150" # in mV, the full voltage for the battery
	batVoltageEmpty="3100" # in mV, the empty voltage for the battery

	_getBatDischargeCurrent () {
		# a step size of 0.5mA
		batDischargeCurrent=$(( $(( $(( $(i2cget -f -y 0 0x34 0x7C) << 5 )) + $(i2cget -f -y 0 0x34 0x7D) )) / 2 ))
	}

	getBatDischargeCurrent (){
		_getBatDischargeCurrent
		rwoPrintout "${batDischargeCurrent}"
	}

	_getBatChargeCurrent () {
		# a step size of 0.5mA
		batChargeCurrent=$(( $(( $(( $(i2cget -f -y 0 0x34 0x7A) << 4 )) + $(i2cget -f -y 0 0x34 0x7B) )) / 2 ))
	}

	getBatChargeCurrent (){
		_getBatChargeCurrent
		rwoPrintout "${batChargeCurrent}"
	}

	_getBatVoltage () {
		batVoltUpper8=$(($(i2cget -y -f 0 0x34 0x78)))
		batVoltLower4=$(($(i2cget -y -f 0 0x34 0x79)))
		# a step size of 1.1mV so multiply by 11 and divide by 10
		batVoltage=$(( ($batVoltUpper8 * 16 + ($batVoltLower4 & 0x0F) ) * 11 / 10 ))
	}

	getBatVoltage (){
		_getBatVoltage
		rwoPrintout "${batVoltage}"
	}

	_getBatNumLeds () {
		_getBatVoltage # start by getting the battery voltage
		# %   mV    mv/17.6mV
		# 20	3477	197
		# 35	3592	204
		# 45	3674	209
		# 55	3796	215
		# 65	3917	222
		# 85	4170	237
		batNumLeds=0
		if [[ $batVoltage -lt 3477 ]]; then
			batNumLeds=1
		elif [[ $batVoltage -lt 3592 ]]; then
			batNumLeds=2
		elif [[ $batVoltage -lt 3674 ]]; then
			batNumLeds=3
		elif [[ $batVoltage -lt 3796 ]]; then
			batNumLeds=4
		elif [[ $batVoltage -lt 3917 ]]; then
			batNumLeds=5
		elif [[ $batVoltage -lt 4170 ]]; then
			batNumLeds=6
		else
			batNumLeds=7
		fi
	}

	getBatNumLeds () {
		_getBatNumLeds
		rwoPrintout "${batNumLeds}"
	}

	_getBatPct () {
		# the standard way of reading the battery percentage is below
		# it tends to be incorrect, instead let's use the voltage and do math
		#i2cget -y -f 0 0x34 0xb9
		#batPct=$(($(i2cget -y -f 0 0x34 0xb9)))

		_getBatVoltage # start by getting the battery voltage
		if [[ $batVoltage -lt $batVoltageEmpty ]]; then
			batPct=0
		elif [[ $batVoltage -gt $batVoltageFull ]]; then
			batPct=100
		else
			vUp=$(( $batVoltage - $batVoltageEmpty ))
			batPct=$(( $vUp * 100 / ( $batVoltageFull-$batVoltageEmpty ) ))
		fi
	}

	getBatPct (){
		_getBatPct
		rwoPrintout "${batPct}"
	}

	_getChargerIsPlugged (){
		chargerIsPlugged=0
		chargerIsPlugged=$(($(i2cget -y -f 0 0x34 0x00) & 0x80))
		if [[ $chargerIsPlugged -gt 0 ]]; then
			chargerIsPlugged=1
		elif [[ $cargerIsPlugged -lt 0 ]]; then
			chargerIsPlugged=0
		fi
	}

	getChargerIsPlugged (){
		_getChargerIsPlugged
		rwoPrintout "${chargerIsPlugged}"
	}


	_getBatIsCharging (){
		batIsCharging=0
		batIsCharging=$(($(i2cget -y -f 0 0x34 0x01) & 0x40))
		if [[ $batIsCharging -gt 0 ]]; then
			batIsCharging=1
		elif [[ $batIsCharging -lt 0 ]]; then
			batIsCharging=0
		fi
	}

	getBatIsCharging (){
		_getBatIsCharging
		rwoPrintout "${batIsCharging}"
	}

	function batterySetChargeCurrent {
		local current="$1"
		if [[ $current == '' ]]; then
			return 1
		fi
		if [[ $current -lt 300 ]] || [[ $current -gt 1800 ]]; then
			return 1
		fi
		local value=$(printf "%X\n" $(( (${current} - 300) / 100 )))
		i2cset -f -y -m 0x0F 0 0x34 0x33 0xC${value}
	}

	setAxp209OnCharge () {
		# REFERENCE: http://linux-sunxi.org/AXP209#Overview
		# vbus no voltage drop limit, vbus current no limit
		i2cset -f -y 0 0x34 0x30 0x03
		# open, 4.2v, <10%, 1200mA max charge current
		i2cset -f -y 0 0x34 0x33 0xC9
	}

	setAxp209CoulombCountEnable () {
		# REFERENCE: http://linux-sunxi.org/AXP209#Overview
		i2cset -f -y 0 0x34 0xB8 0x80
	}
elif [[ "${hwPlatform}" == 'pocket v2' ]]; then
	# TODO ...
	command
elif [[ "${hwPlatform}" =~ ^rpi[1-3]$ ]]; then
	rwoLog "ERROR: trying to call battery script on RPi platform"
	exit 1
fi

##################### MAIN ###################
# this allows the first argument to be a function
# the function is called with the other arguments after the function
helperCallFirstArgIfFunctionAndExit $@
rwoLog "ERROR: unkonwn command \"${1}\""
