#!/usr/bin/python2
#
# RENEW OUTREACH, LLC ("COMPANY") CONFIDENTIAL
# Unpublished Copyright (c) 2013 Renew Outreach, LLC, All Rights Reserved.
#
# NOTICE:  All information contained herein is, and remains the property of COMPANY. The intellectual and technical concepts contained
# herein are proprietary to COMPANY and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
# Dissemination of this information or reproduction of this material is strictly forbidden unless prior written permission is obtained
# from COMPANY.  Access to the source code contained herein is hereby forbidden to anyone except current COMPANY employees, managers or contractors who have executed
# Confidentiality and Non-disclosure agreements explicitly covering such access.
#
# The copyright notice above does not evidence any actual or intended publication or disclosure of this source code, which includes
# information that is confidential and/or proprietary, and is a trade secret, of COMPANY.  ANY REPRODUCTION, MODIFICATION, DISTRIBUTION, PUBLIC PERFORMANCE,
# OR PUBLIC DISPLAY OF OR THROUGH USE OF THIS SOURCE CODE WITHOUT THE EXPRESS WRITTEN CONSENT OF COMPANY IS STRICTLY PROHIBITED, AND IN VIOLATION OF APPLICABLE
# LAWS AND INTERNATIONAL TREATIES.  THE RECEIPT OR POSSESSION OF THIS SOURCE CODE AND/OR RELATED INFORMATION DOES NOT CONVEY OR IMPLY ANY RIGHTS
# TO REPRODUCE, DISCLOSE OR DISTRIBUTE ITS CONTENTS, OR TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT MAY DESCRIBE, IN WHOLE OR IN PART.

import ConfigParser
import dbus
import dbus.glib
import dbus.mainloop.glib
import dbus.service
import fcntl
import gobject
import grp
import logging.handlers
import os
import pwd
import Queue
import rwo_hciconfig
import rwo_logging
import stat
import sqlite3
import subprocess
import sys
import threading
import time
import traceback
from datetime import date
from datetime import datetime
from collections import namedtuple
from optparse import OptionParser

# import argparse
# import logging
_G_LOG_FMT = '%(asctime)s %(levelname)s %(message)s'
_G_DATE_FMT = '%Y-%m-%d %H:%M:%S'
# _DAYS_LIST = ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday']

BLUEZ_SERVICE_NAME = 'org.bluez'
BLUEZ_PATH = '/org/bluez'
BLUEZ_AGENT_MANAGER_INTERFACE = BLUEZ_SERVICE_NAME + '.AgentManager1'
BLUEZ_AGENT_INTERFACE = BLUEZ_SERVICE_NAME + '.Agent1'

# logging
_g_rwoLog = rwo_logging.RwoLogger()
_g_dbFile = '/etc/lightstream/data/lsbt.sqlite3'
_g_inProcessList = None  # SafeSimpleList()
_g_btDeviceName = 'LightStream'
_g_exiting = False
_g_srcdev_writer_object_dict = {'hci0': None, 'hci1': None, 'hci2': None, 'hci3': None, 'hci4': None, 'hci5': None, 'hci6': None, 'hci7': None, 'hci8': None, 'hci9': None, 'hci10': None, 'hci11': None}
_g_sqlite3WriteLock = threading.RLock()
DataFileTup = namedtuple('DataFileTup', ['dFileName', 'dFileType', 'dFilePath'])


class DeviceWriter(threading.Thread):

	def __init__(self):
		threading.Thread.__init__(self)
		self.srcDev = None
		self.srcAddr = None
		self.channel = None
		self.cancelThread = False
		self.mediaDir = ''
		self.sleepBeforeStartTime = 5
		self.dbFile = ''
		self.mediaDbFile = ''
		self.scan_for_devices = True
		self.sqlConn = None
		self.mediaSql = None
		self.bluetooth_device_list_inbound = SafeSimpleList()
		self.scan_delay = 15
		self.time_of_last_scan = None
		self.fail_reject_break_count = 1
		self.max_push_retry = 3
		self.reject_delay = 24
		self.scanned_devices_list = []

	def __str__(self):
		# returns the string representation of the class
		return (
				'{srcDev:%s, srcAddr:%s '
				'channel:%s, mediaDir:%s, '
				'sleepBeforeStartTime:%s, '
				'dbFile:%s, sqlConn:%s, '
				'bluetooth_device_list_inbound:%s, '
				'scan_delay:%s}'
				% (
					str(self.srcDev), str(self.srcAddr),
					str(self.channel), str(self.mediaDir),
					str(self.sleepBeforeStartTime),
					str(self.dbFile), str(self.sqlConn),
					str(self.bluetooth_device_list_inbound),
					str(self.scan_delay)))

	def run(self):
		global _g_rwoLog
		global _g_exiting
		# try getting a new sqlite3 connection
		self.sqlConn = sqlGetConn(self.dbFile)
		self.mediaSql = sqlGetConn(self.mediaDbFile)
		while not self.cancelThread and not _g_exiting:
			try:
				# this is the list of scanned devices that this thread has found
				self.scanned_devices_list = []
				# if this thread scans for devices
				if self.scan_for_devices:
					# scan for devices, and write to the sql
					if self._isTimeForScan():
						self._scanForDevices()
					# check to see if any of the scanned devices are on the inbound list
					# for this_bt_dev in scanned_devices_list:
					# 	if (self.bluetooth_device_list_inbound.contains(this_bt_dev)):
					# 		scanned_devices_list.remove(this_bt_dev)
					# 	else:
					# 		_g_rwoLog.debug('DeviceWriter.run adding:{DeviceWriter:%s BluetoothDevice:%s}'
					# 						% (str(self), str(this_bt_dev)))
					else:
						# if it isn't time for a scan sleep for just a bit just to keep the CPU from taking on too much of a load
						time.sleep(0.20)
				# process and push to your inbound & outbound bluetooth devices
				self._pushToDevices()
			except Exception as e:
				_g_rwoLog.exception('DeviceWriter.run processing exception:{ DeviceWriter:%s exception:%s}' % (str(self), str(e)))
		# close the sqlite_conn
		if self.sqlConn:
			self.sqlConn.close()
			self.sqlConn = None
		# close the sqlite_conn
		if self.mediaSql:
			self.mediaSql.close()
			self.mediaSql = None

	def _scanForDevices(self):
		scanStartTime = time.time()
		self.scanned_devices_list = scan_for_devices(srcDev=self.srcDev, getDevNames=True)
		scanDuration = time.time() - scanStartTime
		self.time_of_last_scan = datetime.today()

		# write them all to the sql database
		for scanned_dev in self.scanned_devices_list:
			# if there weren't any scans of that device since the top of the hour, put it in the list ...
			scans_this_hour = sqlCountScansThisHour(self.sqlConn, scanned_dev.address, self.srcDev)
			_g_rwoLog.debug('DeviceWriter.run scanned bluetooth device DeviceWriter:%s scanned_dev:%s' % (str(self), str(scanned_dev)))
			# only store one scan per hour in the database, that way you won't fill up the database too fast
			if scans_this_hour == 0:
				sqlWriteBluetoothInteractionStats(self.sqlConn, "", scanned_dev.address, scanned_dev.name, sqlGetDateTimeStringUTCNow(), scanDuration, "scan", self.srcDev, -1)

	def _pushToDevices(self):
		# if there are any devices either in the inbound list, or in the scanned devices
		while (self.bluetooth_device_list_inbound.count() > 0) or (len(self.scanned_devices_list) > 0):
			this_bt_dev = None
			try:
				# get the next bluetooth device
				if(self.bluetooth_device_list_inbound.count() > 0):
					this_bt_dev = self.bluetooth_device_list_inbound.getObjectAtIndex(0)
					self.bluetooth_device_list_inbound.remove(this_bt_dev)
				else:
					this_bt_dev = self.scanned_devices_list[0]
				# remove the device from the scanned devices list
				if this_bt_dev in self.scanned_devices_list:
					self.scanned_devices_list.remove(this_bt_dev)
				# try to get an "inProcess" lock on this device
				# function returns 'True' if it was able to add it to the inProcessList
				# returns 'False' if it was unable to add (eg device already in process on another thread)
				if inProcessAppendBtAddrIfDoesNotExist(this_bt_dev.address):
					_g_rwoLog.debug('DeviceWriter.run processing:{DeviceWriter:%s BluetoothDevice:%s}' % (str(self), str(this_bt_dev)))
					try:
						self._do_bluetooth_device(this_bt_dev)
					except Exception as e:
						_g_rwoLog.exception('DeviceWriter.run doBluetootDevice exception:{DeviceWriter:%s BluetoothDevice:%s exception:%s}' % (str(self), str(this_bt_dev), str(e)))
					# remove the device from the in-process list
					inProcessRemoveBtAddr(this_bt_dev.address)
			except Exception as e:
				_g_rwoLog.exception('DeviceWriter.run processing exception:{DeviceWriter:%s BluetoothDevice:%s exception:%s}' % (str(self), str(this_bt_dev), str(e)))
			finally:
				# remove this device from the scanned list and the inbound list
				if this_bt_dev:
					if this_bt_dev in self.scanned_devices_list:
						self.scanned_devices_list.remove(this_bt_dev)
					self.bluetooth_device_list_inbound.remove(this_bt_dev)

	def _isTimeForScan(self):
		return (not self.time_of_last_scan) or ((datetime.today() - self.time_of_last_scan).seconds/1 > self.scan_delay)

	def _do_bluetooth_device(self, bluetooth_device):
		reject_or_fail_counter = 0
		reject_count = 0
		if bluetooth_device.is_inbound:
			# if it is an inbound connection, we may need to sleep some time to let it finish the conneciton process
			secondsAgo = (datetime.today() - bluetooth_device.timestamp).seconds/1
			sleepTime = self.sleepBeforeStartTime - secondsAgo
			_g_rwoLog.info('DeviceWriter checking:{ DeviceWriter:%s BluetoothDevice:%s seconds_ago:%s sleep_time:%s}' % (str(self), str(bluetooth_device), str(secondsAgo), str(sleepTime)))
			if (sleepTime > 0):
				time.sleep(sleepTime)
		else:
			# it is an outbound connection
			# check if it is on the bluetooth blacklist
			_g_rwoLog.info('DeviceWriter will check outbound device DeviceWriter:%s too many rejected or failed pushes for device:%s' % (str(self), str(bluetooth_device)))
			reject_count = sqlCountRejectAndFailPushes(self.sqlConn, bluetooth_device.address, within_hours=self.reject_delay)
			if (reject_count >= self.max_push_retry):
				_g_rwoLog.info('DeviceWriter checking DeviceWriter:%s too many rejected or failed pushes for device:%s' % (str(self), str(bluetooth_device)))
				return
		# the success delay in hours is the number of files * 24 hours
		datafiles = sqlGetFilesEnabledForBluetooth(self.mediaSql)
		success_delay = (len(datafiles)*24)
		for thisDataFile in datafiles:
			shortenedFileNameAndPath = thisDataFile['parent_path_rel'] + thisDataFile['file_name']
			if thisDataFile['file_extension'] is not None and len(thisDataFile['file_extension']) > 0:
				shortenedFileNameAndPath += '.' + thisDataFile['file_extension']
			fullFileNameAndPath = self.mediaDir + '/' + shortenedFileNameAndPath
			file_id = thisDataFile['file_id']
			# gets the success_count for that particular device / file_id
			success_count = sqlCountSuccessPushes(self.sqlConn, bluetooth_device.address, file_id, within_hours=success_delay)
			# an attempt count of 0 means do not try
			if success_count > 0:
				_g_rwoLog.info('DeviceWriter not attempting at this time:{DeviceWriter:%s BluetoothDevice:%s file:%s}' % (str(self), str(bluetooth_device), str(fullFileNameAndPath)))
				continue
			_g_rwoLog.info('DeviceWriter will attempt push:{DeviceWriter:%s BluetoothDevice:%s file:%s success_count:%s}' % (str(self), str(bluetooth_device), str(fullFileNameAndPath), str(success_count)))
			pushedts = datetime.today().replace(microsecond=0)
			obexPushStartTime = time.time()
			obexReturnStatus = self.writeFileByObex(fullFileNameAndPath, bluetooth_device)
			obexPushDuration = time.time() - obexPushStartTime
			_g_rwoLog.info('DeviceWriter push attempt complete:{DeviceWriter:%s BluetoothDevice:%s file:%s success_count:%s return_status:%s push_durration:%s}' % (str(self), str(bluetooth_device), str(fullFileNameAndPath), str(success_count), str(obexReturnStatus), str(obexPushDuration)))
			# write the interaction to the sqlite3 database
			# def sqlWriteBluetoothInteractionStats(sqlConn, file_path, device_id, device_name, date_time, push_durration, interaction_type, hciDevice):
			sqlWriteBluetoothInteractionStats(self.sqlConn, shortenedFileNameAndPath, bluetooth_device.address, bluetooth_device.name, sqlGetDateTimeStringUTCNow(), obexPushDuration, obexReturnStatus, self.srcDev, file_id)
			# check to see if it is a rejection or a failure
			#  break after certain number of rejections or failures
			# allow the writer to move on to the next device
			if obexReturnStatus != 'success':
				reject_or_fail_counter += 1
				if (reject_or_fail_counter >= self.fail_reject_break_count):
					break
			else:
				reject_or_fail_counter = 0

	def writeFileByObex(self, aFile, btDev):
		global _g_rwoLog
		rtcode = obexPush(destAddrStr=btDev.address, fileStr=aFile, channelStr=self.channel, srcAddrStr=self.srcAddr)
		if rtcode == 255:
			return 'success'
		elif rtcode == 1:
			return 'rejected'
		else:
			return 'failed'

##########################################################################
# ############################ OBEX #######################################
##########################################################################


# do an obexpush, return an integer return code
def obexPush(destAddrStr='', fileStr='', channelStr=None, srcAddrStr=None):
	global _g_rwoLog
	try:
		# use dbus-run-session instead of dbus-launch to run obexpush,\
		# dbus-run-session will close after obexpush exits
		cmdlineList = ['dbus-run-session', 'rwo_bluetoothObexPush.py', '-d', destAddrStr, '-f', fileStr]
		if (channelStr):
			cmdlineList.append('-c')
			cmdlineList.append(channelStr)
		if (srcAddrStr):
			cmdlineList.append('-s')
			cmdlineList.append(srcAddrStr)

		_g_rwoLog.debug('rwo_bluetooth.obexPush cmd:%s' % (str(cmdlineList)))

		# rtcode = -1
		print str(cmdlineList)
		rtcode = subprocess.call(cmdlineList)

		_g_rwoLog.debug('rwo_bluetooth.obexPush finished returncode:%s cmd:%s' % (str(rtcode), str(cmdlineList)))
	except Exception as e:
		_g_rwoLog.exception('rwo_bluetooth.obexPush exception exception:%s cmd:%s' % (str(e), str(cmdlineList)))
		rtcode = -1
	return rtcode

##########################################################################
# ############################ AGENT ######################################
##########################################################################

class BluezErrorRejected(dbus.DBusException):
	_dbus_error_name = 'org.bluez.Error.Rejected'


class BluezErrorCanceled(dbus.DBusException):
	_dbus_error_name = 'org.bluez.Error.Canceled'


class BluezAgent(dbus.service.Object):

	def __init__(self, conn=None, obj_path=None):
		_g_rwoLog.info('BluezAgent: init')
		dbus.service.Object.__init__(self, conn, obj_path)

	@dbus.service.method(BLUEZ_AGENT_INTERFACE, in_signature='', out_signature='')
	def Release(self):
		_g_rwoLog.info('BluezAgent: Release')

	@dbus.service.method(BLUEZ_AGENT_INTERFACE, in_signature='o', out_signature='s')
	def RequestPinCode(self, device):
		global _g_rwoLog
		_g_rwoLog.info('BluezAgent: RequestPinCode (%s)' % (device))
		processBluezDbusDeviceIncomingConnection(device)
		return '0000'
		# to reject
		# raise BluezErrorRejected('Connection rejected by user')

	@dbus.service.method(BLUEZ_AGENT_INTERFACE, in_signature='os', out_signature='')
	def DisplayPinCode(self, device, pincode):
		g_rwoLog.info('BluezAgent: DisplayPinCode (%s, %s)' % (device, pincode))

	@dbus.service.method(BLUEZ_AGENT_INTERFACE, in_signature='o', out_signature='u')
	def RequestPasskey(self, device):
		_g_rwoLog.info('BluezAgent: RequestPasskey (%s)' % (device))
		return 1234

	@dbus.service.method(BLUEZ_AGENT_INTERFACE, in_signature='ouq', out_signature='')
	def DisplayPasskey(self, device, passkey, entered):
		g_rwoLog.info('BluezAgent: DisplayPasskey (%s, %06u entered %u)' % (device, passkey, entered))

	@dbus.service.method(BLUEZ_AGENT_INTERFACE, in_signature='ou', out_signature='')
	def RequestConfirmation(self, device, passkey):
		global _g_rwoLog
		_g_rwoLog.info('BluezAgent: RequestConfirmation (%s, %d)' % (device, passkey))
		processBluezDbusDeviceIncomingConnection(device)
		return
		# raise BluezErrorRejected('Connection rejected by user')

	@dbus.service.method(BLUEZ_AGENT_INTERFACE, in_signature='o', out_signature='')
	def RequestAuthorization(self, device):
		global _g_rwoLog
		_g_rwoLog.info('BluezAgent: RequestAuthorization (%s)' % (device))
		processBluezDbusDeviceIncomingConnection(device)
		return
		# to reject
		# raise BluezErrorRejected('Connection rejected by user')

	@dbus.service.method(BLUEZ_AGENT_INTERFACE, in_signature='os', out_signature='')
	def AuthorizeService(self, device, uuid):
		_g_rwoLog.info('BluezAgent: AuthorizeService (%s, %s)' % (device, uuid))
		# rrauthorize = ask('Authorize connection (yes/no): ')
		# authorize = 'yes'
		# if (authorize == 'yes'):
		return
		# raise BluezErrorRejected('Connection rejected by user')

	@dbus.service.method(BLUEZ_AGENT_INTERFACE, in_signature='', out_signature='')
	def Cancel(self):
		_g_rwoLog.info('BluezAgent: Cancel')
		return


# this is the new thread for the agent class
class BluezAgentThread(threading.Thread):

	def __init__(self):
		threading.Thread.__init__(self)
		self.mainLoop = None

	def run(self):
		gobject.threads_init()
		# dbus.glib.init_threads() # this seemed to cause issues
		dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
		# SessionBus did not work
		bus = dbus.SystemBus()
		self.mainLoop = gobject.MainLoop()
		manager = dbus.Interface(bus.get_object(BLUEZ_SERVICE_NAME, BLUEZ_PATH), BLUEZ_AGENT_MANAGER_INTERFACE)
		path = '/lightstream/agent'
		agent = BluezAgent(bus, path)
		# FROM_DOC: /usr/share/doc/bluez/dbus-apis/agent-api.txt
		manager.RegisterAgent(path, 'DisplayYesNo')
		manager.RequestDefaultAgent(path)
		_g_rwoLog.info('Agent: RequestDefault')
		self.mainLoop.run()
		manager.UnregisterAgent(path)
		_g_rwoLog.info('Agent: Unregister')
		# sys.exit(_g_returncode)

	def quit(self):
		self.mainLoop.quit()

##########################################################################
# ########################## DEVICES ######################################
##########################################################################


class BluetoothDevice:

	def __init__(self):
		self.address = ""
		self.name = ""
		self.classnum = 0x000100
		self.timestamp = datetime.today().replace(microsecond=0)
		self.hcidevice = ""
		self.is_inbound = False

	def __str__(self):
		string = '{address:%s, name:%s, is_inbound:%s}' \
				% (str(self.address), str(self.name), str(self.is_inbound))
		return string

	def __eq__(self, other):
		# if addresses are equal, devices are equal
		is_equal = False
		try:
			is_equal = self.address == other.address
		except Exception as e:
			is_equal = False
		finally:
			return is_equal


class BluezDevice:

	# please see documentation at:
	# /usr/share/doc/bluez/dbus-apis/device-api.txt (for arch linux)

	# Built while using:
	# bluez 5.28-1
	# bluez-libs 5.28-1
	# bluez-utils 5.25-2

	Bluez_Service_Name = 'org.bluez'
	Bluez_Interface = 'org.bluez.Device1'

	def __init__(self, bluez_path):
		self.bluez_path = bluez_path

		bus = dbus.SystemBus()
		try:
			self.bluezObject = bus.get_object(BluezDevice.Bluez_Service_Name, self.bluez_path)
			self.bluezInterface = dbus.Interface(self.bluezObject, BluezDevice.Bluez_Interface)
			self.bluezProps = dbus.Interface(self.bluezObject, 'org.freedesktop.DBus.Properties')
		except Exception as e:
			_g_rwoLog.exception(
								'BluezDevice.__init__(self, %s) threw an exception:%s'
								% (str(bluez_path), str(e))
								)

	def __str__(self):
		return str(self.__dict__)

	def getProperty(self, str_prop_name):
		prop_value = None
		try:
			prop_value = self.bluezProps.Get(BluezDevice.Bluez_Interface, str_prop_name)
			_g_rwoLog.debug(
							'BluezDevice.getProperty(%s, %s) value equals:%s'
							% (str(self), str(str_prop_name), str(prop_value))
							)
		except Exception as e:
			_g_rwoLog.exception(
								'BluezDevice.getProperty(%s, %s) threw exception:%s'
								% (str(self), str(str_prop_name), str(e))
								)
			prop_value = None
		return prop_value

	def setProperty(self, str_prop_name, prop_value):
		success = True
		try:
			prop_value = self.bluezProps.Set(BluezDevice.Bluez_Interface, str_prop_name, prop_value)
			_g_rwoLog.debug(
							'BluezDevice.setProperty(%s, %s, %s) value equals:%s'
							% (str(self), str(str_prop_name), str(prop_value), str(prop_value))
							)
		except Exception as e:
			_g_rwoLog.exception(
								'BluezDevice.setProperty(%s, %s, %s) threw exception:%s'
								% (str(self), str(str_prop_name), str(prop_value), str(e))
								)
			success = False
		return success

	def getAddress(self):
		return self.getProperty('Address')

	def getName(self):
		return self.getProperty('Name')

	def getIcon(self):
		return self.getProperty('Icon')

	def getClass(self):
		return self.getProperty('Class')

	def getAppearance(self):
		return self.getProperty('Appearance')

	def getUUIDs(self):
		return self.getProperty('UUIDs')

	def getPaired(self):
		return self.getProperty('Paired')

	def getTrusted(self):
		return self.getProperty('Trusted')

	def setTrusted(self, value):
		return self.setProperty('Trusted', value)

	def getBlocked(self):
		return self.getProperty('Blocked')

	def setBlocked(self, value):
		return self.setProperty('Blocked', value)

	def getAlias(self):
		return self.getProperty('Alias')

	def setAlias(self, value):
		return self.setProperty('Alias', value)

	def getAdapter(self):
		return self.getProperty('Adapter')

	def getLegacyPairing(self):
		return self.getProperty('LegacyPairing')

	def getModalias(self):
		return self.getProperty('Modalias')

	def getRSSI(self):
		return self.getProperty('RSSI')

	def connect(self):
		self.bluezInterface.Connect()

	def disconnect(self):
		self.bluezInterface.Disconnect()

	def connectProfile(self, uuid):
		self.bluezInterface.ConnectProfile(uuid)

	def disconnectProfile(self, uuid):
		self.bluezInterface.DisconnectProfile(uuid)


class BluezAdapter:

	# please see documentation at:
	# /usr/share/doc/bluez/dbus-apis/adapter-api.txt (for arch linux)

	# Built while using:
	# bluez 5.28-1
	# bluez-libs 5.28-1
	# bluez-utils 5.25-2

	Bluez_Service_Name = 'org.bluez'
	Bluez_Interface = 'org.bluez.Adapter1'

	def __init__(self, bluez_path):
		self.bluez_path = bluez_path

		bus = dbus.SystemBus()
		try:
			self.bluezObject = bus.get_object(BluezAdapter.Bluez_Service_Name, self.bluez_path)
			self.bluezInterface = dbus.Interface(self.bluezObject, BluezAdapter.Bluez_Interface)
			self.bluezProps = dbus.Interface(self.bluezObject, 'org.freedesktop.DBus.Properties')
		except Exception as e:
			_g_rwoLog.exception(
								'BluezAdapter.__init__(self, %s) threw an exception:%s'
								% (str(bluez_path), str(e))
								)

	def __str__(self):
		return str(self.__dict__)

	def getProperty(self, str_prop_name):
		prop_value = None
		try:
			prop_value = self.bluezProps.Get(BluezAdapter.Bluez_Interface, str_prop_name)
			_g_rwoLog.debug(
							'BluezAdapter.getProperty(%s, %s) value equals:%s'
							% (str(self), str(str_prop_name), str(prop_value))
							)
		except Exception as e:
			_g_rwoLog.exception(
								'BluezAdapter.getProperty(%s, %s) threw exception:%s'
								% (str(self), str(str_prop_name), str(e))
								)
			prop_value = None
		return prop_value

	def setProperty(self, str_prop_name, prop_value):
		success = True
		try:
			prop_value = self.bluezProps.Set(BluezAdapter.Bluez_Interface, str_prop_name, prop_value)
			_g_rwoLog.debug(
							'BluezAdapter.setProperty(%s, %s, %s) value equals:%s'
							% (str(self), str(str_prop_name), str(prop_value), str(prop_value))
							)
		except Exception as e:
			_g_rwoLog.exception(
								'BluezAdapter.setProperty(%s, %s, %s) threw exception:%s'
								% (str(self), str(str_prop_name), str(prop_value), str(e)))
			success = False
		return success

	def getAddress(self):
		return self.getProperty('Address')

	def getName(self):
		return self.getProperty('Name')

	def getAlias(self):
		return self.getProperty('Alias')

	def setAlias(self, value):
		return self.setProperty('Alias', value)

	def getClass(self):
		return self.getProperty('Class')

	def getPowered(self):
		return self.getProperty('Powered')

	def setPowered(self, value):
		return self.setProperty('Powered', value)

	def getDiscoverable(self):
		return self.getProperty('Discoverable')

	def setDiscoverable(self, value):
		return self.setProperty('Discoverable', value)

	def getPairable(self):
		return self.getProperty('Pairable')

	def setPairable(self, value):
		return self.setProperty('Pairable', value)

	def getPairableTimeout(self):
		return self.getProperty('PairableTimeout')

	def setPairableTimeout(self, value):
		return self.setProperty('PairableTimeout', value)

	def getDiscoverableTimeout(self):
		return self.getProperty('DiscoverableTimeout')

	def setDiscoverableTimeout(self, value):
		return self.setProperty('DiscoverableTimeout', value)

	def getDiscovering(self):
		return self.getProperty('Discovering')

	def getUUIDs(self):
		return self.getProperty('UUIDs')

	def getModalias(self):
		return self.getProperty('Modalias')

	def startDiscovery(self):
		self.bluezInterface.StartDiscovery()

	def stopDiscovery(self):
		self.bluezInterface.StopDiscovery()

	def removeDevice(self, bluez_device):
		self.bluezInterface.RemoveDevice(bluez_device)


def scan_for_devices(srcDev=None, getDevNames=False):
	global _g_rwoLog

	_g_rwoLog.info('rwo_bluetooth.scan_for_devices:{srcdev:%s getDevNames:%s}' % (str(srcDev), str(getDevNames)))
	devlist = []
	out = None

	try:
		cmdList = ['hcitool']
		if (srcDev):
			cmdList.append('-i')
			cmdList.append(srcDev)
		cmdList.append('inq')
		p = subprocess.Popen(cmdList, stdout=subprocess.PIPE)
		out = p.communicate()
	except Exception as e:
		exc_type, exc_value, exc_traceback = sys.exc_info()
		lines = traceback.format_exception(exc_type, exc_value, exc_traceback)
		# print ''.join('!! ' + line for line in lines)  # Log it or whatever here
	if p.returncode == 0:

		# scannedts = datetime.today().replace(microsecond=0)
		scannedts = datetime.today()

		s = out[0].splitlines()
		# printMsg('scan finished:' + out[0])
		for line in s:
			if line.lstrip().startswith('Inquiring'):
				continue
			devinfo = line.lstrip().split('\t')

			# check whether the class of the remote device is either
			# 0x000200 (phone) or 0x000100 (computer)
			btDevClassInt = int(devinfo[2][7:], 16)
			isvalidclass = ((btDevClassInt & 0x000200) == 0x000200) or ((btDevClassInt & 0x000100) == 0x000100)
			if isvalidclass:
				btDevAddr = devinfo[0].strip(' \t\n\r')
				btDevName = ''

				if (getDevNames):
					btDevName = hcitool_get_name_for_device(btDevAddr, srcDev=srcDev)

				bluetoothDevice = BluetoothDevice()
				bluetoothDevice.address = btDevAddr
				bluetoothDevice.name = btDevName
				bluetoothDevice.classnum = btDevClassInt
				bluetoothDevice.is_inbound = False
				bluetoothDevice.timestamp = scannedts
				bluetoothDevice.hcidevice = str(srcDev)

				devlist.append(bluetoothDevice)
	else:
		_g_rwoLog.error('scan tool failed with return code %d' % p.returncode)

	return devlist


def hcitool_get_name_for_device(btDevAddr, srcDev=None):
	btDevName = ''
	try:
		cmdList = ['hcitool']
		if (srcDev):
			cmdList.append('-i')
			cmdList.append(srcDev)
		cmdList.append('name')
		cmdList.append(btDevAddr)

		p = subprocess.Popen(cmdList, stdout=subprocess.PIPE)
		out = p.communicate()

		if p.returncode == 0:
			btDevName = out[0].rstrip('\n')
	except Exception as e:
		return ''
	finally:
		return btDevName

##########################################################################
# ########################## ADAPTERS #####################################
##########################################################################


# configures all the adapters that the system can find
def adaptersConfigureAll():
	hciDevicesList = adaptersGetList()
	adaptersConfigureInList(hciDevicesList)


# configures that adapters in a specified list
def adaptersConfigureInList(hciDevicesList):
	global _g_rwoLog
	global _g_btDeviceName

	_g_rwoLog.info('Configuring hci devices:%s' % (str(hciDevicesList)))

	if (not hciDevicesList) or (type(hciDevicesList) != list):
		_g_rwoLog.error('rwo_bluetooth unable to get list of hci devices:%s' % (str(hciDevicesList)))
		return
	if (len(hciDevicesList) == 0):
		_g_rwoLog.error('rwo_bluetooth unable to find any local hci devices:%s' % (str(hciDevicesList)))
		return

	aclMtu = '9999:8'
	scanMode = 'piscan'
	authMode = 'noauth'
	encryptMode = 'noencrypt'

	# first reset all the hci devices
	_g_rwoLog.info('rwo_bluetooth resetting hci devices in list:%s' % (str(hciDevicesList)))
	rwo_hciconfig.runCmdOnDevicesInList(hciDevicesList, ['reset'])

	_g_rwoLog.info('rwo_bluetooth bringing up hci devices in list:%s' % (str(hciDevicesList)))
	rwo_hciconfig.runCmdOnDevicesInList(hciDevicesList, ['up'])

	_g_rwoLog.info('rwo_bluetooth setting aclmtu:%s hci devices in list:%s' % (str(aclMtu), str(str(hciDevicesList))))
	rwo_hciconfig.runCmdOnDevicesInList(hciDevicesList, ['aclmtu', aclMtu])

	_g_rwoLog.info('rwo_bluetooth setting bt scan mode:%s hci devices in list:%s' % (str(scanMode), str(str(hciDevicesList))))
	rwo_hciconfig.runCmdOnDevicesInList(hciDevicesList, [scanMode])

	_g_rwoLog.info('rwo_bluetooth setting bt auth mode:%s hci devices in list:%s' % (str(authMode), str(str(hciDevicesList))))
	rwo_hciconfig.runCmdOnDevicesInList(hciDevicesList, [authMode])

	_g_rwoLog.info('rwo_bluetooth setting bt encrypt mode:%s hci devices in list:%s' % (str(encryptMode), str(str(hciDevicesList))))
	rwo_hciconfig.runCmdOnDevicesInList(hciDevicesList, [encryptMode])

	for i in range(0, len(hciDevicesList)):
		hciXStr = hciDevicesList[i]
		hciNumStr = hciXStr.lstrip('hci')
		appendstring = ''
		if hciNumStr != '0':
			appendstring = '(%s)' % (str(hciNumStr))
		appendedName = _g_btDeviceName + appendstring
		bluezAdapter = BluezAdapter("/org/bluez/hci%d" % i)
		bluezAdapter.setAlias(appendedName)


# get a full list of hci adapters that are up
def adaptersGetListUp():
	return rwo_hciconfig.getDevListWithDevicesThatHaveStringInState('UP ')


# get a full list of hci adapters that are down
def adaptersGetListDown():
	return rwo_hciconfig.getDevListWithDevicesThatHaveStringInState('DOWN ')


# gets the full list of hci adapters
def adaptersGetList():
	return rwo_hciconfig.getDevList()


# will wait in a loop until one adapter is up
def adaptersConfigureAllAndWaitForUp():
	global _g_rwoLog
	adaptersConfigureAll()
	hciDevicesUp = adaptersGetListUp()
	while (len(hciDevicesUp) == 0):
		_g_rwoLog.info('Waiting for hci devices to be up ...')
		time.sleep(10)
		adaptersConfigureAll()
		hciDevicesUp = adaptersGetListUp()
	_g_rwoLog.info('Done waiting for hci devices to be up:%s' % (str(hciDevicesUp)))

##########################################################################
# ############################ LOGGER #####################################
##########################################################################


def loggerConfigure(logFile, loggerName='rwo_bluetooth', level='DEBUG'):
	global _g_rwoLog
	global _G_LOG_FMT
	logger = None
	try:
		logger = logging.getLogger(loggerName)
		logger.setLevel(level)
		handler = logging.handlers.RotatingFileHandler(logFile, maxBytes=200000, backupCount=5)
		handler.setLevel(level)
		formatter = logging.Formatter(_G_LOG_FMT)
		handler.setFormatter(formatter)
		logger.addHandler(handler)
	except Exception as e:
		_g_rwoLog.exception(
							'Unable to configure logger:%s level:%s exception:%s'
							% (str(loggerName), str(level), str(e)))
	return logger

##########################################################################
# ############################ In Process #################################
##########################################################################


# returns TRUE if the BT address is in process
def inProcessIsBtAddr(btAddrStr):
	global _g_rwoLog
	global _g_inProcessList
	isInProcess = _g_inProcessList.contains(btAddrStr)
	_g_rwoLog.debug('rwo_bluetooth.inProcessIsBtAddr:{ device_addr:%s inProcessList:%s isInProcess:%s}' % (str(btAddrStr), str(_g_inProcessList), str(isInProcess)))
	return isInProcess


# appends a bluetooth address to the list
# return true if the item was appended
# return false if the item was not
def inProcessAppendBtAddrIfDoesNotExist(btAddrStr):
	global _g_rwoLog
	global _g_inProcessList
	_g_rwoLog.debug('rwo_bluetooth.inProcessAppendBtAddrIfDoesNotExist:{device_addr:%s inProcessList:%s}' % (str(btAddrStr), str(_g_inProcessList)))
	return _g_inProcessList.append(btAddrStr, allow_duplicates=False)


# removes a bluetooth address from the list
def inProcessRemoveBtAddr(btAddrStr):
	global _g_rwoLog
	global _g_inProcessList
	_g_rwoLog.debug('rwo_bluetooth.inProcessRemoveBtAddr:{ device_addr:%s inProcessList:%s}' % (str(btAddrStr), str(_g_inProcessList)))
	_g_inProcessList.remove(btAddrStr)


def inProcessCount():
	global _g_rwoLog
	global _g_inProcessList
	aCount = count(_g_inProcessList)
	_g_rwoLog.debug('rwo_bluetooth.inProcessCount{ count:%s inProcessList:%s}' % (str(aCount), str(_g_inProcessList)))
	return aCount


##########################################################################
# ############################## SQL ######################################
##########################################################################

# return the datetime to be used by sqlite3 "YYYY-MM-DD HH:MM:SS.sss"
def sqlGetDateTimeStringUTCNow():
	return datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]  # trim last 3 digits for .sss

# # will return >1 if it is on the blacklist and <1 if it is not on the blacklist
# def sqlCheckDeviceIdIsOnBlacklist(sqlConn, device_id):
# 	 global _g_rwoLog
# 	 global _g_sqlite3WriteLock
# 	 count = 0
# 	 _g_sqlite3WriteLock.acquire(blocking = 1)
# 	 try:
# 		 _g_rwoLog.info('rwo_bluetooth.sqlCheckDeviceIdIsOnBlacklist sqlConn:%s btDevAddr:%s' % (str(sqlConn), str(device_id)))
# 		 sqlConn.text_factory = str
# 		 cursor = sqlConn.execute("SELECT COUNT(device_id) FROM bluetooth_blacklist WHERE device_id=?", (device_id,))
# 		 count = cursor.fetchone()[0]

# 	 except Exception as e:
# 		 _g_rwoLog.exception('rwo_bluetooth.sqlCheckDeviceIdIsOnBlacklist exception:%s' % (str(e)))
# 		 count = 0

# 	 finally:
# 		 _g_sqlite3WriteLock.release()
# 		 return count

#  def sqlAddDeviceIdToBlacklist(sqlConn, device_id, device_name=''):
# 	 global _g_rwoLog
# 	 global _g_sqlite3WriteLock
# 	 _g_sqlite3WriteLock.acquire(blocking = 1)
# 	 try:
# 		 _g_rwoLog.info('rwo_bluetooth.sqlAddDeviceIdToBlacklist sqlConn:%s device_id:%s' % (str(sqlConn), str(device_id)))
# 		 sqlConn.text_factory = str
# 		 sqlConn.execute("INSERT INTO bluetooth_blacklist (device_id, device_name) VALUES(?,?)", (device_id, device_name))
# 		 sqlConn.commit()
# 	 except Exception as e:
# 		 _g_rwoLog.exception('rwo_bluetooth.sqlAddDeviceIdToBlacklist exception:%s' % (str(e)))
# 	 finally:
# 		 _g_sqlite3WriteLock.release()

# def sqlDeleteDeviceIdFromBlacklist(sqlConn, device_id):
# 	 global _g_rwoLog
# 	 global _g_sqlite3WriteLock
# 	 _g_sqlite3WriteLock.acquire(blocking = 1)
# 	 try:
# 		 _g_rwoLog.info('rwo_bluetooth.sqlDeleteDeviceIdFromBlacklist sqlConn:%s device_id:%s' % (str(sqlConn), str(device_id)))
# 		 sqlConn.text_factory = str
# 		 sqlConn.execute("DELETE FROM bluetooth_blacklist WHERE device_id=?", (device_id,))
# 		 sqlConn.commit()
# 	 except Exception as e:
# 		 _g_rwoLog.exception('rwo_bluetooth.sqlDeleteDeviceIdFromBlacklist exception:%s' % (str(e)))
# 	 finally:
# 		 _g_sqlite3WriteLock.release()


def sqlGetFilesEnabledForBluetooth(sqlConn):
	global _g_rwoLog
	global _g_sqlite3WriteLock
	media_array = []
	_g_sqlite3WriteLock.acquire(blocking=1)
	try:
		_g_rwoLog.info('rwo_bluetooth.sqlGetFilesEnabledForBluetooth sqlConn:%s' % (str(sqlConn)))
		sqlConn.text_factory = str
		sqlConn.row_factory = sqlite3.Row  # this should make each row a dictionary
		cursor = sqlConn.execute("SELECT * FROM media2 WHERE bluetooth_status > 0 AND mime_part1 != 'directory' AND is_deleted = 0 ORDER BY parent_path_rel, file_name ASC")
		media_array = cursor.fetchall()  # should be a list of dictionaries
	except Exception as e:
		_g_rwoLog.exception('rwo_bluetooth.sqlGetFilesEnabledForBluetooth exception:%s' % (str(e)))
		media_array = []
	finally:
		_g_sqlite3WriteLock.release()
		return media_array


def sqlCountScansThisHour(sqlConn, btDevAddr, hciDevice):
	global _g_rwoLog
	global _g_sqlite3WriteLock
	count = 0
	_g_sqlite3WriteLock.acquire(blocking=1)
	try:
		_g_rwoLog.info('rwo_bluetooth.sqlCountScansThisHour is file push allowed devid:%s hciDevice:%s' % (str(btDevAddr), str(hciDevice)))
		sqlConn.text_factory = str
		cursor = sqlConn.execute(
		  """
			SELECT count(interaction_id) FROM bluetooth_interaction_stats
			WHERE device_id=?
			AND interaction_type = 'scan'
			AND hci_device =?
			AND date_time BETWEEN strftime('%Y-%m-%d %H:00:00.000', 'now') AND strftime('%Y-%m-%d %H:00:00.000', 'now', '+1 hour')
		""", (btDevAddr, hciDevice)
			)

		# return the number of scans this hour
		count = cursor.fetchone()[0]

	except Exception as e:
		_g_rwoLog.exception('rwo_bluetooth.sqlCountScansThisHour exception:%s' % (str(e)))
		count = -1

	finally:
		_g_sqlite3WriteLock.release()
		return count


def sqlWriteBluetoothInteractionStats(sqlConn, file_path, device_id, device_name, date_time, push_durration, interaction_type, hciDevice, file_id):
	global _g_rwoLog
	global _g_sqlite3WriteLock
	_g_sqlite3WriteLock.acquire(blocking=1)
	try:
		ta = []
		_g_rwoLog.debug('inserting the data to bluetooth_interaction_stats')
		query = "INSERT INTO bluetooth_interaction_stats"
		query += "(file_path,device_id,device_name,date_time,push_durration,interaction_type, hci_device, file_id)"
		query += "VALUES(?,?,?,?,?,?,?,?)"
		sqlConn.execute(query, (file_path, device_id, device_name, date_time, push_durration, interaction_type, hciDevice, file_id))
		sqlConn.commit()
	except Exception as e:
		_g_rwoLog.exception('rwo_bluetooth.sqlWriteBluetoothInteractionStats exception:%s' % (str(e)))
	finally:
		_g_sqlite3WriteLock.release()


def sqlCountRejectAndFailPushes(sqlConn, device_id, within_hours=24):
	global _g_rwoLog
	global _G_DATE_FMT
	global _g_sqlite3WriteLock
	count = 0
	_g_sqlite3WriteLock.acquire(blocking=1)
	try:
		within_hours_str = "-%i hours" % (within_hours)
		_g_rwoLog.info('rwo_bluetooth.sqlCountRejectAndFailPushes device_id:%s within_hours_str:%s' % (str(device_id), str(within_hours_str)))
		sqlConn.text_factory = str
		query = "SELECT count(interaction_id) FROM bluetooth_interaction_stats "
		query += "WHERE device_id=? "
		query += "AND (interaction_type = 'rejected' OR interaction_type = 'failed') "
		query += "AND date_time BETWEEN DATETIME('now', ?) AND DATETIME('now') "
		cursor = sqlConn.execute(query, (device_id, within_hours_str))
		count = cursor.fetchone()[0]
		# return a 0 if there was a successful push already, this means don't try t push again now
		if (count < 0):
			count = 0
		_g_rwoLog.info('rwo_bluetooth.sqlCountRejectAndFailPushes device_id:%s within_hours_str:%s count:%s' % (str(device_id), str(within_hours_str), str(count)))

	except Exception as e:
		_g_rwoLog.exception('rwo_bluetooth.sqlCountRejectAndFailPushes device_id:%s within_hours_str:%s exception:%s' % (str(device_id), str(within_hours_str), str(e)))

	finally:
		_g_sqlite3WriteLock.release()
		return count


def sqlCountSuccessPushes(sqlConn, device_id, file_id, within_hours=24):
	global _g_rwoLog
	global _G_DATE_FMT
	global _g_sqlite3WriteLock
	count = 1
	_g_sqlite3WriteLock.acquire(blocking=1)
	try:
		within_hours_str = "-%i hours" % (within_hours)
		_g_rwoLog.info('rwo_bluetooth.sqlCountSuccessPushes is file push allowed device_id:%s file_id:%s within_hours_str:%s' % (str(device_id), str(file_id), str(within_hours_str)))
		sqlConn.text_factory = str
		query = "SELECT count(interaction_id) FROM bluetooth_interaction_stats "
		query += "WHERE device_id=? "
		query += "AND file_id=? "
		query += "AND interaction_type = 'success' "
		query += "AND date_time BETWEEN DATETIME('now', ?) AND DATETIME('now')"
		cursor = sqlConn.execute(query, (device_id, int(file_id), within_hours_str))
		count = cursor.fetchone()[0]
		# return a 0 if there was a successful push already, this means don't try t push again now
		if (count < 0):
			count = 0

		_g_rwoLog.info('rwo_bluetooth.sqlCountSuccessPushes is file push allowed device_id:%s file_id:%s within_hours_str:%s count:%s' % (str(device_id), str(file_id), str(within_hours_str), str(count)))

	except Exception as e:
		_g_rwoLog.exception('rwo_bluetooth.sqlCountSuccessPushes is file push allowed device_id:%s file_id:%s within_hours_str:%s exception:%s' % (str(device_id), str(file_id), str(within_hours_str), str(e)))
		count = 0  # if exception, then let it through

	finally:
		_g_sqlite3WriteLock.release()
		return count


def sqlGetConn(dbFile):
	# returns a sql database connection
	global _g_rwoLog
	sqlConn = None
	try:
		_g_rwoLog.debug("rwo_bluetooth.sqlGetConn dbFile:%s" % (str(dbFile)))
		sqlConn = sqlite3.connect(dbFile)
		_g_rwoLog.debug("rwo_bluetooth.sqlGetConn sqlConn:%s" % (str(sqlConn)))
	except Exception as e:
		_g_rwoLog.exception("rwo_bluetooth.sqlGetConn exception:%s" % (str(e)))
	return sqlConn


def sqlGetSetting(sqlConn, key):
	global _g_rwoLog
	value = None
	try:
		cur = sqlConn.cursor()
		cur.execute("SELECT value FROM settings WHERE key='"+key+"'")
		value = cur.fetchone()[0]
	except Exception as e:
		_g_rwoLog.exception("rwo_bluetooth.sqlUpdateSetting exception:%s" % (str(e)))

	finally:
		return value


def sqlUpdateSetting(sqlConn, key, value):
	global _g_rwoLog
	try:
		cur = sqlConn.cursor()
		cur.execute("UPDATE settings SET value='%s' WHERE key='%s'" % (value, key))
	except Exception as e:
		_g_rwoLog.exception("rwo_bluetooth.sqlUpdateSetting exception:%s" % (str(e)))

##########################################################################
# ############################### Utils ###################################
##########################################################################


def printMsg(msg):
	if msg:
		print msg


def parseBluezDbusDeviceObjectStr(device_str):
	global _g_rwoLog
	if not device_str:
		return (None, None)
	devinfo = device_str.split('/')
	hciDevice = None
	remoteMac = None
	for line in devinfo:
		if line.startswith('hci'):
			hciDevice = line
			continue
		if line.startswith('dev_'):
			remoteMac = line.lstrip('dev_').replace('_', ':')
	_g_rwoLog.debug('parseBluezDbusDeviceObject: %s , %s' % (hciDevice, remoteMac))
	return (hciDevice, remoteMac)


# there are 79 total channels in bluetooth , 32 0f them are use for inquiry
# that means there are 47 "free" channels
def getChanelForObexPushForAdapterAtIndex(adapter_index, total_adapters_count):
	channelDelta = 47//total_adapters_count  # the operator // is integer division
	thisChannel = 79 - (channelDelta * (adapter_index))
	return thisChannel


def processBluezDbusDeviceIncomingConnection(bluezdbus_device_str):
	global _g_rwoLog
	bluezdbusdevtuple = parseBluezDbusDeviceObjectStr(bluezdbus_device_str)
	# check to see if the device is in -process
	# if the device isn't in process, assume incoming
	# if it is in process, do not proceed further
	device_mac = bluezdbusdevtuple[1]
	if not inProcessIsBtAddr(device_mac):
		t = threading.Thread(target=try_to_write_to_inbound_bluez_device, args=[bluezdbus_device_str])
		t.daemon = True
		t.start()


def try_to_write_to_inbound_bluez_device(bluezdbus_device_str):
	global _g_srcdev_writer_object_dict
	bluezdbusdevtuple = parseBluezDbusDeviceObjectStr(bluezdbus_device_str)

	bluetooth_device = BluetoothDevice()
	bluetooth_device.address = bluezdbusdevtuple[1]
	bluetooth_device.hcidevice = bluezdbusdevtuple[0]
	bluetooth_device.timestamp = datetime.today()
	bluetooth_device.classnum = None
	bluetooth_device.is_inbound = True

	thisWriterObject = _g_srcdev_writer_object_dict[bluetooth_device.hcidevice]

	_g_rwoLog.debug('rwo_bluetooth.try_to_write_to_btDev trying to write to device:%s' % (str(bluetooth_device)))

	# record the inbound connection to the interaction stats
	sqlConn = sqlGetConn(thisWriterObject.dbFile)
	sqlWriteBluetoothInteractionStats(sqlConn, '', bluetooth_device.address, bluetooth_device.name, sqlGetDateTimeStringUTCNow(), 0, 'inbound', thisWriterObject.srcDev, -1)
	sqlConn.close()
	sqlConn = None

	thisWriterObject.bluetooth_device_list_inbound.append(bluetooth_device)


class SafeSimpleList:

	def __init__(self):
		# :rtype : SafeSimpleList
		global _g_rwoLog
		self.aLock = threading.RLock()
		self.aList = []
		_g_rwoLog.debug('SafeSimpleList.__init__(%s)' % (str(self)))

	def __str__(self):
		return str(self.aList)

	def count(self):
		global _g_rwoLog
		aCount = 0
		try:
			self.aLock.acquire(blocking=1)
			aCount = len(self.aList)
		except Exception as e:
			_g_rwoLog.exception('SafeSimpleList.count() exception:%s' % (str(e)))
		finally:
			self.aLock.release()
			return aCount

	def contains(self, anObject):
		global _g_rwoLog
		contains = False
		try:
			self.aLock.acquire(blocking=1)
			contains = anObject in self.aList
		except Exception as e:
			_g_rwoLog.exception('SafeSimpleList.contains(%s) exception:%s' % (str(anObject), str(e)))
		finally:
			self.aLock.release()
		return contains

	def append(self, anObject, allow_duplicates=False):
		global _g_rwoLog
		success = False
		try:
			self.aLock.acquire(blocking=1)
			if allow_duplicates or (not self.contains(anObject)):
				self.aList.append(anObject)
				success = True
		except Exception as e:
			_g_rwoLog.exception(
								'SafeSimpleList.append(%s, allow_duplicates = %s) exception:%s'
								% (str(anObject), str(allow_duplicates), str(e)))
			success = False
		finally:
			self.aLock.release()
			return success

	def remove(self, anObject):
		global _g_rwoLog
		try:
			self.aLock.acquire(blocking=1)
			if self.contains(anObject):
				self.aList.remove(anObject)
		except Exception as e:
			_g_rwoLog.exception('SafeSimpleList.remove(%s) exception:%s' % (str(anObject), str(e)))
		finally:
			self.aLock.release()

	def getObjectAtIndex(self, anIndex):
		global _g_rwoLog
		aValue = None
		try:
			self.aLock.acquire(blocking=1)
			aValue = self.aList[anIndex]
		except Exception as e:
			_g_rwoLog.exception('SafeSimpleList.getObjectAtIndex(%s) exception:%s' % (str(anIndex), str(e)))
		finally:
			self.aLock.release()
			return aValue

	def getListCopy(self):
		global _g_rwoLog
		list_copy = None
		try:
			self.aLock.acquire(blocking=1)
			list_copy = list(self.aList)
		except Exception as e:
			_g_rwoLog.exception('SafeSimpleList.getListCopy(self) exception:%s' % (str(e)))
		finally:
			self.aLock.release()
			return list_copy


##########################################################################
# ############################### Main ####################################
##########################################################################

def main():

	global _g_btDeviceName
	global _g_rwoLog
	global _G_DATE_FMT
	global _g_exiting
	global _g_srcdev_writer_object_dict
	global _g_inProcessList
	global _g_dbFile

	PID_FILE = '/var/lock/rwo_bluetooth.pid'
	USE_DAEMON_THREADS = True
	RUN_AS_PROCESS = True
	exit_code = 0
	agentThread = None

	try:
		# check to see if it is being run as a process
		if RUN_AS_PROCESS:
			# try to obtain an exclusive non-blocking lock to the rwo_bluetooth pid file,
			# and if it is already running, exit
			fd = open(PID_FILE, 'w')
			try:
				fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
				# pid = str(os.getpid())
				# f.write(pid)
			except IOError:
				printMsg('rwo_bluetooth running already. exiting')
				os.sys.exit(0)

		# start the bluetooth service ... required
		cmdList = ['systemctl']
		cmdList.append('restart')
		cmdList.append('bluetooth')
		p = subprocess.Popen(cmdList, stdout=subprocess.PIPE)
		time.sleep(1)

		_g_dbFile = "/etc/lightstream/data/lsbt.sqlite3"
		mediaDbFile = "/etc/lightstream/data/media.sqlite3"
		sqlConn = sqlGetConn(_g_dbFile)
		#mediaDir = sqlGetSetting(sqlConn, 'data_dir')
		# hardcoded by CV 20180302
		mediaDir = '/srv/http/htdocs/media/master'
		max_push_retry = int(sqlGetSetting(sqlConn, 'max_push_retry'))
		reject_delay = int(sqlGetSetting(sqlConn, 'reject_delay'))
		logFile = sqlGetSetting(sqlConn, 'engine_log')
		loglevel = sqlGetSetting(sqlConn, 'engine_log_level')
		scan_delay = int(sqlGetSetting(sqlConn, 'scan_delay'))
		_g_btDeviceName = sqlGetSetting(sqlConn, 'device_name')
		scan_for_devices = int(sqlGetSetting(sqlConn, 'scan_for_devices'))
		sqlConn.close()
		# Main logger
		rwo_bluetoothLog = loggerConfigure(logFile, loggerName='rwo_bluetooth', level=loglevel)
		_g_rwoLog.set_logger(rwo_bluetoothLog)

		# This is a workaround for a BUG
		datetime.strptime('2013-03-03 00:00:00', _G_DATE_FMT)

		_g_rwoLog.info('Starting the LightStream Bluetooth Engine...')

		# log the configuration
		_g_rwoLog.info(
			'Engine Configuration:{Media_Directory:%s Database_File:%s '
			'Reject_Delay(h):%s Max_Push_Retry:%s Engine_Log:%s Device_Name:%s '
			'Engine_Log_Level:%s scan_delay:%s}'
			% (
				str(mediaDir), str(_g_dbFile),
				str(reject_delay), str(max_push_retry), str(logFile), str(_g_btDeviceName),
				str(loglevel), str(scan_delay)))

		# check to see if the mediaDir is actually a directory on the system
		if not os.path.isdir(mediaDir):
			_g_rwoLog.error('data directory %s does not exist. please check' % mediaDir)
			os.sys.exit(1)

		# setup the inProcessList
		_g_inProcessList = SafeSimpleList()

		# setup the sql database
		# sqlInit(_g_dbFile) DEPRECIATED. this is now done in rwo_update.sh when the user runs an update  =-)

		# check for some bluetooth adapters
		allAdapters = adaptersGetList()
		if len(allAdapters) == 0:
			printMsg('EXITING: No bluetooth adatpers detected')
			_g_rwoLog.info('EXITING: No bluetooth adatpers detected')
			exit(2)

		# wait for one adapter to come up
		adaptersConfigureAllAndWaitForUp()
		adaptersUpList = adaptersGetListUp()
		_g_rwoLog.info('Done bringing up and configuring hci devices up:%s' % (str(adaptersUpList)))
		_g_rwoLog.info('Hci devices up:%s' % (str(adaptersUpList)))

		# initialize the agent (bt security)
		agentThread = BluezAgentThread()
		agentThread.daemon = USE_DAEMON_THREADS
		agentThread.start()

		# the 'hciconfig -a' dict of dictionaries
		hciconfig_dict_of_cfg_dicts = rwo_hciconfig.getAllDictOfCfgDicts()
		numberOfUpAdapters = len(adaptersUpList)

		scanOffset = scan_delay
		if numberOfUpAdapters > 0:
			scanOffset = scan_delay/numberOfUpAdapters

		# spawn off a thread for each writer
		for i in range(0, numberOfUpAdapters):

			# thisChannel = getChanelForObexPushForAdapterAtIndex(i, numberOfWriters)
			thisChannel = None
			srcDev = adaptersUpList[i]
			thisCfgDict = hciconfig_dict_of_cfg_dicts[srcDev]
			thisAddress = thisCfgDict[rwo_hciconfig.KEY_BD_ADDR]
			print "HCI Device:" + str(srcDev) + " Address:" + str(thisAddress) + " channel:" + str(thisChannel)

			thisWriter = DeviceWriter()
			# TODO - remove this and figure out how to know when the device is finished pairing
			thisWriter.sleepBeforeStartTime = 6  # hack to give the device some time to connect
			thisWriter.max_push_retry = max_push_retry
			thisWriter.reject_delay = reject_delay
			thisWriter.mediaDir = mediaDir
			thisWriter.dbFile = _g_dbFile
			thisWriter.mediaDbFile = mediaDbFile
			thisWriter.srcDev = srcDev
			thisWriter.srcAddr = thisAddress.strip(' \t\n\r')
			thisWriter.scan_for_devices = (scan_for_devices > 0)
			# thisWriter.channel = str(thisChannel)
			thisWriter.daemon = USE_DAEMON_THREADS
			thisWriter.scan_delay = scan_delay
			time.sleep(1)  # offset the scanners so they both don't scan at exactly the same time
			thisWriter.start()

			# add this device writer to the list of device writers
			_g_srcdev_writer_object_dict[srcDev] = thisWriter

			# append this device to the in process list
			# in case of multiple adapters trying to write to each other
			inProcessAppendBtAddrIfDoesNotExist(thisAddress)

		# main loop
		while True:
			time.sleep(60)

	except KeyboardInterrupt as e:
		_g_rwoLog.exception('rwo_bluetooth KeyboardInterrupt exception:%s' % (str(e)))
		exit_code = 2

	except Exception as e:
		_g_rwoLog.exception('rwo_bluetooth exception:%s' % (str(e)))
		exit_code = 3

	finally:
		_g_rwoLog.info('Shutting down LightStream Bluetooth Engine ...')
		# clean up
		_g_exiting = True
		if agentThread:
			agentThread.quit()
		time.sleep(2)
		os.unlink(PID_FILE)
		sys.exit(exit_code)
		_g_rwoLog.info('Exiting LightStream Bluetooth Engine ...')


if __name__ == '__main__':
	main()
