#!/usr/bin/python2
#
# RENEW OUTREACH, LLC ("COMPANY") CONFIDENTIAL
# Unpublished Copyright (c) 2013 Renew Outreach, LLC, All Rights Reserved.
#
# NOTICE:  All information contained herein is, and remains the property of COMPANY. The intellectual and technical concepts contained
# herein are proprietary to COMPANY and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
# Dissemination of this information or reproduction of this material is strictly forbidden unless prior written permission is obtained
# from COMPANY.  Access to the source code contained herein is hereby forbidden to anyone except current COMPANY employees, managers or contractors who have executed
# Confidentiality and Non-disclosure agreements explicitly covering such access.
#
# The copyright notice above does not evidence any actual or intended publication or disclosure of this source code, which includes
# information that is confidential and/or proprietary, and is a trade secret, of COMPANY.  ANY REPRODUCTION, MODIFICATION, DISTRIBUTION, PUBLIC PERFORMANCE,
# OR PUBLIC DISPLAY OF OR THROUGH USE OF THIS SOURCE CODE WITHOUT THE EXPRESS WRITTEN CONSENT OF COMPANY IS STRICTLY PROHIBITED, AND IN VIOLATION OF APPLICABLE
# LAWS AND INTERNATIONAL TREATIES.  THE RECEIPT OR POSSESSION OF THIS SOURCE CODE AND/OR RELATED INFORMATION DOES NOT CONVEY OR IMPLY ANY RIGHTS
# TO REPRODUCE, DISCLOSE OR DISTRIBUTE ITS CONTENTS, OR TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT MAY DESCRIBE, IN WHOLE OR IN PART.

import sys
import dbus
import gobject
import dbus.mainloop.glib
import os.path
from optparse import OptionParser

BUS_NAME = 'org.bluez.obex'
PATH = '/org/bluez/obex'
CLIENT_INTERFACE = 'org.bluez.obex.Client1'
SESSION_INTERFACE = 'org.bluez.obex.Session1'
OBJECT_PUSH_INTERFACE = 'org.bluez.obex.ObjectPush1'

returncode = 0
mainloop = None

RETURN_CODE_SUCCESS = 255
RETURN_CODE_BADPARAM = 9
RETURN_CODE_NOSERVICERECORD = 2
RETURN_CODE_FAILED = 2
RETURN_CODE_REJECTED = 1

# Push file to a device using Obex Push
class ObexPush:

    def __init__(self, session_path):
        self.transfer_path = None
        bus = dbus.SessionBus()
        obj = bus.get_object(BUS_NAME, session_path)
        self.session = dbus.Interface(obj, SESSION_INTERFACE)
        self.opp = dbus.Interface(obj, OBJECT_PUSH_INTERFACE)
        bus.add_signal_receiver(self.propertieschanged,
                                dbus_interface="org.freedesktop.DBus.Properties",
                                signal_name="PropertiesChanged",
                                path_keyword="path")

    def reply(self, path, properties):
        self.transfer_path = path
        print 'ObexPush.reply(%s,%s)' \
              % (str(path), str(properties))

    def error(self, err):
        global returncode
        global RETURN_CODE_REJECTED
        global mainloop

        print 'ObexPush.error(%s)' % (str(err))
        returncode = RETURN_CODE_REJECTED
        mainloop.quit()

    def propertieschanged(self, interface, properties, invalidated, path):
        global returncode
        global RETURN_CODE_SUCCESS
        global RETURN_CODE_FAILED
        global mainloop

        #print 'ObexPush.propertieschanged(%s, %s, %s, %s)' \
        #      % (str(interface), str(properties), str(invalidated), str(path))

        if path != self.transfer_path:
            return

        if "Status" in properties:
            if properties["Status"] == "complete":
                returncode = RETURN_CODE_SUCCESS
                mainloop.quit()
                return
            elif properties["Status"] == "error":
                returncode = RETURN_CODE_FAILED
                mainloop.quit()
                return

    def pushfile(self, filename):
        self.opp.SendFile(os.path.abspath(filename),
                          reply_handler=self.reply,
                          error_handler=self.error)




def main():

    global returncode
    global RETURN_CODE_NOSERVICERECORD
    global mainloop

    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

    parser = OptionParser()
    parser.add_option("-d", "--device", dest="device", help="(REQUIRED) target device mac addr", metavar="DEVICE")
    parser.add_option("-f", "--file", dest="file", help="(REQUIRED) file path", metavar="FILE")
    parser.add_option("-s", "--source", dest="source", help="local address to be used", metavar="SOURCE")
    parser.add_option("-c", "--channel", dest="channel", help="channel to be used 0-79", metavar="CHANNEL")

    (options, args) = parser.parse_args()

    # check for required arguments
    if not options.device or not options.file:
        parser.print_help()
        sys.exit(RETURN_CODE_BADPARAM)

    bus = dbus.SessionBus()

    print bus

    mainloop = gobject.MainLoop()

    client = dbus.Interface(bus.get_object(BUS_NAME, PATH), CLIENT_INTERFACE)

    # create obex client CreateSession Dict
    clientCreateSessionDict = {}
    clientCreateSessionDict["Target"] = "OPP"  # hard coded
    if options.source:
        clientCreateSessionDict["Source"] = options.source
    if options.channel:
        clientCreateSessionDict["Channel"] = options.channel

    path = None
    try:
        #create the session
        path = client.CreateSession(options.device, clientCreateSessionDict)
    except Exception as e:
        print 'Exception:' + str(e)
        returncode = RETURN_CODE_NOSERVICERECORD
        return

    # create a new obex-push object
    obexClient = ObexPush(path)

    # push the file to the remote device using OPP
    obexClient.pushfile(options.file)

    # wait in a main loop
    mainloop.run()

    # remove the session
    #client.RemoveSession(path)


if __name__ == '__main__':
    try:
        main()
        # return the return code
        sys.exit(returncode)
    except Exception as e:
        print 'Exception:' + str(e)
        returncode = 1
        sys.exit(returncode)
