'''
Created on Oct 16, 2014

@author: root
'''

import rwo_logging
import threading

# logging
_rwo_bluetooth_log = rwo_logging.RwoLogger()
def setLogger(logger):
    _rwo_bluetooth_log.set_logger(logger)

class BtDeviceClass(object):
    
    '''
    Contains functions that deal with the class of a bluetooth device
    The class of a bluetooth device is a 24 bit value that can be received during inquiry
    
    [23][22][21][20][19][18][17][16][15][14][13][12][11][10][9][8][7][6][5][4][3][2][1]
    Bits:
        [0-1]    Format Type
        [2-7]    Minor device class
        [8-12]   Major device class 
        [13-23]  Major service classes
    
    '''
    
    __CLASS_VALUE_LIST = []
    __CLASS_NAME_LIST = []
    
    MISC = 0x00000000
    
    COMPUTER = 0x00000100
    __CLASS_VALUE_LIST.append(COMPUTER)
    __CLASS_NAME_LIST.append("COMPUTER")
    COMPUTER_DESKTOP = 0x00000104
    __CLASS_VALUE_LIST.append(COMPUTER_DESKTOP)
    __CLASS_NAME_LIST.append("COMPUTER_DESKTOP")
    COMPUTER_HANDHELD_PC_PDA = 0x00000110
    __CLASS_VALUE_LIST.append(COMPUTER_HANDHELD_PC_PDA)
    __CLASS_NAME_LIST.append("COMPUTER_HANDHELD_PC_PDA")
    COMPUTER_LAPTOP = 0x0000010c
    __CLASS_VALUE_LIST.append(COMPUTER_LAPTOP)
    __CLASS_NAME_LIST.append("COMPUTER_LAPTOP")
    COMPUTER_PALM_SIZE_PC_PDA = 0x00000114
    __CLASS_VALUE_LIST.append(COMPUTER_PALM_SIZE_PC_PDA)
    __CLASS_NAME_LIST.append("COMPUTER_PALM_SIZE_PC_PDA")
    COMPUTER_SERVER = 0x00000108
    __CLASS_VALUE_LIST.append(COMPUTER_SERVER)
    __CLASS_NAME_LIST.append("COMPUTER_SERVER")
    COMPUTER_UNCATEGORIZED = 0x00000100
    __CLASS_VALUE_LIST.append(COMPUTER_UNCATEGORIZED)
    __CLASS_NAME_LIST.append("COMPUTER_UNCATEGORIZED")
    COMPUTER_WEARABLE = 0x00000118
    __CLASS_VALUE_LIST.append(COMPUTER_WEARABLE)
    __CLASS_NAME_LIST.append("COMPUTER_WEARABLE")
    
    PHONE = 0x00000200
    __CLASS_VALUE_LIST.append(PHONE)
    __CLASS_NAME_LIST.append("PHONE")
    PHONE_CELLULAR = 0x00000204
    __CLASS_VALUE_LIST.append(PHONE_CELLULAR)
    __CLASS_NAME_LIST.append("PHONE_CELLULAR")
    PHONE_CORDLESS = 0x00000208
    __CLASS_VALUE_LIST.append(PHONE_CORDLESS)
    __CLASS_NAME_LIST.append("PHONE_CORDLESS")
    PHONE_ISDN = 0x00000214
    __CLASS_VALUE_LIST.append(PHONE_ISDN)
    __CLASS_NAME_LIST.append("PHONE_ISDN")
    PHONE_MODEM_OR_GATEWAY = 0x00000210
    __CLASS_VALUE_LIST.append(PHONE_MODEM_OR_GATEWAY)
    __CLASS_NAME_LIST.append("PHONE_MODEM_OR_GATEWAY")
    PHONE_SMART = 0x0000020c
    __CLASS_VALUE_LIST.append(PHONE_SMART)
    __CLASS_NAME_LIST.append("PHONE_SMART")
    
    NETWORKING = 0x00000300
    __CLASS_VALUE_LIST.append(NETWORKING)
    __CLASS_NAME_LIST.append("NETWORKING")
    
    AUDIO_VIDEO = 0x00000400
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO")
    AUDIO_VIDEO_CAMCORDER = 0x00000434
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_CAMCORDER)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_CAMCORDER")
    AUDIO_VIDEO_CAR_AUDIO = 0x00000420
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_CAR_AUDIO)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_CAR_AUDIO")
    AUDIO_VIDEO_HANDSFREE = 0x00000408
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_HANDSFREE)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_HANDSFREE")
    AUDIO_VIDEO_HEADPHONES = 0x00000418
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_HEADPHONES)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_HEADPHONES")
    AUDIO_VIDEO_HIFI_AUDIO = 0x00000428
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_HIFI_AUDIO)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_HIFI_AUDIO")
    AUDIO_VIDEO_LOUDSPEAKER = 0x00000414
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_LOUDSPEAKER)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_LOUDSPEAKER")
    AUDIO_VIDEO_MICROPHONE = 0x00000410
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_MICROPHONE)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_MICROPHONE")
    AUDIO_VIDEO_PORTABLE_AUDIO = 0x0000041c
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_PORTABLE_AUDIO)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_PORTABLE_AUDIO")
    AUDIO_VIDEO_SET_TOP_BOX = 0x00000424
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_SET_TOP_BOX)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_SET_TOP_BOX")
    AUDIO_VIDEO_VCR = 0x0000042c
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_VCR)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_VCR")
    AUDIO_VIDEO_VIDEO_CAMERA = 0x00000430
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_VIDEO_CAMERA)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_VIDEO_CAMERA")
    AUDIO_VIDEO_VIDEO_CONFERENCING = 0x00000440
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_VIDEO_CONFERENCING)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_VIDEO_CONFERENCING")
    AUDIO_VIDEO_VIDEO_DISPLAY_AND_LOUDSPEAKER = 0x0000043c
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_VIDEO_DISPLAY_AND_LOUDSPEAKER)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_VIDEO_DISPLAY_AND_LOUDSPEAKER")
    AUDIO_VIDEO_VIDEO_GAMING_TOY = 0x00000448
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_VIDEO_GAMING_TOY)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_VIDEO_GAMING_TOY")
    AUDIO_VIDEO_VIDEO_MONITOR = 0x00000438
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_VIDEO_MONITOR)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_VIDEO_MONITOR")
    AUDIO_VIDEO_WEARABLE_HEADSET = 0x00000404
    __CLASS_VALUE_LIST.append(AUDIO_VIDEO_WEARABLE_HEADSET)
    __CLASS_NAME_LIST.append("AUDIO_VIDEO_WEARABLE_HEADSET")
    
    PERIPHERAL = 0x00000500
    __CLASS_VALUE_LIST.append(PERIPHERAL)
    __CLASS_NAME_LIST.append("PERIPHERAL")
    
    IMAGING = 0x00000600
    __CLASS_VALUE_LIST.append(IMAGING)
    __CLASS_NAME_LIST.append("IMAGING")
    
    WEARABLE = 0x00000700
    __CLASS_VALUE_LIST.append(WEARABLE)
    __CLASS_NAME_LIST.append("WEARABLE")
    WEARABLE_GLASSES = 0x00000714
    __CLASS_VALUE_LIST.append(WEARABLE_GLASSES)
    __CLASS_NAME_LIST.append("WEARABLE_GLASSES")
    WEARABLE_HELMET = 0x00000710
    __CLASS_VALUE_LIST.append(WEARABLE_HELMET)
    __CLASS_NAME_LIST.append("WEARABLE_HELMET")
    WEARABLE_JACKET = 0x0000070c
    __CLASS_VALUE_LIST.append(WEARABLE_JACKET)
    __CLASS_NAME_LIST.append("WEARABLE_JACKET")
    WEARABLE_PAGER = 0x00000708
    __CLASS_VALUE_LIST.append(WEARABLE_PAGER)
    __CLASS_NAME_LIST.append("WEARABLE_PAGER")
    WEARABLE_WRIST_WATCH = 0x00000704
    __CLASS_VALUE_LIST.append(WEARABLE_WRIST_WATCH)
    __CLASS_NAME_LIST.append("WEARABLE_WRIST_WATCH")
    
    TOY = 0x00000800
    __CLASS_VALUE_LIST.append(TOY)
    __CLASS_NAME_LIST.append("TOY")
    TOY_CONTROLLER = 0x00000810
    __CLASS_VALUE_LIST.append(TOY_CONTROLLER)
    __CLASS_NAME_LIST.append("TOY_CONTROLLER")
    TOY_DOLL_ACTION_FIGURE = 0x0000080c
    __CLASS_VALUE_LIST.append(TOY_DOLL_ACTION_FIGURE)
    __CLASS_NAME_LIST.append("TOY_DOLL_ACTION_FIGURE")
    TOY_GAME = 0x00000814
    __CLASS_VALUE_LIST.append(TOY_GAME)
    __CLASS_NAME_LIST.append("TOY_GAME")
    TOY_ROBOT = 0x00000804
    __CLASS_VALUE_LIST.append(TOY_ROBOT)
    __CLASS_NAME_LIST.append("TOY_ROBOT")
    TOY_VEHICLE = 0x00000808
    __CLASS_VALUE_LIST.append(TOY_VEHICLE)
    __CLASS_NAME_LIST.append("TOY_VEHICLE")
    
    HEALTH = 0x00000900
    __CLASS_VALUE_LIST.append(HEALTH)
    __CLASS_NAME_LIST.append("HEALTH")
    HEALTH_BLOOD_PRESSURE = 0x00000904
    __CLASS_VALUE_LIST.append(HEALTH_BLOOD_PRESSURE)
    __CLASS_NAME_LIST.append("HEALTH_BLOOD_PRESSURE")
    HEALTH_DATA_DISPLAY = 0x0000091c
    __CLASS_VALUE_LIST.append(HEALTH_DATA_DISPLAY)
    __CLASS_NAME_LIST.append("HEALTH_DATA_DISPLAY")
    HEALTH_GLUCOSE = 0x00000910
    __CLASS_VALUE_LIST.append(HEALTH_GLUCOSE)
    __CLASS_NAME_LIST.append("HEALTH_GLUCOSE")
    HEALTH_PULSE_OXIMETER = 0x00000914
    __CLASS_VALUE_LIST.append(HEALTH_PULSE_OXIMETER)
    __CLASS_NAME_LIST.append("HEALTH_PULSE_OXIMETER")
    HEALTH_PULSE_RATE = 0x00000918
    __CLASS_VALUE_LIST.append(HEALTH_PULSE_RATE)
    __CLASS_NAME_LIST.append("HEALTH_PULSE_RATE")
    HEALTH_THERMOMETER = 0x00000908
    __CLASS_VALUE_LIST.append(HEALTH_THERMOMETER)
    __CLASS_NAME_LIST.append("HEALTH_THERMOMETER")
    HEALTH_WEIGHING = 0x0000090c
    __CLASS_VALUE_LIST.append(HEALTH_WEIGHING)
    __CLASS_NAME_LIST.append("HEALTH_WEIGHING")
    
    UNCATEGORIZED = 0x00001f00
    __CLASS_VALUE_LIST.append(UNCATEGORIZED)
    __CLASS_NAME_LIST.append("UNCATEGORIZED")
    
    
    __MAJOR_CLASS_MASK = 0x00001f00 
    __MINOR_CLASS_MASK = 0x00001f00
    
    @staticmethod
    def majorClassesList():
        return [BtDeviceClass.COMPUTER, BtDeviceClass.PHONE, BtDeviceClass.AUDIO_VIDEO, \
                BtDeviceClass.WEARABLE, BtDeviceClass.TOY, BtDeviceClass.HEALTH, BtDeviceClass.UNCATEGORIZED]
    
    @staticmethod
    def isMajorClassEqual(dev_class_1, dev_class_2):
        
        if (dev_class_1 == None) or (dev_class_2 == None):
            return False
        
        if (type(dev_class_1) == int) and (type(dev_class_2) == int):
            return (dev_class_1 & BtDeviceClass.__MAJOR_CLASS_MASK) \
                == (dev_class_2 & BtDeviceClass.__MAJOR_CLASS_MASK)
                
'''
class BtDevice(object):
    
    # 
    # line 1
    KEY_HCI = "hci"
    KEY_TYPE = "Type"
    KEY_BUS = "Bus"
    # line 2
    KEY_BD_ADDR = "BD Address"
    KEY_ACL_MTU = "ACL MTU"
    KEY_SCO_MTU = "SCO MTU"
    # line 3
    KEY_STATE_LINE_3 = "STATE (Line 3)"
    # line 4
    KEY_RX_BYTES = "RX bytes"
    # line 5
    KEY_TX_BYTES = "TX bytes"
    # line 6
    KEY_FEATURES = "Features"
    # line 7
    KEY_PACKET_TYPE = "Packet type"
    # line 8
    KEY_LINK_POLICY = "Link policy"
    # line 9
    KEY_LINK_MODE = "Link mode"
    # line 10
    KEY_NAME = "Name"
    # line 11
    KEY_CLASS = "Class"
    # line 12
    KEY_SERVICE_CLASSES = "Service Classes"
    #line 13
    KEY_DEVICE_CLASS = "Device Class"
    # line 14
    KEY_HCI_VERSION = "HCI Version"
    KEY_REVISION = "Revision"
    # line 15
    KEY_LMP_VERSION = "LMP Version"
    KEY_SUBVERSION = "Subversion"
    # line 16
    KEY_MANUFACTURER = "Manufacturer"
    
    def __init__(self, configDict={}):
        #the dict that contains the configuration for this bluetooth device
        self.__configDict = {}
        self.__threadingLock = threading.Lock()
        self.updateWithValuesFromDict(configDict)
    
    def updateWithValuesFromDict(self, configDict):
        try:
            with self.__threadingLock:
                if (configDict != None) and (type(configDict) == dict):
                    self.__configDict.update(configDict)
        except Exception as e:
            _rwo_bluetooth_log.exception("BtDevice.updateWithValuesFromDict self:%s \
            configDict:%s exception:%s" % (str(self), str(configDict), str(e)))
            
    def getValueFromDict(self, aKey):
        value = None
        try:
            with self.__threadingLock:
                if (aKey != None) and (aKey in self.__configDict):
                    self.__configDict.update(self.__configDict)
        except Exception as e:
            _rwo_bluetooth_log.exception("BtDevice.getValueFromDict self:%s \
            aKey:%s exception:%s" % (str(self), str(aKey), str(e)))
        return value
    
    def setValue(self, aKey, aValue):
        value = None
        try:
            with self.__threadingLock:
                self.__configDict[aKey] = value
        except Exception as e:
            _rwo_bluetooth_log.exception("BtDevice.setValue self:%s \
            aKey:%s value:%s exception:%s" % (str(self), str(aKey), str(aValue), str(e)))
        return value
        
'''

