#!/bin/bash
# Include logging to log stderr and stdout to specific file
. "/usr/bin/rwo_includeLogging.sh"

function expandPartition {
	rwoLog "Check size of disk device..."
	diskSizeGB=$(( $(blockdev --getsize64 /dev/mmcblk0) / 1073741824 ))
	rwoLog "Size of disk device: ${diskSizeGB}G"

	rwoLog "Checking size of root partition..."
	rootSizeGB=$(( $(blockdev --getsize64 ${rootPartitonDev}) / 1073741824 ))
	rwoLog "Size of root partition: ${rootSizeGB}G"

	#  Check the difference between the two measurements above. If the difference is greater than 0
	if [[ $(( ${diskSizeGB} - ${rootSizeGB} )) -gt 0 ]]; then
		rwoLog "Expanding the root partition..."
		if [[ "${rootPartitonDev}" == "/dev/mmcblk0p2" ]]; then

			fdisk /dev/mmcblk0 <<-EOF
				d
				2
				n
				p
				2


				w
			EOF

		elif [[ "${rootPartitonDev}" == "/dev/mmcblk0p5"  ]]; then

			fdisk /dev/mmcblk0 <<-EOF
				d
				5
				d
				2
				n
				e
				2


				n
				l
				5


				w
			EOF

		fi
	echo "1" > "${partitionStatusFile}"
	sync
	shutdown -r now
	else
		rwoLog "the partition does not need to be expanded"
		expandFilesystem
	fi
}

function expandFilesystem {
	rwoLog "Attempting to expand filesystem..."
	resize2fs -f /dev/mmcblk0p2
	echo "2" > "${partitionStatusFile}"
	sync
	shutdown -r now
}

rootPartitonDev=$(mount | grep " on / " | cut -d" " -f1)
if [[ "$rootPartitonDev" != "/dev/mmcblk0p2" ]] && [[ "$rootPartitonDev" != "/dev/mmcblk0p5" ]]; then
	rwoLog "unrecognized root partition device:${rootPartitonDev}"
	exit 1
fi

# time to check the partition status file`
partitionStatusFile="/var/log/lightstream/partition_status.txt"
if [ ! -f $partitionStatusFile ]; then
	echo '0' > $partitionStatusFile
fi

partitionStatus=$(cat "${partitionStatusFile}")
if [[ ${partitionStatus} == 0 ]]; then
	expandPartition
elif [[ ${partitionStatus} == 1 ]]; then
	expandFilesystem
fi

# no need to expand at this time
exit 0
