#!/bin/bash
#. "/usr/bin/rwo_includeLogging.sh"

inputPath="$1"
outputPath16x9="${inputPath}.16x9.jpeg"
outputPath4x3="${inputPath}.4x3.jpeg"
outputPathBg="${inputPath}.bg.jpeg"

if [ ! -e "${inputPath}" ]; then
	echo "Error, the input file does not exist"
fi

width=$(gm identify -format %w "${inputPath}")
height=$(gm identify -format %h "${inputPath}")
orientation=$(gm identify -format '%[EXIF:Orientation]' "${inputPath}");

# we may be able to use the file command instead of the identify command
#dimensions=$(file "${inputPath}" | grep -Eo '([0-9]* x [0-9]*)')
#width=$(echo "${dimensions}" | cut -d' ' -f 1)
#height=$(echo "${dimensions}" | cut -d' ' -f 3)
#orientation="normal"

# Used for bg images
originalWidth=$width
originalHeight=$height

# set some default sizes
width4x3="240"
height4x3="180"
width16x9="1280"
height16x9="720"
maxSize="1280"

# check the orientation, and make the width and height proper
if ((orientation == 5 || orientation == 6 || orientation == 7 || orientation == 8)); then
	orientation="inverted"
	temp=$width
	width=$height
	height=$temp

	temp4x3=$width4x3
	width4x3=$height4x3
	height4x3=$temp4x3

	temp16x9=$width16x9
	width16x9=$height16x9
	height16x9=$temp16x9
else
	orientation="normal"
fi

function resizePixels (){
	local wid=$1
	local hei=$2
	local widNew=$3
	local heiNew=$4
	outWidth=$widNew
	outHeight=$heiNew
	# check to see if the image is smaller than the 16x9 image and scale accordingly
	if (((${wid} < ${widNew})) || ((${hei} < ${heiNew}))); then
		outWidth=$wid
		outHeight=$((${outWidth} * ${heiNew} / ${widNew}))
		if ((${hei} < ${outHeight})); then
			outHeight=${hei}
			outWidth=$((${outHeight} * ${widNew} / ${heiNew}))
		fi
	fi
	outResize="${outWidth}x${outHeight}^" #wide images
	outCrop="${outWidth}x${outHeight}+0+0"
}

# TODO: remove -interlace Plane for non-jpeg formats
resizePixels $width $height $width16x9 $height16x9
gm convert -size "$outWidth"x"$outHeight" "$inputPath" -resize "$outResize" -gravity center -crop "$outCrop" -auto-orient +repage -strip -interlace Plane -quality 60% "$outputPath16x9"

resizePixels $width $height $width4x3 $height4x3
gm convert -size "$outWidth"x"$outHeight" "$inputPath" -resize "$outResize" -gravity center -crop "$outCrop" -auto-orient +repage -strip -interlace Plane -quality 85% "$outputPath4x3"

# Bg doesn't need to be cropped
resizePixels $maxSize $maxSize $originalWidth $originalHeight
gm convert -size "$outWidth"x"$outHeight" "$inputPath" -resize "$outResize" -auto-orient +repage -strip -interlace Plane -quality 70% "$outputPathBg"

# change permissions and owner
chmod 777 "${outputPathBg}" "${outputPath4x3}" "${outputPath16x9}"
chown http:http "${outputPathBg}" "${outputPath4x3}" "${outputPath16x9}"
