#!/bin/bash
# this script is started by a systemd service of the same name
# it will poll eth0 for event changes and then fire off a corresponding systemd service based on changed state


function onEth0Up {
	# configure dnsmasq
	echo "configuring for no dhcp server via eth0"
	rm "/etc/dnsmasq.conf"
	ln -T "/etc/dnsmasq.wlan0.conf" "/etc/dnsmasq.conf"
	systemctl restart dnsmasq
	# get a DHCP address
	pkill -9 dhcpcd
	sudo dhcpcd --timeout 5 eth0
	dhcpReqStatus=$?
	# check for failure of dhcp address
	if [ $dhcpReqStatus -eq 0 ]; then
		command
	else
		echo "unable to obtain ip address:${dhcpReqStatus}"
		echo "configuring for dhcp server via eth0"
		rm "/etc/dnsmasq.conf"
		ln -T "/etc/dnsmasq.eth0.wlan0.conf" "/etc/dnsmasq.conf"
		systemctl restart dnsmasq
		# nginx rewrite now handled by rwo_wifi
		# used to configure nginx for no-rewrite
	fi

	# configure the ip addresses
	ip addr add 192.168.8.2/24 dev eth0
	ip addr add 192.168.200.1/24 dev eth0
}

function onEth0Down {
	# configure dnsmasq
	echo "configuring for no dhcp server via eth0"
	rm "/etc/dnsmasq.conf"
	ln -T "/etc/dnsmasq.wlan0.conf" "/etc/dnsmasq.conf"
	systemctl restart dnsmasq
	echo "starting up rwo_wifi@wlan0Ap in case it had been stopped"
	systemctl start rwo_wifi@wlan0Ap
}


# include logging script to go to central log
. /usr/bin/rwo_includeLogging.sh

lastIfState=200 # assume you start down
ifName="eth0"
pollingInterval=10.0
apLastState="none"

# infinite loop
while true; do
	# poll every X seconds
	sleep "${pollingInterval}"
	# check if the ethernet is up
	currentIfState=$(ip link | grep .*${ifName}:.*state\ UP.* | wc -l)
	if [ $currentIfState -ne $lastIfState ]; then
		rwoLog "${ifName} state changed:${currentIfState}"
		lastIfState=${currentIfState}
		if [ $currentIfState -gt 0 ]; then
			onEth0Up
		else
			apLastState="none"
			onEth0Down
		fi
	elif [ $currentIfState -gt 0 ]; then
			apCurrentState=$(rwo_eth0WifiApState.sh)
			if [[ "${apCurrentState}" != "${apLastState}" ]]; then
				apLastState="${apCurrentState}"
				if [[ "${apCurrentState}" == 'initialized' ]]; then
					rwoLog "detected an initialized Gl-INet router on the network"
					systemd-run --on-active=1 /usr/bin/rwo_eth0WifiApConfigure.sh
				elif [[ "${apCurrentState}" == 'uninitialized' ]]; then
				rwoLog "detected an uninitialized Gl-INet router on the network"
				systemd-run --on-active=1 /usr/bin/rwo_eth0WifiApInitialize.sh
			fi
		fi
	fi
done
