#!/bin/bash
# this script is used to initialize a WiFi AP that is connected via eth0
# once the AP is initialized, it shouldn't need to be initialized again, unless it gets uninitialized or changed

# include logging script to go to central log
. /usr/bin/rwo_includeLogging.sh
apIpAddrUninit="192.168.8.1"
apIpAddrInit="192.168.200.2"

# check if platform is rapsberry-pi
apState=$(rwo_eth0WifiApState.sh)
if [[ $apState == "initialized" ]]; then
	rwoLog "found initialized glinet router"
	ssid=$(sqlite3 /srv/http/data/lswifi.sqlite3 "SELECT value FROM settings WHERE key = 'ssid'")
	channel=$(sqlite3 /srv/http/data/lswifi.sqlite3 "SELECT value FROM settings WHERE key = 'channel'")
	txpower="20" # txpower in dbm ( mw=10^(dbm/10) ) 20dbm=100mw 30dbm=1000mw 40dbm=10000mw
	country="US"
	secret=$(rwo_generateSecret.sh)
	htmode="HT40+"
	if (("${channel}">=1 && "${channel}"<=7)); then
		htmode="HT40+"
	elif (("${channel}">=8 && "${channel}"<=11)); then
		htmode="HT40-"
	else
		rwoLog "ERROR: invalid channel detected:\"${channel}\" , defaulting to channel 1"
		htmode="HT40+"
		channel="1"
	fi
	commandStr="uci set wireless.@wifi-iface[0].ssid=\"${ssid}\" && uci set wireless.@wifi-iface[0].encryption=none && "
	commandStr="${commandStr}uci set wireless.@wifi-device[0].channel=${channel} && uci set wireless.@wifi-device[0].htmode=${htmode} && "
	commandStr="${commandStr}uci set wireless.@wifi-device[0].country=${country} && uci set wireless.@wifi-device[0].txpower=${txpower} && "
	# check if there are any staged changes, and if there are, commit them and restart
	commandStr="${commandStr}[ \$(uci changes wireless | wc -l) -gt 0 ] && uci commit wireless && /etc/init.d/network restart &"
	sshpass -p "${secret}" ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no root@${apIpAddrInit} "${commandStr}"
	success=$?
	rwoLog "tried to configure access point and got exit code:${success}"
	if [ $success -eq 0 ]; then
		rwoLog "will stop hostapd"
		systemctl stop rwo_wifi@wlan0Ap
		rwoLog "configuring for dhcp server via eth0"
		rm "/etc/dnsmasq.conf"
		ln -T "/etc/dnsmasq.eth0.wlan0.conf" "/etc/dnsmasq.conf"
		systemctl restart dnsmasq
	fi
fi
