#!/usr/bin/python2
#
# RENEW OUTREACH, LLC ("COMPANY") CONFIDENTIAL
# Unpublished Copyright (c) 2014 Renew Outreach, LLC, All Rights Reserved.
#
# NOTICE:  All information contained herein is, and remains the property of COMPANY. The intellectual and technical concepts contained
# herein are proprietary to COMPANY and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
# Dissemination of this information or reproduction of this material is strictly forbidden unless prior written permission is obtained
# from COMPANY.  Access to the source code contained herein is hereby forbidden to anyone except current COMPANY employees, managers or contractors who have executed 
# Confidentiality and Non-disclosure agreements explicitly covering such access.
#
# The copyright notice above does not evidence any actual or intended publication or disclosure of this source code, which includes  
# information that is confidential and/or proprietary, and is a trade secret, of COMPANY.  ANY REPRODUCTION, MODIFICATION, DISTRIBUTION, PUBLIC PERFORMANCE, 
# OR PUBLIC DISPLAY OF OR THROUGH USE OF THIS SOURCE CODE WITHOUT THE EXPRESS WRITTEN CONSENT OF COMPANY IS STRICTLY PROHIBITED, AND IN VIOLATION OF APPLICABLE 
# LAWS AND INTERNATIONAL TREATIES.  THE RECEIPT OR POSSESSION OF THIS SOURCE CODE AND/OR RELATED INFORMATION DOES NOT CONVEY OR IMPLY ANY RIGHTS  
# TO REPRODUCE, DISCLOSE OR DISTRIBUTE ITS CONTENTS, OR TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT MAY DESCRIBE, IN WHOLE OR IN PART.     

from __builtin__ import str, int
import rwo_strings
import rwo_subprocess
import rwo_list
import time

# logging
import rwo_logging
_rwo_hciconfig_log = rwo_logging.RwoLogger()
def setLogger(logger):
    _rwo_hciconfig_log.set_logger(logger)

# __logLock = threading.Lock()
__HCICONFIG_CMD_STR = 'hciconfig'

#TODO .. possibly move these keys in to a separate class

# line 1
KEY_HCI = "hci"
KEY_TYPE = "Type"
KEY_BUS = "Bus"
# line 2
KEY_BD_ADDR = "BD Address"
KEY_ACL_MTU = "ACL MTU"
KEY_SCO_MTU = "SCO MTU"
# line 3
KEY_STATE_LINE_3 = "STATE (Line 3)"
# line 4
KEY_RX_BYTES = "RX bytes"
# line 5
KEY_TX_BYTES = "TX bytes"
# line 6
KEY_FEATURES = "Features"
# line 7
KEY_PACKET_TYPE = "Packet type"
# line 8
KEY_LINK_POLICY = "Link policy"
# line 9
KEY_LINK_MODE = "Link mode"
# line 10
KEY_NAME = "Name"
# line 11
KEY_CLASS = "Class"
# line 12
KEY_SERVICE_CLASSES = "Service Classes"
#line 13
KEY_DEVICE_CLASS = "Device Class"
# line 14
KEY_HCI_VERSION = "HCI Version"
KEY_REVISION = "Revision"
# line 15
KEY_LMP_VERSION = "LMP Version"
KEY_SUBVERSION = "Subversion"
# line 16
KEY_MANUFACTURER = "Manufacturer"


def __isValidHciXStr(hciXStr):
    if (hciXStr != None) and (type(hciXStr) is str) and ("hci" in hciXStr):
        return True
    else:
        return False

# used as a helper for parsing the hciconfig -a lines
def __seperateStringFromHciconfigLineByItemInListAndAddPairInDict(aString, sep_items_lst, dict_to_append):
    global _rwo_hciconfig_log
    real_sep_items = []
    try:
        for sep_item in sep_items_lst:
            this_real_sep_item = sep_item + ": "
            if sep_items_lst.index(sep_item) == 0:
                this_real_sep_item = '\t' + this_real_sep_item
            else:
                this_real_sep_item = " " + this_real_sep_item
            real_sep_items.append(this_real_sep_item)
        values = rwo_strings.splitStringByListOfStringsOnFirstOccurrence(aString, real_sep_items)[0]
        for indexOfItem in range(0,len(real_sep_items)):
            sep_item = sep_items_lst[indexOfItem]
            real_sep_item = real_sep_items[indexOfItem]
            value = rwo_list.getItemAfterObjectInList(real_sep_item, values)
            if (value != None) and (type(value) == str):
                dict_to_append[sep_item] = value
    except Exception as e:
        message = "rwo_hciconfig.__seperateStringFromHciconfigLineByItemInListAndAddPairInDict \
        Exception: " + str(e) + "aString:" + str(aString) +\
        "sep_items_lst:" + str(sep_items_lst) + "dict_to_append:" + str(dict_to_append)
        _rwo_hciconfig_log.exception(message)
        
# takes in a aString of the format hciX and an array of options that correspond with an hcxconfig command
# returns the command output if successful and None if there was an error
def runCmdOnDevice(hciXStr, commandWithOptionsList):
    global _rwo_hciconfig_log
    subprocessResponse = None
    try:
        if (__isValidHciXStr(hciXStr)) and (commandWithOptionsList != None) and (type(commandWithOptionsList) == list):
            completeCommandWithOptionsList = [__HCICONFIG_CMD_STR , hciXStr]
            completeCommandWithOptionsList.extend(commandWithOptionsList)
            _rwo_hciconfig_log.info("running hciconfig command:%s" % (rwo_strings.iterableToString(commandWithOptionsList)))
            subprocessResponse = rwo_subprocess.openAndCommunicate(completeCommandWithOptionsList)
        else:
            _rwo_hciconfig_log.error("error running hciconfig command:%s on hci device:%s" % (str(commandWithOptionsList) , str(hciXStr)))
    except Exception as e:
        message = "rwo_hciconfig.runCmdOnDevice \
        Exception: " + str(e) + "hciXStr:" + str(hciXStr) +\
        "commandWithOptionsList:" + str(commandWithOptionsList)
        _rwo_hciconfig_log.exception(message)
    return subprocessResponse
    
# takes in a list of hciX devices and an array of options that correspond with an hcxconfig command
# returns a list of dictionaries
def runCmdOnDevicesInList(hciXDevicesList, commandWithOptionsList):
    global _rwo_hciconfig_log
    listOfSubprocessResponses = []
    try:
        if (hciXDevicesList != None) and (type(hciXDevicesList) == list) \
           and (commandWithOptionsList != None) and (type(commandWithOptionsList) == list):
            for thisHciDevice in hciXDevicesList:
                thisCommandOutput = runCmdOnDevice(thisHciDevice, commandWithOptionsList)
                listOfSubprocessResponses.append(thisCommandOutput)
                time.sleep(1)
        else:
            _rwo_hciconfig_log.error("error running hciconfig commands \
            on devices in list:%s devices in list:%s" \
            % (str(commandWithOptionsList) , str(hciXDevicesList)))
    except Exception as e:
        message = "rwo_hciconfig.runCmdOnDevicesInList \
        Exception: " + str(e) + "hciXDevicesList:" + str(hciXDevicesList) +\
        "commandWithOptionsList:" + str(commandWithOptionsList)
        _rwo_hciconfig_log.exception(message)    
    return listOfSubprocessResponses

# returns a string hciconfig -a , and None for a non-zero exit value
def getAllString():
    global _rwo_hciconfig_log
    output = ""
    try:
        subpDict = rwo_subprocess.openAndCommunicate(["hciconfig", "-a"])
        p = subpDict[rwo_subprocess.PROCESS_KEY]
        out = subpDict[rwo_subprocess.OUTPUT_KEY]
        if p.returncode == 0:
            output = out[0]
            _rwo_hciconfig_log.info("rwo_hciconfig.getAllString hciconfig -a returned:" + output)
        # if the command returned with a non-zero exit value
        else:
            _rwo_hciconfig_log.error('rwo_hciconfig.getAllString hciconfig -a returned witn non-zero code:' + p.returncode)
    except Exception as e:
        message = "rwo_hciconfig.getAllString Exception: " + str(e)
        _rwo_hciconfig_log.exception(message)
    return output
   
# returns each device in a list, returns None if hciconfig -a returns with an error
def parseAllStringToAllList(hciconfig_a):
    global _rwo_hciconfig_log
    hciconfig_a_list = []
    try:
        if (hciconfig_a != None) and (type(hciconfig_a) == str) and \
        (len(hciconfig_a) > 0):
            hciconfig_a_list = hciconfig_a.split('\n\n')
            #this is for the possible new line at the end of the list,
            # removes the extra entry so the count represents the number of devices
            if "" in hciconfig_a_list:
                hciconfig_a_list.remove("")
        
            if " " in hciconfig_a_list:
                hciconfig_a_list.remove(" ")
            _rwo_hciconfig_log.info("rwo_hciconfig.parseAllStringToAllList found %d devices" % (len(hciconfig_a_list)))
        else:
            _rwo_hciconfig_log.warning("rwo_hciconfig.parseAllStringToAllList hciconfig_a is None when trying to get devices in list")
    except Exception as e:
        message = "rwo_hciconfig.parseAllStringToAllList hciconfig_a:" + str(hciconfig_a) + "Exception: " + str(e)
        _rwo_hciconfig_log.exception(message)
    return hciconfig_a_list
   
def parseAllStringToDictOfCfgDicts(hciconfig_a):
    global _rwo_hciconfig_log
    hciconfig_a_dict_of_dicts = None
    try:
        hciconfig_a_list = parseAllStringToAllList(hciconfig_a)
        hciconfig_a_dict_of_dicts = parseAllListToDictOfCfgDicts(hciconfig_a_list)
    except Exception as e:
        message = "rwo_hciconfig.parseAllStringToDictOfCfgDicts hciconfig_a:" + str(hciconfig_a) + "Exception: " + str(e)
        _rwo_hciconfig_log.exception(message)
    
    return hciconfig_a_dict_of_dicts

def getAllDictOfCfgDicts():
    global _rwo_hciconfig_log
    hciconfig_a_dict_of_dicts = None
    try:
        hciconfig_all_str = getAllString()
        hciconfig_a_dict_of_dicts = parseAllStringToDictOfCfgDicts(hciconfig_all_str)
    except Exception as e:
        message = "rwo_hciconfig.getAllDictOfCfgDicts " + "Exception: " + str(e)
        _rwo_hciconfig_log.exception(message)
    return hciconfig_a_dict_of_dicts;
   
def parseAllListToDictOfCfgDicts(hciconfig_a_list):
    global _rwo_hciconfig_log
    hciconfig_a_dict_of_dicts = {}
    try:
        if (hciconfig_a_list != None) and (type(hciconfig_a_list) == list) and \
        (type(hciconfig_a_list) == list):
            for indexOfDevice in range(0,len(hciconfig_a_list)):
                thisDeviceConfig = hciconfig_a_list[indexOfDevice]
                _rwo_hciconfig_log.debug("rwo_hciconfig.parseAllListToDictOfCfgDicts Parsing hciconfig dict for hci device config:%s" % (str(thisDeviceConfig)))
                dictOfThisDeviceConfig = _parseAllDeviceStringToCfgDict(thisDeviceConfig)
                # add the device's cfg dict to the dictionary of dictionaries
                if(dictOfThisDeviceConfig != None) and (dictOfThisDeviceConfig.has_key(KEY_HCI)):
                    hciconfig_a_dict_of_dicts[dictOfThisDeviceConfig[KEY_HCI]] = dictOfThisDeviceConfig
        else:
            _rwo_hciconfig_log.error("rwo_hciconfig.parseAllListToDictOfCfgDicts Error getting list of dictionaries from hciconfig -a list:%s" \
                          % (str(hciconfig_a_list)))
    except Exception as e:
        message = "rwo_hciconfig.parseAllListToDictOfCfgDicts hciconfig_a_list:" + str(hciconfig_a_list) + "Exception: " + str(e)
        _rwo_hciconfig_log.exception(message)
    
    return hciconfig_a_dict_of_dicts
   
def _parseAllDeviceStringToCfgDict(hciconfig_a_single_device):
    global _rwo_hciconfig_log
    dictOfThisDeviceConfig = {}
    try:
        thisDeviceConfigLines = hciconfig_a_single_device.split('\n')
        for indexOfLine in range(0,len(thisDeviceConfigLines)):
            thisLineInDeviceConfig = thisDeviceConfigLines[indexOfLine]
            
            if indexOfLine == 0 :
                hciName = parseAllDeviceStringGetHciDev(hciconfig_a_single_device)
                dictOfThisDeviceConfig[KEY_HCI] = hciName
                __seperateStringFromHciconfigLineByItemInListAndAddPairInDict(thisLineInDeviceConfig,\
                                                                       [KEY_TYPE, KEY_BUS],\
                                                                        dictOfThisDeviceConfig)
            elif indexOfLine == 1:
                __seperateStringFromHciconfigLineByItemInListAndAddPairInDict(thisLineInDeviceConfig,\
                                                                       [KEY_BD_ADDR, KEY_ACL_MTU, KEY_SCO_MTU],\
                                                                        dictOfThisDeviceConfig)
            elif indexOfLine == 2:
                dictOfThisDeviceConfig[KEY_STATE_LINE_3] = thisLineInDeviceConfig.lstrip('\t')
            elif indexOfLine == 3:
                dictOfThisDeviceConfig[KEY_RX_BYTES] = thisLineInDeviceConfig.lstrip('\t')
            elif indexOfLine == 4:
                dictOfThisDeviceConfig[KEY_TX_BYTES] = thisLineInDeviceConfig.lstrip('\t')
            elif indexOfLine == 5:
                __seperateStringFromHciconfigLineByItemInListAndAddPairInDict(thisLineInDeviceConfig,\
                                                                       [KEY_FEATURES],\
                                                                        dictOfThisDeviceConfig)
            elif indexOfLine == 6:
                __seperateStringFromHciconfigLineByItemInListAndAddPairInDict(thisLineInDeviceConfig,\
                                                                       [KEY_PACKET_TYPE],\
                                                                        dictOfThisDeviceConfig)
            elif indexOfLine == 7:
                __seperateStringFromHciconfigLineByItemInListAndAddPairInDict(thisLineInDeviceConfig,\
                                                                       [KEY_LINK_POLICY],\
                                                                        dictOfThisDeviceConfig)
            elif indexOfLine == 8:
                __seperateStringFromHciconfigLineByItemInListAndAddPairInDict(thisLineInDeviceConfig,\
                                                                       [KEY_LINK_MODE],\
                                                                        dictOfThisDeviceConfig)
            elif indexOfLine == 9:
                __seperateStringFromHciconfigLineByItemInListAndAddPairInDict(thisLineInDeviceConfig,\
                                                                       [KEY_NAME],\
                                                                        dictOfThisDeviceConfig)
            elif indexOfLine == 10:
                __seperateStringFromHciconfigLineByItemInListAndAddPairInDict(thisLineInDeviceConfig,\
                                                                       [KEY_CLASS],\
                                                                        dictOfThisDeviceConfig)
            elif indexOfLine == 11:
                __seperateStringFromHciconfigLineByItemInListAndAddPairInDict(thisLineInDeviceConfig,\
                                                                       [KEY_SERVICE_CLASSES],\
                                                                        dictOfThisDeviceConfig)
            elif indexOfLine == 12:
                __seperateStringFromHciconfigLineByItemInListAndAddPairInDict(thisLineInDeviceConfig,\
                                                                       [KEY_DEVICE_CLASS],\
                                                                        dictOfThisDeviceConfig)
            elif indexOfLine == 13:
                __seperateStringFromHciconfigLineByItemInListAndAddPairInDict(thisLineInDeviceConfig,\
                                                                       [KEY_HCI_VERSION, KEY_REVISION],\
                                                                        dictOfThisDeviceConfig)
            elif indexOfLine == 14:
                __seperateStringFromHciconfigLineByItemInListAndAddPairInDict(thisLineInDeviceConfig,\
                                                                       [KEY_LMP_VERSION, KEY_SUBVERSION],\
                                                                        dictOfThisDeviceConfig)
            elif indexOfLine == 12:
                __seperateStringFromHciconfigLineByItemInListAndAddPairInDict(thisLineInDeviceConfig,\
                                                                       [KEY_MANUFACTURER],\
                                                                 dictOfThisDeviceConfig)
    except Exception as e:            
        message = "rwo_hciconfig._parseAllDeviceStringToCfgDict \
        Exception: " + str(e) + "single device config:" + str(hciconfig_a_single_device)
        _rwo_hciconfig_log.exception(message)
    return dictOfThisDeviceConfig

# return a list of names of hci devices in the form hci<X> where X is the number
def parseAllStringGetHciDevList(hciconfig_a):
    global _rwo_hciconfig_log
    try:
        hciconfig_a_list = parseAllStringToAllList(hciconfig_a)
        hci_dev_lst = parseAllListGetHciDevList(hciconfig_a_list)  
    except Exception as e:
        message = "rwo_hciconfig.parseAllStringGetHciDevList hciconfig_a:" + str(hciconfig_a) + "Exception: " + str(e)
        _rwo_hciconfig_log.exception(message)
      
    return hci_dev_lst

# returns a list of hci names given the list
def parseAllListGetHciDevList(hciconfig_a_list):
    global _rwo_hciconfig_log
    hci_dev_lst = []   
    try:
        if (hciconfig_a_list != None) and (type(hciconfig_a_list) == list):
            for deviceConfig in hciconfig_a_list:
                hciName = parseAllDeviceStringGetHciDev(deviceConfig)
                if (hciName != None) and (type(hciName) == str) and\
                (len(hciName) > 0):
                    hci_dev_lst.append(hciName)                 
        # if the command returned with a bad exit value
        else:
            _rwo_hciconfig_log.warning("rwo_hciconfig.parseAllStringGetHciDevList when trying to get device names:%s" % (str(hciconfig_a_list))) 
    except Exception as e:
        message = "rwo_hciconfig.parseAllStringGetHciDevList hciconfig_a_list:" + str(hciconfig_a_list) + "Exception: " + str(e)
        _rwo_hciconfig_log.exception(message)  
    return hci_dev_lst

def parseAllDeviceStringGetHciDev(hciconfig_a_single_device):
    global _rwo_hciconfig_log
    hciName = ""
    try:
        if hciconfig_a_single_device.lstrip().startswith('hci'):
            words = hciconfig_a_single_device.split();
        if (words != None) and (len(words) > 0):
            hciName = words[0].strip(':')
            _rwo_hciconfig_log.info('rwo_hciconfig.parseAllDeviceStringGetHciDev found hci device with name:' + hciName)  
    except Exception as e:
        message = "rwo_hciconfig.parseAllDeviceStringGetHciDev hciconfig_a_single_device:" + str(hciconfig_a_single_device) + "Exception: " + str(e)
        _rwo_hciconfig_log.exception(message)
    return hciName

# returns a list of hciX device string that have a state that contains the given state string
def parseAllDictOfCfgDictsGetDevListWithDevicesThatContainState(dict_of_cfg_dicts, string_in_state):
    global _rwo_hciconfig_log
    hciXList = []
    try:
        _rwo_hciconfig_log.debug("rwo_hciconfig.parseAllDictOfCfgDictsGetDevListWithDevicesThatContainState \
        dict_of_cfg_dicts:%s string_in_state:%s" % (str(dict_of_cfg_dicts) , str(string_in_state)))  
        keys = dict_of_cfg_dicts.keys()
        for hciXStr in keys:
            if hciXStr in dict_of_cfg_dicts:
                hciXCfgDict = dict_of_cfg_dicts[hciXStr]
                if KEY_STATE_LINE_3 in hciXCfgDict:
                    state_str = hciXCfgDict[KEY_STATE_LINE_3]
                    if string_in_state in state_str:
                        hciXList.append(hciXStr)
    except Exception as e:
        message = "rwo_hciconfig.parseAllDictOfCfgDictsGetDevListWithDevicesThatContainState dict_of_cfg_dicts:" \
         + str(dict_of_cfg_dicts) + "string_in_state:" + str(string_in_state) + "Exception: " + str(e)
        _rwo_hciconfig_log.exception(message)
    return hciXList

# returns a list of devices that have the specified state
def getDevListWithDevicesThatHaveStringInState(string_in_state):
    global _rwo_hciconfig_log
    hciXList = []
    try:
        hciconfig_a_str = getAllString()
        hci_a_dict = parseAllStringToDictOfCfgDicts(hciconfig_a_str)
        hci_x_list = parseAllDictOfCfgDictsGetDevListWithDevicesThatContainState(hci_a_dict, string_in_state)
        if (hci_x_list != None) and (type(hci_x_list) == list):
            hciXList.extend(hci_x_list)
    except Exception as e:
        message = "rwo_hciconfig.parseAllDictOfCfgDictsGetDevListWithDevicesThatContainState "\
         + "string_in_state:" + str(string_in_state) + "hci_a_dict:" \
         + str(hci_a_dict) + "Exception: " + str(e)
        _rwo_hciconfig_log.exception(message)
    return hciXList

def getDevList():
    global _rwo_hciconfig_log
    hciXList = []
    try:
        hciconfig_a_str = getAllString()
        hciXList = parseAllStringGetHciDevList(hciconfig_a_str)
    except Exception as e:
        message = "rwo_hciconfig.parseAllDictOfCfgDictsGetDevListWithDevicesThatContainState " \
        + "Exception: " + str(e)
        _rwo_hciconfig_log.exception(message)
    return hciXList
    
    

# test this code
def __test():
    return 0

if __name__ == '__main__':
    __test()
    

