'''
Created on Oct 16, 2014

@author: root
'''

import rwo_subprocess
import rwo_logging
import rwo_strings
import rwo_list

_HCITOOL_CMD = "hcitool"
_HCITOOL_INQ_CLOCK_OFFSET_SEP = '\tclock offset: '
_HCITOOL_INQ_CLASS_SEP = '\tclass: '

# logging
_rwo_hcitool_log = rwo_logging.RwoLogger()
def setLogger(logger):
    _rwo_hcitool_log.set_logger(logger)

# run a hcitool cmd
# put in the options after the hcitool cmd
# eg. rwo_hcitool.runCmd(["-i", "hci1", "inq"])
# returns the response from rwo_subprocess
def runCmd(cmd_lst):
    global _rwo_hcitool_log
    global _HCITOOL_CMD
    value = None
    try:
        _rwo_hcitool_log.debug("rwo_hcitool.runCmd cmd_lst:" + str(cmd_lst))
        if (cmd_lst != None) and (type(cmd_lst) == list) \
        and (len(cmd_lst) > 0):
            cmdWithOptions = [_HCITOOL_CMD].extend(cmd_lst)
            value = rwo_subprocess.openAndCommunicate(cmdWithOptions)
    except Exception as e:
        message = "rwo_hcitool.runCmd cmd_lst:" + str(cmd_lst) \
        + "Exception:" + str(e)
        _rwo_hcitool_log.exception(message)
    return value

#uses the -i option for hci_dev
def runCmdOnDevice(cmd_lst, hci_dev):
    global _rwo_hcitool_log
    value = None
    try:
        _rwo_hcitool_log.debug("rwo_hcitool.runCmdOnDevice cmd_lst:" + str(cmd_lst))
        if (cmd_lst != None) and (type(cmd_lst) == list) \
        and (hci_dev != None) and (type(hci_dev) == str):
            complete_cmd_lst = ["-i", hci_dev].extend(cmd_lst)
            value = runCmd(complete_cmd_lst)
    except Exception as e:
        message = "rwo_hcitool.runCmdOnDevice cmd_lst:" + str(cmd_lst) \
        + "hci_dev:"+ str(hci_dev) + "Exception:" + str(e)
        _rwo_hcitool_log.exception(message)
    return value

#returns an array of responses for each device
def runCmdOnDevicesInList(cmd_lst, hci_dev_lst):
    global _rwo_hcitool_log
    value = []
    try:
        _rwo_hcitool_log.debug("rwo_hcitool.runCmdOnDevsInList cmd_lst:" \
                          + str(cmd_lst) + "hci_dev_lst:" + str(hci_dev_lst))
        if (cmd_lst != None) and (type(cmd_lst) == list) \
        and (hci_dev_lst != None) and (type(hci_dev_lst) == str):
            for hci_dev in hci_dev_lst:
                value.append(runCmdOnDevice(cmd_lst, hci_dev))
    except Exception as e:
        message = "rwo_hcitool.runCmdOnDevice cmd_lst:" + str(cmd_lst) \
        + "hci_dev:"+ str(hci_dev) + "Exception:" + str(e)
        _rwo_hcitool_log.exception(message)
    return value


def inqOnDevice(hci_dev):

    '''
    performs an inq on the specified hci device and returns the raw string
    you can use the parse functions to parse the inq
    '''

    global _rwo_hcitool_log
    value = ""
    try:
        subpDict = runCmdOnDevice(["inq"] , hci_dev)
        p = subpDict[rwo_subprocess.PROCESS_KEY]
        out = subpDict[rwo_subprocess.OUTPUT_KEY]
        if p.returncode == 0:
            value = out[0]
            _rwo_hcitool_log.info("rwo_hcitool.inqOnDevice hci_dev:" + str(hci_dev) + "returned:" + value)
        # if the command returned with a non-zero exit value
        else:
            _rwo_hcitool_log.error("rwo_hcitool.inqOnDevice hci_dev:" + str(hci_dev) + \
                             "non-zero return code:" + str(p.returncode))
    except Exception as e:
        message = "rwo_hcitool.inqOnDevice hci_dev:" + str(hci_dev) + \
        "Exception:" + str(e)
        _rwo_hcitool_log.exception(message)
    return value


def parseInqStr(inq_str):

    '''
    parse the result from the inqOnDevice function
    returns a list of bluetooth device mac addresses
    returns a list of lists: [devLst1, devLst2 ..., devLstN]
    devLst = [<bdAddr>, <clockOffset>, <bdClass>]
    '''

    global _rwo_hcitool_log
    global _HCITOOL_INQ_CLASS_SEP
    global _HCITOOL_INQ_CLOCK_OFFSET_SEP
    listOfDevLists = []
    try:
        _rwo_hcitool_log.debug("rwo_hcitool.parseInqGetBdAddrList inq_str:" + str(inq_str))
        seperatorItems = [_HCITOOL_INQ_CLOCK_OFFSET_SEP, _HCITOOL_INQ_CLASS_SEP ]
        if (inq_str != None) and (type(inq_str) == str):
            s = inq_str.splitlines()
            for line in s:
                if line.lstrip().startswith('Inquiring'): # skip the line that says "Inquiring"
                    continue
                line = line.lstrip('\t') # remove the leading tab
                valuesList = rwo_strings.splitStringByListOfStringsOnFirstOccurrence(line, seperatorItems)[0]
                if (valuesList != None) and (type(valuesList) == list) and (len(valuesList) > 0):
                    bdAddr = valuesList[0]
                    if (bdAddr == None):
                        bdAddr = ""
                    clockOffset = rwo_list.getItemAfterObjectInList(_HCITOOL_INQ_CLOCK_OFFSET_SEP, valuesList)
                    if (clockOffset == None):
                        clockOffset = ""
                    bdClass = rwo_list.getItemAfterObjectInList(_HCITOOL_INQ_CLASS_SEP, valuesList)
                    if (bdClass == None):
                        bdClass = ""
                    thisInqDevList = [bdAddr, clockOffset, bdClass]
                    listOfDevLists.append(thisInqDevList)

    except Exception as e:
        message = "rwo_hcitool.parseInqGetBdAddrList inq_str:" + str(inq_str) \
        + "Exception:" + str(e)
        _rwo_hcitool_log.exception(message)
    return listOfDevLists
