
ledGpiosSd=( 48 131 138 139 123 122 121 )
ledGpioSdLock=120
ledGpioSdUnlock=119
ledGpioSdSixOclock=47
ledGpiosWifi=( 117 116 115 114 111 110 109 )
ledGpiosBatt=( 107 106 103 108 102 101 100 )
ledGpiosAllBlue=( "${ledGpiosSd[@]}" "${ledGpiosWifi[@]}" "${ledGpiosBatt[@]}" )
ledGpiosAll=( "${ledGpiosAllBlue[@]}" "${ledGpioSdLock}" "${ledGpioSdUnlock}" "${ledGpioSdSixOclock}" )
PIDLEDSPROGRESS=''
PIDFLASHLED=''
PIDLEDSWIFIUPDATE=''
LEDSPROGRESSFILE='/tmp/rwo/rwo_ledsProgress.percentage'
touch "$LEDSPROGRESSFILE"

#################### GPIO #######################
# initialize the LED by exporting it and then turn it off
ledsGpiosInit () {
	gpioDir="/sys/class/gpio/gpio${1}"
	if [[ ! -d  "${gpioDir}" ]]; then
		echo "${1}" > "/sys/class/gpio/export"
		ledsGpioOff "${1}" # turn the gpio off as default
	fi
}

# turning on the LED opens the drain (low and output)
ledsGpioOn () {
	gpioDir="/sys/class/gpio/gpio${1}"
	if [[ ! -d "${gpioDir}" ]]; then
		return 0
	fi
	echo "out" > "${gpioDir}/direction"
	echo "0" > "${gpioDir}/value"
}

# turning off the LED is preferred high-z
ledsGpioOff () {
	echo "in" > "/sys/class/gpio/gpio${1}/direction"
}

ledsGpioSet (){
	if [[ $2 -gt 0 ]]; then
		ledsGpioOn $1
	else
		ledsGpioOff $1
	fi
}

##################### ALL UTILS #####################
ledsAllInit () {
	for gpioNum in "${ledGpiosAll[@]}"; do
		ledsGpiosInit "${gpioNum}"
	done
}

ledsAllOn () {
	for gpioNum in "${ledGpiosAll[@]}"; do
		ledsGpioOn "${gpioNum}"
	done
	ledsBatterySetSixOclockLed 1
	ledsWifiSetSixOclockLed 1
	ledsSdSetSixOclockLed 1
	ledsSdSetLockLed 1
	ledsSdSetUnlockLed 1
}

ledsAllBlueOnOnly () {
	ledsAllBlueOn
	ledsBatterySetSixOclockLed 0
	ledsWifiSetSixOclockLed 0
	ledsSdSetSixOclockLed 0
	ledsSdSetLockLed 0
	ledsSdSetUnlockLed 0
}

ledsAllBlueOn () {
	for gpioNum in "${ledGpiosAllBlue[@]}"; do
		ledsGpioOn "${gpioNum}"
	done
}

ledsAllBlueOff () {
	for gpioNum in "${ledGpiosAllBlue[@]}"; do
		ledsGpioOff "${gpioNum}"
	done
}

ledsAllOff () {
	for gpioNum in "${ledGpiosAll[@]}"; do
		ledsGpioOff "${gpioNum}"
	done
	ledsBatterySetSixOclockLed 0
	ledsWifiSetSixOclockLed 0
	ledsSdSetSixOclockLed 0
	ledsSdSetLockLed 0
	ledsSdSetUnlockLed 0
}

ledsDance () {
	for numLeds in $(seq 1 1 7); do
		ledsSdSetLedsOn $numLeds
		ledsWifiSetLedsOn $numLeds
		ledsBatterySetLedsOn $numLeds
		sleep 0.1
	done
	sleep 0.5
	for numLeds in $(seq 7 -1 0); do
		ledsSdSetLedsOn $numLeds
		ledsWifiSetLedsOn $numLeds
		ledsBatterySetLedsOn $numLeds
		sleep 0.1
	done
}

##################### SD #########################
ledsSdSetLedsOn () {
	local numLeds="${1}"
	local ledNum=0
	for ledNum in $(seq 1 1 ${numLeds}); do
		ledsSdSetLedOn "${ledNum}"
	done
	for ledNum in $(seq 7 -1 $(( ${numLeds} + 1 ))); do
		ledsSdSetLedOff "${ledNum}"
	done
}

ledsSdSetLedOn () {
	local ledNum=$1
	local gpioNum=${ledGpiosSd[$((ledNum - 1))]}
	ledsGpioOn "${gpioNum}"
}

ledsSdSetLedOff () {
	local ledNum=$1
	local gpioNum=${ledGpiosSd[$((ledNum - 1))]}
	ledsGpioOff "${gpioNum}"
}

ledsSdSetLockLed () {
	ledsGpioSet "$ledGpioSdLock" "$1"
}

ledsSdSetUnlockLed () {
	ledsGpioSet "$ledGpioSdUnlock" "$1"
}

ledsSdSetSixOclockLed () {
	ledsGpioSet "$ledGpioSdSixOclock" "$1"
}

##################### WIFI ###########################

function ledsWifiSetLedsToNumConnectedToInterface {
	local iface="$1"
	local thisNumSta=$(iw dev $iface station dump | grep "^Station" | wc -l)
	ledsWifiSetLedsOn "${thisNumSta}"
}

ledsWifiSetSixOclockLed () {
	# register 0x93 corresponds to GPIO2 on the AXP209
	# LOW (0) = ON , FLOATING (1) = OFF
	if [[ $1 -gt 0 ]]; then
		i2cset -y -f 0 0x34 0x93 0x00
	else
		i2cset -y -f 0 0x34 0x93 0x01
	fi
}

ledsWifiSetLedsOn () {
	local numLeds="${1}"
	local ledNum=0
	for ledNum in $(seq 1 1 ${numLeds}); do
		ledsWifiSetLedOn "${ledNum}"
	done
	for ledNum in $(seq 7 -1 $(( ${numLeds} + 1 ))); do
		ledsWifiSetLedOff "${ledNum}"
	done
}

ledsWifiSetLedOn () {
	local ledNum=$1
	local gpioNum=${ledGpiosWifi[$((ledNum - 1))]}
	ledsGpioOn "${gpioNum}"
}

ledsWifiSetLedOff () {
	local ledNum=$1
	local gpioNum=${ledGpiosWifi[$((ledNum - 1))]}
	ledsGpioOff "${gpioNum}"
}


############# BATTERY ######################
ledsBatterySetSixOclockLed () {
	# register 0x92 corresponds to GPIO1 on the AXP209
	# HIGH (1) = ON , FLOATING (8) = OFF
	if [[ $1 -gt 0 ]]; then
		i2cset -y -f 0 0x34 0x92 0x01
	else
		i2cset -y -f 0 0x34 0x92 0x08
	fi
}

ledsBatterySetLedsOn () {
	local numLeds="${1}"
	local ledNum=0
	for ledNum in $(seq 1 1 ${numLeds}); do
		ledsBatterySetLedOn "${ledNum}"
	done
	for ledNum in $(seq 7 -1 $(( ${numLeds} + 1 ))); do
		ledsBatterySetLedOff "${ledNum}"
	done
}

ledsBatterySetLedOn () {
	local ledNum=$1
	local gpioNum=${ledGpiosBatt[$((ledNum - 1))]}
	ledsGpioOn "${gpioNum}"
}

ledsBatterySetLedOff () {
	local ledNum=$1
	local gpioNum=${ledGpiosBatt[$((ledNum - 1))]}
	ledsGpioOff "${gpioNum}"
}

ledsBatterySetLedsPercent () {
	battPct="${1}"
	numLedsOn=$((battPct *  7 / 100))
	ledsBatterySetLedsOn $numLedsOn
}

################# RINGS ###################
function ledsSetRingsAllLedOn {
	ledsSetRingsLed 'all' "$1" 'on'
}

function ledsSetRingsAllLedOff {
	ledsSetRingsLed 'all' "$1" 'off'
}

function ledsSetRingsAllLedsOn {
	ledsWifiSetLedsOn "$1"
	ledsBatterySetLedsOn "$1"
	ledsSdSetLedsOn "$1"
}

function ledsSetRingsLed {
	local ledRing="$1"
	local ledNum="$2"
	local onOff="$3"
	if [[ $onOff == 'on' ]]; then
		if [[ $ledRing == 'all' ]]; then
			ledsSdSetLedOn $ledNum
			ledsWifiSetLedOn $ledNum
			ledsBatterySetLedOn $ledNum
		elif [[ $ledRing == 'sd' ]]; then
			ledsSdSetLedOn $ledNum
		elif [[ $ledRing == 'wifi' ]]; then
			ledsWifiSetLedOn $ledNum
		elif [[ $ledRing == 'battery' ]]; then
			ledsBatterySetLedOn $ledNum
		fi
	elif [[ $onOff == 'off' ]]; then
		if [[ $ledRing == 'all' ]]; then
			ledsSdSetLedOff $ledNum
			ledsWifiSetLedOff $ledNum
			ledsBatterySetLedOff $ledNum
		elif [[ $ledRing == 'sd' ]]; then
			ledsSdSetLedOff $ledNum
		elif [[ $ledRing == 'wifi' ]]; then
			ledsWifiSetLedOff $ledNum
		elif [[ $ledRing == 'battery' ]]; then
			ledsBatterySetLedOff $ledNum
		fi
	elif [[ $onOff == 'numLedsOn' ]]; then
		if [[ $ledRing == 'all' ]]; then
			ledsSdSetLedsOn $ledNum
			ledsWifiSetLedsOn $ledNum
			ledsBatterySetLedsOn $ledNum
		elif [[ $ledRing == 'sd' ]]; then
			ledsSdSetLedsOn $ledNum
		elif [[ $ledRing == 'wifi' ]]; then
			ledsWifiSetLedsOn $ledNum
		elif [[ $ledRing == 'battery' ]]; then
			ledsBatterySetLedsOn $ledNum
		fi
	fi
}

################### FLASHING ##################
function ledsFlashLockLed {
	local numTimes=$1
	for (( i = 0; i < ${numTimes}; i++ )); do
		ledsSdSetLockLed 1
		sleep 0.25
		ledsSdSetLockLed 0
		sleep 0.25
	done
}

function ledsFlashWifiSixOclockLed {
	local numTimes=$1
	for (( i = 0; i < ${numTimes}; i++ )); do
		ledsWifiSetSixOclockLed 1
		sleep 0.25
		ledsWifiSetSixOclockLed 0
		sleep 0.25
	done
	# turn the wifi six-oclock back on if the wifi is running
	isActive=$( systemctl is-active rwo_wifi@wlan0Ap )
	if [[ $isActive == "active" ]]; then
		ledsWifiSetSixOclockLed 1
	fi
}

##################### PROGRESS ##################
function ledsSdDoProgressTransferBgStart {
	ledsSdDoProgressTransferBgStop
	ledsSdDoProgressTransfer "$1" "$2" "$3" "$4" "$5" &
	# save the pid of the bachground process
	PIDLEDSPROGRESS=$!
}

function ledsSdDoProgressTransferBgStop {
	[[ "${PIDLEDSPROGRESS}" != "" ]] && kill $PIDLEDSPROGRESS >/dev/null 2>&1
}

function ledsSdDoProgressTransfer {
	local destinationMountPoint="$1"
	local totalSizeInBytes="$2"
	local numberOfBlinks="$3"
	local delayInSeconds="$4"
	local ledRing="$5"
	if [[ $ledRing == '' ]]; then
		local ledRing='sd'
	fi
	local totalSizeTransferredInBytes=0
	local freeDiskSpaceInBytesOrig=$(df -B1 "${destinationMountPoint}" | tail -1 | awk '{print $4}')
	local lastLedsToTurnOn=0

	if [[ ${totalSizeInBytes} == 0 ]]; then
		ledsSdSetLedsOn 7
		return 1
	fi

	local ledsToTurnOn=0
	while true; do
		local freeDiskSpaceInBytesNow=$(df -B1 "${destinationMountPoint}" | tail -1 | awk '{print $4}')
		if [[ $freeDiskSpaceInBytesNow != '' ]]; then
			local totalSizeTransferredInBytes=$(( $freeDiskSpaceInBytesOrig - $freeDiskSpaceInBytesNow ))
			echo $(( ($totalSizeTransferredInBytes * 100) / $totalSizeInBytes )) > $LEDSPROGRESSFILE
			local ledsToTurnOn=$(( ($totalSizeTransferredInBytes * 100) / $totalSizeInBytes * 7 / 100 ))
			if [[ $lastLedsToTurnOn != $ledsToTurnOn ]]; then
				lastLedsToTurnOn=$ledsToTurnOn
				ledsSetRingsLed $ledRing $ledsToTurnOn 'numLedsOn'
			fi
		fi
		local blinkLed=$(( $ledsToTurnOn + 1 ))
		if [[ $blinkLed -gt 7 ]]; then
			blinkLed=7
		fi
		for (( i = 0; i < $numberOfBlinks; i++ )); do
			ledsSetRingsLed $ledRing $blinkLed 'off'
			sleep ${delayInSeconds}
			ledsSetRingsLed $ledRing $blinkLed 'on'
			sleep ${delayInSeconds}
		done
	done
}

function ledsSdDoFlashGreenLedForever {
	local delayInSeconds=$1
	while true; do
			ledsSdSetUnlockLed 1
			sleep ${delayInSeconds}
			ledsSdSetUnlockLed 0
			sleep ${delayInSeconds}
	done
}

function ledsSdShowFinalProgress {
	local successful=$1
	local attempted=$2
	ledsSdSetSixOclockLed 1
	if [[ ${attempted} == 0 ]]; then
		ledsSdSetLedsOn 7
	else
		local progressSdLeds=$(( (${successful} * 100) / ${attempted} * 7 / 100 ))
		if [[ ${progressSdLeds} == 0 ]]; then
			local progressSdLeds=1
		fi
		ledsSdSetLedsOn ${progressSdLeds}
	fi
}

function ledsFlashLoopStop {
	[[ "${PIDFLASHLED}" != "" ]] && kill $PIDFLASHLED >/dev/null 2>&1
}

function ledsFlashLoopStart {
	ledsFlashLoopStop
	ledsFlashLoop "$1" "$2" "$3" "$4" "$5" &
	# save the pid of the bachground process
	PIDFLASHLED=$!
}

function ledsFlashLoop {
	local delayInSeconds=$5
	while true; do
			$1 $2
			sleep ${delayInSeconds}
			$3 $4
			sleep ${delayInSeconds}
	done
}


function ledsWifiUpdateLoopStop {
	[[ "${PIDLEDSWIFIUPDATE}" != "" ]] && kill $PIDLEDSWIFIUPDATE >/dev/null 2>&1
}

function ledsWifiUpdateLoopStart {
	ledsFlashLoopStop
	ledsWifiUpdateLoop &
	# save the pid of the bachground process
	PIDLEDSWIFIUPDATE=$!
}

function ledsWifiUpdateLoop {
	# designed to run in the background to show progress of the update
	index=0
	while true; do
		ledsWifiSetLedOff "$index"
		index=$(( $index + 1 ))
		if [[ "${index}" -gt 7 ]]; then
			sleep 0.7
			index=1
		fi
		ledsWifiSetLedOn "$index"
		sleep 0.1
	done
}
