#!/bin/bash

. "/usr/bin/rwo_includeLogging.sh"
. "/usr/bin/rwo_includeLeds.sh"
. "/usr/bin/rwo_includeHelpers.sh"

############## FUNCTIONS #############
function checkWirelessNamesAndSetDefaultIfNeeded {
	# Check to make sure wifi has a proper ssid, and if not, assign _onExit
	rwo_system.sh setWiFiToNewDefaultsIfNeeded

	# Setup the bluetooth name ... some of the pilot run lightstreams had "1" as the bluetooth name, so correct for that
	deviceName=$(helperSqlite3 "/etc/lightstream/data/lsbt.sqlite3" "SELECT value FROM settings WHERE key = 'device_name'")
	if [[ "${deviceName}" == "1" ]] || [[ "${deviceName}" =~ ^GoWifi.org-[0-9a-fA-F]{4}$ ]]; then
		rwo_system.sh setBluetoothDeviceNameToDefault
	fi
}

################ MAIN #################
hwPlatform=$(/usr/bin/rwo_system.sh getHwPlatform)
systemMode=$(helperSqlite3 "/etc/lightstream/data/lswifi.sqlite3" "SELECT value FROM settings WHERE key = 'system_mode'")
rootFsDevice=$(helperGetRootFsDevice)
partuuidRootFs=$(helperGetPartUuidForDevice "$rootFsDevice")

helperCreateSystemTmpDir

# perform platform specific functions
rwoLog "initializing LEDs"
ledsAllInit

if [[ "$partuuidRootFs" == "aa1212cd-02" ]]; then
	rwoLog "detected mirror apply ... "
	# don't startup wifi, and don't startup button listener
	ledsSetRingsAllLedsOn 1
	# pause the udev rules for now
	udevadm control -s
else
	rwoLog "Normal boot mode ... "
	checkWirelessNamesAndSetDefaultIfNeeded
	ledsAllBlueOff
	ledsDance
	ledsBatterySetSixOclockLed 1
	# start to blink the wifi led to let the user know you plan to start up the wifi
	systemctl start rwo_ledBlink@wifiSixOclock
fi
