#!/bin/bash
# poll the state of the power button for the LightStream pocket

# include logging script to go to central log
#. /usr/bin/rwo_includeLogging.sh
. "/usr/bin/rwo_includeLeds.sh"

function setupSecondaryButton {
	btnGpio=192
	btnGpioDir="/sys/class/gpio/gpio${btnGpio}"
	if [[ ! -d "${btnGpioDir}" ]]; then
		echo "${btnGpio}" > "/sys/class/gpio/export"
		sleep 0.1
		echo "in" > "${btnGpioDir}/direction"
		sleep 0.1
		echo "both" > "${btnGpioDir}/edge"
		sleep 0.1
	fi
}

function setupAxp209Options {
	# enable the IRQ for the PEK (power-switch) on the AXP209
	i2cset -y -f 0 0x34 0x44 0x00
	# reset the IRQ register to 0
	i2cset -y -f 0 0x34 0x4C 0xFF
}

function showBatteryPctLeds {
	ledsBatterySetLedsOn "$(rwo_battery.sh getBatNumLeds)"
	sleep 1.5
	ledsBatterySetLedsOn "0"
}

function listenToButtons {
	# infinite loop
	lastTimeNs=$(date +"%s%N")
	while true; do
		pekStatus=$(( $( i2cget -y -f 0 0x34 0x4C ) & 0x60 ))
		secondaryStatus=$( cat "/sys/class/gpio/gpio192/value" )


		# HOLDING PEK ONLY
		if [[ $pekStatus == 32 ]] && [[ ${secondaryStatus} != 0 ]]; then
			battLeds=$(( ${battLeds} - 1 ))
			if [[ $battLeds == -1 ]]; then
				rwo_system.sh systemShutdownNow
			elif [[ $battLeds -lt 8 ]]; then
				ledsBatterySetLedsOn "${battLeds}"
			fi
			sleep 0.1
		# TODO .. holding both buttons for secure mode
		elif [[ $pekStatus == 32 ]] && [[ ${secondaryStatus} == 0 ]]; then
			command
		# HOLDING SECONDRY BUTTON ONLY
		elif [[ ${secondaryStatus} == 0 ]]; then
			wifiLeds=$(( ${wifiLeds} - 1 ))
			isActive=$( systemctl is-active rwo_wifi@wlan0Ap )
			if [[ $isActive == "active" ]]; then
				if [[ $wifiLeds == -1 ]]; then
					systemctl stop rwo_wifi@wlan0Ap
					ledsWifiSetLedsOn "0"
				elif [[ $wifiLeds -lt 8 ]] && [[ $wifiLeds -gt -1 ]]; then
					ledsWifiSetLedsOn "$wifiLeds"
				fi
			else
				if [[ $wifiLeds == -1 ]]; then
					systemctl start rwo_wifi@wlan0Ap
					ledsWifiSetLedsOn "0"
				elif [[ $wifiLeds -lt 8 ]] && [[ $wifiLeds -gt -1 ]]; then
					ledsWifiSetLedsOn $(( 7 - "$wifiLeds" ))
				fi
			fi

			# extra long wifi button hold = startup with no password
			if [[ $wifiLeds == -15 ]]; then
				touch '/tmp/rwo/rwo_wifi@wlan0Ap.startNoPassword'
				systemctl restart rwo_wifi@wlan0Ap
				ledsFlashWifiSixOclockLed 5
				ledsWifiSetSixOclockLed 1
			fi

			sleep 0.1
		# PEK WAS RELEASED
		elif [[ $pekStatus == 96 ]]; then
			i2cset -y -f 0 0x34 0x4C 0xFF
			showBatteryPctLeds &

		# BOTH KEYS ARE RELEASED
		else
			if [[ "${wifiLeds}" != "10" ]]; then
				isActive=$( systemctl is-active rwo_wifi@wlan0Ap )
				if [[ $isActive == "active" ]]; then
					# check if hostapd is running .... important b/c iw station dump does not report correctly
					# iw station dump keeps reporting connected stations even when wifi is down
					hostapdRunning=$(ps -aux | grep /usr/bin/hostap.* | wc -l)
					if [[ "${hostapdRunning}" -ge 2 ]]; then
						ledsWifiSetLedsToNumConnectedToInterface 'wlan0'
					else
						ledsWifiSetLedsOn "0"
					fi
				else
					ledsWifiSetLedsOn "0"
				fi
				wifiLeds=10
			fi
			battLeds=11
			bothLeds=10
			sleep 0.3 # give the cpu some time to sleep
		fi
	done
}


######## MAIN #########
setupSecondaryButton
setupAxp209Options
listenToButtons
