#!/usr/bin/python2

# ########## SETUP #############################
# wget https://pypi.python.org/packages/source/R/RPi.GPIO/RPi.GPIO-0.6.2.tar.gz
# tar -xvzf RPi.GPIO-0.6.2.tar.gz
# cd RPi.GPIO-0.6.2
# sudo python2 setup.py install

# wget https://pypi.python.org/packages/df/c9/d9da7fafaf2a2b323d20eee050503ab08237c16b0119c7bbf1597d53f793/pyserial-2.7.tar.gz#md5=794506184df83ef2290de0d18803dd11
# tar -xvzf pyserial-2.7.tar.gz
# cd pyserial-27
# sudo python2 setup.py install
# +++++++OR+++++++++
# pacman -Sy python2-pyserial

# packages on ARCH LINUX on the RPi get installed in: /usr/lib/python2.7/site-packages

# cat /boot/cmdline.txt
# >> root=/dev/mmcblk0p2 rw rootwait console=tty1 selinux=0 plymouth.enable=0 smsc95xx.turbo_mode=N dwc_otg.lpm_enable=0 elevator=noop

import time
import sqlite3
import os
import fcntl
import sys
import logging
import logging.handlers
import signal
from datetime import date
from datetime import datetime
from datetime import timedelta
import re


G_LOGGER = None
G_SERIAL = None
G_GPIO_CLEANEDUP = False  # set to true when the GPIO has been cleaned up
G_SHOULD_LOOP = True  # main loop continues as long as True
G_SERIALDEVICE = "/dev/smartone"

G_GPIOTX = 8
G_GPIORX = 10
G_GPIOHANDSHAKE = 12
# GPIO 14 is GROUND
G_GPIOMONITOR = 16

G_DB_WIFI = '/etc/lightstream/data/lswifi.sqlite3'
G_DB_USB = '/etc/lightstream/data/lssd.sqlite3'
G_DB_SATCOM = '/etc/lightstream/data/satellite.sqlite3'

try:
	import RPi.GPIO as GPIO
except RuntimeError:
	print ("Error importing RPI.GPIO. May be because you need sudo privilages")

try:
	import serial
except RuntimeError:
	print ("Error importing the serial modult. Try installing the pacman package python2-pyserial")


# returns the day of the week as a number
# Monday is 0 and Sunday is 6 ...
def getWeekDayNumber():
	return datetime.today().weekday()


def sqlInsertSatcomInteractionStats(date_time, msg_tx, msg_rx, msg_type, msg_description):
	try:
		global G_DB_SATCOM
		conn = sqlite3.connect(G_DB_SATCOM)
		conn.execute("INSERT INTO satellite_interaction_stats (date_time, msg_tx, msg_rx, msg_type, msg_description) VALUES (?,?,?,?,?)", (date_time, msg_tx, msg_rx, msg_type, msg_description))
		conn.commit()
		conn.close()
	except Exception as e:
		printAndLogException(e)


# return the datetime to be used by sqlite3 "YYYY-MM-DD HH:MM:SS.sss"
def sqlGetDateTimeStringUTCNow():
	return datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]  # trim last 3 digits for .sss


# return a tuple of SQL timestamps YYYY-MM-DD hh:mm:ss.sss
def sqlGetStatsMsgDateRange(offset1, offset2):
	dates = (None, None)
	try:
		global G_DB_SATCOM
		conn = sqlite3.connect(G_DB_SATCOM)
		cur = conn.cursor()

		cur.execute("SELECT strftime('%Y-%m-%d 00:00:00.000', 'now', '"+offset1+"')")
		date_1 = cur.fetchone()[0]

		cur.execute("SELECT strftime('%Y-%m-%d 23:59:59.999', 'now', '"+offset2+"')")
		date_2 = cur.fetchone()[0]

		dates = (date_1, date_2)
		conn.close()

	except Exception as e:
		printAndLogException(e)

	finally:
		return dates


# return a tuple of UNIX timestamps
def sqlTimeToUnixTime(sqlTime):
	timestamp = None
	try:
		global G_DB_SATCOM
		conn = sqlite3.connect(G_DB_SATCOM)
		cur = conn.cursor()
		cur.execute("SELECT strftime('%s', '"+sqlTime+"')")  # convert to unix timestamp
		timestamp = cur.fetchone()[0]
		conn.close()

	except Exception as e:
		printAndLogException(e)

	finally:
		return timestamp


def sqlGetSetting(key):
	value = None
	try:
		global G_DB_SATCOM
		conn = sqlite3.connect(G_DB_SATCOM)
		cur = conn.cursor()
		cur.execute("SELECT value FROM settings WHERE key='"+key+"'")
		value = cur.fetchone()[0]
		conn.close()
	except Exception as e:
		printAndLogException(e)

	finally:
		return value


def sqlUpdateSetting(key, value):
	try:
		global G_DB_SATCOM
		conn = sqlite3.connect(G_DB_SATCOM)
		conn.execute("UPDATE settings SET value='%s' WHERE key='%s'" % (value, key))
		conn.commit()
		conn.close()
	except Exception as e:
		printAndLogException(e)


# 2 offsets ... for example
# for yesterdays stats ("-1 days", "-1 days")
# for last weeks satas ("-7 days", "-1 days")
def getStatsMsgStrPayload(offsetBeginning, offsetEnd):
	msgBody = None
	try:

		global G_DB_WIFI
		global G_DB_BT
		global G_DB_USB

		wifi_conn = sqlite3.connect(G_DB_WIFI)
		cur = wifi_conn.cursor()

		# get date ranges for yesterdays stats
		dates = sqlGetStatsMsgDateRange(offsetBeginning, offsetEnd)
		date_1 = dates[0]
		date_2 = dates[1]
		unixepoch_1 = sqlTimeToUnixTime(date_1)
		unixepoch_2 = sqlTimeToUnixTime(date_2)

		cur.execute("SELECT count(interaction_id) FROM wifi_interaction_stats WHERE date_time BETWEEN '%s' AND '%s' AND interaction_type='download'" % (date_1, date_2))
		wifi_downloads = cur.fetchone()[0]

		# get the number of wifi streams
		cur.execute("SELECT count(interaction_id) FROM wifi_interaction_stats WHERE date_time BETWEEN '%s' AND '%s' AND interaction_type='stream'" % (date_1, date_2))
		wifi_streams = cur.fetchone()[0]
		wifi_conn.close()

		# get the number of bluetooth successful pushes
		btdb_conn = sqlite3.connect(G_DB_BT)
		curbt = btdb_conn.cursor()

		curbt.execute("SELECT count(interaction_id) FROM bluetooth_interaction_stats WHERE date_time BETWEEN '%s' AND '%s' AND interaction_type='success'" % (date_1, date_2))
		bluetooth_xfers = curbt.fetchone()[0]
		btdb_conn.close()

		# USB Stats
		usbdb_conn = sqlite3.connect(G_DB_USB)
		curusb = usbdb_conn.cursor()
		curusb.execute("SELECT count(interaction_id) FROM sd_interaction_stats WHERE date_time BETWEEN '%s' AND '%s' AND status='success'" % (date_1, date_2))
		usb_downloads = curusb.fetchone()[0]
		usbdb_conn.close()

		# TODO
		file_id_1 = 0
		file_xfers_1 = 0
		file_id_2 = 0
		file_xfers_2 = 0
		file_id_3 = 0
		file_xfers_3 = 0

		msgBody = chr(4 << 2)  # the format-id 4 for LightStream
		msgBody += chr(0)  # the version identifier ... means this type of lightstream msg
		msgBody += longToStrMsbLsb(long(unixepoch_1))  # the start of stats range
		msgBody += longToStrMsbLsb(long(unixepoch_2))  # the end of stats range
		msgBody += intToStrMsbLsb(wifi_downloads)
		msgBody += intToStrMsbLsb(wifi_streams)
		msgBody += intToStrMsbLsb(bluetooth_xfers)
		msgBody += intToStrMsbLsb(usb_downloads)
		msgBody += longToStrMsbLsb(file_id_1)
		msgBody += intToStrMsbLsb(file_xfers_1)
		msgBody += longToStrMsbLsb(file_id_2)
		msgBody += intToStrMsbLsb(file_xfers_2)
		msgBody += longToStrMsbLsb(file_id_3)
		msgBody += intToStrMsbLsb(file_xfers_3)

	except Exception as e:
		printAndLogException(e)

	finally:
		return msgBody


# 2 offsets ... for example
# for yesterdays stats ("-1 days", "-1 days")
# for last weeks satas ("-7 days", "-1 days")
def getStatsMsgStrPayload1():
	msgBody = None
	try:

		global G_DB_WIFI
		global G_DB_BT
		global G_DB_USB

		wifi_conn = sqlite3.connect(G_DB_WIFI)
		cur = wifi_conn.cursor()
		# get date ranges for yesterdays stats
		yesterday = datetime.utcnow() - timedelta(1)
		date_1 = yesterday.strftime("%Y-01-01 00:00:00.000")  # go from the first of the year
		date_2 = yesterday.strftime("%Y-%m-%d 23:59:59.999")  # to yesterday at midnight
		unixepoch_1 = sqlTimeToUnixTime(date_1)
		unixepoch_2 = sqlTimeToUnixTime(date_2)

		cur.execute("SELECT count(interaction_id) FROM wifi_interaction_stats WHERE date_time BETWEEN '%s' AND '%s' AND interaction_type='download'" % (date_1,  date_2))
		wifi_downloads = cur.fetchone()[0]

		# get the number of wifi streams
		cur.execute("SELECT count(interaction_id) FROM wifi_interaction_stats WHERE date_time BETWEEN '%s' AND '%s' AND interaction_type='stream'" % (date_1, date_2))
		wifi_streams = cur.fetchone()[0]

		# get the number of bluetooth successful pushes
		btdb_conn = sqlite3.connect(G_DB_BT)
		curbt = btdb_conn.cursor()

		curbt.execute("SELECT count(interaction_id) FROM bluetooth_interaction_stats WHERE date_time BETWEEN '%s' AND '%s' AND interaction_type='success'" % (date_1, date_2))
		bluetooth_xfers = curbt.fetchone()[0]

		# USB Stats
		usbdb_conn = sqlite3.connect(G_DB_USB)
		curusb = usbdb_conn.cursor()
		curusb.execute("SELECT count(interaction_id) FROM sd_interaction_stats WHERE date_time BETWEEN '%s' AND '%s' AND status='success'" % (date_1, date_2))
		usb_downloads = curusb.fetchone()[0]

		# TODO
		file_id_1 = 0
		file_xfers_1 = 0
		file_id_2 = 0
		file_xfers_2 = 0
		file_id_3 = 0
		file_xfers_3 = 0

		msgBody = chr(4 << 2)  # the format-id 4 for LightStream
		msgBody += chr(1)  # the version identifier ... means this type of lightstream msg
		msgBody += longToStrMsbLsb(long(unixepoch_2))  # the start of stats range
		msgBody += intToStrMsbLsb(wifi_downloads)
		msgBody += intToStrMsbLsb(wifi_streams)
		msgBody += intToStrMsbLsb(bluetooth_xfers)
		msgBody += intToStrMsbLsb(usb_downloads)
		msgBody += longToStrMsbLsb(file_id_1)
		msgBody += intToStrMsbLsb(file_xfers_1)
		msgBody += longToStrMsbLsb(file_id_2)
		msgBody += intToStrMsbLsb(file_xfers_2)
		msgBody += longToStrMsbLsb(file_id_3)
		msgBody += intToStrMsbLsb(file_xfers_3)

	except Exception as e:
		printAndLogException(e)

	finally:
		return msgBody


def getRawMessageStringWithPayload(payload):
	messageBody = None
	try:
		global G_LOGGER
		# get the message length
		messageLenInt = len(payload)

		# the message is too large
		if messageLenInt > 53:
			G_LOGGER.log(logging.WARN, "message payload is too large > 32B")
			return 0

		messageLenChr = chr((messageLenInt+5))

		messageBody = "\xAA"+messageLenChr+"\x27"+payload
		messageBodyHexStr = messageBody.encode("hex")
		crcInt = crc16(messageBodyHexStr)
		crcChrLsb = chr((crcInt & 0x00FF))
		crcChrMsb = chr((crcInt & 0xFF00) >> 8)
		messageBody += (crcChrLsb+crcChrMsb)

	except Exception as e:
		printAndLogException(e)

	finally:
		return messageBody


# returns the entire getID message to be sent to the modem
# the modem should respond with it's ESN number
def getIdMessage():
	return "\xAA\x05\x01\x50\xD5"


# the message payload string can be a maximum of XX bytes
# pass in the serial object and the complete message string 0xAA -> [CRC.LSB+CRC.MSB]
# return the response from the modem
def serialTransmitMessage(ser, strMessage):
	out = ''  # the reply from the modem
	try:
		global G_LOGGER
		G_LOGGER.log(logging.INFO, "will attempt to send message payload:%s" % (str(strMessage.encode("hex"))))
		# ser.flushInput() # clear the input buffer
		# ser.flushOutput() # clear the output buffer
		ser.write(strMessage)
		ser.flush()  # write the message now
		# GPIO.setup(G_GPIOHANDSHAKE,GPIO.IN) # handshake high-z (impedence)
	except Exception as e:
		printAndLogException(e)
		exit_code = 4


# will look for the next preamble (AA) and read message from there...
# returns the message after reading
def serialReadNextMessage(ser):
	msg = ''
	try:
		G_LOGGER.log(logging.INFO, "will attempt to read next message from satellite modem")
		while (True):
			if (ser.inWaiting() < 1):
				G_LOGGER.log(logging.WARN, "there are no bytes waiting on the serial connection")
				break

			msg = ser.read(1)

			if (len(msg) == 0):
				G_LOGGER.log(logging.WARN, "got zero length when trying to read preamble byte from input")
				break

			if (msg != "\xAA"):  # didn't find the preamble yet
				G_LOGGER.log(logging.WARN, "got a non-preamble byte from the input:%s" % (str(msg.encode('hex'))))
				msg = ''
				continue

			G_LOGGER.log(logging.DEBUG, "got a preamble byte from the input:%s" % (str(msg.encode('hex'))))

			l = ser.read(1)  # now try to get the length
			msg += l

			if (len(l) < 1):
				G_LOGGER.log(logging.WARN, "got no length while reading the message:%s" % (str(l.encode('hex'))))
				msg = ''
				break

			bytes_left = ord(l) - 2  # because you have already read 2 bytes of the message

			the_rest = ''
			the_rest = ser.read(bytes_left)

			msg += the_rest

			if (len(the_rest) != bytes_left):
				G_LOGGER.log(logging.WARN, "missing the rest of the body:%s" % (str(msg.encode('hex'))))
				msg = ''

			break
	except Exception as e:
		printAndLogException(e)
		exit_code = 5
	finally:
		G_LOGGER.log(logging.INFO, "read entire message from modem:%s" % (str(msg.encode('hex'))))
		return msg


# give it a hex string representation (eg. crc16("AA0501") = 0xd550)
# You can use "hello".encode("hex") = '68656c6c6f', and then '68656c6c6f'.decode("hex") = hello
def crc16(data, bits=8):
	crc = 0xFFFF
	for op, code in zip(data[0::2], data[1::2]):
		thisSlice = int(op+code, 16)
		crc = crc ^ thisSlice
		for bit in range(0, bits):
			if (crc & 0x0001) == 0x0001:
				crc = ((crc >> 1) ^ 0x8408)
			else:
				crc = crc >> 1
	crc = ~crc
	# unsign the result
	crc = crc & 0xFFFF
	return crc


# will swap the least and most signifigant Byte in a 16bit integer
# eg. swap16(0x2211) = 0x1122
def swap16(int16):
	return ((int16 & 0x00FF) << 8) | ((int16 & 0xFF00) >> 8)


def intToStrMsbLsb(int16):
	chrLsb = chr((int16 & 0x00FF))
	chrMsb = chr((int16 & 0xFF00) >> 8)
	return (chrMsb+chrLsb)


def longToStrMsbLsb(lng32):
	chr1 = chr((lng32 & 0x000000FF))
	chr2 = chr((lng32 & 0x0000FF00) >> 8)
	chr3 = chr((lng32 & 0x00FF0000) >> 16)
	chr4 = chr((lng32 & 0xFF000000) >> 24)
	return (chr4+chr3+chr2+chr1)


def getLogger(logFile='/var/log/lightstream/satellite.log', loggerName='satellite', level='DEBUG'):
	logger = None
	try:
		logger = logging.getLogger(loggerName)
		logger.setLevel(level)
		handler = logging.handlers.RotatingFileHandler(logFile, maxBytes=1000000, backupCount=5)
		handler.setLevel(level)
		formatter = logging.Formatter("%(asctime)s %(levelname)s %(lineno)d %(message)s")
		handler.setFormatter(formatter)
		logger.addHandler(handler)
	except Exception as e:
		printAndLogException(e)
	return logger


def printAndLogException(e):
	exec_type, exec_obj, tb = sys.exc_info()
	lineno = tb.tb_lineno
	global G_LOGGER
	msg = 'exception:%s line:%s' % (str(e), str(lineno))
	print msg
	if G_LOGGER:
		G_LOGGER.log(logging.ERROR, msg)


def signalHandler(sigNum, frame):
	global G_LOGGER
	logString = "signal caught:" + str(sigNum) + " frame:" + str(frame)
	print logString
	if G_LOGGER:
		G_LOGGER.log(logging.INFO, logString)
	cleanup()


def cleanup():

	global G_LOGGER
	global G_SERIAL
	global G_GPIO_CLEANEDUP
	G_LOGGER.log(logging.INFO, "cleanup function called ... will clean up")

	# if G_LOGGER != None:
	# G_LOGGER = None

	if G_SERIAL:
		G_LOGGER.log(logging.INFO, "cleaning up serial object")
	# G_SERIAL.flushInput()
	# G_SERIAL.flushOutput()
	# G_SERIAL.flush()
		G_SERIAL.close()
	# G_SERIAL.__del__()
		G_SERIAL = None

	# if G_GPIO_CLEANEDUP != True:
	# G_GPIO_CLEANEDUP = True
	# GPIO.cleanup()


def pathExists(path):
	exists = True
	try:
		os.stat(path)
	except Exception as e:
		exists = False
	finally:
		return exists


def main():
	# define an exit code
	exit_code = 0

	try:
		# try to get a lock on the file
		# fd = open('/var/lock/satellite.lock', 'w')
		# try:
		# 	fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
		# except IOError:
		# 	print 'satellite running already. exiting'
		# 	os.sys.exit(0)

		# now time to register signal handler(s)
		# signal.signal(signal.SIGTERM, signalHandler)
		# signal.signal(signal.SIGHUP, signalHandler)

		global G_LOGGER
		global G_SERIALDEVICE
		global G_SERIAL
		global G_SHOULD_LOOP
		G_LOGGER = getLogger()
		G_LOGGER.log(logging.INFO, "will attempt to use serial device:%s" % (str(G_SERIALDEVICE)))

		# talk/listen on the serial port at 9600baud rate
		G_SERIAL = serial.Serial(G_SERIALDEVICE, 9600, timeout=60)
		# G_SERIAL.rts = 0
		# G_SERIAL.dtr = 0
		G_SHOULD_LOOP = True

		G_LOGGER.log(logging.INFO, "will start satellite main loop")
		while G_SHOULD_LOOP:
			# don't loop
			G_SHOULD_LOOP = False

			transmit_test = sqlGetSetting("transmit_test")
			if (transmit_test != "true"):
				# check the tx frequency
				txfrequency = sqlGetSetting("txfrequency")
				if (txfrequency != "1 day" and txfrequency != "7 days"):
					G_LOGGER.log(logging.ERROR, "unkonwn txfrequency:%s defaulting to '1 day'" % (str(txfrequency)))
					txfrequency = "1 day"

				# only transmit weekly transmissions on Sunday, and on 01/01
				weekDayNumber = getWeekDayNumber()
				if (txfrequency == "7 days" and weekDayNumber != 6):
					today = datetime.utcnow().strftime("%m-%d")
					if today != "01-01":
						G_LOGGER.log(logging.INFO, "transmit weekly stats on sunday (day 6), and 01-01 only, today is day %s" % (str(weekDayNumber)))
						exit_code = 7
			else:
				G_LOGGER.log(logging.INFO, "will attempt to send test message")
				sqlUpdateSetting("transmit_test", "false")
				txfrequency = "test"

			# if so far there is no reason to exit ...
			if (exit_code == 0):
				# msg_tx = getIdMessage()
				# serialTransmitMessage(G_SERIAL, msg_tx)
				# G_SERIAL.flushOutput()
				# G_SERIAL.flushInput()
				# time.sleep(2)
				# resp =  serialReadNextMessage(G_SERIAL)
				# date_time = sqlGetDateTimeStringUTCNow()
				msgPayload = getStatsMsgStrPayload1()
				rawMessage = getRawMessageStringWithPayload(msgPayload)
				msg_tx = rawMessage.encode("hex")
				G_LOGGER.log(logging.INFO, "will attempt to send message:\"%s\" to satellite modem" % (str(msg_tx)))
				serialTransmitMessage(G_SERIAL, rawMessage)

				# wait 10s because if I don't it does not tend to go well with receiving the message
				time.sleep(10)

				resp = serialReadNextMessage(G_SERIAL)
				msg_rx = resp.encode("hex")
				sqlTime = sqlGetDateTimeStringUTCNow()
				G_LOGGER.log(logging.INFO, "received a response from the satellite:\"%s\" at time:\"%s\"" % (str(msg_rx), str(sqlTime)))

				response = "unknown"
				if msg_rx == "aa05276491":  # successful message
					response = "success"
					G_LOGGER.log(logging.INFO, "the transmission to the satellite modem was a success")
				elif re.match('^aa05ff.*', msg_rx):
					response = "fail"
					G_LOGGER.log(logging.ERROR, "the transmission to the satellite modem was a failure")
				elif len(str(msg_rx)) > 0:
					response = "unknown:" + str(msg_rx)
					G_LOGGER.log(logging.ERROR, "the transmission to the satellite modem received an unknown response from the modem:\"%s\"" % (str(msg_rx)))
				else:
					response = "empty"
					G_LOGGER.log(logging.ERROR, "the transmission to the satellite modem received an empty response, please check to make sure the modem is connected, and replace batteries if needed")
				# write to the satellite database
				sqlInsertSatcomInteractionStats(sqlTime, msg_tx, msg_rx, "statsmsg-"+txfrequency, "response="+response+";")
				# ACK = 0xAA05276491
				# NAK = 0xAA05FFXXXX
				# G_SHOULD_LOOP = False
	except KeyboardInterrupt as e:
		msg = 'KeyboardInterrupt exception:%s' % (str(e))
		print msg
		G_LOGGER.log(logging.ERROR, msg)
		exit_code = 2

	except Exception as e:
		printAndLogException(e)
		exit_code = 3

	finally:
		msg = 'Exiting LightStream Satellite Engine ... exit_code:%s' % (str(exit_code))
		cleanup()
		sys.exit(exit_code)


# ############ MAIN #####################################
if __name__ == '__main__':
	main()
