#!/bin/bash

# README ... use this script to do things that are firmware update-related
# include logging
LOG_SILENT_STDOUT=1
. /usr/bin/rwo_includeLogging.sh
. /usr/bin/rwo_includeHelpers.sh
. /usr/bin/rwo_includeLeds.sh


# check for which hardware you are running on
hwPlatform=$(rwo_system.sh getHwPlatform)

function runUpdateFileWithPath {
	firmwareTarballPath="${1}"
	rwoLog "Will attempt to run firmware update file at:${firmwareTarballPath}"
	if [[ ! -f "${firmwareTarballPath}" ]]; then
		rwoLog "ERROR: firmware update file does not exist at:${firmwareTarballPath}"
		return 1
	fi

	rwoLog "check to see if this update has been run already"
	updateFirmwareSig=$(md5sum "${firmwareTarballPath}")
	updateFirmwareSigTmpFile="/tmp/rwo_updateFirmware.md5"
	touch "${updateFirmwareSigTmpFile}"
	lastUpdateFirmwareSig=$(cat "${updateFirmwareSigTmpFile}")

	if [[ "${lastUpdateFirmwareSig}" == "${updateFirmwareSig}" ]]; then
		rwoLog "WILL EXIT .. firmware update file has already been run"
		return 0
	else
		echo "${updateFirmwareSig}" > "${updateFirmwareSigTmpFile}"
	fi

	# go ahead and start the update leds
	ledsWifiUpdateLoopStart

	rwoLog "Will check integrity of firmware update file at:${firmwareTarballPath}"
	gunzip -t "${firmwareTarballPath}"
	if [[ $? -ne 0 ]]; then
		#rm -f "${firmwareTarballPath}"
		rwoLog "ERROR: firmware gzip file has bad integrity check at:${firmwareTarballPath}"
		return 2
	fi

	tempFwDir="/tmp/UpdateFirmware"
	rwoLog "Remove any old UpdateFirmware directory"
	#rm -Rf "${tempFwDir}"
	rwoLog "Create temporary firmware update directory"
	mkdir -p "${tempFwDir}"
	trap 'rm -rf "${tempFwDir}"' EXIT INT HUP TERM
	rwoLog "Extracting firmware files to temporary directory..."
	tar --touch -xzf "${firmwareTarballPath}" -C "${tempFwDir}"
	rwoLog "Remove firmware tarball:${firmwareTarballPath}"
	#rm -f "${firmwareTarballPath}"
	rwoLog "Attempting to initiate firmware update..."
	# stop the update leds, and the rwo_update.sh should have it's own leds too..
	ledsWifiUpdateLoopStop
	# Replace current shell with the specified program.
	exec "${tempFwDir}/usr/bin/rwo_update.sh"
	updateExitCode=$?
	rwoLog "Remove firmware tarball directory"
	rm -Rf "${tempFwDir}"/*
	if [[ ${updateExitCode} -ne 0 ]]; then
		rwoLog "ERROR: Unable to initiate firmware update. Please confirm the file is a firmware update file."
		return 5
	fi
}

# TODO ... at a later date think about including this function
function downloadUpdateFromRenewAndRun {
	command
}

function findAndRunAdminPanelUpload {
	rwoLog "Check for potential firmware update file in admin panel uploads directory"
	wifiFwUploadDir="/srv/http/htdocs/uploads/firmware/"
	# try to find a file ending in .tar.gz in the firmware uploads directory
	firmwareTarballPath=$(find "${wifiFwUploadDir}" -regex .*\.tar\.gz$ | head -n 1)
	if [[ $(echo "${firmwareTarballPath}" | wc -w) -gt 0 ]]; then
		runUpdateFileWithPath "${firmwareTarballPath}"
		rm -f "${firmwareTarballPath}"
		return $?
	else
		rwoLog "ERROR: no potential firmware update file was found in the admin panel uploads dir"
		return 5
	fi
}

function _onExit {
	ledsWifiUpdateLoopStop
	ledsWifiSetLedsToNumConnectedToInterface "wlan0"
}

	##################### MAIN ###################
	# this allows the first argument to be a function
	# the function is called with the other arguments after the function
	trap '_onExit' EXIT INT HUP TERM
	helperCallFirstArgIfFunctionAndExit $@
	# check to see if the first argument is a firmware update file, if so, lets try to run it
	if [[ -f "${1}" ]]; then
		runUpdateFileWithPath "${1}"
		ledsWifiUpdateLoopStop
	fi
	rwoLog "ERROR: unkonwn command \"$@\""
